Package: coresynth
Title: Fast and Unified Synthetic Control Methods
Version: 0.2.0
Authors@R: 
    person("Yosuke Abe", role = c("aut", "cre"),
           email = "yosuke.abe0507@gmail.com")
Description: A unified 'Formula' interface to the Synthetic Control Method
    (SCM) and related panel-data causal inference estimators: Synthetic
    Difference-in-Differences (SDID), Generalized Synthetic Control (GSC),
    Matrix Completion (MC), Time-Aware Synthetic Control (TASC), and Synthetic
    Interventions (SI), together with an experimental-design variant.
    Computational bottlenecks (quadratic programming, singular value
    decomposition, and Kalman filtering) are implemented in 'C++' via
    'RcppArmadillo'. Methods are described in Abadie, Diamond and Hainmueller
    (2010) <doi:10.1198/jasa.2009.ap08746>, Arkhangelsky, Athey, Hirshberg,
    Imbens and Wager (2021) <doi:10.1257/aer.20190159>, Xu (2017)
    <doi:10.1017/pan.2016.2>, Athey, Bayati, Doudchenko, Imbens and Khosravi
    (2021) <doi:10.1080/01621459.2021.1891924>, and Agarwal, Shah and Shen
    (2025) <doi:10.1287/opre.2025.1590>.
License: MIT + file LICENSE
URL: https://github.com/yo5uke/coresynth, https://yo5uke.com/coresynth/
BugReports: https://github.com/yo5uke/coresynth/issues
Encoding: UTF-8
Config/roxygen2/markdown: TRUE
RoxygenNote: 8.0.0
Depends: R (>= 4.1.0)
LinkingTo: Rcpp, RcppArmadillo
Imports: Rcpp, Formula, ggplot2, broom, jsonlite
Suggests: testthat (>= 3.0.0), knitr, rmarkdown
VignetteBuilder: knitr
Config/testthat/edition: 3
NeedsCompilation: yes
Packaged: 2026-06-06 07:28:44 UTC; yo5uk
Author: Yosuke Abe [aut, cre]
Maintainer: Yosuke Abe <yosuke.abe0507@gmail.com>
Repository: CRAN
Date/Publication: 2026-06-12 10:10:02 UTC
Built: R 4.5.2; aarch64-apple-darwin20; 2026-06-12 10:45:23 UTC; unix
Archs: coresynth.so.dSYM

Package: coresynth
Title: Fast and Unified Synthetic Control Methods
Version: 0.2.0
Authors@R: 
    person("Yosuke Abe", role = c("aut", "cre"),
           email = "yosuke.abe0507@gmail.com")
Description: A unified 'Formula' interface to the Synthetic Control Method
    (SCM) and related panel-data causal inference estimators: Synthetic
    Difference-in-Differences (SDID), Generalized Synthetic Control (GSC),
    Matrix Completion (MC), Time-Aware Synthetic Control (TASC), and Synthetic
    Interventions (SI), together with an experimental-design variant.
    Computational bottlenecks (quadratic programming, singular value
    decomposition, and Kalman filtering) are implemented in 'C++' via
    'RcppArmadillo'. Methods are described in Abadie, Diamond and Hainmueller
    (2010) <doi:10.1198/jasa.2009.ap08746>, Arkhangelsky, Athey, Hirshberg,
    Imbens and Wager (2021) <doi:10.1257/aer.20190159>, Xu (2017)
    <doi:10.1017/pan.2016.2>, Athey, Bayati, Doudchenko, Imbens and Khosravi
    (2021) <doi:10.1080/01621459.2021.1891924>, and Agarwal, Shah and Shen
    (2025) <doi:10.1287/opre.2025.1590>.
License: MIT + file LICENSE
URL: https://github.com/yo5uke/coresynth, https://yo5uke.com/coresynth/
BugReports: https://github.com/yo5uke/coresynth/issues
Encoding: UTF-8
Config/roxygen2/markdown: TRUE
RoxygenNote: 8.0.0
Depends: R (>= 4.1.0)
LinkingTo: Rcpp, RcppArmadillo
Imports: Rcpp, Formula, ggplot2, broom, jsonlite
Suggests: testthat (>= 3.0.0), knitr, rmarkdown
VignetteBuilder: knitr
Config/testthat/edition: 3
NeedsCompilation: yes
Packaged: 2026-06-06 07:28:44 UTC; yo5uk
Author: Yosuke Abe [aut, cre]
Maintainer: Yosuke Abe <yosuke.abe0507@gmail.com>
Repository: CRAN
Date/Publication: 2026-06-12 10:10:02 UTC
Built: R 4.6.0; aarch64-apple-darwin23; 2026-06-12 11:05:05 UTC; unix
Archs: coresynth.so.dSYM

# Generated by roxygen2: do not edit by hand

S3method(augment,coresynth)
S3method(glance,coresynth)
S3method(glance,coresynth_inference)
S3method(plot,coresynth)
S3method(plot,scm_design)
S3method(print,conformal_inference)
S3method(print,coresynth)
S3method(print,coresynth_inference)
S3method(print,pred_spec)
S3method(print,scm_design)
S3method(print,sdid_inference)
S3method(summary,coresynth)
S3method(summary,scm_design)
S3method(tidy,coresynth)
S3method(tidy,coresynth_inference)
export(augment_scm)
export(conformal_inference)
export(export_json)
export(gsc_boot)
export(gsc_ife_cpp)
export(gsc_inference)
export(kalman_smoother_cpp)
export(mspe_ratio_pval)
export(pred)
export(scm_design)
export(scm_fit)
export(scm_inner_weights_cpp)
export(scm_placebo_cpp)
export(scm_weights_cpp)
export(sdid_estimate_cpp)
export(sdid_inference)
export(sdid_placebo_cpp)
export(sdid_time_weights_cpp)
export(sdid_unit_weights_cpp)
export(si_inference)
export(si_pcr_cpp)
export(soft_impute_cpp)
export(tensor_unfold_cpp)
import(ggplot2)
importFrom(Rcpp,sourceCpp)
importFrom(broom,augment)
importFrom(broom,glance)
importFrom(broom,tidy)
importFrom(stats,complete.cases)
importFrom(stats,median)
importFrom(stats,optim)
importFrom(stats,pnorm)
importFrom(stats,qnorm)
importFrom(stats,quantile)
importFrom(stats,reorder)
importFrom(stats,rnorm)
importFrom(stats,sd)
importFrom(stats,setNames)
importFrom(stats,var)
importFrom(utils,combn)
importFrom(utils,packageVersion)
useDynLib(coresynth, .registration = TRUE)

<!-- README.md is generated from README.Rmd. Please do not edit README.md directly. -->

# coresynth <img src="man/figures/logo.png" align="right" height="140" />

<!-- badges: start -->

[![R-CMD-check](https://github.com/yo5uke/coresynth/actions/workflows/R-CMD-check.yaml/badge.svg)](https://github.com/yo5uke/coresynth/actions/workflows/R-CMD-check.yaml)
[![License:
MIT](https://img.shields.io/badge/License-MIT-yellow.svg)](https://opensource.org/licenses/MIT)
[![Lifecycle:
experimental](https://img.shields.io/badge/lifecycle-experimental-orange.svg)](https://lifecycle.r-lib.org/articles/stages.html#experimental)
<!-- badges: end -->

**coresynth** is a high-performance R package that provides six causal
inference methods for panel data through a unified formula interface.
All core optimizations (QP solving, SVD, Kalman filtering) are
implemented in C++ via RcppArmadillo, so estimation stays fast even on
larger donor pools (see the [Performance](#performance) section for
timings).

## Installation

``` r
# From GitHub (requires Rtools on Windows, Xcode on macOS)
pak::pak("yo5uke/coresynth")

# Via devtools
devtools::install_github("yo5uke/coresynth")
```

## Quick Start

``` r
library(coresynth)

# Generate a balanced panel (10 units, 20 periods, true ATT = 2.0)
set.seed(42)
N <- 10; TT <- 20; T_pre <- 10
f   <- cumsum(rnorm(TT, 0, 0.5))
lam <- rnorm(N, 1, 0.3)
dat <- expand.grid(time = seq_len(TT), id = paste0("u", seq_len(N)))
dat$y <- as.vector(outer(f, lam)) + rnorm(nrow(dat), 0, 0.3)
dat$d <- as.integer(dat$id == "u1" & dat$time > T_pre)
dat$y[dat$d == 1] <- dat$y[dat$d == 1] + 2.0   # true ATT = 2

# Run all six methods
methods <- c("scm", "sdid", "gsc", "mc", "tasc", "si")
fits    <- lapply(methods, \(m) scm_fit(y ~ d | id + time, data = dat, method = m))
names(fits) <- methods

# Compare ATT estimates (true value = 2.0)
data.frame(
  method   = methods,
  estimate = round(sapply(fits, `[[`, "estimate"), 3)
)
#>      method estimate
#> scm     scm    2.271
#> sdid   sdid    2.150
#> gsc     gsc    2.255
#> mc       mc    2.696
#> tasc   tasc    1.154
#> si       si    2.346
```

``` r
# SDID trend plot (observed vs. synthetic)
plot(fits$sdid, type = "trend")
```

<img src="man/figures/README-plot-trend-1.png" alt="" width="100%" />

``` r
# SCM gap plot (treatment effect over time)
plot(fits$scm, type = "gap")
```

<img src="man/figures/README-plot-gap-1.png" alt="" width="100%" />

``` r
# SCM donor weights
plot(fits$scm, type = "weights")
```

<img src="man/figures/README-plot-weights-1.png" alt="" width="100%" />

## Supported Methods

| Method | Reference | Treatment | Covariates | Inference |
|----|----|----|:--:|----|
| `scm` | Abadie, Diamond & Hainmueller (2010) | Sharp & Staggered | `pred()` list | `mspe_ratio_pval()`, `conformal_inference()` |
| `sdid` | Arkhangelsky et al. (2021) | Sharp & Staggered | `covariates=` | `sdid_inference()`, `conformal_inference()` |
| `gsc` | Xu (2017) | Sharp & Staggered | `covariates=` time-varying | `gsc_boot()`, `gsc_inference()`, `conformal_inference()` |
| `mc` | Athey et al. (2021) | Sharp & Staggered | — | `conformal_inference()` |
| `tasc` | Rho et al. (2026) | Sharp & Staggered | — | — |
| `si` | Agarwal et al. (2025) | Sharp, Staggered & Multi-arm | — | `si_inference()`, `conformal_inference()` |

`conformal_inference()` (Chernozhukov, Wüthrich & Zhu 2021) provides
permutation-based p-values and confidence intervals for **sharp** fits
across `scm`/`sdid`/`gsc`/`mc`/`si`.

## Staggered Adoption

All six methods support staggered adoption using a cohort-based approach
(Clarke et al. 2023): each adoption cohort is fitted separately and the
cohort ATTs are aggregated with weights proportional to
`N_treated × T_post`.

``` r
# u1: treated from t=11, u2: treated from t=16
dat_s        <- dat
dat_s$d      <- 0L
dat_s$d[dat_s$id == "u1" & dat_s$time > 10] <- 1L
dat_s$d[dat_s$id == "u2" & dat_s$time > 15] <- 1L
dat_s$y[dat_s$d == 1] <- dat_s$y[dat_s$d == 1] + 2.0

# All methods detect staggered timing automatically
fit_sdid <- scm_fit(y ~ d | id + time, data = dat_s, method = "sdid")
fit_gsc  <- scm_fit(y ~ d | id + time, data = dat_s, method = "gsc")
fit_mc   <- scm_fit(y ~ d | id + time, data = dat_s, method = "mc")
fit_si   <- scm_fit(y ~ d | id + time, data = dat_s, method = "si")

# Cohort-level estimates are accessible
fit_sdid$cohort_estimates
#>   cohort estimate weight n_treated T_pre T_post
#> 1     11    1.97  0.667         1    10      9
#> 2     16    2.03  0.333         1    10      4

# control_group = "clean" (default) uses never-treated + future-adopters as donors
# control_group = "never_treated" restricts to never-treated only
fit_sdid_clean <- scm_fit(y ~ d | id + time, data = dat_s, method = "sdid",
                          control_group = "never_treated")
```

## Covariates

### SCM: Predictor Variables via `pred()`

SCM supports covariate-based matching following Abadie et al. (2010)
§2.3. Use `pred(vars, times, op)` to specify which variables and time
windows to include in the predictor matrix:

``` r
# Assume dat has extra columns: income, unemp
fit_scm_cov <- scm_fit(
  y ~ d | id + time,
  data   = dat,
  method = "scm",
  predictors = list(
    pred(c("income", "unemp"), 1:8),   # average income & unemp over pre-period
    pred("y", 5),                       # outcome at a specific pre-treatment year
    pred("y", 1:4, op = "mean")         # outcome averaged over early pre-period
  )
)
summary(fit_scm_cov)   # shows predictor balance table
```

Each `pred()` call aggregates one or more variables over a time window
using `op = "mean"` (default), `"median"`, or `"sum"`. Multiple `pred()`
calls with different windows can be combined freely in the list.

### GSC: Time-Varying Covariates

GSC supports time-varying covariate adjustment via the full EM algorithm
of Xu (2017). Pass a character vector of column names as `covariates`:

``` r
# Assume dat has a time-varying column: gdp_growth
fit_gsc_cov <- scm_fit(
  y ~ d | id + time,
  data       = dat,
  method     = "gsc",
  r          = 2,
  covariates = "gdp_growth"
)
fit_gsc_cov$beta   # estimated beta coefficient(s)
```

The EM loop alternates between:

- **E-step**: SVD of $\tilde{Y}_{it} = Y_{it} - x_{it}'\hat\beta$ to
  update factors $\hat{F}$, $\hat\Lambda$
- **M-step**: Ridge OLS of residuals on $x_{it}$ to update $\hat\beta$

Treated unit loadings are estimated from covariate-demeaned
pre-treatment data per Xu (2017) Step 2. When `covariates = NULL`
(default), the plain 3-step SVD estimator ($\hat\beta = 0$) is used.

## Inference

``` r
# SCM: MSPE ratio placebo test (Abadie et al. 2010)
scm_p <- mspe_ratio_pval(fits$scm)
cat("p-value:", scm_p$p_value, "\n")

# SDID: four inference methods — placebo, bootstrap, jackknife, jackknife_global
sdid_inf <- sdid_inference(fits$sdid, method = "placebo")
tidy(sdid_inf)   # broom-style one-row data.frame

sdid_boot <- sdid_inference(fits$sdid, method = "bootstrap", n_boot = 200, seed = 1)
tidy(sdid_boot)

# GSC: parametric bootstrap under H0 (sharp only)
gsc_ci <- gsc_boot(fits$gsc, B = 200, alpha = 0.05)
cat("95% CI: [", gsc_ci$ci_lower, ",", gsc_ci$ci_upper, "]\n")

# GSC / SI: non-parametric inference (sharp + staggered)
gsc_inf <- gsc_inference(fits$gsc, method = "jackknife")
si_inf  <- si_inference(fits$si,  method = "bootstrap", n_boot = 200, seed = 1)
tidy(gsc_inf)
tidy(si_inf)

# Conformal inference (Chernozhukov, Wüthrich & Zhu 2021) — sharp fits for
# scm / sdid / gsc / mc / si. Re-estimates the counterfactual under the null
# and inverts a moving-block permutation test for a p-value and CI.
conf <- conformal_inference(fits$scm, tau0 = 0, level = 0.95)
tidy(conf)
```

## tidyverse / broom Integration

``` r
library(broom)

# Extract weights as a data frame
tidy(fits$scm)

# Summary row
glance(fits$scm)

# JSON export (for reproducibility and AI workflows)
export_json(fits$scm, file = "scm_result.json")
```

## Performance

Estimation stays fast even as the donor pool grows. SCM fit times
(Windows 11 / R 4.6.0 / GCC 14.2.0):

| N_co | T_pre | coresynth |
|-----:|------:|----------:|
|   16 |    10 |    105 ms |
|   20 |    20 |     72 ms |
|   50 |    30 |    916 ms |
|  100 |    40 |  6,300 ms |

A full SCM fit on a 100-donor pool completes in a few seconds.

## References

- Abadie, A., Diamond, A., & Hainmueller, J. (2010). Synthetic control
  methods for comparative case studies. *JASA*, 105(490), 493–505.
- Agarwal, A., Shah, D., & Shen, D. (2025). Synthetic interventions:
  extending synthetic controls to multiple treatments. *Operations
  Research*. <doi:10.1287/opre.2025.1590>
- Arkhangelsky, D., Athey, S., Hirshberg, D. A., Imbens, G. W., &
  Wager, S. (2021). Synthetic difference-in-differences. *AER*, 111(12),
  4088–4118.
- Athey, S., Bayati, M., Doudchenko, N., Imbens, G., & Khosravi, K.
  (2021). Matrix completion methods for causal panel data models.
  *JASA*, 116(536), 1716–1730.
- Chernozhukov, V., Wüthrich, K., & Zhu, Y. (2021). An exact and robust
  conformal inference method for counterfactual and synthetic controls.
  *JASA*, 116(536), 1849–1864.
- Rho, H., et al. (2026). Time-aware synthetic control. *Working paper*.
- Xu, Y. (2017). Generalized synthetic control method. *Political
  Analysis*, 25(1), 57–76.

#' Fit Matrix Completion (Athey et al. 2021)
#'
#' Treats treated post-intervention outcomes as missing and recovers them
#' via nuclear-norm-regularised matrix completion (Soft-Impute algorithm).
#'
#' @param y      Outcome vector (long format)
#' @param d      Treatment indicator (long format)
#' @param id     Unit identifier (long format)
#' @param time   Time identifier (long format)
#' @param lambda Nuclear norm penalty. If NULL, defaults to 1% of the
#'               spectral norm of the observed data matrix.
#' @return A list of class `coresynth`.
#' @noRd
fit_mc_cpp <- function(y, d, id, time, lambda = NULL, ...) {
  pan <- panel_to_matrices(y, d, id, time)
  Y   <- pan$Y; T_pre <- pan$T_pre
  idx_tr <- pan$idx_treat; idx_co <- pan$idx_control

  TT <- nrow(Y); N <- ncol(Y)

  # Per-unit observation mask (0 = treated post-adoption, 1 = observed)
  O <- matrix(1.0, nrow = TT, ncol = N)
  for (j in idx_tr) {
    t0 <- pan$T_adopt[j]
    if (!is.na(t0) && t0 <= TT) O[t0:TT, j] <- 0.0
  }

  # Replace NAs in Y with 0 (unobserved positions will be ignored via mask)
  Y_obs <- Y; Y_obs[is.na(Y_obs)] <- 0.0

  # Default lambda: small fraction of spectral norm
  if(is.null(lambda)) {
    sv <- svd(Y_obs, nu = 0, nv = 0)$d
    lambda <- 0.01 * sv[1]
  }

  L_hat <- soft_impute_cpp(Y_obs, O, lambda)

  Y_treat <- Y[, idx_tr, drop = FALSE]
  Y_hat   <- L_hat[, idx_tr, drop = FALSE]

  gap <- Y_treat - Y_hat
  # ATT: average gap over each unit's own post-treatment period
  gap_post_vals <- vapply(seq_along(idx_tr), function(k) {
    j  <- idx_tr[k]
    t0 <- pan$T_adopt[j]
    if (is.na(t0) || t0 > TT) return(NA_real_)
    mean(gap[t0:TT, k, drop = FALSE], na.rm = TRUE)
  }, numeric(1L))
  att <- mean(gap_post_vals, na.rm = TRUE)

  list(
    method       = "mc",
    lambda       = lambda,
    L_hat        = L_hat,
    Y_hat        = Y_hat,
    Y_treat      = Y_treat,
    gap          = gap,
    times        = pan$times,
    T_pre        = T_pre,
    idx_co       = idx_co,
    idx_tr       = idx_tr,
    Y_co_all     = Y[, idx_co, drop = FALSE],  # T x N_co -- needed by conformal_inference()
    unit_weights = NULL,
    estimate     = att
  )
}

#' Export coresynth Results to JSON
#'
#' Generates a comprehensive, standardized JSON record covering all six
#' estimators. Suitable for reproducibility workflows (Xu & Yang 2026) and
#' downstream tooling. Pass the result of [mspe_ratio_pval()] or [gsc_boot()]
#' via the `inference` argument to include inference results.
#'
#' @param x A `coresynth` object from [scm_fit()].
#' @param file Output file path. Default `"coresynth_results.json"`. Pass
#'   `NULL` to skip writing and return the R list invisibly.
#' @param inference Optional list from [mspe_ratio_pval()] or [gsc_boot()].
#'   When provided, populates the `inference` section and updates `estimate`
#'   with `p_value`, `se`, `ci_lower`, `ci_upper`.
#' @param digits Number of significant digits applied to numeric values
#'   (default 6L).
#' @return Invisibly, the R list that was (or would be) serialized.
#' @export
export_json <- function(x, file = "coresynth_results.json",
                        inference = NULL, digits = 6L) {
  if (!requireNamespace("jsonlite", quietly = TRUE))
    stop("Package 'jsonlite' is required for JSON export. Please install it.")
  if (!inherits(x, "coresynth"))
    stop("x must be a coresynth object.")

  rd <- function(v) if (is.numeric(v)) signif(v, digits) else v

  # ── 1. meta ──────────────────────────────────────────────────────────────────
  meta <- list(
    package    = "coresynth",
    version    = as.character(packageVersion("coresynth")),
    method     = x$method,
    created_at = format(Sys.time(), "%Y-%m-%dT%H:%M:%SZ", tz = "UTC")
  )

  # ── 2. data ───────────────────────────────────────────────────────────────────
  T_total <- length(x$times)
  T_pre   <- x$T_pre
  T_post  <- T_total - T_pre

  unit_names_co <- if (!is.null(x$unit_weights)) names(x$unit_weights)
                   else if (!is.null(x$Y_co_all)) colnames(x$Y_co_all)
                   else NULL
  n_controls <- if (!is.null(x$unit_weights)) length(x$unit_weights)
                else if (!is.null(x$L_co))    nrow(x$L_co)
                else NULL
  n_treated <- if (!is.null(x$L_tr))      nrow(x$L_tr)
               else if (is.matrix(x$Y_treat)) ncol(x$Y_treat)
               else 1L

  data_section <- list(
    n_controls = n_controls,
    n_treated  = n_treated,
    T_total    = T_total,
    T_pre      = T_pre,
    T_post     = T_post,
    times      = as.numeric(x$times),
    unit_names = unit_names_co
  )

  # ── 3. estimate ───────────────────────────────────────────────────────────────
  estimate_section <- list(
    att      = rd(x$estimate),
    se       = if (!is.null(x$se))       rd(x$se)       else NULL,
    p_value  = if (!is.null(x$p_value))  rd(x$p_value)  else NULL,
    ci_lower = if (!is.null(x$ci_lower)) rd(x$ci_lower) else NULL,
    ci_upper = if (!is.null(x$ci_upper)) rd(x$ci_upper) else NULL
  )
  if (!is.null(inference)) {
    if (!is.null(inference$p_value))  estimate_section$p_value  <- rd(inference$p_value)
    if (!is.null(inference$se))       estimate_section$se       <- rd(inference$se)
    if (!is.null(inference$ci_lower)) estimate_section$ci_lower <- rd(inference$ci_lower)
    if (!is.null(inference$ci_upper)) estimate_section$ci_upper <- rd(inference$ci_upper)
  }

  # ── 4. weights ────────────────────────────────────────────────────────────────
  weights_section <- list()
  if (!is.null(x$unit_weights))
    weights_section$unit <- rd(as.list(x$unit_weights))
  if (!is.null(x$time_weights))
    weights_section$time <- rd(as.numeric(x$time_weights))
  if (!is.null(x$v_weights))
    weights_section$covariate <- rd(as.list(x$v_weights))

  # ── 5. time_series ────────────────────────────────────────────────────────────
  flatten_ts <- function(m) {
    if (is.matrix(m)) {
      if (ncol(m) == 1L) return(rd(drop(m)))
      return(lapply(seq_len(ncol(m)), function(j) rd(m[, j])))
    }
    rd(as.numeric(m))
  }
  Y_synth_out <- if (!is.null(x$Y_synth))   x$Y_synth
                 else if (!is.null(x$Y_tr_hat)) x$Y_tr_hat
                 else if (!is.null(x$Y_hat))    x$Y_hat
                 else NULL

  ts_section <- list(
    times   = as.numeric(x$times),
    Y_treat = flatten_ts(x$Y_treat),
    Y_synth = if (!is.null(Y_synth_out)) flatten_ts(Y_synth_out) else NULL,
    gap     = flatten_ts(x$gap)
  )

  # ── 6. method_specific ───────────────────────────────────────────────────────
  ms_section <- switch(x$method,
    "scm"  = list(
      loss      = rd(x$loss),
      v_weights = rd(as.list(x$v_weights))
    ),
    "sdid" = list(
      zeta2      = rd(x$zeta2),
      sigma2_hat = rd(x$sigma2_hat),
      omega0     = rd(x$omega0),
      lambda0    = rd(x$lambda0)
    ),
    "gsc"  = list(
      r               = x$r,
      singular_values = rd(as.numeric(x$singular_values)),
      F               = lapply(seq_len(ncol(x$F)),   function(j) rd(x$F[, j])),
      L_co            = lapply(seq_len(ncol(x$L_co)), function(j) rd(x$L_co[, j])),
      L_tr            = lapply(seq_len(ncol(x$L_tr)), function(j) rd(x$L_tr[, j]))
    ),
    "mc"   = list(lambda = rd(x$lambda)),
    "tasc" = list(
      r = x$r,
      A = lapply(seq_len(nrow(x$A)), function(i) rd(x$A[i, ]))
    ),
    "si"   = list(
      k       = x$k,
      weights = rd(as.numeric(x$unit_weights))
    ),
    list()
  )

  # ── 7. inference ─────────────────────────────────────────────────────────────
  inf_section <- list()
  if (!is.null(inference)) {
    if (!is.null(inference$mspe_ratios_all)) {
      inf_section <- list(
        type               = "mspe_permutation",
        p_value            = rd(inference$p_value),
        mspe_ratio_treated = rd(inference$mspe_ratio_treated),
        mspe_ratios_all    = rd(as.numeric(inference$mspe_ratios_all)),
        n_placebo_used     = inference$n_placebo_used,
        placebo_effects    = rd(as.numeric(inference$placebo_effects))
      )
    } else if (!is.null(inference$boot_dist)) {
      inf_section <- list(
        type      = "parametric_bootstrap",
        B         = length(inference$boot_dist),
        p_value   = rd(inference$p_value),
        se        = rd(inference$se),
        ci_lower  = rd(inference$ci_lower),
        ci_upper  = rd(inference$ci_upper),
        boot_dist = rd(inference$boot_dist)
      )
    }
  }

  # ── Assemble ──────────────────────────────────────────────────────────────────
  result <- list(
    meta            = meta,
    data            = data_section,
    estimate        = estimate_section,
    weights         = if (length(weights_section) > 0L) weights_section else NULL,
    time_series     = ts_section,
    method_specific = if (length(ms_section) > 0L) ms_section else NULL,
    inference       = if (length(inf_section) > 0L) inf_section else NULL
  )

  if (!is.null(file))
    jsonlite::write_json(result, path = file, auto_unbox = TRUE,
                         pretty = TRUE, null = "null")
  invisible(result)
}

#' @importFrom broom tidy
#' @export
tidy.coresynth <- function(x, ...) {
  if (isTRUE(x$staggered)) {
    if (isTRUE(x$multi_arm) && !is.null(x$cohort_arm_estimates)) {
      ca <- x$cohort_arm_estimates
      return(data.frame(
        term      = paste0("cohort_", ca$cohort, "_arm_", ca$arm),
        estimate  = ca$estimate,
        weight    = ca$weight,
        n_treated = ca$n_treated,
        T_pre     = ca$T_pre,
        T_post    = ca$T_post,
        type      = "cohort_arm_estimate",
        stringsAsFactors = FALSE
      ))
    }
    ce <- x$cohort_estimates
    if (!is.null(ce) && nrow(ce) > 0L) {
      return(data.frame(
        term     = paste0("cohort_", ce$cohort),
        estimate = ce$estimate,
        weight   = ce$weight,
        n_treated = ce$n_treated,
        T_pre    = ce$T_pre,
        T_post   = ce$T_post,
        type     = "cohort_estimate",
        stringsAsFactors = FALSE
      ))
    }
    return(data.frame())
  }
  if (is.null(x$unit_weights)) {
    return(data.frame())
  }
  data.frame(
    term     = names(x$unit_weights) %||% paste0("Unit_", seq_along(x$unit_weights)),
    estimate = as.numeric(x$unit_weights),
    type     = "unit_weight",
    stringsAsFactors = FALSE
  )
}

#' @importFrom broom glance
#' @export
glance.coresynth <- function(x, ...) {
  T_all   <- length(x$times) %||% NA_integer_
  T_pre   <- x$T_pre %||% NA_integer_
  T_post  <- if (is.finite(T_all) && is.finite(T_pre)) T_all - T_pre else NA_integer_

  n_treated <- if (!is.null(x$Y_treat) && is.matrix(x$Y_treat)) {
    ncol(x$Y_treat)
  } else if (!is.null(x$Y_treat)) {
    1L
  } else {
    NA_integer_
  }

  n_controls <- if (!is.null(x$unit_weights)) {
    length(x$unit_weights)
  } else if (!is.null(x$idx_co)) {
    length(x$idx_co)
  } else if (isTRUE(x$staggered) && !is.null(x$cohort_fits)) {
    round(mean(vapply(x$cohort_fits,
                      function(cf) length(cf$idx_co), numeric(1L))))
  } else {
    NA_integer_
  }

  data.frame(
    method     = x$method,
    estimate   = x$estimate,
    n_controls = n_controls,
    n_treated  = n_treated,
    T_pre      = T_pre,
    T_post     = T_post,
    staggered  = isTRUE(x$staggered),
    multi_arm  = isTRUE(x$multi_arm),
    stringsAsFactors = FALSE
  )
}

#' Tidy an inference result
#'
#' Coerces a `coresynth_inference` or `sdid_inference` object to a one-row
#' tidy `data.frame` with broom-style column names so it can be combined
#' with regression output for paper tables.
#'
#' @param x A `coresynth_inference` (or `sdid_inference`) object returned by
#'   [sdid_inference()], [gsc_inference()], or [si_inference()].
#' @param conf.int Logical. Include `conf.low`/`conf.high` columns when CI is
#'   available (default `TRUE`). Permutation placebo SE/CI are `NA`.
#' @param ... Unused.
#' @return A one-row `data.frame` with columns `term`, `estimate`, `std.error`,
#'   `statistic`, `p.value`, `conf.low`, `conf.high`, `method`, `alternative`,
#'   `n_controls`, `staggered`.
#' @importFrom broom tidy
#' @export
tidy.coresynth_inference <- function(x, conf.int = TRUE, ...) {
  se   <- if (is.null(x$se)) NA_real_ else x$se
  stat <- if (is.finite(se) && se > 0) x$estimate / se else NA_real_
  ci_l <- if (is.null(x$ci_lower)) NA_real_ else x$ci_lower
  ci_u <- if (is.null(x$ci_upper)) NA_real_ else x$ci_upper

  out <- data.frame(
    term        = "ATT",
    estimate    = x$estimate,
    std.error   = se,
    statistic   = stat,
    p.value     = x$p_value,
    conf.low    = ci_l,
    conf.high   = ci_u,
    method      = x$method,
    alternative = x$alternative %||% NA_character_,
    n_controls  = x$n_controls,
    staggered   = isTRUE(x$staggered),
    stringsAsFactors = FALSE
  )
  if (!isTRUE(conf.int)) {
    out$conf.low  <- NULL
    out$conf.high <- NULL
  }
  out
}

#' Glance at an inference result
#'
#' One-row summary of a `coresynth_inference` (or `sdid_inference`) object.
#'
#' @param x  An inference object.
#' @param ... Unused.
#' @return A one-row `data.frame` with columns `method`, `n_controls`,
#'   `staggered`, `estimate`, `std.error`, `p.value`, `conf.low`,
#'   `conf.high`, `alternative`, `n_boot_valid`.
#' @importFrom broom glance
#' @export
glance.coresynth_inference <- function(x, ...) {
  n_boot_valid <- if (is.null(x$boot_ests)) NA_integer_ else
    sum(!is.na(x$boot_ests))
  data.frame(
    method       = x$method,
    n_controls   = x$n_controls,
    staggered    = isTRUE(x$staggered),
    estimate     = x$estimate,
    std.error    = x$se %||% NA_real_,
    p.value      = x$p_value,
    conf.low     = x$ci_lower %||% NA_real_,
    conf.high    = x$ci_upper %||% NA_real_,
    alternative  = x$alternative %||% NA_character_,
    n_boot_valid = n_boot_valid,
    stringsAsFactors = FALSE
  )
}

#' @importFrom broom augment
#' @export
augment.coresynth <- function(x, include_donors = FALSE, ...) {
  # ── Staggered path ──────────────────────────────────────────────────────────
  if (isTRUE(x$staggered)) {
    return(.augment_staggered(x, include_donors = include_donors))
  }

  # Resolve Y_treat
  Y_treat_raw <- x$Y_treat
  if (is.null(Y_treat_raw))
    stop("augment.coresynth: cannot find treated outcome in fit object.",
         call. = FALSE)
  Y_treat <- if (is.matrix(Y_treat_raw)) rowMeans(Y_treat_raw) else as.numeric(Y_treat_raw)

  # Resolve Y_fitted in priority order
  Y_fitted <- NULL
  if (!is.null(x$Y_synth)) {
    Y_fitted <- if (is.matrix(x$Y_synth)) rowMeans(x$Y_synth) else as.numeric(x$Y_synth)
  } else if (!is.null(x$Y_tr_hat)) {
    Y_fitted <- rowMeans(as.matrix(x$Y_tr_hat))
  } else if (!is.null(x$Y_hat)) {
    Y_fitted <- rowMeans(as.matrix(x$Y_hat))
  } else if (!is.null(x$gap)) {
    gap_raw  <- if (is.matrix(x$gap)) rowMeans(x$gap) else as.numeric(x$gap)
    Y_fitted <- Y_treat - gap_raw
  } else {
    stop("augment.coresynth: cannot find fitted values in fit object.",
         call. = FALSE)
  }

  T_total   <- length(Y_treat)
  T_pre     <- x$T_pre %||% 0L
  is_post   <- c(rep(FALSE, T_pre), rep(TRUE, T_total - T_pre))
  times     <- x$times %||% seq_len(T_total)
  unit_name <- if (!is.null(names(x$Y_treat))) names(x$Y_treat)[1L] else "treated"

  treated_df <- data.frame(
    .unit    = unit_name,
    .type    = "treated",
    .time    = times,
    .observed= Y_treat,
    .fitted  = Y_fitted,
    .resid   = Y_treat - Y_fitted,
    .treated = is_post,
    .period  = factor(ifelse(is_post, "post", "pre"), levels = c("pre", "post")),
    stringsAsFactors = FALSE
  )

  if (!include_donors) {
    # Drop .unit / .type for backwards compatibility
    treated_df$.unit <- NULL
    treated_df$.type <- NULL
    return(treated_df)
  }

  # Build control rows from Y_co_pre + Y_co_post (SCM/SDID) or Y_co_all (GSC)
  Y_co_mat <- NULL
  if (!is.null(x$Y_co_pre) && !is.null(x$Y_co_post)) {
    Y_co_mat <- rbind(x$Y_co_pre, x$Y_co_post)
  } else if (!is.null(x$Y_co_all)) {
    Y_co_mat <- x$Y_co_all
  }

  if (is.null(Y_co_mat)) {
    warning(
      "include_donors = TRUE: control unit outcomes not available in this fit. ",
      "Returning treated unit rows only.",
      call. = FALSE
    )
    return(treated_df)
  }

  donor_names <- colnames(Y_co_mat) %||% paste0("Unit_", seq_len(ncol(Y_co_mat)))
  control_df <- do.call(rbind, lapply(seq_along(donor_names), function(j) {
    data.frame(
      .unit    = donor_names[j],
      .type    = "control",
      .time    = times,
      .observed= as.numeric(Y_co_mat[, j]),
      .fitted  = NA_real_,
      .resid   = NA_real_,
      .treated = FALSE,
      .period  = factor(ifelse(is_post, "post", "pre"), levels = c("pre", "post")),
      stringsAsFactors = FALSE
    )
  }))

  rbind(treated_df, control_df)
}

# ── Internal: augment() for staggered fits ────────────────────────────────────
# Returns a long-format data.frame stacking the treated outcome and synthetic
# counterfactual for each cohort (and each treatment arm, if multi-arm).
.augment_staggered <- function(x, include_donors = FALSE) {
  cohort_fits <- x$cohort_fits
  if (is.null(cohort_fits) || length(cohort_fits) == 0L) {
    warning(
      "augment.coresynth: staggered fit has no cohort_fits to expand.",
      call. = FALSE
    )
    return(data.frame())
  }
  times_all <- x$times %||% seq_len(nrow(x$Y_treat %||% matrix(NA_real_, 0, 0)))
  is_multi  <- isTRUE(x$multi_arm)

  Y_all_parent <- x$Y_all %||% x$Y_mat  # GSC/SI uses Y_all, SDID uses Y_mat

  rows <- lapply(cohort_fits, function(cf) {
    # Determine TT from cf or parent
    if (!is.null(cf$Y_treat)) {
      TT <- if (is.matrix(cf$Y_treat)) nrow(cf$Y_treat) else length(cf$Y_treat)
    } else if (!is.null(Y_all_parent)) {
      TT <- nrow(Y_all_parent)
    } else {
      return(NULL)
    }
    if (is.null(TT) || TT == 0L) return(NULL)
    T_pre_g  <- cf$T_pre
    is_post  <- c(rep(FALSE, T_pre_g), rep(TRUE, TT - T_pre_g))
    period   <- factor(ifelse(is_post, "post", "pre"), levels = c("pre", "post"))
    times_g  <- if (length(times_all) >= TT) times_all[seq_len(TT)] else seq_len(TT)

    # Resolve treated path
    Y_treat_g <- if (!is.null(cf$Y_treat)) {
      if (is.matrix(cf$Y_treat)) rowMeans(cf$Y_treat) else as.numeric(cf$Y_treat)
    } else if (!is.null(Y_all_parent) && !is.null(cf$idx_tr)) {
      rowMeans(Y_all_parent[, cf$idx_tr, drop = FALSE])
    } else {
      rep(NA_real_, TT)
    }

    # Resolve fitted in priority order (SCM/SI/GSC carry Y_synth or Y_tr_hat;
    # SDID staggered carries unit_weights + omega0 → reconstruct from Y_mat)
    Y_fit_g <- if (!is.null(cf$Y_synth)) {
      if (is.matrix(cf$Y_synth)) rowMeans(cf$Y_synth) else as.numeric(cf$Y_synth)
    } else if (!is.null(cf$Y_tr_hat)) {
      rowMeans(as.matrix(cf$Y_tr_hat))
    } else if (!is.null(cf$unit_weights) && !is.null(Y_all_parent) &&
               !is.null(cf$idx_co)) {
      omega0 <- cf$omega0 %||% 0
      drop(Y_all_parent[, cf$idx_co, drop = FALSE] %*% cf$unit_weights) + omega0
    } else {
      rep(NA_real_, TT)
    }

    n_treat_units <- if (!is.null(cf$Y_treat) && is.matrix(cf$Y_treat)) {
      ncol(cf$Y_treat)
    } else if (!is.null(cf$idx_tr)) {
      length(cf$idx_tr)
    } else {
      cf$n_treated %||% 1L
    }
    unit_label    <- sprintf("cohort_%s", as.character(cf$cohort))
    if (is_multi && !is.null(cf$arm))
      unit_label <- sprintf("%s_arm_%s", unit_label, as.character(cf$arm))

    treated_df <- data.frame(
      .cohort   = cf$cohort,
      .arm      = if (is_multi) cf$arm %||% NA_integer_ else NA_integer_,
      .unit     = unit_label,
      .type     = "treated",
      .time     = times_g,
      .observed = Y_treat_g,
      .fitted   = Y_fit_g,
      .resid    = Y_treat_g - Y_fit_g,
      .treated  = is_post,
      .period   = period,
      n_treated = n_treat_units,
      stringsAsFactors = FALSE
    )

    if (!include_donors) return(treated_df)

    Y_co_mat <- NULL
    if (!is.null(cf$Y_co_pre) && !is.null(cf$Y_co_post)) {
      Y_co_mat <- rbind(cf$Y_co_pre, cf$Y_co_post)
    } else if (!is.null(cf$idx_co) && !is.null(Y_all_parent)) {
      Y_co_mat <- Y_all_parent[, cf$idx_co, drop = FALSE]
    }
    if (is.null(Y_co_mat)) return(treated_df)

    donor_names <- colnames(Y_co_mat) %||% paste0("Unit_", seq_len(ncol(Y_co_mat)))
    control_df <- do.call(rbind, lapply(seq_along(donor_names), function(j) {
      data.frame(
        .cohort   = cf$cohort,
        .arm      = if (is_multi) cf$arm %||% NA_integer_ else NA_integer_,
        .unit     = donor_names[j],
        .type     = "control",
        .time     = times_g,
        .observed = as.numeric(Y_co_mat[, j]),
        .fitted   = NA_real_,
        .resid    = NA_real_,
        .treated  = FALSE,
        .period   = period,
        n_treated = n_treat_units,
        stringsAsFactors = FALSE
      )
    }))
    rbind(treated_df, control_df)
  })

  out <- do.call(rbind, rows[!vapply(rows, is.null, logical(1L))])
  if (is.null(out)) return(data.frame())
  rownames(out) <- NULL
  if (!is_multi) out$.arm <- NULL
  out
}

#' Panel Data Helper: Reshape long-format data to matrices
#'
#' Internal utility used by all fit_* functions.
#'
#' @param y      Numeric outcome vector (long format)
#' @param d      Binary treatment indicator (0/1, long format)
#' @param id     Unit identifier (long format)
#' @param time   Time identifier (long format)
#' @return A named list with:
#'   * `Y`: Outcome matrix (T x N), units as columns
#'   * `D`: Treatment matrix (T x N)
#'   * `units`: Unique unit identifiers
#'   * `times`: Unique time identifiers
#'   * `T_pre`: Number of pre-treatment periods (global minimum across treated units)
#'   * `T_adopt`: Integer vector (length N). Per-unit first treated row; NA for controls.
#'   * `is_sharp`: Logical. TRUE iff all treated units share one adoption date.
#'   * `idx_treat`: Column indices of treated units
#'   * `idx_control`: Column indices of control units
#' @noRd
panel_to_matrices <- function(y, d, id, time) {
  # Sort by id then time
  ord <- order(id, time)
  y <- y[ord]
  d <- d[ord]
  id <- id[ord]
  time <- time[ord]

  units <- unique(id)
  times <- unique(sort(time))
  N <- length(units)
  TT <- length(times)

  Y <- matrix(
    NA_real_,
    nrow = TT,
    ncol = N,
    dimnames = list(as.character(times), as.character(units))
  )
  D <- matrix(
    0L,
    nrow = TT,
    ncol = N,
    dimnames = list(as.character(times), as.character(units))
  )

  # Vectorised fill (avoids O(n * (T + N)) which() lookups inside a loop)
  idx_mat <- cbind(match(time, times), match(id, units))
  Y[idx_mat] <- y
  D[idx_mat] <- d

  # Determine treated units: any unit that is ever treated
  ever_treated <- which(colSums(D) > 0)
  ever_control <- which(colSums(D) == 0)

  # Per-unit adoption row index (first row where D[,j] > 0, i.e. any treatment arm)
  T_adopt <- rep(NA_integer_, N)
  for (j in ever_treated) {
    first_t <- which(D[, j] > 0L)
    if (length(first_t) > 0L) T_adopt[j] <- first_t[1L]
  }

  # Global T_pre = min(T_adopt) - 1 (backward-compatible semantics)
  T_pre <- min(T_adopt[ever_treated], na.rm = TRUE) - 1L
  is_sharp <- length(unique(T_adopt[ever_treated])) == 1L

  list(
    Y = Y,
    D = D,
    units = units,
    times = times,
    T_pre = T_pre,
    T_adopt = T_adopt,
    is_sharp = is_sharp,
    idx_treat = ever_treated,
    idx_control = ever_control
  )
}

#' NULL-coalescing helper
#'
#' Returns `l` unless it is `NULL`, in which case `r` is returned. Shared
#' across the package (broom, plot, fit helpers).
#' @noRd
`%||%` <- function(l, r) if (is.null(l)) r else l

#' Predictor Specification for SCM
#'
#' Creates a single predictor specification for use in [scm_fit()] with
#' `method = "scm"`. Pass a `list()` of `pred()` calls as the `predictors`
#' argument to define the full covariate matrix.
#'
#' @param vars  Character vector of variable names. All variables share the
#'   same `times` window and `op` operator. Use separate `pred()` calls for
#'   variables with different time windows.
#' @param times Numeric/integer vector of time values to aggregate over.
#' @param op    Aggregation operator applied to each variable over `times`.
#'   One of `"mean"` (default), `"median"`, or `"sum"`.
#'
#' @return A `pred_spec` object (a named list with class `"pred_spec"`).
#'
#' @seealso [scm_fit()] for the `predictors` argument that consumes a `list()`
#'   of `pred_spec` objects.
#'
#' @export
#'
#' @examples
#' # Three variables averaged over the same window
#' pred(c("lnincome", "retprice", "age15to24"), 1980:1988)
#'
#' # Single variable at a specific year
#' pred("cigsale", 1975)
#'
#' # Single variable averaged over a range
#' pred("beer", 1984:1988)
#'
#' # Abadie, Diamond & Hainmueller (2010) California Prop 99 style: combine
#' # several covariates aggregated over different windows plus three outcome
#' # lags at specific years. The resulting list is passed to
#' # scm_fit(..., predictors = predictors).
#' predictors <- list(
#'   pred(c("lnincome", "retprice", "age15to24"), 1980:1988),
#'   pred("beer",    1984:1988),
#'   pred("cigsale", 1988),
#'   pred("cigsale", 1980),
#'   pred("cigsale", 1975)
#' )
#' predictors
pred <- function(vars, times, op = "mean") {
  if (!is.character(vars) || length(vars) == 0L) {
    stop("'vars' must be a non-empty character vector.", call. = FALSE)
  }
  if (length(times) == 0L) {
    stop("'times' must be a non-empty vector of time values.", call. = FALSE)
  }
  if (!op %in% c("mean", "median", "sum")) {
    stop("'op' must be one of \"mean\", \"median\", or \"sum\".", call. = FALSE)
  }
  structure(list(vars = vars, times = times, op = op), class = "pred_spec")
}

#' @export
print.pred_spec <- function(x, ...) {
  times_str <- if (length(x$times) == 1L) {
    as.character(x$times)
  } else {
    sprintf("%s:%s", min(x$times), max(x$times))
  }
  cat(sprintf(
    "pred(%s, %s, op = \"%s\")\n",
    paste(x$vars, collapse = ", "),
    times_str,
    x$op
  ))
  invisible(x)
}

#' Build predictor matrices X0 and X1 for SCM
#'
#' Constructs the (k x N_co) predictor matrix X0 and (k x 1) vector X1
#' from a list of [pred()] specifications, following Abadie et al. (2010)
#' Section 2.3. Each `pred()` entry expands to one row per variable.
#'
#' @param data       Full long-format data frame.
#' @param id_var     Name of the unit identifier column.
#' @param time_var   Name of the time identifier column.
#' @param units      All unit identifiers (length N), same order as Y columns.
#' @param idx_co     Integer indices of control units in `units`.
#' @param idx_tr     Integer index of the treated unit in `units`.
#' @param predictors List of `pred_spec` objects created by [pred()].
#' @return A list with:
#'   * `X0`: k x N_co numeric matrix (predictors x control units)
#'   * `X1`: numeric vector of length k (predictors for treated unit)
#'   * `pred_names`: character vector of length k with predictor labels
#' @noRd
build_predictor_matrices <- function(
  data,
  id_var,
  time_var,
  units,
  idx_co,
  idx_tr,
  predictors
) {
  co_units <- units[idx_co]

  agg_unit <- function(var, times, op) {
    fn <- match.fun(op)
    vapply(
      units,
      function(u) {
        sub <- data[[var]][data[[id_var]] == u & data[[time_var]] %in% times]
        if (length(sub) == 0L) NA_real_ else fn(sub, na.rm = TRUE)
      },
      numeric(1L)
    )
  }

  pred_label <- function(var, times) {
    if (length(times) == 1L) {
      sprintf("%s[%s]", var, times)
    } else {
      sprintf("%s[%s:%s]", var, min(times), max(times))
    }
  }

  rows_X0 <- list()
  rows_X1 <- list()
  pred_names <- character(0L)

  for (p in predictors) {
    if (!inherits(p, "pred_spec")) {
      stop(
        paste0(
          "Each element of 'predictors' must be a pred_spec object ",
          "created by pred()."
        ),
        call. = FALSE
      )
    }
    for (var in p$vars) {
      if (!var %in% names(data)) {
        stop(sprintf("Variable '%s' not found in data.", var), call. = FALSE)
      }
      vals <- agg_unit(var, p$times, p$op)
      rows_X0 <- c(rows_X0, list(vals[idx_co]))
      rows_X1 <- c(rows_X1, list(vals[idx_tr]))
      pred_names <- c(pred_names, pred_label(var, p$times))
    }
  }

  X0 <- do.call(rbind, rows_X0) # k x N_co
  X1 <- unlist(rows_X1) # length k

  colnames(X0) <- as.character(co_units)
  rownames(X0) <- pred_names

  list(X0 = X0, X1 = X1, pred_names = pred_names)
}

#' Build a time-varying covariate array for GSC
#'
#' Constructs a T x N x p 3D R array suitable for passing to gsc_ife_cpp as an
#' arma::cube. `arr[t, i, j]` is the value of the j-th covariate for unit i at
#' time t.
#'
#' @param data            Long-format data frame.
#' @param id_var          Name of the unit identifier column.
#' @param time_var        Name of the time identifier column.
#' @param covariate_names Character vector of covariate column names (length p).
#' @param units           Character vector of unit IDs (length N).
#' @param times           Vector of time values (length T).
#' @return A T x N x p numeric array.
#' @noRd
build_covariate_array <- function(
  data,
  id_var,
  time_var,
  covariate_names,
  units,
  times
) {
  T_all <- length(times)
  N <- length(units)
  p <- length(covariate_names)
  arr <- array(NA_real_, dim = c(T_all, N, p))
  for (j in seq_len(p)) {
    var <- covariate_names[j]
    if (!var %in% names(data)) {
      stop(sprintf("covariate '%s' not found in data.", var), call. = FALSE)
    }
    for (i in seq_len(N)) {
      idx <- data[[id_var]] == units[i]
      sub_t <- data[[time_var]][idx]
      sub_v <- data[[var]][idx]
      arr[match(sub_t, times), i, j] <- sub_v
    }
  }
  arr
}

#' Panel Data Helper: Reshape long-format multi-arm data to tensor structure
#'
#' Extends `panel_to_matrices()` for the multi-arm Synthetic Interventions
#' setting (Agarwal et al. 2025). Each unit belongs to exactly one treatment
#' arm (d = 0 for control, d = 1,...,K for treatment arms). Before the
#' treatment date, all treatment-arm units have d = 0; at the treatment date
#' they switch to their assigned arm value.
#'
#' @param y    Numeric outcome vector (long format)
#' @param d    Treatment arm indicator (integer, 0 = control, 1,...,K = arms)
#' @param id   Unit identifier (long format)
#' @param time Time identifier (long format)
#' @return All fields returned by `panel_to_matrices()` plus:
#'   * `arm_levels`: sorted integer vector of unique arm values (`c(0L, 1L, ..., KL)`)
#'   * `idx_by_arm`: named list of column indices, one entry per arm level
#' @noRd
panel_to_tensor <- function(y, d, id, time) {
  d_int <- as.integer(d)
  pan   <- panel_to_matrices(y, d_int, id, time)

  # Per-unit arm = max(D[, j]): control units always 0, treated units = their arm
  arm_of_unit <- as.integer(apply(pan$D, 2, max))
  arm_levels  <- sort(unique(arm_of_unit))

  if (!0L %in% arm_levels)
    stop("panel_to_tensor: arm 0 (control) not found.", call. = FALSE)
  if (any(arm_levels < 0L))
    stop("panel_to_tensor: arm values must be non-negative integers.", call. = FALSE)

  idx_by_arm <- setNames(
    lapply(arm_levels, function(a) which(arm_of_unit == a)),
    as.character(arm_levels)
  )

  c(pan, list(arm_levels = arm_levels, idx_by_arm = idx_by_arm))
}

## Integration tests for coresynth
## Run: devtools::test()

library(testthat)

# ── Shared helpers ────────────────────────────────────────────────────────────

# Balanced panel: 1 treated unit (u1), rest control, treatment from T_pre+1 onward.
# DGP: Y_it = alpha_i + f_t * lambda_i + tau*D_it + eps
make_panel <- function(N = 10, T = 20, T_pre = 10, effect = 2.0, seed = 42) {
  set.seed(seed)
  times <- 1:T
  units <- paste0("u", 1:N)
  f <- cumsum(rnorm(T, 0, 0.5)) # common factor
  lam <- rnorm(N, 1, 0.3) # unit loadings
  rows <- expand.grid(time = times, id = units, stringsAsFactors = FALSE)
  rows$y <- as.vector(outer(f, lam)) + rnorm(nrow(rows), 0, 0.3)
  rows$d <- as.integer(rows$id == "u1" & rows$time > T_pre)
  rows$y[rows$d == 1] <- rows$y[rows$d == 1] + effect
  rows[order(rows$id, rows$time), ]
}

panel <- make_panel()

# ── SCM ──────────────────────────────────────────────────────────────────────
test_that("SCM weights lie on the unit simplex", {
  fit <- scm_fit(y ~ d | id + time, data = panel, method = "scm")
  expect_s3_class(fit, "coresynth")
  expect_equal(fit$method, "scm")
  expect_equal(sum(fit$unit_weights), 1, tolerance = 1e-4)
  expect_true(all(fit$unit_weights >= -1e-5))
  expect_equal(length(fit$Y_synth), 20)
  expect_equal(length(fit$gap), 20)
})

test_that("SCM estimate is in a reasonable range for effect=2", {
  fit <- scm_fit(y ~ d | id + time, data = panel, method = "scm")
  expect_true(abs(fit$estimate - 2.0) < 3.0)
})

test_that("SCM pre-treatment fit is good (low MSPE)", {
  fit <- scm_fit(y ~ d | id + time, data = panel, method = "scm")
  pre <- seq_len(10)
  mspe <- mean((fit$Y_treat[pre] - fit$Y_synth[pre])^2)
  total_var <- var(fit$Y_treat[pre])
  expect_true(mspe / total_var < 0.5) # synthetic tracks pre-period reasonably
})

# ── SDID ─────────────────────────────────────────────────────────────────────
test_that("SDID unit weights lie on the unit simplex", {
  fit <- scm_fit(y ~ d | id + time, data = panel, method = "sdid")
  expect_s3_class(fit, "coresynth")
  expect_equal(sum(fit$unit_weights), 1, tolerance = 1e-4)
  expect_true(all(fit$unit_weights >= -1e-5))
})

test_that("SDID time weights lie on the unit simplex", {
  fit <- scm_fit(y ~ d | id + time, data = panel, method = "sdid")
  expect_equal(sum(fit$time_weights), 1, tolerance = 1e-4)
  expect_true(all(fit$time_weights >= -1e-5))
  expect_equal(length(fit$time_weights), 10L) # T_pre = 10
})

test_that("SDID zeta2 follows Arkhangelsky et al. (2021) Eq.(2.2)", {
  fit <- scm_fit(y ~ d | id + time, data = panel, method = "sdid")
  # zeta2 = sqrt(N_tr * T_post) * sigma2_hat
  # N_tr=1, T_post=10, sigma2_hat from fit
  expected_zeta2 <- sqrt(1 * 10) * fit$sigma2_hat
  expect_equal(fit$zeta2, expected_zeta2, tolerance = 1e-10)
  expect_true(fit$zeta2 > 0)
})

test_that("SDID omega_0 and lambda_0 intercepts are finite", {
  fit <- scm_fit(y ~ d | id + time, data = panel, method = "sdid")
  expect_true(is.finite(fit$omega0))
  expect_true(is.finite(fit$lambda0))
})

test_that("SDID demeaning shifts unit weights vs. no-intercept baseline", {
  # With intercept (current): demeaned QP allows level shifts
  fit_new <- scm_fit(y ~ d | id + time, data = panel, method = "sdid")
  # The estimate should still be in a reasonable range
  expect_true(abs(fit_new$estimate - 2.0) < 4.0)
})

test_that("SDID estimate matches synthdid package (California smoke data)", {
  skip_if_not_installed("synthdid")
  data("california_prop99", package = "synthdid")
  setup <- panel.matrices(california_prop99)
  ref <- synthdid_estimate(setup$Y, setup$N0, setup$T0)
  # Convert data to long format for coresynth
  Y_wide <- setup$Y
  df <- data.frame(
    y = as.vector(Y_wide),
    id = rep(rownames(Y_wide), ncol(Y_wide)),
    time = rep(colnames(Y_wide), each = nrow(Y_wide))
  )
  N0 <- setup$N0
  T0 <- setup$T0
  post_cols <- (T0 + 1):ncol(Y_wide)
  treat_rows <- (N0 + 1):nrow(Y_wide)
  df$d <- as.integer(
    df$id %in%
      rownames(Y_wide)[treat_rows] &
      df$time %in% colnames(Y_wide)[post_cols]
  )
  fit_cs <- scm_fit(y ~ d | id + time, data = df, method = "sdid")
  # Reference: Arkhangelsky et al. (2021) Table 1 reports SDID = -15.6
  expect_equal(fit_cs$estimate, as.numeric(ref), tolerance = 0.5)
})

# ── GSC ──────────────────────────────────────────────────────────────────────
test_that("GSC returns factor matrix with correct dimensions", {
  fit <- scm_fit(y ~ d | id + time, data = panel, method = "gsc", r = 2)
  expect_s3_class(fit, "coresynth")
  expect_equal(ncol(fit$F), 2L) # r = 2 factors
  expect_equal(nrow(fit$F), 20L) # T periods
  expect_equal(ncol(fit$L_co), 2L) # r loadings per control unit
  expect_equal(nrow(fit$L_co), 9L) # N_co = 9
})

test_that("GSC counterfactual has correct dimension", {
  fit <- scm_fit(y ~ d | id + time, data = panel, method = "gsc", r = 2)
  expect_equal(dim(fit$Y_tr_hat), c(20L, 1L)) # T x N_tr
})

test_that("GSC estimate is finite and in reasonable range", {
  fit <- scm_fit(y ~ d | id + time, data = panel, method = "gsc", r = 2)
  expect_true(is.finite(fit$estimate))
  expect_true(abs(fit$estimate - 2.0) < 4.0)
})

test_that("GSC singular values are decreasing", {
  fit <- scm_fit(y ~ d | id + time, data = panel, method = "gsc", r = 2)
  sv <- fit$singular_values
  expect_true(all(diff(sv[sv > 0]) <= 1e-10)) # non-increasing
})

test_that("GSC rejects r larger than N_co or T", {
  # R wrapper fires first (r > N_co = 9): "Need at least r control units"
  expect_error(
    scm_fit(y ~ d | id + time, data = panel, method = "gsc", r = 20),
    regexp = "at least r"
  )
})

# ── MC ───────────────────────────────────────────────────────────────────────
test_that("MC returns low-rank matrix of correct size", {
  fit <- scm_fit(y ~ d | id + time, data = panel, method = "mc")
  expect_s3_class(fit, "coresynth")
  expect_equal(dim(fit$L_hat), c(20L, 10L))
})

test_that("MC observed entries are well-fitted (low residual on mask)", {
  fit <- scm_fit(y ~ d | id + time, data = panel, method = "mc")
  pan <- coresynth:::panel_to_matrices(
    panel$y,
    panel$d,
    panel$id,
    panel$time
  )
  # Observation mask: post x treated = 0, rest = 1
  O <- matrix(1, nrow = 20, ncol = 10)
  O[11:20, 1] <- 0 # unit u1 post-treatment is "missing"
  Y_obs <- pan$Y
  Y_obs[is.na(Y_obs)] <- 0
  resid_obs <- (Y_obs - fit$L_hat)[O == 1]
  rmse_obs <- sqrt(mean(resid_obs^2))
  # Should be small relative to outcome range
  expect_true(rmse_obs < diff(range(Y_obs[O == 1])) * 0.3)
})

test_that("MC estimate is finite", {
  fit <- scm_fit(y ~ d | id + time, data = panel, method = "mc")
  expect_true(is.finite(fit$estimate))
})

# ── TASC ─────────────────────────────────────────────────────────────────────
test_that("TASC runs and returns valid output structure", {
  fit <- scm_fit(
    y ~ d | id + time,
    data = panel,
    method = "tasc",
    r = 2,
    em_iter = 5
  )
  expect_s3_class(fit, "coresynth")
  expect_true(is.finite(fit$estimate))
  expect_equal(length(fit$times), 20L)
  expect_equal(dim(fit$Y_hat), c(20L, 10L))
})

test_that("TASC general A: ATT is finite", {
  fit <- scm_fit(
    y ~ d | id + time,
    data = panel,
    method = "tasc",
    r = 2,
    em_iter = 10
  )
  expect_true(is.finite(fit$estimate))
})

test_that("TASC A matrix has correct dimensions (r x r)", {
  fit <- scm_fit(
    y ~ d | id + time,
    data = panel,
    method = "tasc",
    r = 2,
    em_iter = 5
  )
  expect_equal(dim(fit$A), c(2L, 2L))
})

test_that("TASC A deviates from identity after EM (learning occurred)", {
  fit <- scm_fit(
    y ~ d | id + time,
    data = panel,
    method = "tasc",
    r = 2,
    em_iter = 20
  )
  expect_true(max(abs(fit$A - diag(2))) > 1e-6)
})

test_that("TASC fix_A=TRUE keeps A as identity", {
  fit <- scm_fit(
    y ~ d | id + time,
    data = panel,
    method = "tasc",
    r = 2,
    em_iter = 10,
    fix_A = TRUE
  )
  expect_equal(fit$A, diag(2))
  expect_true(is.finite(fit$estimate))
})

test_that("TASC fix_A=FALSE and fix_A=TRUE both return finite ATT", {
  fit_free <- scm_fit(
    y ~ d | id + time,
    data = panel,
    method = "tasc",
    r = 2,
    em_iter = 10,
    fix_A = FALSE
  )
  fit_fix <- scm_fit(
    y ~ d | id + time,
    data = panel,
    method = "tasc",
    r = 2,
    em_iter = 10,
    fix_A = TRUE
  )
  expect_true(is.finite(fit_free$estimate))
  expect_true(is.finite(fit_fix$estimate))
})

# ── print / summary ──────────────────────────────────────────────────────────
test_that("print and summary work for all methods", {
  for (m in c("scm", "sdid", "gsc", "mc")) {
    fit <- scm_fit(y ~ d | id + time, data = panel, method = m)
    expect_output(print(fit), toupper(m))
    expect_output(summary(fit), "ATT estimate")
  }
})

# ── broom tidiers ─────────────────────────────────────────────────────────────
test_that("tidy() and glance() work for SDID", {
  fit <- scm_fit(y ~ d | id + time, data = panel, method = "sdid")
  td <- broom::tidy(fit)
  gl <- broom::glance(fit)
  expect_s3_class(td, "data.frame")
  expect_true(nrow(td) > 0)
  expect_true(all(c("term", "estimate") %in% names(td)))
  expect_equal(gl$method, "sdid")
})

# ── plot ─────────────────────────────────────────────────────────────────────
test_that("plot.coresynth returns ggplot objects", {
  fit <- scm_fit(y ~ d | id + time, data = panel, method = "sdid")
  expect_s3_class(plot(fit, type = "trend"), "ggplot")
  expect_s3_class(plot(fit, type = "gap"), "ggplot")
  expect_s3_class(plot(fit, type = "weights"), "ggplot")
})

# ── SI ───────────────────────────────────────────────────────────────────────

test_that("SI returns correct class and structure", {
  fit <- scm_fit(y ~ d | id + time, data = panel, method = "si")
  expect_s3_class(fit, "coresynth")
  expect_s3_class(fit, "coresynth_si")
  expect_equal(fit$method, "si")
  expect_true(is.numeric(fit$k) && fit$k >= 1L)
})

test_that("SI output dimensions are correct", {
  fit <- scm_fit(y ~ d | id + time, data = panel, method = "si")
  # panel: T=20, N=10, N_co=9, N_tr=1, T_pre=10, T_post=10
  expect_equal(dim(fit$Y_treat), c(20L, 1L))
  expect_equal(dim(fit$Y_synth), c(20L, 1L))
  expect_equal(dim(fit$Y_cf), c(10L, 1L))
  expect_equal(dim(fit$weights), c(9L, 1L))
  expect_equal(length(fit$unit_weights), 9L)
  expect_equal(length(fit$times), 20L)
  expect_equal(fit$T_pre, 10L)
})

test_that("SI pre-treatment fit is good (low MSPE)", {
  fit <- scm_fit(y ~ d | id + time, data = panel, method = "si")
  pre <- seq_len(10)
  mspe <- mean((fit$Y_treat[pre, ] - fit$Y_synth[pre, ])^2)
  total_var <- var(as.vector(fit$Y_treat[pre, ]))
  expect_true(mspe / total_var < 0.5)
})

test_that("SI estimate is finite and in reasonable range for effect=2", {
  fit <- scm_fit(y ~ d | id + time, data = panel, method = "si")
  expect_true(is.finite(fit$estimate))
  expect_true(abs(fit$estimate - 2.0) < 3.0)
})

test_that("SI respects user-supplied k", {
  fit1 <- scm_fit(y ~ d | id + time, data = panel, method = "si", k = 1L)
  fit3 <- scm_fit(y ~ d | id + time, data = panel, method = "si", k = 3L)
  expect_equal(fit1$k, 1L)
  expect_equal(fit3$k, 3L)
  expect_true(is.finite(fit1$estimate))
  expect_true(is.finite(fit3$estimate))
})

test_that("SI errors when k > min(T_pre, N_co)", {
  expect_error(
    scm_fit(y ~ d | id + time, data = panel, method = "si", k = 100L),
    regexp = "k"
  )
})

test_that("tensor_unfold_cpp produces correct dimensions for all modes", {
  tc <- array(seq_len(3 * 4 * 2), dim = c(3L, 4L, 2L))
  expect_equal(dim(tensor_unfold_cpp(tc, 1L)), c(3L, 8L))
  expect_equal(dim(tensor_unfold_cpp(tc, 2L)), c(4L, 6L))
  expect_equal(dim(tensor_unfold_cpp(tc, 3L)), c(2L, 12L))
  expect_error(tensor_unfold_cpp(tc, 4L), regexp = "Mode")
})

test_that("SI and MC estimates are in similar range", {
  fit_si <- scm_fit(y ~ d | id + time, data = panel, method = "si")
  fit_mc <- scm_fit(y ~ d | id + time, data = panel, method = "mc")
  expect_true(abs(fit_si$estimate - fit_mc$estimate) < 3.0)
})

test_that("print and summary work for SI", {
  fit <- scm_fit(y ~ d | id + time, data = panel, method = "si")
  expect_output(print(fit), "SI")
  expect_output(summary(fit), "ATT estimate")
})

# ── edge cases ───────────────────────────────────────────────────────────────
test_that("scm_fit errors on bad formula", {
  expect_error(scm_fit(y ~ d, data = panel, method = "sdid"), "after '|'")
})

test_that("scm_fit errors on missing variable", {
  expect_error(
    scm_fit(z ~ d | id + time, data = panel, method = "sdid"),
    "not found"
  )
})

# ── Phase 4: SCM placebo / MSPE-ratio inference ───────────────────────────────

test_that("fit_scm_cpp returns Y_co_pre and Y_co_post", {
  fit <- scm_fit(y ~ d | id + time, data = panel, method = "scm")
  expect_equal(dim(fit$Y_co_pre), c(10L, 9L)) # T_pre x N_co
  expect_equal(dim(fit$Y_co_post), c(10L, 9L)) # T_post x N_co
})

test_that("scm_placebo_cpp returns correct structure", {
  fit <- scm_fit(y ~ d | id + time, data = panel, method = "scm")
  plac <- scm_placebo_cpp(fit$Y_co_pre, fit$Y_co_post)
  expect_equal(length(plac$mspe_pre), 9L)
  expect_equal(length(plac$mspe_post), 9L)
  expect_equal(length(plac$effects), 9L)
  expect_true(all(plac$mspe_pre >= 0))
  expect_true(all(plac$mspe_post >= 0))
  expect_true(all(is.finite(plac$effects)))
})

test_that("mspe_ratio_pval returns p_value in [0, 1]", {
  fit <- scm_fit(y ~ d | id + time, data = panel, method = "scm")
  inf <- mspe_ratio_pval(fit)
  expect_true(inf$p_value >= 0 && inf$p_value <= 1)
  expect_true(is.finite(inf$mspe_ratio_treated))
  expect_true(inf$mspe_ratio_treated >= 0)
  expect_equal(length(inf$mspe_ratios_all), 10L) # 1 treated + 9 controls
  expect_equal(length(inf$placebo_effects), 9L)
})

test_that("mspe_ratio_pval mspe_threshold filters all controls", {
  fit <- scm_fit(y ~ d | id + time, data = panel, method = "scm")
  inf <- mspe_ratio_pval(fit, mspe_threshold = 1e10)
  expect_equal(inf$n_placebo_used, 0L)
})

test_that("mspe_ratio_pval rejects non-SCM fits", {
  fit <- scm_fit(y ~ d | id + time, data = panel, method = "sdid")
  expect_error(mspe_ratio_pval(fit), "method = 'scm'")
})

# ── Phase 4: GSC bootstrap inference ─────────────────────────────────────────

test_that("fit_gsc_cpp returns Y_co_all", {
  fit <- scm_fit(y ~ d | id + time, data = panel, method = "gsc", r = 2)
  expect_equal(dim(fit$Y_co_all), c(20L, 9L)) # T x N_co
})

test_that("gsc_boot returns correct structure", {
  fit <- scm_fit(y ~ d | id + time, data = panel, method = "gsc", r = 2)
  boot <- gsc_boot(fit, B = 29L, seed = 1L)
  expect_named(
    boot,
    c("p_value", "ci_lower", "ci_upper", "se", "boot_dist", "att_obs")
  )
  expect_equal(length(boot$boot_dist), 29L)
  expect_true(boot$p_value >= 0 && boot$p_value <= 1)
  expect_true(boot$ci_lower <= boot$ci_upper)
  expect_true(is.finite(boot$se) && boot$se >= 0)
  expect_equal(boot$att_obs, fit$estimate)
})

test_that("gsc_boot seed ensures reproducibility", {
  fit <- scm_fit(y ~ d | id + time, data = panel, method = "gsc", r = 2)
  boot1 <- gsc_boot(fit, B = 29L, seed = 42L)
  boot2 <- gsc_boot(fit, B = 29L, seed = 42L)
  expect_equal(boot1$boot_dist, boot2$boot_dist)
})

test_that("gsc_boot rejects non-GSC fits", {
  fit <- scm_fit(y ~ d | id + time, data = panel, method = "scm")
  expect_error(gsc_boot(fit), "method = 'gsc'")
})

# ── Phase 4: export_json ──────────────────────────────────────────────────────

test_that("export_json NULL file returns list without writing", {
  fit <- scm_fit(y ~ d | id + time, data = panel, method = "sdid")
  res <- export_json(fit, file = NULL)
  expect_type(res, "list")
  expect_equal(res$meta$method, "sdid")
  expect_equal(res$meta$package, "coresynth")
})

test_that("export_json has required top-level sections", {
  fit <- scm_fit(y ~ d | id + time, data = panel, method = "sdid")
  res <- export_json(fit, file = NULL)
  expect_true(all(c("meta", "data", "estimate", "time_series") %in% names(res)))
})

test_that("export_json estimate section contains att", {
  fit <- scm_fit(y ~ d | id + time, data = panel, method = "scm")
  res <- export_json(fit, file = NULL)
  expect_equal(res$estimate$att, signif(fit$estimate, 6))
})

test_that("export_json time_series has required fields", {
  fit <- scm_fit(y ~ d | id + time, data = panel, method = "scm")
  res <- export_json(fit, file = NULL)
  expect_true(all(
    c("times", "Y_treat", "Y_synth", "gap") %in% names(res$time_series)
  ))
  expect_equal(length(res$time_series$times), 20L)
})

test_that("export_json writes valid JSON for SCM, SDID, GSC, MC", {
  skip_if_not_installed("jsonlite")
  tmp <- tempfile(fileext = ".json")
  on.exit(unlink(tmp), add = TRUE)
  for (m in c("scm", "sdid", "gsc", "mc")) {
    fit <- scm_fit(y ~ d | id + time, data = panel, method = m)
    export_json(fit, file = tmp)
    dat <- jsonlite::read_json(tmp)
    expect_equal(dat$meta$method, m)
    expect_false(is.null(dat$estimate$att))
    expect_equal(dat$data$T_pre, 10L)
  }
})

test_that("export_json with inference includes inference section (SCM)", {
  skip_if_not_installed("jsonlite")
  fit <- scm_fit(y ~ d | id + time, data = panel, method = "scm")
  inf <- mspe_ratio_pval(fit)
  res <- export_json(fit, file = NULL, inference = inf)
  expect_equal(res$inference$type, "mspe_permutation")
  expect_false(is.null(res$inference$p_value))
  expect_false(is.null(res$inference$mspe_ratios_all))
  expect_equal(res$estimate$p_value, signif(inf$p_value, 6))
})

test_that("export_json with inference includes inference section (GSC)", {
  fit <- scm_fit(y ~ d | id + time, data = panel, method = "gsc", r = 2)
  boot <- gsc_boot(fit, B = 29L, seed = 1L)
  res <- export_json(fit, file = NULL, inference = boot)
  expect_equal(res$inference$type, "parametric_bootstrap")
  expect_equal(res$inference$B, 29L)
  expect_false(is.null(res$inference$ci_lower))
})

# ── Phase 5a: Staggered adoption ──────────────────────────────────────────────

make_staggered_panel <- function(N = 12, TT = 24, seed = 7) {
  set.seed(seed)
  f <- cumsum(rnorm(TT, 0, 0.5))
  lam <- rnorm(N, 1, 0.3)
  rows <- expand.grid(
    time = seq_len(TT),
    id = paste0("u", seq_len(N)),
    stringsAsFactors = FALSE
  )
  rows$y <- as.vector(outer(f, lam)) + rnorm(nrow(rows), 0, 0.3)
  # u1 採用 t=9、u2 採用 t=15
  rows$d <- as.integer(
    (rows$id == "u1" & rows$time >= 9) |
      (rows$id == "u2" & rows$time >= 15)
  )
  rows$y[rows$d == 1] <- rows$y[rows$d == 1] + 1.5
  rows[order(rows$id, rows$time), ]
}

staggered <- make_staggered_panel()

test_that("panel_to_matrices detects staggered adoption correctly", {
  pan <- coresynth:::panel_to_matrices(
    staggered$y,
    staggered$d,
    staggered$id,
    staggered$time
  )
  expect_false(pan$is_sharp)
  expect_equal(pan$T_pre, 8L)
  u1c <- which(pan$units == "u1")
  u2c <- which(pan$units == "u2")
  expect_equal(pan$T_adopt[u1c], 9L)
  expect_equal(pan$T_adopt[u2c], 15L)
})

test_that("panel_to_matrices is_sharp TRUE for non-staggered panel", {
  pan <- coresynth:::panel_to_matrices(
    panel$y,
    panel$d,
    panel$id,
    panel$time
  )
  expect_true(pan$is_sharp)
})

test_that("MC handles staggered adoption and returns finite estimate", {
  fit <- scm_fit(y ~ d | id + time, data = staggered, method = "mc")
  expect_true(is.finite(fit$estimate))
})

test_that("MC staggered: L_hat dimensions are correct", {
  fit <- scm_fit(y ~ d | id + time, data = staggered, method = "mc")
  expect_equal(dim(fit$L_hat), c(24L, 12L))
})

test_that("TASC handles staggered adoption and returns finite estimate", {
  fit <- scm_fit(
    y ~ d | id + time,
    data = staggered,
    method = "tasc",
    r = 2L,
    em_iter = 5L
  )
  expect_true(is.finite(fit$estimate))
})

test_that("SCM staggered with predictors arg errors informatively", {
  expect_error(
    scm_fit(y ~ d | id + time, data = staggered, method = "scm",
            predictors = list(pred("y", 1:3))),
    regexp = "predictors"
  )
})

test_that("GSC handles staggered adoption (Phase 16)", {
  fit <- scm_fit(y ~ d | id + time, data = staggered, method = "gsc")
  expect_true(isTRUE(fit$staggered))
  expect_true(is.finite(fit$estimate))
})

test_that("SI handles staggered adoption (Phase 16)", {
  fit <- scm_fit(y ~ d | id + time, data = staggered, method = "si")
  expect_true(isTRUE(fit$staggered))
  expect_true(is.finite(fit$estimate))
})

# ── SCM with covariates (pred() interface) ────────────────────────────────────

# Helper: panel with two extra covariate columns
make_panel_with_cov <- function(
  N = 10,
  T = 20,
  T_pre = 10,
  effect = 2.0,
  seed = 42
) {
  set.seed(seed)
  p <- make_panel(N = N, T = T, T_pre = T_pre, effect = effect, seed = seed)
  units_all <- unique(p$id)
  cov1_vals <- setNames(rnorm(N, mean = 5, sd = 1), units_all)
  cov2_vals <- setNames(rnorm(N, mean = 10, sd = 2), units_all)
  p$cov1 <- cov1_vals[p$id]
  p$cov2 <- cov2_vals[p$id]
  p
}

panel_cov <- make_panel_with_cov()

test_that("SCM with pred() returns valid fit object", {
  fit <- scm_fit(
    y ~ d | id + time,
    data = panel_cov,
    method = "scm",
    predictors = list(pred(c("cov1", "cov2"), 1:10))
  )
  expect_s3_class(fit, "coresynth")
  expect_equal(fit$method, "scm")
  expect_equal(sum(fit$unit_weights), 1, tolerance = 1e-4)
  expect_true(all(fit$unit_weights >= -1e-5))
  expect_equal(length(fit$Y_synth), 20)
})

test_that("SCM with pred(): v_weights have predictor names", {
  fit <- scm_fit(
    y ~ d | id + time,
    data = panel_cov,
    method = "scm",
    predictors = list(pred(c("cov1", "cov2"), 1:10))
  )
  expect_named(fit$v_weights, c("cov1[1:10]", "cov2[1:10]"))
})

test_that("SCM with pred(): predictor_table is a data.frame with correct cols", {
  fit <- scm_fit(
    y ~ d | id + time,
    data = panel_cov,
    method = "scm",
    predictors = list(pred(c("cov1", "cov2"), 1:10))
  )
  expect_s3_class(fit$predictor_table, "data.frame")
  expect_true(all(
    c("predictor", "treated", "synthetic") %in% names(fit$predictor_table)
  ))
  expect_equal(nrow(fit$predictor_table), 2L)
})

test_that("SCM with multiple pred() returns valid fit object", {
  fit <- scm_fit(
    y ~ d | id + time,
    data = panel_cov,
    method = "scm",
    predictors = list(
      pred("y", c(3, 5, 7)),
      pred("cov1", 1:10)
    )
  )
  expect_s3_class(fit, "coresynth")
  expect_equal(sum(fit$unit_weights), 1, tolerance = 1e-4)
  expect_equal(length(fit$v_weights), 2L)
  expect_true(all(is.finite(fit$Y_synth)))
})

test_that("SCM with mixed pred() entries stacks rows", {
  fit <- scm_fit(
    y ~ d | id + time,
    data = panel_cov,
    method = "scm",
    predictors = list(
      pred("cov1", 1:10),
      pred("y", c(5, 8, 10))
    )
  )
  expect_equal(length(fit$v_weights), 2L)
  expect_equal(nrow(fit$predictor_table), 2L)
})

test_that("SCM without predictors matches previous outcome-only behaviour", {
  fit_plain <- scm_fit(y ~ d | id + time, data = panel, method = "scm")
  fit_cov <- scm_fit(y ~ d | id + time, data = panel_cov, method = "scm")
  expect_equal(fit_plain$unit_weights, fit_cov$unit_weights, tolerance = 1e-4)
  expect_null(fit_cov$predictor_table)
  expect_equal(
    names(fit_plain$v_weights),
    paste0("V", seq_along(fit_plain$v_weights))
  )
})

test_that("SCM with pred(): predictor_table synthetic matches W dot X0", {
  fit <- scm_fit(
    y ~ d | id + time,
    data = panel_cov,
    method = "scm",
    predictors = list(pred(c("cov1", "cov2"), 1:10))
  )
  # Synthetic predictor values should be a weighted combination of control values
  expect_true(all(is.finite(fit$predictor_table$synthetic)))
})

test_that("SCM errors when pred() variable not in data", {
  expect_error(
    scm_fit(
      y ~ d | id + time,
      data = panel_cov,
      method = "scm",
      predictors = list(pred("nonexistent_var", 1:5))
    ),
    regexp = "not found in data"
  )
})

test_that("SCM errors when predictors is a character vector (old API)", {
  expect_error(
    scm_fit(
      y ~ d | id + time,
      data = panel_cov,
      method = "scm",
      predictors = "cov1"
    ),
    regexp = "list of pred\\(\\) objects"
  )
})

# ── Phase 7a: GSC covariates (time-varying, EM loop) ─────────────────────────

# Helper: panel with one time-varying covariate column
make_panel_with_timecov <- function(
  N = 10,
  T = 20,
  T_pre = 10,
  effect = 2.0,
  seed = 42
) {
  p <- make_panel(N = N, T = T, T_pre = T_pre, effect = effect, seed = seed)
  set.seed(seed + 1L)
  units_all <- unique(p$id)
  for (u in units_all) {
    base <- rnorm(1, 5, 1)
    p$cov1[p$id == u] <- base +
      seq(0, by = 0.1, length.out = T) +
      rnorm(T, 0, 0.2)
  }
  p
}

panel_timecov <- make_panel_with_timecov()

test_that("GSC with covariates: fit$beta has length p", {
  fit <- scm_fit(
    y ~ d | id + time,
    data = panel_timecov,
    method = "gsc",
    r = 2,
    covariates = "cov1"
  )
  expect_equal(length(fit$beta), 1L)
  expect_true(is.numeric(fit$beta))
})

test_that("GSC with covariates: ATT is finite and in reasonable range", {
  fit <- scm_fit(
    y ~ d | id + time,
    data = panel_timecov,
    method = "gsc",
    r = 2,
    covariates = "cov1"
  )
  expect_true(is.finite(fit$estimate))
  expect_true(abs(fit$estimate - 2.0) < 5.0)
})

test_that("GSC with covariates: Y_hat_co has correct dimensions (T x N_co)", {
  fit <- scm_fit(
    y ~ d | id + time,
    data = panel_timecov,
    method = "gsc",
    r = 2,
    covariates = "cov1"
  )
  expect_equal(dim(fit$Y_hat_co), c(20L, 9L))
})

test_that("GSC with covariates via scm_fit formula interface", {
  fit <- scm_fit(
    y ~ d | id + time,
    data = panel_timecov,
    method = "gsc",
    r = 2,
    covariates = "cov1"
  )
  expect_s3_class(fit, "coresynth")
  expect_equal(fit$method, "gsc")
})

test_that("GSC without covariates backward compat: beta is empty, covariates NULL", {
  fit <- scm_fit(y ~ d | id + time, data = panel, method = "gsc", r = 2)
  expect_equal(length(fit$beta), 0L)
  expect_null(fit$covariates)
  expect_true(is.finite(fit$estimate))
})

test_that("gsc_boot with covariates: p_value in [0,1], CI valid", {
  fit <- scm_fit(
    y ~ d | id + time,
    data = panel_timecov,
    method = "gsc",
    r = 2,
    covariates = "cov1"
  )
  boot <- gsc_boot(fit, B = 19L, seed = 1L)
  expect_true(boot$p_value >= 0 && boot$p_value <= 1)
  expect_true(boot$ci_lower <= boot$ci_upper)
  expect_equal(length(boot$boot_dist), 19L)
})

test_that("build_covariate_array returns correct dimensions (T x N x p)", {
  arr <- coresynth:::build_covariate_array(
    panel_timecov,
    "id",
    "time",
    "cov1",
    units = unique(panel_timecov$id),
    times = 1:20
  )
  expect_equal(dim(arr), c(20L, 10L, 1L))
  expect_true(all(is.finite(arr)))
})

test_that("GSC with covariates: beta deviates from zero (EM learned)", {
  fit <- scm_fit(
    y ~ d | id + time,
    data = panel_timecov,
    method = "gsc",
    r = 2,
    covariates = "cov1"
  )
  expect_true(abs(fit$beta[1]) > 1e-10)
})

# ── Phase 7b: SCM placebo test covariate consistency ─────────────────────────

test_that("fit_scm_cpp with covariates stores X0_mat (k x N_co)", {
  fit <- scm_fit(
    y ~ d | id + time,
    data = panel_cov,
    method = "scm",
    predictors = list(pred(c("cov1", "cov2"), 1:10))
  )
  expect_false(is.null(fit$X0_mat))
  expect_equal(dim(fit$X0_mat), c(2L, 9L))
})

test_that("mspe_ratio_pval use_covariates=TRUE returns p_value in [0,1]", {
  fit <- scm_fit(
    y ~ d | id + time,
    data = panel_cov,
    method = "scm",
    predictors = list(pred(c("cov1", "cov2"), 1:10))
  )
  inf <- mspe_ratio_pval(fit, use_covariates = TRUE)
  expect_true(inf$p_value >= 0 && inf$p_value <= 1)
  expect_equal(length(inf$mspe_ratios_all), 10L)
})

test_that("mspe_ratio_pval use_covariates=TRUE: mspe_ratios_all correct length", {
  fit <- scm_fit(
    y ~ d | id + time,
    data = panel_cov,
    method = "scm",
    predictors = list(pred("cov1", 1:10))
  )
  inf <- mspe_ratio_pval(fit, use_covariates = TRUE)
  expect_equal(length(inf$mspe_ratios_all), 10L) # treated + 9 controls
  expect_true(is.numeric(inf$n_placebo_used))
})

test_that("mspe_ratio_pval use_covariates=TRUE warns and falls back for no-cov fit", {
  fit <- scm_fit(y ~ d | id + time, data = panel, method = "scm")
  expect_null(fit$X0_mat)
  expect_warning(
    inf <- mspe_ratio_pval(fit, use_covariates = TRUE),
    regexp = "no covariates"
  )
  expect_true(inf$p_value >= 0 && inf$p_value <= 1)
})

# ── Phase 9: Abadie (2021) — OOS V selection & one-sided inference ────────────

test_that("SCM v_selection='oos' returns valid fit (Abadie 2021 §3.2)", {
  fit <- scm_fit(y ~ d | id + time, data = panel, method = "scm",
                 v_selection = "oos")
  expect_s3_class(fit, "coresynth")
  expect_equal(fit$method, "scm")
  expect_equal(sum(fit$unit_weights), 1, tolerance = 1e-4)
  expect_true(all(fit$unit_weights >= -1e-5))
  expect_true(all(is.finite(fit$Y_synth)))
})

test_that("SCM v_selection='oos' estimate is in reasonable range", {
  fit <- scm_fit(y ~ d | id + time, data = panel, method = "scm",
                 v_selection = "oos")
  expect_true(abs(fit$estimate - 2.0) < 3.0)
})

test_that("SCM v_selection='insample' and 'oos' give different (but valid) V weights", {
  fit_is  <- scm_fit(y ~ d | id + time, data = panel, method = "scm",
                     v_selection = "insample")
  fit_oos <- scm_fit(y ~ d | id + time, data = panel, method = "scm",
                     v_selection = "oos")
  expect_equal(length(fit_is$v_weights), length(fit_oos$v_weights))
  # Both should be valid simplex weights for V
  expect_equal(sum(fit_is$v_weights),  1, tolerance = 1e-4)
  expect_equal(sum(fit_oos$v_weights), 1, tolerance = 1e-4)
})

test_that("mspe_ratio_pval alternative='two.sided' matches default (Abadie 2021 §3.5)", {
  fit  <- scm_fit(y ~ d | id + time, data = panel, method = "scm")
  inf1 <- mspe_ratio_pval(fit)
  inf2 <- mspe_ratio_pval(fit, alternative = "two.sided")
  expect_equal(inf1$p_value, inf2$p_value)
  expect_equal(inf1$mspe_ratio_treated, inf2$mspe_ratio_treated)
})

test_that("mspe_ratio_pval alternative='greater' returns valid one-sided p-value", {
  fit <- scm_fit(y ~ d | id + time, data = panel, method = "scm")
  inf <- mspe_ratio_pval(fit, alternative = "greater")
  expect_true(inf$p_value >= 0 && inf$p_value <= 1)
  expect_true(is.numeric(inf$treated_effect))
  expect_equal(inf$treated_effect, fit$estimate)
})

test_that("mspe_ratio_pval alternative='less' returns valid one-sided p-value", {
  fit <- scm_fit(y ~ d | id + time, data = panel, method = "scm")
  inf_g <- mspe_ratio_pval(fit, alternative = "greater")
  inf_l <- mspe_ratio_pval(fit, alternative = "less")
  # greater and less p-values should sum to approximately 1 + 1/N (both include treated)
  expect_true(inf_l$p_value >= 0 && inf_l$p_value <= 1)
  expect_true(inf_g$p_value + inf_l$p_value <= 1 + 1/10 + 1e-6)
})

test_that("mspe_ratio_pval one-sided test is more powerful for correct direction", {
  # With a positive effect (effect=2), 'greater' p-value should be <= two-sided
  fit    <- scm_fit(y ~ d | id + time, data = panel, method = "scm")
  inf_2s <- mspe_ratio_pval(fit, alternative = "two.sided")
  inf_gt <- mspe_ratio_pval(fit, alternative = "greater")
  # One-sided can only be <= two-sided for the correct direction (not always guaranteed
  # due to different test statistics, but treated_effect should be accessible)
  expect_true(is.numeric(inf_gt$p_value))
  expect_true(is.numeric(inf_2s$p_value))
})

# ── Phase 10: Abadie & Zhao (2026) — Experimental SCM Design ─────────────────

# Shared helper: balanced panel without a treatment indicator (design setting)
make_design_panel <- function(J = 10, TT = 20, seed = 42) {
  set.seed(seed)
  f_t  <- cumsum(rnorm(TT, 0, 0.5))
  lam  <- rnorm(J, 1, 0.3)
  df   <- expand.grid(unit = paste0("m", seq_len(J)),
                      time = seq_len(TT),
                      stringsAsFactors = FALSE)
  df$outcome <- as.vector(outer(f_t, lam)) + rnorm(nrow(df), 0, 0.2)
  df
}

test_that("scm_design returns scm_design class with correct structure", {
  df  <- make_design_panel()
  res <- scm_design(df, "outcome", "unit", "time", T0 = 15L)
  expect_s3_class(res, "scm_design")
  expect_equal(res$method, "scm_design")
  expect_length(res$treated_units, 1L)
  expect_length(res$tau_hat, 5L)          # 20 - 15 post periods
  expect_equal(res$T_pre, 15L)
})

test_that("scm_design w sums to 1 and v sums to 1", {
  df  <- make_design_panel()
  res <- scm_design(df, "outcome", "unit", "time", T0 = 15L)
  expect_equal(sum(res$w), 1, tolerance = 1e-6)
  expect_equal(sum(res$v), 1, tolerance = 1e-6)
})

test_that("scm_design w and v are non-negative and disjoint", {
  df  <- make_design_panel()
  res <- scm_design(df, "outcome", "unit", "time", T0 = 15L)
  expect_true(all(res$w >= -1e-8))
  expect_true(all(res$v >= -1e-8))
  # No unit has positive weight in both w and v
  expect_true(all(res$w * res$v < 1e-10))
  # treated and control sets are disjoint
  expect_length(intersect(res$treated_units, res$control_units), 0L)
})

test_that("scm_design inference: blank periods produce valid p_value and CI", {
  df  <- make_design_panel()
  res <- scm_design(df, "outcome", "unit", "time", T0 = 15L, T_fit = 10L)
  expect_equal(length(res$u_blank), 5L)    # 15 - 10 blank periods
  expect_true(!is.na(res$p_value))
  expect_true(res$p_value >= 0 && res$p_value <= 1)
  expect_length(res$ci_lower, 5L)
  expect_length(res$ci_upper, 5L)
  expect_true(all(!is.na(res$ci_lower)))
  expect_true(all(res$ci_upper >= res$ci_lower))
})

test_that("scm_design default (no T_fit) gives NA inference without warning", {
  df  <- make_design_panel()
  res <- scm_design(df, "outcome", "unit", "time", T0 = 15L)  # no warning for default
  expect_true(is.na(res$p_value))
  expect_true(all(is.na(res$ci_lower)))
})

test_that("scm_design explicit T_fit consuming all pre-periods warns", {
  df <- make_design_panel()  # TT=20, T0=15 -> T0_idx=15
  expect_warning(
    scm_design(df, "outcome", "unit", "time", T0 = 15L, T_fit = 15L),
    "No blank periods"
  )
})

test_that("scm_design m_max=2 selects 2 treated units", {
  df  <- make_design_panel(J = 10L)
  res <- scm_design(df, "outcome", "unit", "time", T0 = 15L,
                    m_min = 2L, m_max = 2L)
  expect_length(res$treated_units, 2L)
  expect_equal(sum(res$w > 1e-6), 2L)    # exactly 2 units have positive w
})

test_that("scm_design m_min=1, m_max=2 returns between 1 and 2 treated units", {
  df  <- make_design_panel(J = 8L)
  res <- scm_design(df, "outcome", "unit", "time", T0 = 12L,
                    m_min = 1L, m_max = 2L)
  n_treated <- length(res$treated_units)
  expect_true(n_treated >= 1L && n_treated <= 2L)
})

test_that("scm_design weakly_targeted variant works", {
  df  <- make_design_panel()
  res <- scm_design(df, "outcome", "unit", "time", T0 = 15L,
                    design = "weakly_targeted", beta = 1)
  expect_equal(res$design, "weakly_targeted")
  expect_equal(sum(res$w), 1, tolerance = 1e-6)
  expect_equal(sum(res$v), 1, tolerance = 1e-6)
})

test_that("scm_design unit_level variant works and returns disjoint weights", {
  df  <- make_design_panel(J = 8L)
  res <- scm_design(df, "outcome", "unit", "time", T0 = 12L,
                    m_max = 2L, design = "unit_level", xi = 1)
  expect_equal(res$design, "unit_level")
  expect_true(all(res$w * res$v < 1e-10))
})

test_that("scm_design custom population weights f are normalised and applied", {
  J     <- 8L
  units <- paste0("m", seq_len(J))
  df    <- make_design_panel(J = J)
  f_raw <- setNames(c(0.4, 0.2, 0.1, 0.1, 0.1, 0.05, 0.03, 0.02), units)
  res   <- scm_design(df, "outcome", "unit", "time", T0 = 12L, f = f_raw)
  expect_equal(sum(res$f), 1, tolerance = 1e-10)
  # f stored in the object should be normalised
  expect_equal(res$f[["m1"]], 0.4 / sum(f_raw), tolerance = 1e-10)
})

test_that("scm_design with pred() predictors works", {
  df <- make_design_panel(J = 8L, TT = 20L)
  # Add a covariate
  df$cov1 <- rnorm(nrow(df), 5, 1)
  res <- scm_design(
    df, "outcome", "unit", "time", T0 = 15L,
    predictors = list(
      pred("outcome", 1:10),
      pred("cov1",   1:15)
    )
  )
  expect_s3_class(res, "scm_design")
  expect_true(grepl("outcome", res$pred_names[1]))
  expect_true(grepl("cov1", res$pred_names[length(res$pred_names)]))
})

test_that("scm_design estimate and tau_hat are finite", {
  df  <- make_design_panel(J = 10L, TT = 25L)
  res <- scm_design(df, "outcome", "unit", "time", T0 = 18L, T_fit = 12L)
  expect_true(is.finite(res$estimate))
  expect_true(all(is.finite(res$tau_hat)))
  expect_true(all(is.finite(res$u_blank)))
})

test_that("scm_design print output mentions scm_design", {
  df  <- make_design_panel()
  res <- scm_design(df, "outcome", "unit", "time", T0 = 15L, T_fit = 10L)
  expect_output(print(res), "scm_design")
  expect_output(print(res), "Treated units")
})

test_that("scm_design summary output shows design variant and CI", {
  df  <- make_design_panel()
  res <- scm_design(df, "outcome", "unit", "time", T0 = 15L, T_fit = 10L)
  expect_output(summary(res), "Design variant")
  expect_output(summary(res), "Confidence intervals")
})

test_that("scm_design plot outcome type returns ggplot", {
  df  <- make_design_panel()
  res <- scm_design(df, "outcome", "unit", "time", T0 = 15L)
  p   <- plot(res, type = "outcome")
  expect_s3_class(p, "gg")
})

test_that("scm_design plot gap type with blank periods returns ggplot with ribbon", {
  df  <- make_design_panel()
  res <- scm_design(df, "outcome", "unit", "time", T0 = 15L, T_fit = 10L)
  p   <- plot(res, type = "gap")
  expect_s3_class(p, "gg")
})

test_that("scm_design error on invalid m_max >= J", {
  df <- make_design_panel(J = 5L)
  expect_error(
    scm_design(df, "outcome", "unit", "time", T0 = 10L, m_max = 5L),
    "m_max < J"
  )
})

test_that("scm_design error on missing outcome column", {
  df <- make_design_panel()
  expect_error(
    scm_design(df, "no_col", "unit", "time", T0 = 15L),
    "not found"
  )
})

test_that("scm_design error when T0 not in time column", {
  df <- make_design_panel()
  expect_error(
    scm_design(df, "outcome", "unit", "time", T0 = 999L),
    "present in the time column"
  )
})

# ── Phase 11a: Donor pool filtering (Abadie 2021 §4) ─────────────────────────

# Helper: panel where two donors have extreme pre-treatment trajectories
make_panel_outliers <- function() {
  df <- make_panel(N = 8L, T = 20L, T_pre = 10L, seed = 42L)
  df$y[df$id == "u5"] <- df$y[df$id == "u5"] * 20  # clear outlier
  df$y[df$id == "u6"] <- df$y[df$id == "u6"] * 20  # clear outlier
  df
}

test_that("donor_mspe_threshold = Inf (default) keeps all donors", {
  fit <- scm_fit(y ~ d | id + time, data = panel, method = "scm")
  expect_equal(fit$excluded_donors, character(0L))
})

test_that("donor_mspe_threshold too strict raises warning and keeps all donors", {
  # threshold = 1 means keep only donors with exactly min MSPE → < 2 donors → warning
  expect_warning(
    fit <- scm_fit(
      y ~ d | id + time, data = panel, method = "scm",
      donor_mspe_threshold = 1.0
    ),
    "ignoring filter"
  )
  expect_equal(fit$excluded_donors, character(0L))
  expect_equal(sum(fit$unit_weights), 1, tolerance = 1e-4)
})

test_that("donor_mspe_threshold excludes clear outliers", {
  df  <- make_panel_outliers()
  fit <- scm_fit(y ~ d | id + time, data = df, method = "scm",
                 donor_mspe_threshold = 5)
  expect_true(length(fit$excluded_donors) > 0L)
  expect_true("u5" %in% fit$excluded_donors || "u6" %in% fit$excluded_donors)
})

test_that("excluded donors are absent from unit_weights", {
  df  <- make_panel_outliers()
  fit <- scm_fit(y ~ d | id + time, data = df, method = "scm",
                 donor_mspe_threshold = 5)
  if (length(fit$excluded_donors) > 0L) {
    expect_false(any(fit$excluded_donors %in% names(fit$unit_weights)))
  }
})

test_that("weights remain on the simplex after donor filtering", {
  df  <- make_panel_outliers()
  fit <- scm_fit(y ~ d | id + time, data = df, method = "scm",
                 donor_mspe_threshold = 5)
  expect_equal(sum(fit$unit_weights), 1, tolerance = 1e-4)
  expect_true(all(fit$unit_weights >= -1e-5))
})

test_that("ATT is finite after donor filtering", {
  df  <- make_panel_outliers()
  fit <- scm_fit(y ~ d | id + time, data = df, method = "scm",
                 donor_mspe_threshold = 5)
  expect_true(is.finite(fit$estimate))
})

test_that("donor_mspe_threshold via scm_fit() passes through correctly", {
  df  <- make_panel_outliers()
  fit <- scm_fit(y ~ d | id + time, data = df, method = "scm",
                 donor_mspe_threshold = 5)
  expect_type(fit$excluded_donors, "character")
})

# ── Phase 11b: Penalised SCM (Abadie & L'Hour 2021) ─────────────────────────

test_that("lambda_pen = NULL gives standard SCM (lambda_pen field is NA)", {
  fit <- scm_fit(y ~ d | id + time, data = panel, method = "scm",
                 lambda_pen = NULL)
  expect_true(is.na(fit$lambda_pen))
})

test_that("lambda_pen = 0 weights lie on the unit simplex", {
  fit <- scm_fit(y ~ d | id + time, data = panel, method = "scm",
                 lambda_pen = 0)
  expect_equal(sum(fit$unit_weights), 1, tolerance = 1e-4)
  expect_true(all(fit$unit_weights >= -1e-5))
})

test_that("lambda_pen = 0 records lambda_pen = 0 in fit object", {
  fit <- scm_fit(y ~ d | id + time, data = panel, method = "scm",
                 lambda_pen = 0)
  expect_equal(fit$lambda_pen, 0)
})

test_that("lambda_pen = 1 weights lie on the unit simplex", {
  fit <- scm_fit(y ~ d | id + time, data = panel, method = "scm",
                 lambda_pen = 1)
  expect_equal(sum(fit$unit_weights), 1, tolerance = 1e-4)
  expect_true(all(fit$unit_weights >= -1e-5))
  expect_true(is.finite(fit$estimate))
})

test_that("large lambda_pen concentrates weights (fewer donors used)", {
  fit_pen  <- scm_fit(y ~ d | id + time, data = panel, method = "scm",
                      lambda_pen = 1e6)
  fit_std  <- scm_fit(y ~ d | id + time, data = panel, method = "scm")
  # Very large penalty pushes toward nearest-neighbor → max weight should increase
  expect_true(max(fit_pen$unit_weights) >= max(fit_std$unit_weights) - 1e-4)
})

test_that("lambda_pen = 'auto' returns a non-negative finite lambda", {
  fit <- scm_fit(y ~ d | id + time, data = panel, method = "scm",
                 lambda_pen = "auto")
  expect_true(is.numeric(fit$lambda_pen))
  expect_true(is.finite(fit$lambda_pen))
  expect_true(fit$lambda_pen >= 0)
})

test_that("lambda_pen = 'auto' gives finite ATT and valid weights", {
  fit <- scm_fit(y ~ d | id + time, data = panel, method = "scm",
                 lambda_pen = "auto")
  expect_true(is.finite(fit$estimate))
  expect_equal(sum(fit$unit_weights), 1, tolerance = 1e-4)
  expect_true(all(fit$unit_weights >= -1e-5))
})

test_that("invalid lambda_pen raises an error", {
  expect_error(
    scm_fit(y ~ d | id + time, data = panel, method = "scm",
            lambda_pen = -1),
    "non-negative"
  )
  expect_error(
    scm_fit(y ~ d | id + time, data = panel, method = "scm",
            lambda_pen = "bad_value"),
    "NULL"
  )
})

# ── Phase 11c: Augmented SCM (Ben-Michael, Feller & Rothstein 2021) ──────────

test_that("augment_scm returns a list with required fields", {
  fit <- scm_fit(y ~ d | id + time, data = panel, method = "scm")
  aug <- augment_scm(fit)
  expect_type(aug, "list")
  expect_true(all(c("att_aug", "delta", "att_scm", "lambda_ridge", "beta_hat") %in% names(aug)))
})

test_that("augment_scm att_aug and delta are finite", {
  fit <- scm_fit(y ~ d | id + time, data = panel, method = "scm")
  aug <- augment_scm(fit)
  expect_true(is.finite(aug$att_aug))
  expect_true(is.finite(aug$delta))
})

test_that("augment_scm att_scm equals fit$estimate", {
  fit <- scm_fit(y ~ d | id + time, data = panel, method = "scm")
  aug <- augment_scm(fit)
  expect_equal(aug$att_scm, fit$estimate)
})

test_that("augment_scm beta_hat has length equal to T_pre", {
  fit <- scm_fit(y ~ d | id + time, data = panel, method = "scm")
  aug <- augment_scm(fit)
  expect_length(aug$beta_hat, fit$T_pre)
})

test_that("augment_scm auto-selected lambda_ridge is positive", {
  fit <- scm_fit(y ~ d | id + time, data = panel, method = "scm")
  aug <- augment_scm(fit)
  expect_true(aug$lambda_ridge > 0)
})

test_that("augment_scm with very large lambda_ridge gives delta near zero", {
  fit <- scm_fit(y ~ d | id + time, data = panel, method = "scm")
  aug <- augment_scm(fit, lambda_ridge = 1e12)
  # With huge lambda, beta -> 0, so delta = m_tr - W'm_co -> 0
  expect_true(abs(aug$delta) < 1e-3)
  expect_equal(aug$att_aug, aug$att_scm, tolerance = 1e-3)
})

test_that("augment_scm explicit lambda_ridge is honoured", {
  fit <- scm_fit(y ~ d | id + time, data = panel, method = "scm")
  aug <- augment_scm(fit, lambda_ridge = 0.5)
  expect_equal(aug$lambda_ridge, 0.5)
  expect_true(is.finite(aug$att_aug))
})

test_that("augment_scm rejects wrong method or missing Y_co_pre", {
  fit_sdid <- scm_fit(y ~ d | id + time, data = panel, method = "sdid")
  expect_error(augment_scm(fit_sdid), "method = 'scm'")

  fit_scm <- scm_fit(y ~ d | id + time, data = panel, method = "scm")
  fit_bad <- fit_scm
  fit_bad$Y_co_pre <- NULL
  expect_error(augment_scm(fit_bad), "Y_co_pre")
})

# ── Phase 12: SCM 高速化 ──────────────────────────────────────────────────────

test_that("Phase 12b: low-rank path returns valid simplex weights (k=7, N_co=46)", {
  set.seed(42L)
  k <- 7L; N_co <- 46L
  X0     <- matrix(rnorm(k * N_co), k, N_co)
  X1     <- rnorm(k)
  V_diag <- rep(1 / k, k)
  w <- scm_inner_weights_cpp(X0, X1, V_diag)
  expect_equal(length(w), N_co)
  expect_equal(sum(w), 1, tolerance = 1e-4)
  expect_true(all(w >= -1e-5))
})

test_that("Phase 12b: low-rank and full QP give algebraically equivalent weights", {
  set.seed(7L)
  k <- 5L; N_co <- 20L    # 2*k=10 < N_co=20 → low-rank path
  X0     <- matrix(rnorm(k * N_co), k, N_co)
  X1     <- rnorm(k)
  V_diag <- runif(k); V_diag <- V_diag / sum(V_diag)
  w_lr   <- scm_inner_weights_cpp(X0, X1, V_diag)
  # Reference via explicit full QP
  Q_full <- t(X0) %*% diag(V_diag) %*% X0
  c_full <- drop(t(X0) %*% diag(V_diag) %*% X1)
  w_full <- solve_simplex_qp(Q_full, c_full)
  expect_equal(w_lr, w_full, tolerance = 1e-3)
})

test_that("Phase 12b: full path used when k >= N_co/2 (k=10, N_co=16)", {
  set.seed(11L)
  k <- 10L; N_co <- 16L   # 2*k=20 >= N_co=16 → full path
  X0 <- matrix(rnorm(k * N_co), k, N_co)
  X1 <- rnorm(k)
  w  <- scm_inner_weights_cpp(X0, X1, rep(1 / k, k))
  expect_equal(sum(w), 1, tolerance = 1e-4)
  expect_true(all(w >= -1e-5))
})

test_that("Phase 12a: v_optim='bfgs' returns valid simplex weights", {
  fit <- scm_fit(y ~ d | id + time, data = panel, method = "scm",
                 v_optim = "bfgs")
  expect_equal(sum(fit$unit_weights), 1, tolerance = 1e-4)
  expect_true(all(fit$unit_weights >= -1e-5))
  expect_true(is.finite(fit$estimate))
})

test_that("Phase 12a: v_optim='bfgs' + v_selection='oos' works", {
  fit <- scm_fit(y ~ d | id + time, data = panel, method = "scm",
                 v_optim = "bfgs", v_selection = "oos")
  expect_equal(sum(fit$unit_weights), 1, tolerance = 1e-4)
  expect_true(is.finite(fit$estimate))
})

test_that("Phase 12a: v_optim='coord_descent' is identical to default", {
  fit1 <- scm_fit(y ~ d | id + time, data = panel, method = "scm")
  fit2 <- scm_fit(y ~ d | id + time, data = panel, method = "scm",
                  v_optim = "coord_descent")
  expect_equal(fit1$unit_weights, fit2$unit_weights, tolerance = 1e-6)
})

test_that("Phase 12a: v_optim='auto' returns valid weights", {
  # panel: k=T_pre=10, N_co=9 → k=10 > 15? No. But 2*k=20 > N_co=9? Yes
  # auto: k_dim=10 > 15 → coord_descent; still valid
  fit <- scm_fit(y ~ d | id + time, data = panel, method = "scm",
                 v_optim = "auto")
  expect_equal(sum(fit$unit_weights), 1, tolerance = 1e-4)
  expect_true(is.finite(fit$estimate))
})

test_that("Phase 12a: v_optim='bfgs' with lambda_pen works", {
  fit <- scm_fit(y ~ d | id + time, data = panel, method = "scm",
                 v_optim = "bfgs", lambda_pen = 0.5)
  expect_equal(sum(fit$unit_weights), 1, tolerance = 1e-4)
  expect_true(is.finite(fit$estimate))
})

# ── Phase 13a: SDID 共変量対応 ────────────────────────────────────────────────

# Time-varying covariate panel: panel_cov already has cov1/cov2 (time-invariant here;
# sufficient to test the partial-out API)
test_that("Phase 13a: SDID with one covariate returns valid fit", {
  fit <- scm_fit(y ~ d | id + time, data = panel_cov, method = "sdid",
                 covariates = "cov1")
  expect_s3_class(fit, "coresynth")
  expect_true(is.finite(fit$estimate))
})

test_that("Phase 13a: beta_hat length equals number of covariates", {
  fit <- scm_fit(y ~ d | id + time, data = panel_cov, method = "sdid",
                 covariates = c("cov1", "cov2"))
  expect_length(fit$beta_hat, 2L)
})

test_that("Phase 13a: SDID covariates — unit weights stay on simplex", {
  fit <- scm_fit(y ~ d | id + time, data = panel_cov, method = "sdid",
                 covariates = "cov1")
  expect_equal(sum(fit$unit_weights), 1, tolerance = 1e-4)
  expect_true(all(fit$unit_weights >= -1e-8))
})

test_that("Phase 13a: SDID without covariates — beta_hat is length-0", {
  fit <- scm_fit(y ~ d | id + time, data = panel_cov, method = "sdid")
  expect_length(fit$beta_hat, 0L)
})

test_that("Phase 13a: covariate partial-out changes the ATT estimate", {
  fit_no  <- scm_fit(y ~ d | id + time, data = panel_cov, method = "sdid")
  fit_cov <- scm_fit(y ~ d | id + time, data = panel_cov, method = "sdid",
                     covariates = "cov1")
  expect_false(isTRUE(all.equal(fit_no$estimate, fit_cov$estimate, tol = 1e-6)))
})

test_that("Phase 13a: staggered=FALSE for sharp SDID", {
  fit <- scm_fit(y ~ d | id + time, data = panel, method = "sdid")
  expect_false(fit$staggered)
})

# ── Phase 13b: SDID Staggered Adoption ───────────────────────────────────────

test_that("Phase 13b: SDID handles staggered adoption without error", {
  expect_no_error(
    scm_fit(y ~ d | id + time, data = staggered, method = "sdid")
  )
})

test_that("Phase 13b: SDID staggered — staggered field is TRUE", {
  fit <- scm_fit(y ~ d | id + time, data = staggered, method = "sdid")
  expect_true(fit$staggered)
})

test_that("Phase 13b: SDID staggered — estimate is finite", {
  fit <- scm_fit(y ~ d | id + time, data = staggered, method = "sdid")
  expect_true(is.finite(fit$estimate))
})

test_that("Phase 13b: SDID staggered — cohort_estimates has correct columns", {
  fit <- scm_fit(y ~ d | id + time, data = staggered, method = "sdid")
  expected_cols <- c("cohort", "n_treated", "T_pre", "T_post", "estimate", "weight")
  expect_true(all(expected_cols %in% names(fit$cohort_estimates)))
})

test_that("Phase 13b: SDID staggered — cohort weights sum to 1", {
  fit <- scm_fit(y ~ d | id + time, data = staggered, method = "sdid")
  expect_equal(sum(fit$cohort_estimates$weight), 1, tolerance = 1e-8)
})

test_that("Phase 13b: SDID staggered — each cohort estimate is finite", {
  fit <- scm_fit(y ~ d | id + time, data = staggered, method = "sdid")
  expect_true(all(is.finite(fit$cohort_estimates$estimate)))
})

test_that("Phase 13b: SDID staggered control_group='never_treated' works", {
  fit <- scm_fit(y ~ d | id + time, data = staggered, method = "sdid",
                 control_group = "never_treated")
  expect_true(fit$staggered)
  expect_true(is.finite(fit$estimate))
})

test_that("Phase 13b: SDID staggered control_group='clean' is default", {
  fit_clean  <- scm_fit(y ~ d | id + time, data = staggered, method = "sdid",
                        control_group = "clean")
  fit_never  <- scm_fit(y ~ d | id + time, data = staggered, method = "sdid",
                        control_group = "never_treated")
  # Different control groups should (in general) give different estimates
  expect_true(is.finite(fit_clean$estimate))
  expect_true(is.finite(fit_never$estimate))
})

test_that("Phase 13b: SDID staggered — Y_treat dimensions match panel", {
  fit <- scm_fit(y ~ d | id + time, data = staggered, method = "sdid")
  pan <- coresynth:::panel_to_matrices(staggered$y, staggered$d,
                                       staggered$id, staggered$time)
  expect_equal(nrow(fit$Y_treat), nrow(pan$Y))
})

# ── Phase 14a: SDID staggered + covariates ────────────────────────────────────

make_staggered_cov_panel <- function(N = 12, TT = 24, seed = 7) {
  p <- make_staggered_panel(N = N, TT = TT, seed = seed)
  set.seed(seed + 10L)
  p$cov1 <- NA_real_
  for (u in unique(p$id)) {
    base_val <- rnorm(1, 5, 1)
    p$cov1[p$id == u] <- base_val +
      seq(0, by = 0.05, length.out = TT) + rnorm(TT, 0, 0.1)
  }
  p
}

stag_cov <- make_staggered_cov_panel()

test_that("Phase 14a: SDID staggered + covariates — no error", {
  expect_no_error(
    scm_fit(y ~ d | id + time, data = stag_cov, method = "sdid",
            covariates = "cov1")
  )
})

test_that("Phase 14a: SDID staggered + covariates — beta_hat length equals p", {
  fit <- scm_fit(y ~ d | id + time, data = stag_cov, method = "sdid",
                 covariates = "cov1")
  expect_length(fit$beta_hat, 1L)
})

test_that("Phase 14a: SDID staggered + covariates — staggered=TRUE preserved", {
  fit <- scm_fit(y ~ d | id + time, data = stag_cov, method = "sdid",
                 covariates = "cov1")
  expect_true(fit$staggered)
})

test_that("Phase 14a: SDID staggered + covariates — estimate is finite", {
  fit <- scm_fit(y ~ d | id + time, data = stag_cov, method = "sdid",
                 covariates = "cov1")
  expect_true(is.finite(fit$estimate))
})

test_that("Phase 14a: SDID staggered + covariates — cohort_estimates columns correct", {
  fit <- scm_fit(y ~ d | id + time, data = stag_cov, method = "sdid",
                 covariates = "cov1")
  expected_cols <- c("cohort", "n_treated", "T_pre", "T_post", "estimate", "weight")
  expect_true(all(expected_cols %in% names(fit$cohort_estimates)))
})

test_that("Phase 14a: SDID staggered + covariates — cohort weights sum to 1", {
  fit <- scm_fit(y ~ d | id + time, data = stag_cov, method = "sdid",
                 covariates = "cov1")
  expect_equal(sum(fit$cohort_estimates$weight), 1, tolerance = 1e-8)
})

test_that("Phase 14a: SDID staggered + covariates — covariate changes estimate", {
  fit_no_cov <- scm_fit(y ~ d | id + time, data = stag_cov, method = "sdid")
  fit_cov    <- scm_fit(y ~ d | id + time, data = stag_cov, method = "sdid",
                        covariates = "cov1")
  expect_false(isTRUE(all.equal(fit_no_cov$estimate, fit_cov$estimate,
                                tolerance = 1e-10)))
})

test_that("Phase 14a: SDID staggered + covariates — control_group='never_treated' works", {
  fit <- scm_fit(y ~ d | id + time, data = stag_cov, method = "sdid",
                 covariates = "cov1", control_group = "never_treated")
  expect_true(fit$staggered)
  expect_true(is.finite(fit$estimate))
})

# ── Phase 14b: SCM staggered adoption ─────────────────────────────────────────

test_that("Phase 14b: SCM handles staggered adoption without error", {
  expect_no_error(
    scm_fit(y ~ d | id + time, data = staggered, method = "scm")
  )
})

test_that("Phase 14b: SCM staggered — staggered = TRUE", {
  fit <- scm_fit(y ~ d | id + time, data = staggered, method = "scm")
  expect_true(fit$staggered)
})

test_that("Phase 14b: SCM staggered — estimate is finite", {
  fit <- scm_fit(y ~ d | id + time, data = staggered, method = "scm")
  expect_true(is.finite(fit$estimate))
})

test_that("Phase 14b: SCM staggered — cohort_estimates has required columns", {
  fit <- scm_fit(y ~ d | id + time, data = staggered, method = "scm")
  expected_cols <- c("cohort", "n_treated", "T_pre", "T_post", "estimate", "weight")
  expect_true(all(expected_cols %in% names(fit$cohort_estimates)))
})

test_that("Phase 14b: SCM staggered — cohort weights sum to 1", {
  fit <- scm_fit(y ~ d | id + time, data = staggered, method = "scm")
  expect_equal(sum(fit$cohort_estimates$weight), 1, tolerance = 1e-8)
})

test_that("Phase 14b: SCM staggered — each cohort unit_weights on simplex", {
  fit <- scm_fit(y ~ d | id + time, data = staggered, method = "scm")
  for (cf in fit$cohort_fits) {
    w <- cf$unit_weights
    expect_equal(sum(w), 1, tolerance = 1e-5)
    expect_true(all(w >= -1e-5))
  }
})

test_that("Phase 14b: SCM staggered — v_selection='oos' gives finite ATT", {
  fit <- scm_fit(y ~ d | id + time, data = staggered, method = "scm",
                 v_selection = "oos")
  expect_true(is.finite(fit$estimate))
})

test_that("Phase 14b: SCM staggered — predictors != NULL errors", {
  expect_error(
    scm_fit(y ~ d | id + time, data = staggered, method = "scm",
            predictors = list(pred("y", 1:3))),
    regexp = "predictors"
  )
})

test_that("Phase 14b: SCM staggered — estimate within ±4 of true ATT (1.5)", {
  fit <- scm_fit(y ~ d | id + time, data = staggered, method = "scm")
  expect_lt(abs(fit$estimate - 1.5), 4.0)
})

test_that("Phase 14b: SCM staggered — Y_treat nrow matches panel T", {
  fit <- scm_fit(y ~ d | id + time, data = staggered, method = "scm")
  expect_equal(nrow(fit$Y_treat), 24L)
})

test_that("Phase 14b: SCM staggered — control_group='never_treated' gives finite ATT", {
  fit <- scm_fit(y ~ d | id + time, data = staggered, method = "scm",
                 control_group = "never_treated")
  expect_true(is.finite(fit$estimate))
})

# ── Phase 14c: augment.coresynth() ───────────────────────────────────────────

test_that("Phase 14c: augment() returns data.frame for SCM", {
  fit <- scm_fit(y ~ d | id + time, data = panel, method = "scm")
  aug <- broom::augment(fit)
  expect_s3_class(aug, "data.frame")
})

test_that("Phase 14c: augment() column names are correct", {
  fit <- scm_fit(y ~ d | id + time, data = panel, method = "scm")
  aug <- broom::augment(fit)
  expected <- c(".time", ".observed", ".fitted", ".resid", ".treated", ".period")
  expect_true(all(expected %in% names(aug)))
})

test_that("Phase 14c: augment() nrow equals T", {
  fit <- scm_fit(y ~ d | id + time, data = panel, method = "scm")
  aug <- broom::augment(fit)
  expect_equal(nrow(aug), 20L)
})

test_that("Phase 14c: augment() .resid == .observed - .fitted", {
  fit <- scm_fit(y ~ d | id + time, data = panel, method = "scm")
  aug <- broom::augment(fit)
  expect_equal(aug$.resid, aug$.observed - aug$.fitted, tolerance = 1e-10)
})

test_that("Phase 14c: augment() .treated is FALSE pre, TRUE post", {
  fit <- scm_fit(y ~ d | id + time, data = panel, method = "scm")
  aug <- broom::augment(fit)
  T_pre <- fit$T_pre
  expect_true(all(!aug$.treated[seq_len(T_pre)]))
  expect_true(all(aug$.treated[(T_pre + 1L):nrow(aug)]))
})

test_that("Phase 14c: augment() .period factor levels correct", {
  fit <- scm_fit(y ~ d | id + time, data = panel, method = "scm")
  aug <- broom::augment(fit)
  expect_equal(levels(aug$.period), c("pre", "post"))
})

test_that("Phase 14c: augment() works for SDID sharp", {
  fit <- scm_fit(y ~ d | id + time, data = panel, method = "sdid")
  aug <- broom::augment(fit)
  expect_s3_class(aug, "data.frame")
  expect_true(all(c(".time", ".resid") %in% names(aug)))
})

test_that("Phase 14c: augment() works for GSC", {
  fit <- scm_fit(y ~ d | id + time, data = panel, method = "gsc", r = 2L)
  aug <- broom::augment(fit)
  expect_s3_class(aug, "data.frame")
  expect_equal(nrow(aug), 20L)
})

test_that("Phase 14c (Phase 24-updated): augment() staggered SDID returns cohort-long df", {
  # Phase 24: augment.coresynth() now supports staggered fits and returns a
  # long-format data.frame stacking per-cohort treated paths (and donors when
  # include_donors=TRUE). The old behavior (warning + empty df) is replaced.
  fit <- scm_fit(y ~ d | id + time, data = staggered, method = "sdid")
  aug <- broom::augment(fit)
  expect_s3_class(aug, "data.frame")
  expect_gt(nrow(aug), 0L)
  expect_true(".cohort" %in% names(aug))
})

test_that("Phase 14c: tidy() returns cohort_estimates for staggered SCM", {
  fit <- scm_fit(y ~ d | id + time, data = staggered, method = "scm")
  td <- broom::tidy(fit)
  expect_s3_class(td, "data.frame")
  expect_true(nrow(td) > 0L)
  expect_true("cohort" %in% names(td) || grepl("^cohort_", td$term[1]))
})

# ── Phase 15a: SDID 推論 API (sdid_inference) ─────────────────────────────────

test_that("Phase 15a: SDID sharp fit stores Y_co_pre and Y_co_post", {
  fit <- scm_fit(y ~ d | id + time, data = panel, method = "sdid")
  expect_equal(dim(fit$Y_co_pre),  c(10L, 9L))  # T_pre x N_co
  expect_equal(dim(fit$Y_co_post), c(10L, 9L))  # T_post x N_co
})

test_that("Phase 15a: SDID sharp fit stores Y_tr_pre_mean, Y_tr_post_mean, N_tr", {
  fit <- scm_fit(y ~ d | id + time, data = panel, method = "sdid")
  expect_length(fit$Y_tr_pre_mean,  10L)  # T_pre
  expect_length(fit$Y_tr_post_mean, 10L)  # T_post
  expect_equal(fit$N_tr, 1L)
})

test_that("Phase 15a: sdid_inference(placebo) returns valid structure", {
  fit <- scm_fit(y ~ d | id + time, data = panel, method = "sdid")
  inf <- sdid_inference(fit, method = "placebo")
  expect_equal(inf$estimate, fit$estimate)
  expect_true(inf$p_value >= 0 && inf$p_value <= 1)
  expect_length(inf$placebo_effects, 9L)  # N_co = 9
  expect_null(inf$se)
  expect_null(inf$ci_lower)
  expect_equal(inf$n_controls, 9L)
})

test_that("Phase 15a: sdid_inference(placebo) named placebo_effects", {
  fit <- scm_fit(y ~ d | id + time, data = panel, method = "sdid")
  inf <- sdid_inference(fit, method = "placebo")
  expect_equal(names(inf$placebo_effects), colnames(fit$Y_co_pre))
})

test_that("Phase 15a: sdid_inference(bootstrap) returns se and CI", {
  fit <- scm_fit(y ~ d | id + time, data = panel, method = "sdid")
  inf <- sdid_inference(fit, method = "bootstrap", n_boot = 50L, seed = 1L)
  expect_true(is.finite(inf$se) && inf$se > 0)
  expect_true(inf$p_value >= 0 && inf$p_value <= 1)
  expect_true(inf$ci_lower < inf$ci_upper)
  expect_length(inf$boot_ests, 50L)
})

test_that("Phase 15a: sdid_inference(bootstrap) is reproducible with seed", {
  fit <- scm_fit(y ~ d | id + time, data = panel, method = "sdid")
  inf1 <- sdid_inference(fit, method = "bootstrap", n_boot = 30L, seed = 99L)
  inf2 <- sdid_inference(fit, method = "bootstrap", n_boot = 30L, seed = 99L)
  expect_equal(inf1$se, inf2$se)
  expect_equal(inf1$boot_ests, inf2$boot_ests)
})

test_that("Phase 15a: sdid_inference(jackknife) returns se and CI", {
  fit <- scm_fit(y ~ d | id + time, data = panel, method = "sdid")
  inf <- sdid_inference(fit, method = "jackknife")
  expect_true(is.finite(inf$se) && inf$se > 0)
  expect_true(inf$p_value >= 0 && inf$p_value <= 1)
  expect_true(inf$ci_lower < inf$ci_upper)
  expect_null(inf$boot_ests)
})

test_that("Phase 15a: sdid_inference alternative='greater' gives one-sided p-value", {
  fit <- scm_fit(y ~ d | id + time, data = panel, method = "sdid")
  inf_g <- sdid_inference(fit, method = "placebo", alternative = "greater")
  inf_l <- sdid_inference(fit, method = "placebo", alternative = "less")
  inf_t <- sdid_inference(fit, method = "placebo", alternative = "two.sided")
  expect_equal(inf_g$alternative, "greater")
  expect_true(inf_g$p_value >= 0 && inf_g$p_value <= 1)
  # sum of one-sided p-values cannot exceed 1 + 1/N
  N_co <- inf_t$n_controls
  expect_true(inf_g$p_value + inf_l$p_value <= 1 + 1 / (N_co + 1) + 1e-9)
})

test_that("Phase 15a: sdid_inference errors on non-SDID fit", {
  fit <- scm_fit(y ~ d | id + time, data = panel, method = "scm")
  expect_error(sdid_inference(fit), regexp = "method = 'sdid'")
})

test_that("Phase 15a: sdid_inference staggered placebo no longer errors (Phase 20)", {
  fit <- scm_fit(y ~ d | id + time, data = staggered, method = "sdid")
  expect_no_error(sdid_inference(fit, method = "placebo"))
})

test_that("Phase 15a: print.sdid_inference produces output", {
  fit <- scm_fit(y ~ d | id + time, data = panel, method = "sdid")
  inf <- sdid_inference(fit, method = "jackknife")
  expect_output(print(inf), regexp = "Estimate")
  expect_output(print(inf), regexp = "p-value")
})

test_that("Phase 15a: SDID covariates fit also stores Y_co_pre/Y_co_post", {
  fit <- scm_fit(y ~ d | id + time, data = stag_cov[stag_cov$id %in%
    c("u1", paste0("u", 3:12)), ],
    method = "sdid", covariates = "cov1")
  # Sharp fit with covariates should store Y_co_pre
  if (!isTRUE(fit$staggered)) {
    expect_false(is.null(fit$Y_co_pre))
  }
})

# ── Phase 15b: SCM staggered covariates ────────────────────────────────────────

test_that("Phase 15b: SCM staggered + covariates — no error", {
  expect_no_error(
    scm_fit(y ~ d | id + time, data = stag_cov, method = "scm",
            covariates = "cov1")
  )
})

test_that("Phase 15b: SCM staggered + covariates — staggered=TRUE preserved", {
  fit <- scm_fit(y ~ d | id + time, data = stag_cov, method = "scm",
                 covariates = "cov1")
  expect_true(fit$staggered)
})

test_that("Phase 15b: SCM staggered + covariates — beta_hat length equals p", {
  fit <- scm_fit(y ~ d | id + time, data = stag_cov, method = "scm",
                 covariates = "cov1")
  expect_length(fit$beta_hat, 1L)
})

test_that("Phase 15b: SCM staggered + covariates — estimate is finite", {
  fit <- scm_fit(y ~ d | id + time, data = stag_cov, method = "scm",
                 covariates = "cov1")
  expect_true(is.finite(fit$estimate))
})

test_that("Phase 15b: SCM staggered + covariates — cohort_estimates exist", {
  fit <- scm_fit(y ~ d | id + time, data = stag_cov, method = "scm",
                 covariates = "cov1")
  expect_s3_class(fit$cohort_estimates, "data.frame")
  expect_true(nrow(fit$cohort_estimates) > 0L)
})

test_that("Phase 15b: SCM staggered without covariates unchanged (regression)", {
  fit_no  <- scm_fit(y ~ d | id + time, data = staggered, method = "scm")
  fit_cov <- scm_fit(y ~ d | id + time, data = stag_cov, method = "scm",
                     covariates = "cov1")
  # Estimates differ when covariates are included
  expect_false(isTRUE(all.equal(fit_no$estimate, fit_cov$estimate)))
})

test_that("Phase 15b: SCM staggered + predictors still errors", {
  fit_sharp <- scm_fit(y ~ d | id + time, data = panel, method = "scm")
  T_pre <- fit_sharp$T_pre
  times <- sort(unique(panel$time))
  expect_error(
    scm_fit(y ~ d | id + time, data = staggered, method = "scm",
            predictors = list(pred("y", times[1:3]))),
    regexp = "predictors"
  )
})

test_that("Phase 15b: SCM sharp + covariates forwarded (no error)", {
  # Sharp SCM doesn't use covariates the same way, but passing it shouldn't error
  # (covariates is silently ignored for sharp SCM unless wired up in fit_scm_cpp)
  fit <- scm_fit(y ~ d | id + time, data = panel, method = "scm")
  expect_s3_class(fit, "coresynth")
})

# ── Phase 15c: augment() include_donors ──────────────────────────────────────

test_that("Phase 15c: augment(include_donors=FALSE) is same as before", {
  fit <- scm_fit(y ~ d | id + time, data = panel, method = "scm")
  aug_old <- broom::augment(fit)
  aug_new <- broom::augment(fit, include_donors = FALSE)
  expect_equal(nrow(aug_new), nrow(aug_old))
  expect_equal(names(aug_new), names(aug_old))
})

test_that("Phase 15c: augment(include_donors=TRUE) adds control rows for SCM", {
  fit <- scm_fit(y ~ d | id + time, data = panel, method = "scm")
  aug <- broom::augment(fit, include_donors = TRUE)
  T_total <- length(fit$times)
  N_co    <- ncol(fit$Y_co_pre)
  expect_equal(nrow(aug), T_total * (1L + N_co))
})

test_that("Phase 15c: augment(include_donors=TRUE) has .unit and .type columns", {
  fit <- scm_fit(y ~ d | id + time, data = panel, method = "scm")
  aug <- broom::augment(fit, include_donors = TRUE)
  expect_true(".unit" %in% names(aug))
  expect_true(".type" %in% names(aug))
})

test_that("Phase 15c: augment control rows have NA .fitted and .resid", {
  fit <- scm_fit(y ~ d | id + time, data = panel, method = "scm")
  aug <- broom::augment(fit, include_donors = TRUE)
  ctrl <- aug[aug$.type == "control", ]
  expect_true(all(is.na(ctrl$.fitted)))
  expect_true(all(is.na(ctrl$.resid)))
})

test_that("Phase 15c: augment .type values are 'treated' and 'control'", {
  fit <- scm_fit(y ~ d | id + time, data = panel, method = "scm")
  aug <- broom::augment(fit, include_donors = TRUE)
  expect_true(all(aug$.type %in% c("treated", "control")))
  expect_true("treated" %in% aug$.type)
  expect_true("control" %in% aug$.type)
})

test_that("Phase 15c: augment(include_donors=TRUE) works for SDID", {
  fit <- scm_fit(y ~ d | id + time, data = panel, method = "sdid")
  aug <- broom::augment(fit, include_donors = TRUE)
  T_total <- length(fit$times)
  N_co    <- ncol(fit$Y_co_pre)
  expect_equal(nrow(aug), T_total * (1L + N_co))
  expect_true("control" %in% aug$.type)
})

test_that("Phase 15c: augment(include_donors=TRUE) .unit names match donor names", {
  fit <- scm_fit(y ~ d | id + time, data = panel, method = "scm")
  aug <- broom::augment(fit, include_donors = TRUE)
  ctrl_units <- unique(aug$.unit[aug$.type == "control"])
  expect_setequal(ctrl_units, colnames(fit$Y_co_pre))
})

# ── Phase 16a: GSC staggered adoption ─────────────────────────────────────────

make_staggered_panel_multi <- function(N = 12, TT = 24, seed = 7) {
  set.seed(seed)
  f   <- cumsum(rnorm(TT, 0, 0.5))
  lam <- rnorm(N, 1, 0.3)
  rows <- expand.grid(time = seq_len(TT), id = paste0("u", seq_len(N)),
                      stringsAsFactors = FALSE)
  rows$y <- as.vector(outer(f, lam)) + rnorm(nrow(rows), 0, 0.3)
  rows$d <- as.integer(
    (rows$id %in% c("u1", "u2", "u3") & rows$time >= 9) |
    (rows$id %in% c("u4", "u5", "u6") & rows$time >= 15)
  )
  rows$y[rows$d == 1] <- rows$y[rows$d == 1] + 1.5
  rows[order(rows$id, rows$time), ]
}
staggered_multi <- make_staggered_panel_multi()

test_that("Phase 16a: GSC staggered — no error", {
  expect_no_error(
    scm_fit(y ~ d | id + time, data = staggered, method = "gsc")
  )
})

test_that("Phase 16a: GSC staggered — staggered = TRUE", {
  fit <- scm_fit(y ~ d | id + time, data = staggered, method = "gsc")
  expect_true(fit$staggered)
})

test_that("Phase 16a: GSC staggered — estimate is finite", {
  fit <- scm_fit(y ~ d | id + time, data = staggered, method = "gsc")
  expect_true(is.finite(fit$estimate))
})

test_that("Phase 16a: GSC staggered — cohort_estimates has required columns", {
  fit <- scm_fit(y ~ d | id + time, data = staggered, method = "gsc")
  expected_cols <- c("cohort", "n_treated", "T_pre", "T_post", "estimate", "weight")
  expect_true(all(expected_cols %in% names(fit$cohort_estimates)))
})

test_that("Phase 16a: GSC staggered — cohort_estimates weights sum to 1", {
  fit <- scm_fit(y ~ d | id + time, data = staggered, method = "gsc")
  expect_equal(sum(fit$cohort_estimates$weight), 1, tolerance = 1e-8)
})

test_that("Phase 16a: GSC staggered — 2 cohorts present", {
  fit <- scm_fit(y ~ d | id + time, data = staggered, method = "gsc")
  expect_equal(nrow(fit$cohort_estimates), 2L)
})

test_that("Phase 16a: GSC staggered — estimate within ±4 of true ATT (1.5)", {
  fit <- scm_fit(y ~ d | id + time, data = staggered, method = "gsc")
  expect_lt(abs(fit$estimate - 1.5), 4.0)
})

test_that("Phase 16a: GSC staggered — Y_treat nrow matches panel T (24)", {
  fit <- scm_fit(y ~ d | id + time, data = staggered, method = "gsc")
  expect_equal(nrow(fit$Y_treat), 24L)
})

test_that("Phase 16a: GSC staggered — per-cohort F is T x r matrix", {
  fit <- scm_fit(y ~ d | id + time, data = staggered, method = "gsc", r = 2L)
  for (cf in fit$cohort_fits) {
    expect_equal(ncol(cf$F), 2L)
    expect_equal(nrow(cf$F), 24L)
  }
})

test_that("Phase 16a: GSC staggered — per-cohort L_co is N_co x r matrix", {
  fit <- scm_fit(y ~ d | id + time, data = staggered, method = "gsc", r = 2L)
  for (cf in fit$cohort_fits) {
    expect_equal(ncol(cf$L_co), 2L)
    expect_true(nrow(cf$L_co) >= 2L)
  }
})

test_that("Phase 16a: GSC staggered — control_group='never_treated' gives finite ATT", {
  fit <- scm_fit(y ~ d | id + time, data = staggered, method = "gsc",
                 control_group = "never_treated")
  expect_true(is.finite(fit$estimate))
})

test_that("Phase 16a: GSC staggered — multi-unit cohort gives finite ATT", {
  fit <- scm_fit(y ~ d | id + time, data = staggered_multi, method = "gsc")
  expect_true(is.finite(fit$estimate))
})

test_that("Phase 16a: GSC staggered — r=3 changes F dimensions", {
  fit <- scm_fit(y ~ d | id + time, data = staggered, method = "gsc", r = 3L)
  for (cf in fit$cohort_fits) {
    expect_equal(ncol(cf$F), 3L)
  }
})

test_that("Phase 16a: GSC staggered — gsc_boot() throws error", {
  fit <- scm_fit(y ~ d | id + time, data = staggered, method = "gsc")
  expect_error(gsc_boot(fit), regexp = "staggered")
})

test_that("Phase 16a: GSC staggered — tidy() returns cohort rows", {
  fit <- scm_fit(y ~ d | id + time, data = staggered, method = "gsc")
  td <- broom::tidy(fit)
  expect_s3_class(td, "data.frame")
  expect_true(nrow(td) == 2L)
  expect_true(grepl("^cohort_", td$term[1]))
})

# ── Phase 16b: SI staggered adoption ──────────────────────────────────────────

test_that("Phase 16b: SI staggered — no error", {
  expect_no_error(
    scm_fit(y ~ d | id + time, data = staggered, method = "si")
  )
})

test_that("Phase 16b: SI staggered — staggered = TRUE", {
  fit <- scm_fit(y ~ d | id + time, data = staggered, method = "si")
  expect_true(fit$staggered)
})

test_that("Phase 16b: SI staggered — estimate is finite", {
  fit <- scm_fit(y ~ d | id + time, data = staggered, method = "si")
  expect_true(is.finite(fit$estimate))
})

test_that("Phase 16b: SI staggered — cohort_estimates has required columns", {
  fit <- scm_fit(y ~ d | id + time, data = staggered, method = "si")
  expected_cols <- c("cohort", "n_treated", "T_pre", "T_post", "estimate", "weight")
  expect_true(all(expected_cols %in% names(fit$cohort_estimates)))
})

test_that("Phase 16b: SI staggered — cohort_estimates weights sum to 1", {
  fit <- scm_fit(y ~ d | id + time, data = staggered, method = "si")
  expect_equal(sum(fit$cohort_estimates$weight), 1, tolerance = 1e-8)
})

# ── Phase 21: SI multi-arm (K>1, Agarwal et al. 2025) ────────────────────────

# DGP: tensor factor model  Y^(d)_{ti} = sum_l u_tl * Lambda[d,l] * v_il + eps
# Lambda: arm 0 = c(1,1), arm 1 = c(1.5,0.8), arm 2 = c(0.7,1.3)
make_tensor_panel <- function(N_co = 10L, N1 = 3L, N2 = 3L,
                               T = 24L, T_pre = 12L, seed = 42L) {
  set.seed(seed)
  r <- 2L
  N <- N_co + N1 + N2
  U      <- matrix(cumsum(rnorm(T * r)), T, r)
  V      <- matrix(rnorm(N * r, 0, 0.5), N, r)
  Lambda <- rbind(c(1.0, 1.0), c(1.5, 0.8), c(0.7, 1.3))
  arm_unit <- c(rep(0L, N_co), rep(1L, N1), rep(2L, N2))
  ids  <- paste0("u", seq_len(N))
  rows <- expand.grid(time = seq_len(T), id = ids, stringsAsFactors = FALSE)
  rows <- rows[order(rows$id, rows$time), ]
  i_idx <- match(rows$id, ids)
  t_idx <- rows$time
  arm_i <- arm_unit[i_idx]
  rows$d <- ifelse(arm_i == 0L, 0L, ifelse(t_idx > T_pre, arm_i, 0L))
  rows$y <- mapply(function(t, i, arm) {
    sum(U[t, ] * Lambda[arm + 1L, ] * V[i, ]) + rnorm(1L, 0, 0.05)
  }, t_idx, i_idx, arm_i)
  rows
}

tensor_panel <- make_tensor_panel()

# § A: panel_to_tensor() ──────────────────────────────────────────────────────

test_that("Phase 21: panel_to_tensor — arm_levels is c(0,1,2)", {
  tp <- coresynth:::panel_to_tensor(tensor_panel$y, tensor_panel$d,
                                     tensor_panel$id, tensor_panel$time)
  expect_equal(tp$arm_levels, c(0L, 1L, 2L))
})

test_that("Phase 21: panel_to_tensor — idx_by_arm unit counts correct", {
  tp <- coresynth:::panel_to_tensor(tensor_panel$y, tensor_panel$d,
                                     tensor_panel$id, tensor_panel$time)
  expect_equal(length(tp$idx_by_arm[["0"]]), 10L)  # N_co
  expect_equal(length(tp$idx_by_arm[["1"]]), 3L)   # N1
  expect_equal(length(tp$idx_by_arm[["2"]]), 3L)   # N2
})

test_that("Phase 21: panel_to_tensor — error when arm 0 absent", {
  bad <- tensor_panel
  bad$d <- bad$d + 1L  # shift: now arms are 1, 2, 3 (no 0)
  expect_error(
    coresynth:::panel_to_tensor(bad$y, bad$d, bad$id, bad$time),
    "arm 0"
  )
})

test_that("Phase 21: panel_to_tensor — works on binary d (K=1)", {
  tp <- coresynth:::panel_to_tensor(panel$y, panel$d, panel$id, panel$time)
  expect_equal(tp$arm_levels, c(0L, 1L))
  expect_true(!is.null(tp$idx_by_arm[["0"]]))
  expect_true(!is.null(tp$idx_by_arm[["1"]]))
})

test_that("Phase 21: panel_to_tensor — return has required fields", {
  tp <- coresynth:::panel_to_tensor(tensor_panel$y, tensor_panel$d,
                                     tensor_panel$id, tensor_panel$time)
  for (fld in c("Y", "T_pre", "is_sharp", "arm_levels", "idx_by_arm"))
    expect_true(fld %in% names(tp), info = paste("missing field:", fld))
})

# § B: fit_si_cpp() multi-arm basic behaviour ─────────────────────────────────

test_that("Phase 21: SI multi-arm — class is coresynth", {
  fit <- scm_fit(y ~ d | id + time, data = tensor_panel, method = "si")
  expect_s3_class(fit, "coresynth")
  expect_s3_class(fit, "coresynth_si")
})

test_that("Phase 21: SI multi-arm — multi_arm flag is TRUE", {
  fit <- scm_fit(y ~ d | id + time, data = tensor_panel, method = "si")
  expect_true(isTRUE(fit$multi_arm))
})

test_that("Phase 21: SI multi-arm — arm_levels is c(1L, 2L)", {
  fit <- scm_fit(y ~ d | id + time, data = tensor_panel, method = "si")
  expect_equal(fit$arm_levels, c(1L, 2L))
})

test_that("Phase 21: SI multi-arm — arm_estimates length equals K", {
  fit <- scm_fit(y ~ d | id + time, data = tensor_panel, method = "si")
  expect_equal(length(fit$arm_estimates), 2L)
})

test_that("Phase 21: SI multi-arm — estimate is finite", {
  fit <- scm_fit(y ~ d | id + time, data = tensor_panel, method = "si")
  expect_true(is.finite(fit$estimate))
})

test_that("Phase 21: SI multi-arm — arm_fits has required sub-fields", {
  fit <- scm_fit(y ~ d | id + time, data = tensor_panel, method = "si")
  for (arm in c("1", "2")) {
    af <- fit$arm_fits[[arm]]
    for (fld in c("weights", "Y_cf", "Y_treat", "Y_synth", "gap"))
      expect_true(fld %in% names(af),
                  info = paste("arm", arm, "missing field:", fld))
  }
})

test_that("Phase 21: SI multi-arm — arm 1 Y_cf dimension is T_post x N1", {
  fit <- scm_fit(y ~ d | id + time, data = tensor_panel, method = "si")
  T_post <- 24L - 12L
  expect_equal(dim(fit$arm_fits[["1"]]$Y_cf), c(T_post, 3L))
})

test_that("Phase 21: SI multi-arm — arm 1 weights dimension is N_co x N1", {
  fit <- scm_fit(y ~ d | id + time, data = tensor_panel, method = "si")
  expect_equal(dim(fit$arm_fits[["1"]]$weights), c(10L, 3L))
})

# § C: estimation accuracy ────────────────────────────────────────────────────

test_that("Phase 21: SI multi-arm — arm 1 ATT is finite and bounded", {
  fit <- scm_fit(y ~ d | id + time, data = tensor_panel, method = "si")
  expect_true(is.finite(fit$arm_estimates[["1"]]))
  expect_lt(abs(fit$arm_estimates[["1"]]), 20)
})

test_that("Phase 21: SI multi-arm — arm 2 ATT is finite and bounded", {
  fit <- scm_fit(y ~ d | id + time, data = tensor_panel, method = "si")
  expect_true(is.finite(fit$arm_estimates[["2"]]))
  expect_lt(abs(fit$arm_estimates[["2"]]), 20)
})

test_that("Phase 21: SI multi-arm — manual k is respected", {
  fit <- scm_fit(y ~ d | id + time, data = tensor_panel, method = "si", k = 2L)
  expect_equal(fit$k, 2L)
  expect_true(is.finite(fit$estimate))
})

# § D: scm_fit() interface ───────────────────────────────────────────────────

test_that("Phase 21: SI multi-arm via scm_fit — no error with multi-valued d", {
  expect_no_error(scm_fit(y ~ d | id + time, data = tensor_panel, method = "si"))
})

test_that("Phase 21: SI multi-arm via scm_fit — estimate is finite", {
  fit <- scm_fit(y ~ d | id + time, data = tensor_panel, method = "si")
  expect_true(is.finite(fit$estimate))
})

test_that("Phase 21: SI multi-arm — print() outputs Multi-arm", {
  fit <- scm_fit(y ~ d | id + time, data = tensor_panel, method = "si")
  out <- capture.output(print(fit))
  expect_true(any(grepl("Multi-arm", out)))
})

# § E: si_inference() multi-arm ───────────────────────────────────────────────

test_that("Phase 21: si_inference multi-arm bootstrap — class is coresynth_inference", {
  fit <- scm_fit(y ~ d | id + time, data = tensor_panel, method = "si")
  inf <- si_inference(fit, method = "bootstrap", n_boot = 50L, seed = 1L)
  expect_s3_class(inf, "coresynth_inference")
})

test_that("Phase 21: si_inference multi-arm bootstrap — SE finite and positive", {
  fit <- scm_fit(y ~ d | id + time, data = tensor_panel, method = "si")
  inf <- si_inference(fit, method = "bootstrap", n_boot = 50L, seed = 1L)
  expect_true(is.finite(inf$se))
  expect_gt(inf$se, 0)
})

test_that("Phase 21: si_inference multi-arm bootstrap — CI contains estimate", {
  fit <- scm_fit(y ~ d | id + time, data = tensor_panel, method = "si")
  inf <- si_inference(fit, method = "bootstrap", n_boot = 50L, seed = 1L)
  expect_lte(inf$ci_lower, inf$estimate + 1e-9)
  expect_gte(inf$ci_upper, inf$estimate - 1e-9)
})

test_that("Phase 21: si_inference multi-arm jackknife — SE finite and positive", {
  fit <- scm_fit(y ~ d | id + time, data = tensor_panel, method = "si")
  inf <- si_inference(fit, method = "jackknife", seed = 1L)
  expect_true(is.finite(inf$se))
  expect_gt(inf$se, 0)
})

test_that("Phase 21: si_inference multi-arm jackknife_global — error with guidance", {
  fit <- scm_fit(y ~ d | id + time, data = tensor_panel, method = "si")
  expect_error(si_inference(fit, method = "jackknife_global"),
               "jackknife")
})

test_that("Phase 21: si_inference — rejects non-SI fit", {
  fit_mc <- scm_fit(y ~ d | id + time, data = panel, method = "mc")
  expect_error(si_inference(fit_mc), "method = 'si'")
})

# § F: edge cases ─────────────────────────────────────────────────────────────

test_that("Phase 21: SI multi-arm — aggregate ATT equals weighted arm mean", {
  fit <- scm_fit(y ~ d | id + time, data = tensor_panel, method = "si")
  ws   <- vapply(fit$arm_fits, `[[`, numeric(1L), "weight")
  taus <- fit$arm_estimates
  expect_equal(sum(ws * taus) / sum(ws), fit$estimate, tolerance = 1e-10)
})

test_that("Phase 21: SI multi-arm — N_tr_d=1 arm works without error", {
  tp1 <- make_tensor_panel(N_co = 10L, N1 = 1L, N2 = 3L, seed = 99L)
  expect_no_error(scm_fit(y ~ d | id + time, data = tp1, method = "si"))
})

test_that("Phase 21: SI multi-arm — k > min(T_pre,N_co) raises error", {
  expect_error(
    scm_fit(y ~ d | id + time, data = tensor_panel, method = "si", k = 100L),
    "k"
  )
})

test_that("Phase 16b: SI staggered — 2 cohorts present", {
  fit <- scm_fit(y ~ d | id + time, data = staggered, method = "si")
  expect_equal(nrow(fit$cohort_estimates), 2L)
})

test_that("Phase 16b: SI staggered — estimate within ±4 of true ATT (1.5)", {
  fit <- scm_fit(y ~ d | id + time, data = staggered, method = "si")
  expect_lt(abs(fit$estimate - 1.5), 4.0)
})

test_that("Phase 16b: SI staggered — Y_treat nrow matches panel T (24)", {
  fit <- scm_fit(y ~ d | id + time, data = staggered, method = "si")
  expect_equal(nrow(fit$Y_treat), 24L)
})

test_that("Phase 16b: SI staggered — control_group='never_treated' gives finite ATT", {
  fit <- scm_fit(y ~ d | id + time, data = staggered, method = "si",
                 control_group = "never_treated")
  expect_true(is.finite(fit$estimate))
})

test_that("Phase 16b: SI staggered — multi-unit cohort gives finite ATT", {
  fit <- scm_fit(y ~ d | id + time, data = staggered_multi, method = "si")
  expect_true(is.finite(fit$estimate))
})

test_that("Phase 16b: SI staggered — k argument respected", {
  fit <- scm_fit(y ~ d | id + time, data = staggered, method = "si", k = 2L)
  for (cf in fit$cohort_fits) {
    expect_lte(cf$k, 2L)
  }
})

test_that("Phase 16b: SI staggered — per-cohort k auto-adjusted for short pre-period", {
  fit <- scm_fit(y ~ d | id + time, data = staggered, method = "si")
  for (cf in fit$cohort_fits) {
    expect_gte(cf$k, 1L)
    expect_lte(cf$k, min(cf$T_pre, length(cf$idx_co)))
  }
})

test_that("Phase 16b: SI staggered — per-cohort weights matrix has correct dims", {
  fit <- scm_fit(y ~ d | id + time, data = staggered, method = "si")
  for (cf in fit$cohort_fits) {
    expect_equal(nrow(cf$weights), length(cf$idx_co))
    expect_equal(ncol(cf$weights), cf$n_treated)
  }
})

test_that("Phase 16b: SI staggered — tidy() returns cohort rows", {
  fit <- scm_fit(y ~ d | id + time, data = staggered, method = "si")
  td <- broom::tidy(fit)
  expect_s3_class(td, "data.frame")
  expect_true(nrow(td) == 2L)
  expect_true(grepl("^cohort_", td$term[1]))
})

# ── Phase 17: GSC/SI 推論 API（Bootstrap/Jackknife）────────────────────────────

# ── Phase 17a: gsc_inference — sharp ─────────────────────────────────────────

test_that("Phase 17a: gsc_inference sharp bootstrap — no error, class correct", {
  fit <- scm_fit(y ~ d | id + time, data = panel, method = "gsc")
  inf <- gsc_inference(fit, method = "bootstrap", n_boot = 99L, seed = 1L)
  expect_s3_class(inf, "coresynth_inference")
})

test_that("Phase 17a: gsc_inference sharp bootstrap — finite SE and p_value in [0,1]", {
  fit <- scm_fit(y ~ d | id + time, data = panel, method = "gsc")
  inf <- gsc_inference(fit, method = "bootstrap", n_boot = 99L, seed = 2L)
  expect_true(is.finite(inf$se))
  expect_gte(inf$p_value, 0)
  expect_lte(inf$p_value, 1)
})

test_that("Phase 17a: gsc_inference sharp bootstrap — CI contains estimate", {
  fit <- scm_fit(y ~ d | id + time, data = panel, method = "gsc")
  inf <- gsc_inference(fit, method = "bootstrap", n_boot = 99L, seed = 3L)
  expect_lte(inf$ci_lower, inf$estimate + 1e-8)
  expect_gte(inf$ci_upper, inf$estimate - 1e-8)
})

test_that("Phase 17a: gsc_inference sharp jackknife — finite SE and p_value in [0,1]", {
  fit <- scm_fit(y ~ d | id + time, data = panel, method = "gsc")
  inf <- gsc_inference(fit, method = "jackknife")
  expect_true(is.finite(inf$se))
  expect_gte(inf$p_value, 0)
  expect_lte(inf$p_value, 1)
})

test_that("Phase 17a: gsc_inference — rejects non-GSC fit", {
  fit <- scm_fit(y ~ d | id + time, data = panel, method = "sdid")
  expect_error(gsc_inference(fit), "method = 'gsc'")
})

# ── Phase 17b: gsc_inference — staggered ──────────────────────────────────────

test_that("Phase 17b: gsc_inference staggered bootstrap — no error, staggered=TRUE", {
  fit <- scm_fit(y ~ d | id + time, data = staggered, method = "gsc")
  inf <- gsc_inference(fit, method = "bootstrap", n_boot = 99L, seed = 10L)
  expect_s3_class(inf, "coresynth_inference")
  expect_true(inf$staggered)
})

test_that("Phase 17b: gsc_inference staggered bootstrap — finite SE", {
  fit <- scm_fit(y ~ d | id + time, data = staggered, method = "gsc")
  inf <- gsc_inference(fit, method = "bootstrap", n_boot = 99L, seed = 11L)
  expect_true(is.finite(inf$se))
})

test_that("Phase 17b: gsc_inference staggered bootstrap — CI contains estimate", {
  fit <- scm_fit(y ~ d | id + time, data = staggered, method = "gsc")
  inf <- gsc_inference(fit, method = "bootstrap", n_boot = 99L, seed = 12L)
  expect_lte(inf$ci_lower, inf$estimate + 1e-8)
  expect_gte(inf$ci_upper, inf$estimate - 1e-8)
})

test_that("Phase 17b: gsc_inference staggered jackknife — finite SE", {
  fit <- scm_fit(y ~ d | id + time, data = staggered, method = "gsc")
  inf <- gsc_inference(fit, method = "jackknife")
  expect_true(is.finite(inf$se))
  expect_true(inf$staggered)
})

# ── Phase 17c: si_inference — sharp ───────────────────────────────────────────

test_that("Phase 17c: si_inference sharp bootstrap — no error, class correct", {
  fit <- scm_fit(y ~ d | id + time, data = panel, method = "si")
  inf <- si_inference(fit, method = "bootstrap", n_boot = 99L, seed = 20L)
  expect_s3_class(inf, "coresynth_inference")
})

test_that("Phase 17c: si_inference sharp bootstrap — finite SE and p_value in [0,1]", {
  fit <- scm_fit(y ~ d | id + time, data = panel, method = "si")
  inf <- si_inference(fit, method = "bootstrap", n_boot = 99L, seed = 21L)
  expect_true(is.finite(inf$se))
  expect_gte(inf$p_value, 0)
  expect_lte(inf$p_value, 1)
})

test_that("Phase 17c: si_inference sharp bootstrap — CI contains estimate", {
  fit <- scm_fit(y ~ d | id + time, data = panel, method = "si")
  inf <- si_inference(fit, method = "bootstrap", n_boot = 99L, seed = 22L)
  expect_lte(inf$ci_lower, inf$estimate + 1e-8)
  expect_gte(inf$ci_upper, inf$estimate - 1e-8)
})

test_that("Phase 17c: si_inference sharp jackknife — finite SE and p_value in [0,1]", {
  fit <- scm_fit(y ~ d | id + time, data = panel, method = "si")
  inf <- si_inference(fit, method = "jackknife")
  expect_true(is.finite(inf$se))
  expect_gte(inf$p_value, 0)
  expect_lte(inf$p_value, 1)
})

test_that("Phase 17c: si_inference — rejects non-SI fit", {
  fit <- scm_fit(y ~ d | id + time, data = panel, method = "sdid")
  expect_error(si_inference(fit), "method = 'si'")
})

# ── Phase 17d: si_inference — staggered ───────────────────────────────────────

test_that("Phase 17d: si_inference staggered bootstrap — no error, staggered=TRUE", {
  fit <- scm_fit(y ~ d | id + time, data = staggered, method = "si")
  inf <- si_inference(fit, method = "bootstrap", n_boot = 99L, seed = 30L)
  expect_s3_class(inf, "coresynth_inference")
  expect_true(inf$staggered)
})

test_that("Phase 17d: si_inference staggered bootstrap — finite SE", {
  fit <- scm_fit(y ~ d | id + time, data = staggered, method = "si")
  inf <- si_inference(fit, method = "bootstrap", n_boot = 99L, seed = 31L)
  expect_true(is.finite(inf$se))
})

test_that("Phase 17d: si_inference staggered bootstrap — CI contains estimate", {
  fit <- scm_fit(y ~ d | id + time, data = staggered, method = "si")
  inf <- si_inference(fit, method = "bootstrap", n_boot = 99L, seed = 32L)
  expect_lte(inf$ci_lower, inf$estimate + 1e-8)
  expect_gte(inf$ci_upper, inf$estimate - 1e-8)
})

test_that("Phase 17d: si_inference staggered jackknife — finite SE", {
  fit <- scm_fit(y ~ d | id + time, data = staggered, method = "si")
  inf <- si_inference(fit, method = "jackknife")
  expect_true(is.finite(inf$se))
  expect_true(inf$staggered)
})

test_that("Phase 17d: gsc_boot staggered error mentions gsc_inference()", {
  fit <- scm_fit(y ~ d | id + time, data = staggered, method = "gsc")
  expect_error(gsc_boot(fit), "gsc_inference")
})

# ── Phase 18a: sdid_inference() staggered bootstrap / jackknife ───────────────

test_that("Phase 18a: sdid_inference staggered bootstrap returns sdid_inference", {
  fit <- scm_fit(y ~ d | id + time, data = staggered, method = "sdid")
  inf <- sdid_inference(fit, method = "bootstrap", n_boot = 50L, seed = 1L)
  expect_s3_class(inf, "sdid_inference")
  expect_true(isTRUE(inf$staggered))
  expect_true(is.finite(inf$se) && inf$se > 0)
  expect_true(inf$p_value >= 0 && inf$p_value <= 1)
})

test_that("Phase 18a: sdid_inference staggered bootstrap CI straddles estimate", {
  fit <- scm_fit(y ~ d | id + time, data = staggered, method = "sdid")
  inf <- sdid_inference(fit, method = "bootstrap", n_boot = 50L, seed = 1L)
  expect_true(inf$ci_lower <= inf$estimate && inf$estimate <= inf$ci_upper)
  expect_false(is.null(inf$boot_ests))
})

test_that("Phase 18a: sdid_inference staggered jackknife returns sdid_inference", {
  fit <- scm_fit(y ~ d | id + time, data = staggered, method = "sdid")
  inf <- sdid_inference(fit, method = "jackknife")
  expect_s3_class(inf, "sdid_inference")
  expect_true(isTRUE(inf$staggered))
  expect_true(is.finite(inf$se) && inf$se > 0)
  expect_true(inf$p_value >= 0 && inf$p_value <= 1)
})

test_that("Phase 18a: sdid_inference staggered jackknife CI straddles estimate", {
  fit <- scm_fit(y ~ d | id + time, data = staggered, method = "sdid")
  inf <- sdid_inference(fit, method = "jackknife")
  expect_true(inf$ci_lower <= inf$estimate && inf$estimate <= inf$ci_upper)
  expect_null(inf$boot_ests)
})

# ── Phase 18b: jackknife_global for SDID / GSC / SI staggered ────────────────

test_that("Phase 18b: sdid_inference jackknife_global staggered", {
  fit <- scm_fit(y ~ d | id + time, data = staggered, method = "sdid")
  inf <- sdid_inference(fit, method = "jackknife_global")
  expect_s3_class(inf, "sdid_inference")
  expect_true(isTRUE(inf$staggered))
  expect_true(is.finite(inf$se) && inf$se > 0)
  expect_true(inf$ci_lower <= inf$estimate && inf$estimate <= inf$ci_upper)
})

test_that("Phase 18b: sdid_inference jackknife_global errors on sharp fit", {
  fit <- scm_fit(y ~ d | id + time, data = panel, method = "sdid")
  expect_error(sdid_inference(fit, method = "jackknife_global"),
               regexp = "staggered fit")
})

test_that("Phase 18b: gsc_inference jackknife_global staggered", {
  fit <- scm_fit(y ~ d | id + time, data = staggered, method = "gsc")
  inf <- gsc_inference(fit, method = "jackknife_global")
  expect_s3_class(inf, "coresynth_inference")
  expect_true(isTRUE(inf$staggered))
  expect_true(is.finite(inf$se) && inf$se > 0)
  expect_true(inf$ci_lower <= inf$estimate && inf$estimate <= inf$ci_upper)
})

test_that("Phase 18b: si_inference jackknife_global staggered", {
  fit <- scm_fit(y ~ d | id + time, data = staggered, method = "si")
  inf <- si_inference(fit, method = "jackknife_global")
  expect_s3_class(inf, "coresynth_inference")
  expect_true(isTRUE(inf$staggered))
  expect_true(is.finite(inf$se) && inf$se > 0)
  expect_true(inf$ci_lower <= inf$estimate && inf$estimate <= inf$ci_upper)
})

# ── Phase 20: sdid_inference() staggered placebo ─────────────────────────────

test_that("Phase 20: sdid_inference staggered placebo returns sdid_inference", {
  fit <- scm_fit(y ~ d | id + time, data = staggered, method = "sdid")
  inf <- sdid_inference(fit, method = "placebo")
  expect_s3_class(inf, "sdid_inference")
  expect_true(isTRUE(inf$staggered))
  expect_true(inf$p_value >= 0 && inf$p_value <= 1)
  expect_null(inf$se)
  expect_null(inf$ci_lower)
  expect_null(inf$ci_upper)
  expect_null(inf$boot_ests)
})

test_that("Phase 20: staggered placebo_effects length = never-treated controls", {
  fit <- scm_fit(y ~ d | id + time, data = staggered, method = "sdid")
  inf <- sdid_inference(fit, method = "placebo")
  # never_co = intersect({u2..u12}, {u3..u12}) = {u3..u12} = 10 units
  never_co <- Reduce(intersect, lapply(fit$cohort_fits, `[[`, "idx_co"))
  expect_equal(length(inf$placebo_effects), length(never_co))
  expect_false(is.null(names(inf$placebo_effects)))
})

test_that("Phase 20: staggered placebo n_controls = never-treated count", {
  fit <- scm_fit(y ~ d | id + time, data = staggered, method = "sdid")
  inf <- sdid_inference(fit, method = "placebo")
  never_co <- Reduce(intersect, lapply(fit$cohort_fits, `[[`, "idx_co"))
  expect_equal(inf$n_controls, length(never_co))
})

test_that("Phase 20: staggered placebo alternative one-sided p-values sum >= 1", {
  fit <- scm_fit(y ~ d | id + time, data = staggered, method = "sdid")
  p_g <- sdid_inference(fit, method = "placebo", alternative = "greater")$p_value
  p_l <- sdid_inference(fit, method = "placebo", alternative = "less")$p_value
  expect_gte(p_g + p_l, 1 - .Machine$double.eps)
  expect_lte(p_g, 1)
  expect_lte(p_l, 1)
})

test_that("Phase 20: staggered placebo with large effect yields small p-value", {
  d_strong <- staggered
  d_strong$y[d_strong$d == 1] <- d_strong$y[d_strong$d == 1] + 100
  fit <- scm_fit(y ~ d | id + time, data = d_strong, method = "sdid")
  inf <- sdid_inference(fit, method = "placebo")
  never_co <- Reduce(intersect, lapply(fit$cohort_fits, `[[`, "idx_co"))
  # ATT ≈ 100 >> noise; observed effect should rank first in placebo distribution
  expect_lte(inf$p_value, 2 / (length(never_co) + 1))
})

test_that("Phase 20: staggered placebo control_group=never_treated uses all controls", {
  fit <- scm_fit(y ~ d | id + time, data = staggered, method = "sdid",
                 control_group = "never_treated")
  inf <- sdid_inference(fit, method = "placebo")
  # never_treated: intersection == full idx_co (same across all cohorts)
  expect_equal(inf$n_controls, length(fit$cohort_fits[[1]]$idx_co))
  expect_true(isTRUE(inf$staggered))
})

# ── Phase 23: SI multi-arm + staggered adoption ───────────────────────────────
# Two cohorts (g=5, g=7) × two arms (d=1, d=2) + N_co=4 controls, T=10.
# cohort g=5: 1 arm-1 unit + 1 arm-2 unit
# cohort g=7: 1 arm-1 unit + 1 arm-2 unit  →  4 (g,d) cells total
make_stag_tensor_panel <- function(N_co = 4L, TT = 10L, seed = 77L) {
  set.seed(seed)
  r <- 2L
  N <- N_co + 4L  # 4 treated (1 per cohort-arm cell)
  U      <- matrix(cumsum(rnorm(TT * r)), TT, r)
  V      <- matrix(rnorm(N * r, 0, 0.5), N, r)
  Lambda <- rbind(c(1.0, 1.0), c(1.5, 0.8), c(0.7, 1.3))
  # arm assignment: cols 1..N_co = arm 0, N_co+1 = arm1 g=5, N_co+2 = arm2 g=5,
  #                N_co+3 = arm1 g=7, N_co+4 = arm2 g=7
  arm_unit   <- c(rep(0L, N_co), 1L, 2L, 1L, 2L)
  cohort_of  <- c(rep(NA_integer_, N_co), 5L, 5L, 7L, 7L)
  ids  <- paste0("u", seq_len(N))
  rows <- expand.grid(time = seq_len(TT), id = ids, stringsAsFactors = FALSE)
  rows <- rows[order(rows$id, rows$time), ]
  i_idx <- match(rows$id, ids)
  rows$d <- mapply(function(i, t) {
    arm_i <- arm_unit[i]
    if (arm_i == 0L) return(0L)
    g <- cohort_of[i]
    if (!is.na(g) && t >= g) as.integer(arm_i) else 0L
  }, i_idx, rows$time)
  rows$y <- mapply(function(t, i) {
    sum(U[t, ] * Lambda[arm_unit[i] + 1L, ] * V[i, ]) + rnorm(1L, 0, 0.05)
  }, rows$time, i_idx)
  rows
}

stag_tensor_panel <- make_stag_tensor_panel()

# § A: fit structure ──────────────────────────────────────────────────────────

test_that("Phase 23: SI staggered-multi — no error", {
  expect_no_error(
    scm_fit(y ~ d | id + time, data = stag_tensor_panel, method = "si")
  )
})

test_that("Phase 23: SI staggered-multi — multi_arm=TRUE and staggered=TRUE", {
  fit <- scm_fit(y ~ d | id + time, data = stag_tensor_panel, method = "si")
  expect_true(isTRUE(fit$multi_arm))
  expect_true(isTRUE(fit$staggered))
})

test_that("Phase 23: SI staggered-multi — estimate is finite", {
  fit <- scm_fit(y ~ d | id + time, data = stag_tensor_panel, method = "si")
  expect_true(is.finite(fit$estimate))
})

test_that("Phase 23: SI staggered-multi — arm_levels is c(1L, 2L)", {
  fit <- scm_fit(y ~ d | id + time, data = stag_tensor_panel, method = "si")
  expect_equal(fit$arm_levels, c(1L, 2L))
})

test_that("Phase 23: SI staggered-multi — arm_estimates has 2 finite entries", {
  fit <- scm_fit(y ~ d | id + time, data = stag_tensor_panel, method = "si")
  expect_equal(length(fit$arm_estimates), 2L)
  expect_true(all(is.finite(fit$arm_estimates)))
})

test_that("Phase 23: SI staggered-multi — cohort_arm_estimates shape (4 rows)", {
  fit <- scm_fit(y ~ d | id + time, data = stag_tensor_panel, method = "si")
  ca  <- fit$cohort_arm_estimates
  expect_s3_class(ca, "data.frame")
  expect_equal(nrow(ca), 4L)
  for (col in c("cohort", "arm", "n_treated", "T_pre", "T_post", "estimate", "weight"))
    expect_true(col %in% names(ca))
})

test_that("Phase 23: SI staggered-multi — cohort_arm_estimates weights sum to 1", {
  fit <- scm_fit(y ~ d | id + time, data = stag_tensor_panel, method = "si")
  expect_equal(sum(fit$cohort_arm_estimates$weight), 1, tolerance = 1e-8)
})

test_that("Phase 23: SI staggered-multi — Y_all stored with correct dims", {
  fit <- scm_fit(y ~ d | id + time, data = stag_tensor_panel, method = "si")
  expect_false(is.null(fit$Y_all))
  expect_equal(nrow(fit$Y_all), 10L)                # TT
  expect_equal(ncol(fit$Y_all), 4L + 4L)            # N_co + N_treated
})

test_that("Phase 23: SI staggered-multi — control_group='never_treated' works", {
  fit <- scm_fit(y ~ d | id + time, data = stag_tensor_panel, method = "si",
                 control_group = "never_treated")
  expect_true(isTRUE(fit$multi_arm))
  expect_true(isTRUE(fit$staggered))
  expect_true(is.finite(fit$estimate))
})

# § B: aggregate formula ──────────────────────────────────────────────────────

test_that("Phase 23: SI staggered-multi — ATT = sum(w*tau)/sum(w) over (g,d) cells", {
  fit <- scm_fit(y ~ d | id + time, data = stag_tensor_panel, method = "si")
  ws   <- vapply(fit$cohort_fits, `[[`, numeric(1L), "weight")
  taus <- vapply(fit$cohort_fits, `[[`, numeric(1L), "estimate")
  expect_equal(sum(ws * taus) / sum(ws), fit$estimate, tolerance = 1e-10)
})

test_that("Phase 23: SI staggered-multi — cohort_fits flat list with required fields", {
  fit <- scm_fit(y ~ d | id + time, data = stag_tensor_panel, method = "si")
  expect_type(fit$cohort_fits, "list")
  expect_gte(length(fit$cohort_fits), 1L)
  cf1 <- fit$cohort_fits[[1L]]
  for (fld in c("cohort", "arm", "n_treated", "T_pre", "T_post", "estimate",
                "weight", "idx_tr", "idx_co", "k", "weights", "Y_cf",
                "Y_treat", "gap"))
    expect_true(fld %in% names(cf1), info = paste("missing field:", fld))
})

test_that("Phase 23: SI staggered-multi — tidy() returns cohort_arm_estimate rows", {
  fit <- scm_fit(y ~ d | id + time, data = stag_tensor_panel, method = "si")
  td  <- tidy(fit)
  expect_s3_class(td, "data.frame")
  expect_equal(nrow(td), 4L)
  expect_true("type" %in% names(td))
  expect_true(all(td$type == "cohort_arm_estimate"))
})

# § C: inference — all three methods ─────────────────────────────────────────

test_that("Phase 23: si_inference staggered-multi bootstrap — class and staggered flag", {
  fit <- scm_fit(y ~ d | id + time, data = stag_tensor_panel, method = "si")
  inf <- si_inference(fit, method = "bootstrap", n_boot = 49L, seed = 1L)
  expect_s3_class(inf, "coresynth_inference")
  expect_true(isTRUE(inf$staggered))
})

test_that("Phase 23: si_inference staggered-multi bootstrap — SE finite and CI valid", {
  fit <- scm_fit(y ~ d | id + time, data = stag_tensor_panel, method = "si")
  inf <- si_inference(fit, method = "bootstrap", n_boot = 49L, seed = 2L)
  expect_true(is.finite(inf$se))
  expect_lte(inf$ci_lower, inf$estimate)
  expect_gte(inf$ci_upper, inf$estimate)
  expect_gte(inf$p_value, 0); expect_lte(inf$p_value, 1)
})

test_that("Phase 23: si_inference staggered-multi jackknife — SE finite and CI valid", {
  fit <- scm_fit(y ~ d | id + time, data = stag_tensor_panel, method = "si")
  inf <- si_inference(fit, method = "jackknife")
  expect_true(is.finite(inf$se) && inf$se >= 0)
  expect_lte(inf$ci_lower, inf$estimate)
  expect_gte(inf$ci_upper, inf$estimate)
  expect_true(isTRUE(inf$staggered))
})

test_that("Phase 23: si_inference staggered-multi jackknife_global — SE finite and CI valid", {
  fit <- scm_fit(y ~ d | id + time, data = stag_tensor_panel, method = "si")
  inf <- si_inference(fit, method = "jackknife_global")
  expect_s3_class(inf, "coresynth_inference")
  expect_true(is.finite(inf$se) && inf$se >= 0)
  expect_lte(inf$ci_lower, inf$estimate)
  expect_gte(inf$ci_upper, inf$estimate)
  expect_true(isTRUE(inf$staggered))
})

# § D: regression — existing paths unaffected ─────────────────────────────────

test_that("Phase 23: sharp multi-arm still works (regression)", {
  fit <- scm_fit(y ~ d | id + time, data = tensor_panel, method = "si")
  expect_true(isTRUE(fit$multi_arm))
  expect_false(isTRUE(fit$staggered))
  expect_true(is.finite(fit$estimate))
})

test_that("Phase 23: single-arm staggered still works (regression)", {
  fit <- scm_fit(y ~ d | id + time, data = staggered, method = "si")
  expect_true(isTRUE(fit$staggered))
  expect_false(isTRUE(fit$multi_arm))
  expect_true(is.finite(fit$estimate))
})

test_that("Phase 23: sharp multi-arm jackknife_global still errors (regression)", {
  fit <- scm_fit(y ~ d | id + time, data = tensor_panel, method = "si")
  expect_error(si_inference(fit, method = "jackknife_global"), "jackknife")
})

# ── Phase 24: applied-pattern regression tests ────────────────────────────────
# 1. SCM multi-predictor patterns (Abadie, Diamond & Hainmueller 2010 style)
# 2. tidy/glance.coresynth_inference and enriched glance.coresynth
# 3. augment.coresynth() staggered + multi-arm
# 4. Improved error messages (staggered + mspe_ratio_pval, staggered + predictors)
# 5. dead-code cleanup regression (Phase 14b/15b SCM staggered unchanged)

# § A: SCM multi-predictor patterns (Prop99 style) ───────────────────────────

test_that("Phase 24: SCM with three pred() at specific years works", {
  fit <- scm_fit(
    y ~ d | id + time, data = panel_cov, method = "scm",
    predictors = list(pred("y", 3), pred("y", 5), pred("y", 7))
  )
  expect_s3_class(fit, "coresynth")
  expect_equal(length(fit$v_weights), 3L)
  expect_true(is.finite(fit$estimate))
})

test_that("Phase 24: SCM with mixed-aggregation pred() runs", {
  fit <- scm_fit(
    y ~ d | id + time, data = panel_cov, method = "scm",
    predictors = list(
      pred("y", 1:10),
      pred("cov1", 1:10, op = "median"),
      pred("cov2", 1:10, op = "mean")
    )
  )
  expect_s3_class(fit, "coresynth")
  expect_equal(nrow(fit$predictor_table), 3L)
})

test_that("Phase 24: SCM oos + multiple pred() returns finite ATT", {
  fit <- scm_fit(
    y ~ d | id + time, data = panel_cov, method = "scm",
    predictors = list(pred(c("cov1", "cov2"), 1:10),
                      pred("y", c(2, 5, 8))),
    v_selection = "oos"
  )
  expect_true(is.finite(fit$estimate))
  expect_equal(length(fit$v_weights), 3L)
})

# § B: tidy/glance.coresynth_inference ────────────────────────────────────────

test_that("Phase 24: tidy.coresynth_inference SDID sharp returns expected columns", {
  fit <- scm_fit(y ~ d | id + time, data = panel, method = "sdid")
  inf <- sdid_inference(fit, method = "jackknife")
  td  <- broom::tidy(inf)
  expect_s3_class(td, "data.frame")
  expect_equal(nrow(td), 1L)
  required <- c("term", "estimate", "std.error", "statistic", "p.value",
                "conf.low", "conf.high", "method", "alternative",
                "n_controls", "staggered")
  expect_true(all(required %in% names(td)))
  expect_equal(td$term, "ATT")
  expect_false(td$staggered)
})

test_that("Phase 24: tidy.coresynth_inference SDID staggered carries staggered=TRUE", {
  fit <- scm_fit(y ~ d | id + time, data = staggered, method = "sdid")
  inf <- sdid_inference(fit, method = "jackknife")
  td  <- broom::tidy(inf)
  expect_true(td$staggered)
})

test_that("Phase 24: tidy.coresynth_inference GSC works", {
  fit <- scm_fit(y ~ d | id + time, data = panel, method = "gsc", r = 2L)
  inf <- gsc_inference(fit, method = "jackknife")
  td  <- broom::tidy(inf)
  expect_equal(td$term, "ATT")
  expect_true(is.finite(td$estimate))
  expect_true(is.finite(td$std.error))
})

test_that("Phase 24: tidy.coresynth_inference SI works", {
  fit <- scm_fit(y ~ d | id + time, data = panel, method = "si")
  inf <- si_inference(fit, method = "jackknife")
  td  <- broom::tidy(inf)
  expect_equal(td$term, "ATT")
  expect_true(is.finite(td$std.error))
})

test_that("Phase 24: glance.coresynth_inference returns one-row summary", {
  fit <- scm_fit(y ~ d | id + time, data = panel, method = "sdid")
  inf <- sdid_inference(fit, method = "bootstrap", n_boot = 31L, seed = 1L)
  g   <- broom::glance(inf)
  expect_equal(nrow(g), 1L)
  expect_true(all(c("method", "n_controls", "staggered", "estimate",
                    "std.error", "p.value", "conf.low", "conf.high",
                    "alternative", "n_boot_valid") %in% names(g)))
  expect_gte(g$n_boot_valid, 1L)
})

test_that("Phase 24: tidy.coresynth_inference placebo SE/CI are NA", {
  fit <- scm_fit(y ~ d | id + time, data = panel, method = "sdid")
  inf <- sdid_inference(fit, method = "placebo")
  td  <- broom::tidy(inf)
  expect_true(is.na(td$std.error))
  expect_true(is.na(td$conf.low))
  expect_true(is.na(td$conf.high))
  expect_true(is.finite(td$p.value))
})

# § C: enriched glance.coresynth ─────────────────────────────────────────────

test_that("Phase 24: glance.coresynth returns new columns", {
  fit <- scm_fit(y ~ d | id + time, data = panel, method = "scm")
  g <- broom::glance(fit)
  expect_true(all(c("method", "estimate", "n_controls", "n_treated",
                    "T_pre", "T_post", "staggered", "multi_arm") %in% names(g)))
  expect_equal(g$T_pre, 10L)
  expect_equal(g$T_post, 10L)
  expect_equal(g$n_treated, 1L)
  expect_false(g$staggered)
  expect_false(g$multi_arm)
})

test_that("Phase 24: glance.coresynth staggered SDID sets staggered=TRUE", {
  fit <- scm_fit(y ~ d | id + time, data = staggered, method = "sdid")
  g <- broom::glance(fit)
  expect_true(g$staggered)
  expect_false(g$multi_arm)
})

test_that("Phase 24: glance.coresynth multi-arm SI sets multi_arm=TRUE", {
  fit <- scm_fit(y ~ d | id + time, data = tensor_panel, method = "si")
  g <- broom::glance(fit)
  expect_true(g$multi_arm)
  expect_false(g$staggered)
})

# § D: augment.coresynth staggered & multi-arm ────────────────────────────────

test_that("Phase 24: augment.coresynth() SDID staggered returns long df", {
  fit <- scm_fit(y ~ d | id + time, data = staggered, method = "sdid")
  aug <- broom::augment(fit)
  expect_s3_class(aug, "data.frame")
  expect_true(nrow(aug) > 0L)
  expect_true(all(c(".cohort", ".unit", ".type", ".time", ".observed",
                    ".fitted", ".period") %in% names(aug)))
  # SDID staggered reconstructs fitted via unit_weights + omega0
  expect_true(any(is.finite(aug$.fitted)))
})

test_that("Phase 24: augment.coresynth() SI staggered carries .cohort column", {
  fit <- scm_fit(y ~ d | id + time, data = staggered, method = "si")
  aug <- broom::augment(fit)
  expect_true(".cohort" %in% names(aug))
  expect_true(nrow(aug) > 0L)
  # SI cohort_fits carry Y_synth → .fitted should be finite
  expect_true(any(is.finite(aug$.fitted)))
})

test_that("Phase 24: augment.coresynth() multi-arm SI carries .arm column", {
  fit <- scm_fit(y ~ d | id + time, data = stag_tensor_panel, method = "si")
  aug <- broom::augment(fit)
  expect_true(".arm" %in% names(aug))
  expect_true(any(!is.na(aug$.arm)))
})

test_that("Phase 24: augment.coresynth() staggered include_donors adds control rows", {
  fit <- scm_fit(y ~ d | id + time, data = staggered, method = "si")
  aug <- broom::augment(fit, include_donors = TRUE)
  expect_true("control" %in% aug$.type)
  expect_true("treated" %in% aug$.type)
})

test_that("Phase 24: augment.coresynth() sharp unchanged (regression)", {
  fit <- scm_fit(y ~ d | id + time, data = panel, method = "scm")
  aug <- broom::augment(fit)
  expect_false(".cohort" %in% names(aug))
  expect_true(all(c(".time", ".observed", ".fitted", ".resid") %in% names(aug)))
})

# § E: improved error messages ────────────────────────────────────────────────

test_that("Phase 24: mspe_ratio_pval() rejects staggered fits with helpful message", {
  fit <- scm_fit(y ~ d | id + time, data = staggered, method = "scm")
  expect_error(mspe_ratio_pval(fit), regexp = "staggered")
})

test_that("Phase 24: SCM staggered + predictors error message names alternatives", {
  err <- tryCatch(
    scm_fit(y ~ d | id + time, data = staggered, method = "scm",
            predictors = list(pred("y", 1:3))),
    error = function(e) conditionMessage(e)
  )
  expect_true(grepl("covariates", err))
  expect_true(grepl("sdid|gsc|mc|tasc", err))
})

# § F: dead-code cleanup regression ───────────────────────────────────────────

test_that("Phase 24: SCM staggered unchanged after max_iter/tol cleanup", {
  fit <- scm_fit(y ~ d | id + time, data = staggered, method = "scm")
  expect_true(isTRUE(fit$staggered))
  expect_true(is.finite(fit$estimate))
  expect_equal(sum(fit$cohort_estimates$weight), 1, tolerance = 1e-8)
})

test_that("Phase 24: SCM staggered + covariates unchanged after cleanup", {
  fit <- scm_fit(y ~ d | id + time, data = stag_cov, method = "scm",
                 covariates = "cov1")
  expect_true(isTRUE(fit$staggered))
  expect_true(is.finite(fit$estimate))
  expect_length(fit$beta_hat, 1L)
})

test_that("Phase 24: sharp SCM tidy() unchanged (regression)", {
  fit <- scm_fit(y ~ d | id + time, data = panel, method = "scm")
  td <- broom::tidy(fit)
  expect_true("unit_weight" %in% td$type)
})

test_that("Phase 24: sdid_inference object inherits coresynth_inference", {
  fit <- scm_fit(y ~ d | id + time, data = panel, method = "sdid")
  inf <- sdid_inference(fit, method = "placebo")
  expect_s3_class(inf, "sdid_inference")
  expect_s3_class(inf, "coresynth_inference")
})

# ── Conformal inference (Chernozhukov, Wuthrich & Zhu 2021) ───────────────────
test_that("conformal_inference works across supported sharp methods", {
  for (m in c("scm", "sdid", "gsc", "mc", "si")) {
    fit <- scm_fit(y ~ d | id + time, data = panel, method = m)
    ci  <- conformal_inference(fit, n_grid = 60L)
    expect_s3_class(ci, "conformal_inference")
    expect_s3_class(ci, "coresynth_inference")
    expect_equal(ci$method, "conformal")
    expect_true(ci$p_value >= 0 && ci$p_value <= 1)
    expect_true(is.finite(ci$estimate))
    expect_true(ci$n_controls == 9L)
  }
})

test_that("conformal p-value for true effect exceeds p-value for tau0 = 0", {
  fit <- scm_fit(y ~ d | id + time, data = panel, method = "scm")
  p_null   <- conformal_inference(fit, tau0 = 0, ci = FALSE)$p_value
  p_truth  <- conformal_inference(fit, tau0 = fit$estimate, ci = FALSE)$p_value
  # The estimate itself should not be rejected; tau0 = 0 (no effect) should be
  # less compatible than the point estimate.
  expect_true(p_truth >= p_null)
})

test_that("conformal CI contains the point estimate", {
  fit <- scm_fit(y ~ d | id + time, data = panel, method = "sdid")
  ci  <- conformal_inference(fit, n_grid = 120L)
  if (!is.na(ci$ci_lower)) {
    expect_true(ci$ci_lower <= fit$estimate && fit$estimate <= ci$ci_upper)
  }
  expect_true(is.numeric(ci$grid) && length(ci$grid) == 120L)
})

test_that("conformal one-sided alternatives run and tidy()/glance() work", {
  fit <- scm_fit(y ~ d | id + time, data = panel, method = "si")
  ci_g <- conformal_inference(fit, alternative = "greater", ci = FALSE)
  ci_l <- conformal_inference(fit, alternative = "less", ci = FALSE)
  expect_true(ci_g$p_value >= 0 && ci_g$p_value <= 1)
  expect_true(ci_l$p_value >= 0 && ci_l$p_value <= 1)
  td <- broom::tidy(conformal_inference(fit, n_grid = 40L))
  expect_equal(td$term, "ATT")
  expect_equal(td$method, "conformal")
  gl <- broom::glance(conformal_inference(fit, n_grid = 40L))
  expect_equal(gl$method, "conformal")
})

test_that("conformal_inference rejects staggered and tasc fits", {
  stag <- make_panel()
  stag$d <- 0L
  stag$d[stag$id == "u1" & stag$time > 10] <- 1L
  stag$d[stag$id == "u2" & stag$time > 15] <- 1L
  stag$y[stag$d == 1] <- stag$y[stag$d == 1] + 2.0
  fit_stag <- scm_fit(y ~ d | id + time, data = stag, method = "sdid")
  expect_error(conformal_inference(fit_stag), "sharp")

  fit_tasc <- scm_fit(y ~ d | id + time, data = panel, method = "tasc")
  expect_error(conformal_inference(fit_tasc), "tasc")
})

#' @keywords internal
#' @useDynLib coresynth, .registration = TRUE
#' @importFrom Rcpp sourceCpp
#' @importFrom stats complete.cases median optim pnorm qnorm quantile reorder rnorm sd setNames var
#' @importFrom utils combn packageVersion
"_PACKAGE"

utils::globalVariables(c(
  "time", "value", "series", "weight",
  "group", "tau_hat", "ci_lower", "ci_upper",
  "sd"
))

#' Fit Original Synthetic Control Method (Abadie et al. 2010)
#'
#' Finds donor unit weights W (on the simplex) minimising the V-weighted
#' pre-treatment covariate distance:
#'   min_W (X1 - X0 W)' V (X1 - X0 W)
#' where V is jointly optimised via coordinate descent on the pre-treatment
#' outcome prediction error.
#'
#' @param y    Outcome vector (long format)
#' @param d    Treatment indicator (long format)
#' @param id   Unit identifier (long format)
#' @param time Time identifier (long format)
#' @param v_selection How to select the V metric matrix. `"insample"` (default)
#'   minimises in-sample pre-treatment MSPE following Abadie et al. (2010).
#'   `"oos"` uses the out-of-sample validation procedure recommended by
#'   Abadie (2021) S.3.2.
#' @param donor_mspe_threshold Numeric threshold for donor pool filtering
#'   (Abadie 2021 S.4). Each donor's individual pre-treatment MSPE is compared
#'   to the best donor's MSPE: donors with ratio > threshold are excluded before
#'   estimation. `Inf` (default) disables filtering.
#' @param lambda_pen Penalty for penalised SCM (Abadie & L'Hour 2021). `NULL`
#'   (default) runs standard SCM. `"auto"` selects lambda via out-of-sample
#'   pre-treatment MSPE. A non-negative number uses that value directly.
#' @return A list of class `coresynth` with standard fields plus:
#'   * `excluded_donors`: character vector of donors removed by filtering
#'   * `lambda_pen`: penalty value used (NA when standard SCM)
#' @noRd
fit_scm_cpp <- function(
  y,
  d,
  id,
  time,
  data = NULL,
  id_var = NULL,
  time_var = NULL,
  predictors = NULL,
  covariates = NULL,
  v_selection = c("insample", "oos"),
  donor_mspe_threshold = Inf,
  lambda_pen = NULL,
  v_optim = c("coord_descent", "auto", "bfgs"),
  control_group = c("clean", "never_treated"),
  ...
) {
  v_selection   <- match.arg(v_selection)
  v_optim       <- match.arg(v_optim)
  control_group <- match.arg(control_group)
  pan <- panel_to_matrices(y, d, id, time)

  # -- Staggered path (cohort-by-cohort SCM) -----------------------------------
  if (!pan$is_sharp) {
    if (!is.null(predictors) && length(predictors) > 0L)
      stop(
        "SCM staggered adoption does not support 'predictors'.\n",
        "Alternatives:\n",
        "  * Use 'covariates' (time-varying) instead -- these are partialled out before per-cohort SCM.\n",
        "  * Fit each cohort separately as sharp SCM (subset data by adoption time).\n",
        "  * Switch to method = 'sdid', 'gsc', 'mc', or 'tasc', which all support staggered + covariates.",
        call. = FALSE
      )

    # Covariate partial-out for staggered SCM (Clarke et al. 2023 S.2.2)
    use_cov  <- !is.null(covariates) && length(covariates) > 0L && !is.null(data)
    Y_work   <- pan$Y
    beta_hat <- numeric(0)
    if (use_cov) {
      X_arr    <- build_covariate_array(data, id_var, time_var,
                                       covariates, pan$units, pan$times)
      po       <- .sdid_partial_out_staggered(pan$Y, X_arr, pan)
      Y_work   <- po$Y_tilde
      beta_hat <- po$beta_hat
    }

    stag <- .fit_scm_staggered(
      pan,
      Y_matrix             = Y_work,
      v_selection          = v_selection,
      v_optim              = v_optim,
      donor_mspe_threshold = donor_mspe_threshold,
      lambda_pen           = lambda_pen,
      control_group        = control_group
    )
    res <- list(
      method           = "scm",
      staggered        = TRUE,
      estimate         = stag$estimate,
      cohort_estimates = stag$cohort_estimates,
      cohort_fits      = stag$cohort_fits,
      unit_weights     = NULL,
      v_weights        = NULL,
      Y_treat          = pan$Y[, pan$idx_treat, drop = FALSE],
      Y_synth          = NULL,
      gap              = NULL,
      times            = pan$times,
      T_pre            = pan$T_pre,
      excluded_donors  = character(0L),
      lambda_pen       = NA_real_,
      beta_hat         = beta_hat,
      covariates       = covariates
    )
    class(res) <- c("coresynth_scm", "coresynth")
    return(res)
  }

  Y <- pan$Y
  T_pre <- pan$T_pre
  idx_tr <- pan$idx_treat
  idx_co <- pan$idx_control

  if (length(idx_tr) != 1L) {
    stop("SCM currently supports exactly one treated unit.")
  }
  if (length(idx_co) < 2L) {
    stop("SCM requires at least two control units.")
  }

  Y_co_pre <- Y[seq_len(T_pre), idx_co, drop = FALSE]
  Y_co_all <- Y[, idx_co, drop = FALSE]
  Y_tr_pre <- Y[seq_len(T_pre), idx_tr]
  Y_tr_all <- Y[, idx_tr]

  # -- Phase 11a: Donor pool filtering (Abadie 2021 S.4) -----------------------
  excluded_donors <- character(0L)
  if (!is.null(donor_mspe_threshold) && is.finite(donor_mspe_threshold)) {
    ind_mspe <- colMeans((as.vector(Y_tr_pre) - Y_co_pre)^2)
    keep     <- ind_mspe <= donor_mspe_threshold * min(ind_mspe)
    if (sum(keep) < 2L) {
      warning(
        "donor_mspe_threshold = ", donor_mspe_threshold,
        " would retain < 2 donors; ignoring filter.",
        call. = FALSE
      )
    } else {
      excluded_donors <- colnames(Y_co_pre)[!keep]
      idx_co    <- idx_co[keep]
      Y_co_pre  <- Y_co_pre[, keep, drop = FALSE]
      Y_co_all  <- Y_co_all[, keep, drop = FALSE]
    }
  }

  # -- Validate lambda_pen ------------------------------------------------------
  if (!is.null(lambda_pen)) {
    if (!(identical(lambda_pen, "auto") ||
          (is.numeric(lambda_pen) && length(lambda_pen) == 1L &&
           is.finite(lambda_pen) && lambda_pen >= 0))) {
      stop(
        "'lambda_pen' must be NULL, \"auto\", or a non-negative finite number.",
        call. = FALSE
      )
    }
  }

  if (is.character(predictors)) {
    stop(
      paste0(
        "'predictors' must be a list of pred() objects, not a character vector.\n",
        "  Use: predictors = list(pred(c('var1', 'var2'), times))"
      ),
      call. = FALSE
    )
  }

  use_cov <- !is.null(predictors) && length(predictors) > 0L

  if (use_cov) {
    if (is.null(data) || is.null(id_var) || is.null(time_var)) {
      stop(
        "'data', 'id_var', and 'time_var' must be provided when using predictors.",
        call. = FALSE
      )
    }
    pm <- build_predictor_matrices(
      data     = data,
      id_var   = id_var,
      time_var = time_var,
      units    = pan$units,
      idx_co   = idx_co,
      idx_tr   = idx_tr,
      predictors = predictors
    )
    X0         <- pm$X0
    X1         <- pm$X1
    pred_names <- pm$pred_names
  } else {
    X0         <- Y_co_pre
    X1         <- drop(Y_tr_pre)
    pred_names <- paste0("V", seq_len(T_pre))
  }

  Z0 <- Y_co_pre
  Z1 <- drop(Y_tr_pre)

  t_train <- if (v_selection == "oos") T_pre %/% 2L else -1L

  # Determine effective outer optimiser (Phase 12a)
  k_dim <- nrow(X0)
  effective_outer <- switch(v_optim,
    "auto"          = if (k_dim <= 15L) "bfgs" else "coord_descent",
    "bfgs"          = "bfgs",
    "coord_descent" = "coord_descent"
  )

  # -- Phase 11b: Standard or penalised SCM -------------------------------------
  if (is.null(lambda_pen)) {
    # Standard SCM
    res <- if (effective_outer == "bfgs") {
      .scm_bfgs_outer(X0, X1, Z0, Z1, t_train = t_train)
    } else {
      scm_weights_cpp(X0, X1, Z0, Z1, t_train = t_train)
    }
    unit_w <- drop(res$W)
    V_final <- drop(res$V)
    loss    <- res$loss
    lambda_pen_used <- NA_real_
  } else {
    # Step 1: get V* via selected outer optimiser (respects v_selection)
    res_v <- if (effective_outer == "bfgs") {
      .scm_bfgs_outer(X0, X1, Z0, Z1, t_train = t_train)
    } else {
      scm_weights_cpp(X0, X1, Z0, Z1, t_train = t_train)
    }
    V_star <- drop(res_v$V)

    # Step 2: V*-weighted pairwise distances: d_j = (X1 - X0[:,j])' V* (X1 - X0[:,j])
    diff_mat <- X1 - X0  # k x N_co
    d_pen    <- colSums(V_star * diff_mat^2)  # N_co

    # Step 3: precompute Q = X0' V* X0 and c = X0' V* X1 for the inner QP
    V_sqrt <- sqrt(pmax(V_star, 0))
    X0_sc  <- sweep(X0, 1L, V_sqrt, `*`)    # k x N_co
    Q_mat  <- crossprod(X0_sc)              # N_co x N_co
    c_base <- as.vector(t(X0_sc) %*% (V_sqrt * X1))  # N_co

    # Step 4: select lambda
    if (identical(lambda_pen, "auto")) {
      lambda_pen_used <- .scm_tune_lambda_pen(
        Q_mat, c_base, d_pen, Z0, Z1, t_train, T_pre
      )
    } else {
      lambda_pen_used <- as.numeric(lambda_pen)
    }

    # Step 5: solve penalised QP: min W'Q W - 2(c - lambda/2 * d)' W
    c_pen  <- c_base - lambda_pen_used / 2 * d_pen
    unit_w <- drop(solve_simplex_qp(Q_mat, c_pen))
    V_final <- V_star
    resid  <- Z1 - as.vector(Z0 %*% unit_w)
    loss   <- sqrt(sum(resid^2))
  }

  names(unit_w) <- colnames(Y_co_all)
  Y_synth <- drop(Y_co_all %*% unit_w)
  Y_treat <- drop(Y_tr_all)

  if (use_cov) {
    synth_pred <- drop(X0 %*% unit_w)
    predictor_table <- data.frame(
      predictor = pred_names,
      treated   = X1,
      synthetic = synth_pred,
      row.names = NULL
    )
  } else {
    predictor_table <- NULL
  }

  list(
    method          = "scm",
    unit_weights    = unit_w,
    v_weights       = setNames(V_final, pred_names),
    Y_synth         = Y_synth,
    Y_treat         = Y_treat,
    gap             = Y_treat - Y_synth,
    times           = pan$times,
    T_pre           = T_pre,
    estimate        = mean((Y_treat - Y_synth)[-(seq_len(T_pre))]),
    loss            = loss,
    predictor_table = predictor_table,
    X0_mat          = if (use_cov) X0 else NULL,
    Y_co_pre        = Y_co_pre,
    Y_co_post       = Y[-(seq_len(T_pre)), idx_co, drop = FALSE],
    excluded_donors = excluded_donors,
    lambda_pen      = lambda_pen_used
  )
}

# Internal: L-BFGS-B outer V optimisation (Phase 12a).
# ~O(k^2) inner QP calls vs coord_descent's k*11*iter calls.
# Mirrors scm_weights_cpp semantics (OOS window, full-data W refit).
.scm_bfgs_outer <- function(X0, X1, Z0, Z1, t_train = -1L) {
  k     <- nrow(X0)
  T_pre <- nrow(Z0)

  # OOS evaluation window -- same split as scm_weights_cpp
  do_oos <- (t_train > 0L && t_train < T_pre)
  if (do_oos) {
    idx     <- (t_train + 1L):T_pre
    Z0_eval <- Z0[idx, , drop = FALSE]
    Z1_eval <- Z1[idx]
  } else {
    Z0_eval <- Z0
    Z1_eval <- Z1
  }

  # Objective: pre-treatment SSE as a function of raw (unnormalised) V weights.
  # V is normalised to sum-to-1 inside the function, so the optimisation is
  # scale-free (L-BFGS-B with lower=0 handles non-negativity).
  fn_v <- function(v_raw) {
    v_raw <- pmax(v_raw, 0)
    v_s   <- sum(v_raw)
    if (v_s < 1e-14) return(1e10)
    W <- tryCatch(
      drop(scm_inner_weights_cpp(X0, X1, v_raw / v_s)),
      error = function(e) NULL
    )
    if (is.null(W) || !all(is.finite(W))) return(1e10)
    r   <- Z1_eval - drop(Z0_eval %*% W)
    val <- sum(r^2)
    if (!is.finite(val)) 1e10 else val
  }

  v0  <- rep(1 / k, k)

  opt <- tryCatch(
    optim(v0, fn_v, method = "L-BFGS-B",
          lower   = rep(0, k),
          upper   = rep(Inf, k),
          control = list(maxit = 300L, factr = 1e7, trace = 0L)),
    error = function(e) NULL
  )
  # Fallback: Nelder-Mead (gradient-free, robust to discontinuities)
  if (is.null(opt) || !is.finite(opt$value)) {
    opt <- tryCatch(
      optim(v0, fn_v, method = "Nelder-Mead",
            control = list(maxit = 500L, reltol = 1e-5)),
      error = function(e) NULL
    )
  }

  V_opt <- if (!is.null(opt) && is.finite(opt$value)) {
    v_raw <- pmax(opt$par, 0)
    s     <- sum(v_raw)
    if (s < 1e-14) rep(1 / k, k) else v_raw / s
  } else {
    rep(1 / k, k)
  }

  W_final    <- drop(scm_inner_weights_cpp(X0, X1, V_opt))
  final_loss <- sqrt(sum((Z1 - drop(Z0 %*% W_final))^2))
  list(W = W_final, V = V_opt, loss = final_loss)
}

# Internal helper: tune lambda for penalised SCM via OOS pre-treatment MSPE.
.scm_tune_lambda_pen <- function(Q_mat, c_base, d_pen, Z0, Z1, t_train, T_pre) {
  if (t_train < 0L || t_train >= T_pre) t_train <- T_pre %/% 2L
  Z0_val <- Z0[(t_train + 1L):T_pre, , drop = FALSE]
  Z1_val <- Z1[(t_train + 1L):T_pre]

  lambda_grid <- c(0, 10^seq(-4, 2, length.out = 19L))
  best_lam    <- 0
  best_mspe   <- Inf

  for (lam in lambda_grid) {
    c_pen <- c_base - lam / 2 * d_pen
    W_try <- tryCatch(
      drop(solve_simplex_qp(Q_mat, c_pen)),
      error = function(e) NULL
    )
    if (is.null(W_try) || !all(is.finite(W_try))) next
    mspe <- mean((Z1_val - as.vector(Z0_val %*% W_try))^2)
    if (is.finite(mspe) && mspe < best_mspe) {
      best_mspe <- mspe
      best_lam  <- lam
    }
  }
  best_lam
}

# -- Internal: cohort-by-cohort SCM for staggered adoption -------------------
# Cohort g ATT is averaged with weight N_treat_g * T_post_g, following the
# same aggregation used in Arkhangelsky et al. (2021) Appendix S.8.
# Treated units within a cohort are averaged into a single pseudo-unit.
# Only predictors = NULL is supported (all pre-treatment outcomes used as X).
.fit_scm_staggered <- function(pan,
                               Y_matrix             = NULL,
                               v_selection          = "insample",
                               v_optim              = "coord_descent",
                               donor_mspe_threshold = Inf,
                               lambda_pen           = NULL,
                               control_group        = "clean") {
  Y       <- if (!is.null(Y_matrix)) Y_matrix else pan$Y
  TT      <- nrow(Y)
  T_adopt <- pan$T_adopt
  idx_tr  <- pan$idx_treat
  idx_co  <- pan$idx_control

  cohorts <- sort(unique(T_adopt[idx_tr]))

  cohort_list <- lapply(cohorts, function(g) {
    idx_tr_g <- idx_tr[T_adopt[idx_tr] == g]
    T_pre_g  <- g - 1L
    T_post_g <- TT - T_pre_g

    if (control_group == "never_treated") {
      idx_co_g <- idx_co
    } else {
      future_tr <- idx_tr[!is.na(T_adopt[idx_tr]) & T_adopt[idx_tr] > g]
      idx_co_g  <- c(idx_co, future_tr)
    }

    if (length(idx_co_g) < 2L || T_pre_g < 2L) {
      warning(sprintf(
        "SCM staggered: cohort g=%d skipped (T_pre=%d, N_co=%d).",
        g, T_pre_g, length(idx_co_g)
      ), call. = FALSE)
      return(NULL)
    }

    pre_rows  <- seq_len(T_pre_g)
    post_rows <- (T_pre_g + 1L):TT

    Y1_g     <- rowMeans(Y[, idx_tr_g, drop = FALSE])
    Y_co_all <- Y[, idx_co_g, drop = FALSE]
    Y_co_pre <- Y[pre_rows, idx_co_g, drop = FALSE]  # T_pre_g x N_co_g
    Y_tr_pre <- Y1_g[pre_rows]

    # Phase 11a-style donor filtering per cohort
    excl_g <- character(0L)
    if (is.finite(donor_mspe_threshold) && donor_mspe_threshold < Inf) {
      ind_mspe <- colMeans((Y_tr_pre - Y_co_pre)^2)
      keep     <- ind_mspe <= donor_mspe_threshold * min(ind_mspe)
      if (sum(keep) >= 2L) {
        excl_g   <- colnames(Y_co_pre)[!keep]
        idx_co_g <- idx_co_g[keep]
        Y_co_all <- Y_co_all[, keep, drop = FALSE]
        Y_co_pre <- Y_co_pre[, keep, drop = FALSE]
      }
    }

    t_train_g <- if (v_selection == "oos") T_pre_g %/% 2L else -1L
    k_dim_g   <- T_pre_g  # predictors = NULL => k = T_pre_g
    effective_outer_g <- switch(v_optim,
      "auto"          = if (k_dim_g <= 15L) "bfgs" else "coord_descent",
      "bfgs"          = "bfgs",
      "coord_descent" = "coord_descent"
    )

    fit_g <- tryCatch({
      if (is.null(lambda_pen)) {
        # Standard SCM
        res <- if (effective_outer_g == "bfgs") {
          .scm_bfgs_outer(Y_co_pre, Y_tr_pre, Y_co_pre, Y_tr_pre,
                          t_train = t_train_g)
        } else {
          scm_weights_cpp(Y_co_pre, Y_tr_pre, Y_co_pre, Y_tr_pre,
                          t_train = t_train_g)
        }
        unit_w          <- drop(res$W)
        lambda_pen_used <- NA_real_
      } else {
        # Penalised SCM (Phase 11b style, per cohort)
        res_v <- if (effective_outer_g == "bfgs") {
          .scm_bfgs_outer(Y_co_pre, Y_tr_pre, Y_co_pre, Y_tr_pre,
                          t_train = t_train_g)
        } else {
          scm_weights_cpp(Y_co_pre, Y_tr_pre, Y_co_pre, Y_tr_pre,
                          t_train = t_train_g)
        }
        V_star   <- drop(res_v$V)
        diff_mat <- Y_tr_pre - Y_co_pre       # T_pre_g x N_co_g
        d_pen    <- colSums(V_star * diff_mat^2)
        V_sqrt   <- sqrt(pmax(V_star, 0))
        X0_sc    <- sweep(Y_co_pre, 1L, V_sqrt, `*`)
        Q_mat    <- crossprod(X0_sc)
        c_base   <- as.vector(t(X0_sc) %*% (V_sqrt * Y_tr_pre))

        lambda_pen_used <- if (identical(lambda_pen, "auto")) {
          .scm_tune_lambda_pen(Q_mat, c_base, d_pen,
                               Y_co_pre, Y_tr_pre, t_train_g, T_pre_g)
        } else {
          as.numeric(lambda_pen)
        }
        c_pen  <- c_base - lambda_pen_used / 2 * d_pen
        unit_w <- drop(solve_simplex_qp(Q_mat, c_pen))
      }

      names(unit_w) <- colnames(Y_co_all)
      Y_synth_g <- drop(Y_co_all %*% unit_w)
      ATT_g     <- mean((Y1_g - Y_synth_g)[post_rows])

      list(
        cohort          = g,
        n_treated       = length(idx_tr_g),
        T_pre           = T_pre_g,
        T_post          = T_post_g,
        estimate        = ATT_g,
        weight          = as.numeric(length(idx_tr_g)) * T_post_g,
        unit_weights    = unit_w,
        Y_synth         = Y_synth_g,
        Y_treat         = Y1_g,
        excluded_donors = excl_g,
        lambda_pen      = lambda_pen_used
      )
    }, error = function(e) {
      warning(sprintf("SCM staggered: cohort g=%d failed: %s",
                      g, conditionMessage(e)), call. = FALSE)
      NULL
    })
    fit_g
  })

  valid <- !vapply(cohort_list, is.null, logical(1L))
  if (!any(valid)) stop("All cohort-level SCM fits failed.", call. = FALSE)
  r <- cohort_list[valid]

  w   <- vapply(r, `[[`, numeric(1L), "weight")
  tau <- vapply(r, `[[`, numeric(1L), "estimate")
  att <- sum(w * tau) / sum(w)

  cohort_df <- data.frame(
    cohort    = vapply(r, `[[`, integer(1L), "cohort"),
    n_treated = vapply(r, `[[`, integer(1L), "n_treated"),
    T_pre     = vapply(r, `[[`, integer(1L), "T_pre"),
    T_post    = vapply(r, `[[`, integer(1L), "T_post"),
    estimate  = tau,
    weight    = w / sum(w),
    stringsAsFactors = FALSE
  )

  list(estimate = att, cohort_estimates = cohort_df, cohort_fits = r)
}

#' Permutation Inference via MSPE Ratio for SCM
#'
#' Computes the Abadie et al. (2010) / Abadie (2021) permutation p-value.
#' For each control unit, a leave-one-out synthetic control is fitted.
#'
#' When `alternative = "two.sided"` (default), the test statistic is the
#' post/pre MSPE ratio, following Abadie et al. (2010). When
#' `alternative = "greater"` or `"less"`, the test statistic is the signed
#' average post-treatment gap (ATT), giving a one-sided permutation test as
#' recommended by Abadie (2021) S.3.5 for improved power when the direction
#' of the treatment effect is known.
#'
#' @param fit A `coresynth` object from [scm_fit()] with `method = "scm"`.
#' @param mspe_threshold Minimum pre-treatment MSPE for including a control
#'   unit in the two-sided test. Ignored for one-sided tests.
#'   Default: 0 (no filtering).
#' @param max_iter Passed to [scm_placebo_cpp()]. Default 100L.
#' @param tol      Passed to [scm_placebo_cpp()]. Default 1e-4.
#' @param use_covariates If `TRUE` and the fit used predictor covariates,
#'   applies the same covariate spec to each placebo unit (R-level loop).
#'   Default `FALSE` (faster C++ outcomes-only placebos).
#' @param alternative Direction of the alternative hypothesis:
#'   `"two.sided"` (default) uses the MSPE ratio statistic;
#'   `"greater"` tests whether the treatment increased the outcome;
#'   `"less"` tests whether the treatment decreased the outcome.
#'   One-sided tests use the signed ATT as the test statistic.
#' @return A list with:
#'   * `p_value`: Permutation p-value between 0 and 1
#'   * `mspe_ratio_treated`: MSPE_post / MSPE_pre for the treated unit (two.sided only)
#'   * `mspe_ratios_all`: Named numeric vector (treated first, then controls); two.sided only
#'   * `placebo_effects`: Named N_co-vector of placebo ATT estimates
#'   * `treated_effect`: ATT estimate for the treated unit
#'   * `n_placebo_used`: Number of control units used
#' @export
mspe_ratio_pval <- function(
  fit,
  mspe_threshold = 0,
  max_iter = 100L,
  tol = 1e-4,
  use_covariates = FALSE,
  alternative = c("two.sided", "greater", "less")
) {
  alternative <- match.arg(alternative)
  if (!inherits(fit, "coresynth") || fit$method != "scm") {
    stop("mspe_ratio_pval() requires a coresynth object with method = 'scm'.")
  }
  if (isTRUE(fit$staggered)) {
    stop(
      "mspe_ratio_pval() does not support staggered fits.\n",
      "Alternatives:\n",
      "  * Apply mspe_ratio_pval() to each cohort_fit individually after re-fitting it as sharp SCM.\n",
      "  * Use method = 'sdid' with sdid_inference(method = 'placebo') for staggered placebo inference.",
      call. = FALSE
    )
  }
  if (is.null(fit$Y_co_pre) || is.null(fit$Y_co_post)) {
    stop(
      "fit is missing Y_co_pre / Y_co_post. Re-run scm_fit() with the current version."
    )
  }

  T_pre <- fit$T_pre
  T_all <- length(fit$times)
  pre   <- seq_len(T_pre)
  post  <- (T_pre + 1L):T_all

  mspe_pre_tr  <- mean((fit$Y_treat[pre]  - fit$Y_synth[pre])^2)
  mspe_post_tr <- mean((fit$Y_treat[post] - fit$Y_synth[post])^2)
  ratio_tr <- if (mspe_pre_tr > mspe_threshold) {
    mspe_post_tr / mspe_pre_tr
  } else {
    NA_real_
  }

  if (use_covariates && is.null(fit$X0_mat)) {
    warning(
      "use_covariates=TRUE but fit has no covariates (X0_mat is NULL). ",
      "Falling back to outcomes-only placebo."
    )
    use_covariates <- FALSE
  }

  if (use_covariates) {
    N_co         <- ncol(fit$Y_co_pre)
    mspe_pre_vec <- numeric(N_co)
    mspe_post_vec <- numeric(N_co)
    effects_vec  <- numeric(N_co)

    for (i in seq_len(N_co)) {
      X0_loo <- fit$X0_mat[, -i, drop = FALSE]
      X1_loo <- fit$X0_mat[, i]
      Z0_loo <- fit$Y_co_pre[, -i, drop = FALSE]
      Z1_loo <- fit$Y_co_pre[, i]

      res_loo <- tryCatch(
        scm_weights_cpp(X0_loo, X1_loo, Z0_loo, Z1_loo,
                        max_iter = max_iter, tol = tol),
        error = function(e) NULL
      )
      if (is.null(res_loo)) {
        mspe_pre_vec[i]  <- NA_real_
        mspe_post_vec[i] <- NA_real_
        effects_vec[i]   <- NA_real_
        next
      }
      W_loo      <- res_loo$W
      synth_pre  <- fit$Y_co_pre[, -i, drop = FALSE]  %*% W_loo
      synth_post <- fit$Y_co_post[, -i, drop = FALSE] %*% W_loo

      mspe_pre_vec[i]  <- mean((Z1_loo - synth_pre)^2)
      mspe_post_vec[i] <- mean((fit$Y_co_post[, i] - synth_post)^2)
      effects_vec[i]   <- mean(fit$Y_co_post[, i] - synth_post)
    }

    keep      <- !is.na(mspe_pre_vec) & mspe_pre_vec > mspe_threshold
    ratios_co <- ifelse(keep, mspe_post_vec / mspe_pre_vec, NA_real_)
    co_effects <- effects_vec
  } else {
    plac      <- scm_placebo_cpp(fit$Y_co_pre, fit$Y_co_post,
                                  max_iter = max_iter, tol = tol)
    keep      <- plac$mspe_pre > mspe_threshold
    ratios_co <- ifelse(keep, plac$mspe_post / plac$mspe_pre, NA_real_)
    co_effects <- plac$effects
  }

  co_names <- colnames(fit$Y_co_pre)
  if (!is.null(co_names)) {
    names(ratios_co)  <- co_names
    names(co_effects) <- co_names
  }

  treated_effect <- fit$estimate

  if (alternative == "two.sided") {
    all_ratios <- c(treated = ratio_tr, ratios_co)
    valid      <- !is.na(all_ratios)
    p_value    <- mean(all_ratios[valid] >= all_ratios["treated"], na.rm = TRUE)
    n_used     <- sum(keep, na.rm = TRUE)
  } else {
    all_effects <- c(treated = treated_effect, co_effects)
    valid       <- !is.na(all_effects)
    if (alternative == "greater") {
      p_value <- mean(all_effects[valid] >= all_effects["treated"], na.rm = TRUE)
    } else {
      p_value <- mean(all_effects[valid] <= all_effects["treated"], na.rm = TRUE)
    }
    all_ratios <- c(treated = ratio_tr, ratios_co)
    n_used     <- sum(!is.na(co_effects))
  }

  list(
    p_value            = p_value,
    mspe_ratio_treated = ratio_tr,
    mspe_ratios_all    = all_ratios,
    placebo_effects    = co_effects,
    treated_effect     = treated_effect,
    n_placebo_used     = n_used
  )
}

# -- Phase 11c: Augmented SCM (Ben-Michael, Feller & Rothstein 2021) ---------

# Internal helper: select ridge lambda via LOO-CV on control units.
.cv_ridge_scm <- function(A, b, T_pre,
                           lambda_grid = 10^seq(-4, 4, length.out = 30L)) {
  N_co <- nrow(A)
  Ipre <- diag(T_pre)

  cv_errors <- vapply(lambda_grid, function(lam) {
    loo <- vapply(seq_len(N_co), function(i) {
      A_i    <- A[-i, , drop = FALSE]
      b_i    <- b[-i]
      beta_i <- tryCatch(
        solve(crossprod(A_i) + lam * Ipre, drop(t(A_i) %*% b_i)),
        error = function(e) rep(NA_real_, T_pre)
      )
      if (anyNA(beta_i)) return(NA_real_)
      (b[i] - sum(A[i, ] * beta_i))^2
    }, numeric(1L))
    mean(loo, na.rm = TRUE)
  }, numeric(1L))

  lambda_grid[which.min(cv_errors)]
}

#' Augmented Synthetic Control Method (Ridge ASCM)
#'
#' Applies a ridge-regression-based bias correction to a fitted SCM object,
#' following Ben-Michael, Feller & Rothstein (2021, JASA). The corrected
#' estimator is:
#'
#'   tau_aug = tau_SCM + (m_tr_post - sum_j W_j * m_j_post)
#'
#' where m_i_post = Y_pre_i' beta_hat is the ridge outcome model prediction
#' for unit i's mean post-treatment outcome, and beta_hat is estimated by
#' ridge regression across control units.
#'
#' @param fit A `coresynth` object from [scm_fit()] with `method = "scm"`.
#' @param lambda_ridge Ridge penalty (non-negative). `NULL` (default) selects
#'   the penalty by leave-one-out cross-validation on the control units.
#' @return A list with:
#'   * `att_aug`: Augmented ATT estimate
#'   * `delta`: Bias correction term (m_tr_post - sum_j W_j m_j_post)
#'   * `att_scm`: Original SCM ATT for comparison
#'   * `lambda_ridge`: Ridge penalty used
#'   * `beta_hat`: Ridge regression coefficients (length T_pre)
#' @export
augment_scm <- function(fit, lambda_ridge = NULL) {
  if (!inherits(fit, "coresynth") || fit$method != "scm") {
    stop("augment_scm() requires a coresynth object with method = 'scm'.", call. = FALSE)
  }
  if (is.null(fit$Y_co_pre) || is.null(fit$Y_co_post)) {
    stop(
      "fit is missing Y_co_pre / Y_co_post. Re-run scm_fit() with the current version.",
      call. = FALSE
    )
  }
  if (!is.null(lambda_ridge) &&
      (!is.numeric(lambda_ridge) || length(lambda_ridge) != 1L ||
       !is.finite(lambda_ridge) || lambda_ridge < 0)) {
    stop("'lambda_ridge' must be NULL or a non-negative finite number.", call. = FALSE)
  }

  Y_co_pre  <- fit$Y_co_pre
  Y_co_post <- fit$Y_co_post
  W         <- fit$unit_weights
  T_pre     <- fit$T_pre
  N_co      <- ncol(Y_co_pre)

  # Design: A[i, t] = pre-treatment outcome of control unit i at time t
  A <- t(Y_co_pre)          # N_co x T_pre
  b <- colMeans(Y_co_post)  # N_co: mean post-treatment outcome per donor

  if (is.null(lambda_ridge)) {
    lambda_ridge <- .cv_ridge_scm(A, b, T_pre)
  }

  Ipre     <- diag(T_pre)
  AtA      <- crossprod(A)            # T_pre x T_pre
  Atb      <- drop(t(A) %*% b)       # T_pre
  beta_hat <- tryCatch(
    solve(AtA + lambda_ridge * Ipre, Atb),
    error = function(e) {
      warning("Ridge solve failed; increasing regularization.", call. = FALSE)
      lam2 <- max(lambda_ridge * 100, 1e-6)
      tryCatch(
        solve(AtA + lam2 * Ipre, Atb),
        error = function(e2) rep(0, T_pre)
      )
    }
  )

  Y_tr_pre <- fit$Y_treat[seq_len(T_pre)]
  m_hat_co <- as.vector(A %*% beta_hat)  # N_co: predicted post-mean per donor
  m_hat_tr <- sum(Y_tr_pre * beta_hat)   # scalar: predicted post-mean for treated

  delta   <- m_hat_tr - sum(W * m_hat_co)
  att_aug <- fit$estimate + delta

  list(
    att_aug      = att_aug,
    delta        = delta,
    att_scm      = fit$estimate,
    lambda_ridge = lambda_ridge,
    beta_hat     = beta_hat
  )
}

# ── Internal: cohort-by-cohort GSC for staggered adoption ────────────────────
# Clarke et al. (2023); Arkhangelsky et al. (2021) Appendix §8.
# Factors estimated independently per cohort (gsc_ife_cpp per cohort).
# Y_co_g uses all T periods (pragmatic: future-treated contamination treated as
# noise, same rationale as SDID staggered). Covariates handled via per-cohort
# EM (subset of global T×N×p array).
.fit_gsc_staggered <- function(pan, Y, r, covariate_array = NULL,
                                control_group = "clean") {
  TT      <- nrow(Y)
  T_adopt <- pan$T_adopt
  idx_tr  <- pan$idx_treat
  idx_co  <- pan$idx_control
  p       <- if (!is.null(covariate_array)) dim(covariate_array)[3L] else 0L

  cohorts <- sort(unique(T_adopt[idx_tr]))

  cohort_list <- lapply(cohorts, function(g) {
    idx_tr_g <- idx_tr[T_adopt[idx_tr] == g]
    T_pre_g  <- g - 1L
    T_post_g <- TT - T_pre_g

    if (control_group == "never_treated") {
      idx_co_g <- idx_co
    } else {
      future_tr <- idx_tr[!is.na(T_adopt[idx_tr]) & T_adopt[idx_tr] > g]
      idx_co_g  <- c(idx_co, future_tr)
    }

    N_co_g <- length(idx_co_g)
    if (N_co_g < r || T_pre_g < r) {
      warning(sprintf(
        "GSC staggered: cohort g=%d skipped (T_pre=%d, N_co=%d, r=%d).",
        g, T_pre_g, N_co_g, r), call. = FALSE)
      return(NULL)
    }

    pre_rows  <- seq_len(T_pre_g)
    post_rows <- (T_pre_g + 1L):TT

    Y_co_g     <- Y[, idx_co_g, drop = FALSE]
    Y_tr_pre_g <- Y[pre_rows, idx_tr_g, drop = FALSE]
    Y_treat_g  <- Y[, idx_tr_g, drop = FALSE]

    if (p > 0L) {
      X_co_g     <- covariate_array[, idx_co_g, , drop = FALSE]
      X_tr_pre_g <- covariate_array[pre_rows, idx_tr_g, , drop = FALSE]
    } else {
      X_co_g     <- array(0.0, dim = c(TT, N_co_g, 0L))
      X_tr_pre_g <- array(0.0, dim = c(T_pre_g, length(idx_tr_g), 0L))
    }

    res <- tryCatch(
      gsc_ife_cpp(Y_co_g, Y_tr_pre_g, r, X_co_g, X_tr_pre_g),
      error = function(e) {
        warning(sprintf("GSC staggered: cohort g=%d failed: %s",
                        g, conditionMessage(e)), call. = FALSE)
        NULL
      }
    )
    if (is.null(res)) return(NULL)

    Y_tr_hat_g <- res$Y_tr_hat
    if (p > 0L) {
      X_tr_full_g <- covariate_array[, idx_tr_g, , drop = FALSE]
      for (j in seq_len(p)) {
        Y_tr_hat_g <- Y_tr_hat_g +
          matrix(X_tr_full_g[,, j], TT, length(idx_tr_g)) * res$beta[j]
      }
    }

    att_g <- mean(Y_treat_g[post_rows, , drop = FALSE] -
                    Y_tr_hat_g[post_rows, , drop = FALSE])

    list(
      cohort    = g,
      n_treated = length(idx_tr_g),
      T_pre     = T_pre_g,
      T_post    = T_post_g,
      estimate  = att_g,
      weight    = as.numeric(length(idx_tr_g)) * T_post_g,
      idx_tr    = idx_tr_g,
      idx_co    = idx_co_g,
      Y_tr_hat  = Y_tr_hat_g,
      Y_treat   = Y_treat_g,
      F         = res$F,
      L_co      = res$L_co,
      L_tr      = res$L_tr,
      beta      = res$beta
    )
  })

  valid <- !vapply(cohort_list, is.null, logical(1L))
  if (!any(valid)) stop("All cohort-level GSC fits failed.", call. = FALSE)
  r_list <- cohort_list[valid]

  w   <- vapply(r_list, `[[`, numeric(1L), "weight")
  tau <- vapply(r_list, `[[`, numeric(1L), "estimate")
  att <- sum(w * tau) / sum(w)

  cohort_df <- data.frame(
    cohort    = vapply(r_list, `[[`, integer(1L), "cohort"),
    n_treated = vapply(r_list, `[[`, integer(1L), "n_treated"),
    T_pre     = vapply(r_list, `[[`, integer(1L), "T_pre"),
    T_post    = vapply(r_list, `[[`, integer(1L), "T_post"),
    estimate  = tau,
    weight    = w / sum(w),
    stringsAsFactors = FALSE
  )

  list(estimate = att, cohort_estimates = cohort_df, cohort_fits = r_list)
}

#' Fit Generalized Synthetic Control (Xu, 2017)
#'
#' Uses the Interactive Fixed Effects (IFE) model:
#'   Y_it(0) = x_it'beta + lambda_i' f_t + eps_it
#' Factors f_t and control loadings Lambda_co are extracted from control units
#' via an EM loop that alternates truncated SVD (E-step) with panel OLS for
#' beta (M-step). When no covariates are supplied (covariates = NULL), the
#' plain 3-step SVD estimator (beta = 0) is used.
#'
#' @param y    Outcome vector (long format)
#' @param d    Treatment indicator (long format)
#' @param id   Unit identifier (long format)
#' @param time Time identifier (long format)
#' @param r    Number of latent factors (default 2)
#' @param data Long-format data frame. Required when `covariates` is non-NULL.
#' @param id_var   Name of the unit identifier column in `data`.
#' @param time_var Name of the time identifier column in `data`.
#' @param covariates Character vector of time-varying covariate column names in
#'   `data`. When non-NULL, `data`, `id_var`, and `time_var` must be supplied.
#' @return A list of class `coresynth`.
#' @noRd
fit_gsc_cpp <- function(
  y,
  d,
  id,
  time,
  r = 2,
  data = NULL,
  id_var = NULL,
  time_var = NULL,
  covariates = NULL,
  control_group = c("clean", "never_treated"),
  ...
) {
  control_group <- match.arg(control_group)
  pan <- panel_to_matrices(y, d, id, time)

  # ── Staggered adoption path ──────────────────────────────────────────────
  if (!pan$is_sharp) {
    use_cov_st <- !is.null(covariates) && !is.null(data)
    if (use_cov_st) {
      if (is.null(id_var) || is.null(time_var))
        stop("'id_var' and 'time_var' must be provided with covariates.",
             call. = FALSE)
      X_all_arr <- build_covariate_array(data, id_var, time_var, covariates,
                                          pan$units, pan$times)
    } else {
      X_all_arr <- NULL
    }
    res_st <- .fit_gsc_staggered(pan, pan$Y, r,
                                  covariate_array = X_all_arr,
                                  control_group   = control_group)
    res_st$method              <- "gsc"
    res_st$staggered           <- TRUE
    res_st$r                   <- r
    res_st$Y_treat             <- pan$Y[, pan$idx_treat, drop = FALSE]
    res_st$Y_synth             <- NULL
    res_st$gap                 <- NULL
    res_st$times               <- pan$times
    res_st$T_pre               <- pan$T_pre
    res_st$unit_weights        <- NULL
    res_st$covariates          <- covariates
    res_st$Y_all               <- pan$Y            # T × N — needed by gsc_inference()
    res_st$idx_co              <- pan$idx_control
    res_st$idx_tr              <- pan$idx_treat
    res_st$covariate_array_all <- if (use_cov_st) X_all_arr else NULL
    return(res_st)
  }

  Y <- pan$Y
  T_pre <- pan$T_pre
  idx_tr <- pan$idx_treat
  idx_co <- pan$idx_control

  if (length(idx_co) < r) {
    stop("Need at least r control units for GSC.")
  }
  if (T_pre < r) {
    stop("Need at least r pre-treatment periods for GSC.")
  }

  Y_co_all <- Y[, idx_co, drop = FALSE] # T x N_co
  Y_tr_pre <- Y[seq_len(T_pre), idx_tr, drop = FALSE] # T_pre x N_tr

  # ── Build covariate arrays ────────────────────────────────────────────────
  use_cov <- !is.null(covariates) && !is.null(data)
  if (use_cov) {
    if (is.null(id_var) || is.null(time_var)) {
      stop(
        "'id_var' and 'time_var' must be provided with covariates.",
        call. = FALSE
      )
    }
    co_units <- pan$units[idx_co]
    tr_units <- pan$units[idx_tr]
    X_co_arr <- build_covariate_array(
      data,
      id_var,
      time_var,
      covariates,
      co_units,
      pan$times
    ) # T x N_co x p
    X_tr_arr <- build_covariate_array(
      data,
      id_var,
      time_var,
      covariates,
      tr_units,
      pan$times
    ) # T x N_tr x p
    # Pre-treatment slice of X_tr for Xu (2017) Step 2 loading estimation
    X_tr_pre_arr <- X_tr_arr[seq_len(T_pre), , , drop = FALSE] # T_pre x N_tr x p
    p <- length(covariates)
  } else {
    T_full <- nrow(Y_co_all)
    N_co_n <- ncol(Y_co_all)
    X_co_arr <- array(0.0, dim = c(T_full, N_co_n, 0L)) # empty cube (p=0)
    X_tr_arr <- NULL
    X_tr_pre_arr <- NULL
    p <- 0L
  }

  # Build empty X_tr_pre cube for the no-covariate case
  if (is.null(X_tr_pre_arr)) {
    N_tr_n <- length(idx_tr)
    X_tr_pre_arr <- array(0.0, dim = c(T_pre, N_tr_n, 0L))
  }

  res <- gsc_ife_cpp(Y_co_all, Y_tr_pre, r, X_co_arr, X_tr_pre_arr)
  beta <- res$beta # numeric(0) when p=0

  # ── Add beta contribution to Y_tr_hat and compute Y_hat_co ───────────────
  Y_tr_hat <- res$Y_tr_hat # T x N_tr, factor part
  Y_hat_co <- res$F %*% t(res$L_co) # T x N_co, factor part

  if (use_cov && p > 0L) {
    T_all <- nrow(Y_tr_hat)
    N_tr_n <- ncol(Y_tr_hat)
    for (j in seq_len(p)) {
      cov_tr <- matrix(X_tr_arr[,, j], T_all, N_tr_n)
      Y_tr_hat <- Y_tr_hat + cov_tr * beta[j]
      Y_hat_co <- Y_hat_co + X_co_arr[,, j] * beta[j]
    }
  }

  Y_treat <- Y[, idx_tr, drop = FALSE] # T x N_tr

  # ATT: average post-treatment residual across treated units and post periods
  post_rows <- (T_pre + 1L):nrow(Y)
  gap_post <- Y_treat[post_rows, , drop = FALSE] -
    Y_tr_hat[post_rows, , drop = FALSE]
  att <- mean(gap_post)

  list(
    method = "gsc",
    r = r,
    F = res$F,
    L_co = res$L_co,
    L_tr = res$L_tr,
    singular_values = res$singular_values,
    beta = beta, # numeric(0) when no covariates
    Y_tr_hat = Y_tr_hat, # T x N_tr (beta included if covariates)
    Y_treat = Y_treat,
    gap = Y_treat - Y_tr_hat,
    times = pan$times,
    T_pre = T_pre,
    unit_weights = NULL, # GSC uses loadings, not simplex weights
    estimate = att,
    Y_co_all = Y_co_all, # T x N_co — needed by gsc_boot()
    Y_hat_co = Y_hat_co, # T x N_co, full prediction (beta included)
    covariate_array_co = if (use_cov) X_co_arr else NULL,
    covariate_array_tr = if (use_cov) X_tr_arr else NULL,
    covariates = covariates # variable names or NULL
  )
}

#' Parametric Bootstrap Inference for GSC (Xu 2017 §3)
#'
#' Generates the null distribution of the ATT under H0 (no treatment effect)
#' by parametric resampling from the estimated IFE factor model. Under H0,
#' both the control panel and treated unit are generated from the fitted
#' factor model with homoskedastic noise. When the fit includes covariate
#' adjustment (beta), the covariate contribution is included in the simulated
#' DGP and re-estimated in each bootstrap replicate.
#'
#' @param fit   A `coresynth` object from [scm_fit()] with `method = "gsc"`.
#' @param B     Bootstrap replications (default 499L).
#' @param alpha Significance level for the confidence interval (default 0.05).
#' @param seed  RNG seed for reproducibility (default NULL).
#' @return A list with:
#'   * `p_value`:   Two-sided p-value: mean(|ATT*| >= |ATT_obs|)
#'   * `ci_lower`:  Lower bound of (1-alpha)*100% bootstrap CI
#'   * `ci_upper`:  Upper bound of (1-alpha)*100% bootstrap CI
#'   * `se`:        Bootstrap standard error
#'   * `boot_dist`: Numeric vector of length B (bootstrap ATT* values)
#'   * `att_obs`:   Observed ATT from the original fit
#' @export
gsc_boot <- function(fit, B = 499L, alpha = 0.05, seed = NULL) {
  if (!inherits(fit, "coresynth") || fit$method != "gsc") {
    stop("gsc_boot() requires a coresynth object with method = 'gsc'.")
  }
  if (isTRUE(fit$staggered)) {
    stop("gsc_boot() does not support staggered adoption fits. ",
         "Use gsc_inference() instead.", call. = FALSE)
  }
  if (is.null(fit$Y_co_all)) {
    stop("fit is missing Y_co_all. Re-run scm_fit() with the current version.")
  }
  if (!is.null(seed)) {
    set.seed(seed)
  }

  F_hat <- fit$F
  L_co_hat <- fit$L_co
  L_tr_hat <- fit$L_tr
  r <- fit$r
  T_pre <- fit$T_pre
  att_obs <- fit$estimate
  Y_co_all <- fit$Y_co_all
  beta <- fit$beta # numeric(0) if no covariates
  X_tr_arr <- fit$covariate_array_tr # T x N_tr x p  or NULL
  X_co_arr <- fit$covariate_array_co # T x N_co x p  or NULL

  T <- nrow(F_hat)
  N_co <- ncol(Y_co_all)
  N_tr <- nrow(L_tr_hat)
  T_post <- T - T_pre
  pre <- seq_len(T_pre)
  post <- (T_pre + 1L):T

  # Full prediction for controls (includes beta contribution if covariates used)
  Y_hat_co <- if (!is.null(fit$Y_hat_co)) {
    fit$Y_hat_co
  } else {
    F_hat %*% t(L_co_hat)
  }

  sigma2_hat <- mean((Y_co_all - Y_hat_co)^2)
  sd_hat <- sqrt(sigma2_hat)

  # Full prediction for treated pre/post (includes beta contribution)
  Y_hat_tr_pre <- F_hat[pre, , drop = FALSE] %*% t(L_tr_hat) # T_pre x N_tr
  Y_hat_tr_post <- F_hat[post, , drop = FALSE] %*% t(L_tr_hat) # T_post x N_tr
  if (length(beta) > 0L && !is.null(X_tr_arr)) {
    T_pre_n <- length(pre)
    T_post_n <- length(post)
    for (j in seq_along(beta)) {
      Y_hat_tr_pre <- Y_hat_tr_pre +
        matrix(X_tr_arr[pre, , j], T_pre_n, N_tr) * beta[j]
      Y_hat_tr_post <- Y_hat_tr_post +
        matrix(X_tr_arr[post, , j], T_post_n, N_tr) * beta[j]
    }
  }

  # Covariate cubes to pass to gsc_ife_cpp in each replicate
  X_co_arr_boot <- if (!is.null(X_co_arr)) {
    X_co_arr
  } else {
    array(0.0, dim = c(T, N_co, 0L))
  }
  # Pre-treatment slice of treated covariates (for Xu 2017 Step 2 demeaning)
  X_tr_pre_boot <- if (!is.null(X_tr_arr)) {
    # T_pre x N_tr x p
    X_tr_arr[seq_len(T_pre), , , drop = FALSE]
  } else {
    array(0.0, dim = c(T_pre, N_tr, 0L))
  }

  boot_dist <- numeric(B)

  for (b in seq_len(B)) {
    eps_co_star <- matrix(rnorm(T * N_co, 0, sd_hat), T, N_co)
    eps_tr_pre_star <- matrix(rnorm(T_pre * N_tr, 0, sd_hat), T_pre, N_tr)
    eps_tr_post_star <- matrix(rnorm(T_post * N_tr, 0, sd_hat), T_post, N_tr)

    Y_star_co <- Y_hat_co + eps_co_star
    Y_star_tr_pre <- Y_hat_tr_pre + eps_tr_pre_star

    res_star <- tryCatch(
      gsc_ife_cpp(Y_star_co, Y_star_tr_pre, r, X_co_arr_boot, X_tr_pre_boot),
      error = function(e) NULL
    )
    if (is.null(res_star)) {
      boot_dist[b] <- NA_real_
      next
    }

    Y_star_tr_post <- Y_hat_tr_post + eps_tr_post_star
    Y_hat_star_post <- res_star$Y_tr_hat[post, , drop = FALSE] # factor part

    # Add beta_star covariate contribution to Y_hat_star_post
    beta_star <- res_star$beta
    if (length(beta_star) > 0L && !is.null(X_tr_arr)) {
      T_post_n <- length(post)
      for (j in seq_along(beta_star)) {
        Y_hat_star_post <- Y_hat_star_post +
          matrix(X_tr_arr[post, , j], T_post_n, N_tr) * beta_star[j]
      }
    }

    boot_dist[b] <- mean(Y_star_tr_post - Y_hat_star_post)
  }

  n_fail <- sum(is.na(boot_dist))
  if (n_fail > 0.1 * B) {
    warning(
      sprintf(
        "%.0f%% of bootstrap replications failed (NA). ",
        100 * n_fail / B
      ),
      "Consider reducing r or checking model fit."
    )
  }

  valid <- boot_dist[!is.na(boot_dist)]
  ci <- quantile(valid, c(alpha / 2, 1 - alpha / 2), names = FALSE)

  list(
    p_value = mean(abs(valid) >= abs(att_obs)),
    ci_lower = ci[1],
    ci_upper = ci[2],
    se = sd(valid),
    boot_dist = boot_dist,
    att_obs = att_obs
  )
}

# ── Internal helper: re-estimate one GSC cohort with given control indices ─────
.refit_gsc_cohort <- function(Y_all, cf, r, cov_array_all) {
  TT        <- nrow(Y_all)
  T_pre_g   <- cf$T_pre
  pre_rows  <- seq_len(T_pre_g)
  post_rows <- (T_pre_g + 1L):TT
  N_co_g    <- length(cf$idx_co)

  Y_co_g   <- Y_all[, cf$idx_co, drop = FALSE]
  Y_tr_pre <- Y_all[pre_rows, cf$idx_tr, drop = FALSE]

  p <- if (!is.null(cov_array_all)) dim(cov_array_all)[3L] else 0L
  if (p > 0L) {
    X_co_g     <- cov_array_all[, cf$idx_co, , drop = FALSE]
    X_tr_pre_g <- cov_array_all[pre_rows, cf$idx_tr, , drop = FALSE]
    X_tr_full  <- cov_array_all[, cf$idx_tr, , drop = FALSE]
  } else {
    X_co_g     <- array(0.0, dim = c(TT, N_co_g, 0L))
    X_tr_pre_g <- array(0.0, dim = c(T_pre_g, length(cf$idx_tr), 0L))
    X_tr_full  <- NULL
  }

  res <- tryCatch(
    gsc_ife_cpp(Y_co_g, Y_tr_pre, r, X_co_g, X_tr_pre_g),
    error = function(e) NULL
  )
  if (is.null(res)) return(NA_real_)

  Y_tr_hat <- res$Y_tr_hat  # T × N_tr_g (factor part)
  if (p > 0L) {
    for (j in seq_len(p))
      Y_tr_hat <- Y_tr_hat +
        matrix(X_tr_full[,, j], TT, length(cf$idx_tr)) * res$beta[j]
  }

  Y_treat_post <- Y_all[post_rows, cf$idx_tr, drop = FALSE]
  mean(Y_treat_post - Y_tr_hat[post_rows, , drop = FALSE])
}

#' Non-parametric Inference for GSC (Xu 2017)
#'
#' Estimates SE and confidence intervals for the ATT via non-parametric cluster
#' bootstrap or jackknife over control units. Works for both sharp and staggered
#' GSC fits. For staggered fits, bootstrap resamples each cohort's control pool
#' independently, and jackknife uses a per-cohort LOO with delta-method variance
#' aggregation.
#'
#' Note: `gsc_boot()` performs a *parametric* bootstrap under H0 (hypothesis
#' testing). `gsc_inference()` provides *non-parametric* SE and CIs suitable
#' for inference about the ATT magnitude.
#'
#' @param fit   A `coresynth` object from [scm_fit()] with `method = "gsc"`.
#' @param method `"bootstrap"` (default) or `"jackknife"`.
#' @param n_boot Number of bootstrap replications (default 499L; ignored for jackknife).
#' @param level  Confidence level (default 0.95).
#' @param alternative `"two.sided"` (default), `"greater"`, or `"less"`.
#' @param seed  RNG seed for reproducibility (default NULL).
#' @return A list of class `coresynth_inference`.
#' @export
gsc_inference <- function(
  fit,
  method      = c("bootstrap", "jackknife", "jackknife_global"),
  n_boot      = 499L,
  level       = 0.95,
  alternative = c("two.sided", "greater", "less"),
  seed        = NULL
) {
  method      <- match.arg(method)
  alternative <- match.arg(alternative)

  if (!inherits(fit, "coresynth") || !identical(fit$method, "gsc"))
    stop("gsc_inference() requires a coresynth fit with method = 'gsc'.",
         call. = FALSE)

  tau_hat   <- fit$estimate
  alpha     <- 1 - level
  staggered <- isTRUE(fit$staggered)

  if (!is.null(seed)) set.seed(seed)

  # ── Sharp path ──────────────────────────────────────────────────────────────
  if (!staggered) {
    if (method == "jackknife_global")
      stop("gsc_inference() method='jackknife_global' requires a staggered fit.",
           call. = FALSE)
    if (is.null(fit$Y_co_all))
      stop("fit does not contain Y_co_all. Re-estimate with the current version.",
           call. = FALSE)

    Y_co  <- fit$Y_co_all
    r     <- fit$r
    T_pre <- fit$T_pre
    N_co  <- ncol(Y_co)
    TT    <- nrow(Y_co)
    post_rows  <- (T_pre + 1L):TT
    Y_tr_pre   <- fit$Y_treat[seq_len(T_pre), , drop = FALSE]

    X_co_arr   <- if (!is.null(fit$covariate_array_co)) fit$covariate_array_co else
                    array(0.0, dim = c(TT, N_co, 0L))
    X_tr_pre   <- if (!is.null(fit$covariate_array_tr))
                    fit$covariate_array_tr[seq_len(T_pre), , , drop = FALSE] else
                    array(0.0, dim = c(T_pre, ncol(fit$Y_treat), 0L))
    X_tr_full  <- fit$covariate_array_tr
    p          <- dim(X_co_arr)[3L]

    .refit_sharp <- function(co_idx) {
      N_b <- length(co_idx)
      X_co_b <- if (p > 0L) X_co_arr[, co_idx, , drop = FALSE] else
                  array(0.0, dim = c(TT, N_b, 0L))
      res <- tryCatch(
        gsc_ife_cpp(Y_co[, co_idx, drop = FALSE], Y_tr_pre, r, X_co_b, X_tr_pre),
        error = function(e) NULL
      )
      if (is.null(res)) return(NA_real_)
      Y_hat_post <- res$Y_tr_hat[post_rows, , drop = FALSE]
      if (p > 0L && !is.null(X_tr_full)) {
        N_tr <- ncol(Y_tr_pre)
        T_post <- length(post_rows)
        for (j in seq_len(p))
          Y_hat_post <- Y_hat_post +
            matrix(X_tr_full[post_rows,, j], T_post, N_tr) * res$beta[j]
      }
      mean(fit$Y_treat[post_rows, , drop = FALSE] - Y_hat_post)
    }

    if (method == "bootstrap") {
      boot_ests <- replicate(n_boot, .refit_sharp(sample(N_co, replace = TRUE)))
      n_fail <- sum(is.na(boot_ests))
      if (n_fail > 0.1 * n_boot)
        warning(sprintf("%.0f%% of bootstrap replications failed.", 100 * n_fail / n_boot))
      valid    <- boot_ests[!is.na(boot_ests)]
      se       <- sd(valid)
      ci_lower <- unname(quantile(valid, alpha / 2))
      ci_upper <- unname(quantile(valid, 1 - alpha / 2))
      z        <- tau_hat / max(se, .Machine$double.eps)
      p_value  <- switch(alternative,
        two.sided = 2 * pnorm(-abs(z)),
        greater   = pnorm(-z),
        less      = pnorm(z)
      )
      return(structure(list(
        estimate    = tau_hat, se = se, p_value = p_value,
        ci_lower    = ci_lower, ci_upper = ci_upper,
        method      = method, staggered = FALSE, n_controls = N_co,
        alternative = alternative, boot_ests = boot_ests
      ), class = "coresynth_inference"))
    }

    # jackknife
    jack_ests <- vapply(seq_len(N_co),
                        function(i) .refit_sharp(seq_len(N_co)[-i]),
                        numeric(1))
    jack_var <- (N_co - 1) / N_co * sum((jack_ests - mean(jack_ests))^2)
    se       <- sqrt(jack_var)
    z        <- tau_hat / max(se, .Machine$double.eps)
    ci_lower <- tau_hat - qnorm(1 - alpha / 2) * se
    ci_upper <- tau_hat + qnorm(1 - alpha / 2) * se
    p_value  <- switch(alternative,
      two.sided = 2 * pnorm(-abs(z)),
      greater   = pnorm(-z),
      less      = pnorm(z)
    )
    return(structure(list(
      estimate    = tau_hat, se = se, p_value = p_value,
      ci_lower    = ci_lower, ci_upper = ci_upper,
      method      = method, staggered = FALSE, n_controls = N_co,
      alternative = alternative, boot_ests = NULL
    ), class = "coresynth_inference"))
  }

  # ── Staggered path ──────────────────────────────────────────────────────────
  if (is.null(fit$Y_all))
    stop("fit does not contain Y_all. Re-estimate with the current version.",
         call. = FALSE)

  Y_all       <- fit$Y_all
  r           <- fit$r
  cov_all     <- fit$covariate_array_all  # NULL if no covariates
  cohort_fits <- fit$cohort_fits
  w_g         <- vapply(cohort_fits, `[[`, numeric(1L), "weight")  # unnorm
  W           <- sum(w_g)

  .boot_one_gsc <- function() {
    att_total <- 0
    for (cf in cohort_fits) {
      N_co_g  <- length(cf$idx_co)
      idx_b   <- cf$idx_co[sample(N_co_g, replace = TRUE)]
      cf_b    <- cf
      cf_b$idx_co <- idx_b
      att_g_b <- .refit_gsc_cohort(Y_all, cf_b, r, cov_all)
      att_total <- att_total + cf$weight * att_g_b
    }
    att_total / W
  }

  if (method == "bootstrap") {
    boot_ests <- replicate(n_boot, .boot_one_gsc())
    n_fail <- sum(is.na(boot_ests))
    if (n_fail > 0.1 * n_boot)
      warning(sprintf("%.0f%% of staggered bootstrap replications failed.",
                      100 * n_fail / n_boot))
    valid    <- boot_ests[!is.na(boot_ests)]
    se       <- sd(valid)
    ci_lower <- unname(quantile(valid, alpha / 2))
    ci_upper <- unname(quantile(valid, 1 - alpha / 2))
    z        <- tau_hat / max(se, .Machine$double.eps)
    p_value  <- switch(alternative,
      two.sided = 2 * pnorm(-abs(z)),
      greater   = pnorm(-z),
      less      = pnorm(z)
    )
    n_co_mean <- round(mean(vapply(cohort_fits,
                                   function(cf) length(cf$idx_co), numeric(1L))))
    return(structure(list(
      estimate    = tau_hat, se = se, p_value = p_value,
      ci_lower    = ci_lower, ci_upper = ci_upper,
      method      = method, staggered = TRUE, n_controls = n_co_mean,
      alternative = alternative, boot_ests = boot_ests
    ), class = "coresynth_inference"))
  }

  if (method == "jackknife_global") {
    # Global jackknife: drop one unique control across ALL cohorts simultaneously
    # (Phase 18b). Captures cross-cohort correlation ignored by per-cohort LOO.
    all_co    <- sort(unique(unlist(lapply(cohort_fits, `[[`, "idx_co"))))
    orig_ests <- vapply(cohort_fits, `[[`, numeric(1L), "estimate")
    jack_ests <- vapply(all_co, function(i) {
      att_total <- 0
      for (k in seq_along(cohort_fits)) {
        cf <- cohort_fits[[k]]
        if (i %in% cf$idx_co) {
          keep <- cf$idx_co[cf$idx_co != i]
          if (length(keep) < 2L) return(NA_real_)
          cf_loo        <- cf
          cf_loo$idx_co <- keep
          att_g <- .refit_gsc_cohort(Y_all, cf_loo, r, cov_all)
          if (is.na(att_g)) return(NA_real_)
        } else {
          att_g <- orig_ests[k]
        }
        att_total <- att_total + cf$weight * att_g
      }
      att_total / W
    }, numeric(1L))
    valid <- jack_ests[!is.na(jack_ests)]
    N_v   <- length(valid)
    se    <- sqrt((N_v - 1L) / N_v * sum((valid - mean(valid))^2))
    z        <- tau_hat / max(se, .Machine$double.eps)
    ci_lower <- tau_hat - qnorm(1 - alpha / 2) * se
    ci_upper <- tau_hat + qnorm(1 - alpha / 2) * se
    p_value  <- switch(alternative,
      two.sided = 2 * pnorm(-abs(z)),
      greater   = pnorm(-z),
      less      = pnorm(z)
    )
    return(structure(list(
      estimate    = tau_hat, se = se, p_value = p_value,
      ci_lower    = ci_lower, ci_upper = ci_upper,
      method      = method, staggered = TRUE, n_controls = length(all_co),
      alternative = alternative, boot_ests = NULL
    ), class = "coresynth_inference"))
  }

  # staggered jackknife: per-cohort LOO + delta-method
  var_components <- vapply(cohort_fits, function(cf) {
    N_co_g <- length(cf$idx_co)
    if (N_co_g < 2L) return(0)
    jack_g <- vapply(seq_len(N_co_g), function(i) {
      cf_loo <- cf
      cf_loo$idx_co <- cf$idx_co[-i]
      .refit_gsc_cohort(Y_all, cf_loo, r, cov_all)
    }, numeric(1))
    valid_g <- jack_g[!is.na(jack_g)]
    if (length(valid_g) < 2L) return(0)
    (N_co_g - 1) / N_co_g * sum((valid_g - mean(valid_g))^2)
  }, numeric(1))

  var_st   <- sum((w_g / W)^2 * var_components)
  se       <- sqrt(var_st)
  z        <- tau_hat / max(se, .Machine$double.eps)
  ci_lower <- tau_hat - qnorm(1 - alpha / 2) * se
  ci_upper <- tau_hat + qnorm(1 - alpha / 2) * se
  p_value  <- switch(alternative,
    two.sided = 2 * pnorm(-abs(z)),
    greater   = pnorm(-z),
    less      = pnorm(z)
  )
  n_co_mean <- round(mean(vapply(cohort_fits,
                                 function(cf) length(cf$idx_co), numeric(1L))))
  structure(list(
    estimate    = tau_hat, se = se, p_value = p_value,
    ci_lower    = ci_lower, ci_upper = ci_upper,
    method      = method, staggered = TRUE, n_controls = n_co_mean,
    alternative = alternative, boot_ests = NULL
  ), class = "coresynth_inference")
}

# -- Internal: multi-arm SI for K>1 treatment arms (Agarwal et al. 2025) -----
# Latent unit factors v_il are arm-invariant, so the SVD basis from control arm
# (d=0) is reused for all K treatment arms. si_pcr_cpp is called once per arm.
.fit_si_multi <- function(Y, idx_by_arm, arm_levels, T_pre, k, times) {
  TT        <- nrow(Y)
  T_post    <- TT - T_pre
  pre_rows  <- seq_len(T_pre)
  post_rows <- (T_pre + 1L):TT
  idx_co    <- idx_by_arm[["0"]]
  treat_arms <- arm_levels[arm_levels != 0L]  # c(1L, ..., KL)

  N_co <- length(idx_co)
  if (N_co < 2L)
    stop("SI multi-arm: at least 2 control units required.", call. = FALSE)
  if (T_pre < 1L)
    stop("SI multi-arm: T_pre must be > 0.", call. = FALSE)
  if (T_post < 1L)
    stop("SI multi-arm: T_post must be > 0.", call. = FALSE)

  # Common donor matrices shared across all arms (unit factors are arm-invariant)
  Y_pre_co  <- Y[pre_rows,  idx_co, drop = FALSE]  # T_pre  x N_co
  Y_post_co <- Y[post_rows, idx_co, drop = FALSE]  # T_post x N_co

  if (is.null(k)) k <- max(1L, floor(sqrt(min(T_pre, N_co))))
  k <- as.integer(k)
  if (k > min(T_pre, N_co))
    stop(sprintf("k (%d) must be <= min(T_pre=%d, N_co=%d).", k, T_pre, N_co),
         call. = FALSE)

  arm_fits <- lapply(as.character(treat_arms), function(a) {
    idx_tr_d   <- idx_by_arm[[a]]
    Y_pre_tr_d <- Y[pre_rows,  idx_tr_d, drop = FALSE]
    Y_treat_d  <- Y[, idx_tr_d, drop = FALSE]

    res <- tryCatch(
      si_pcr_cpp(Y_pre_co, Y_post_co, Y_pre_tr_d, k),
      error = function(e)
        stop(sprintf("SI multi-arm arm %s: %s", a, conditionMessage(e)),
             call. = FALSE)
    )
    W_d       <- res$W       # N_co x N_tr_d
    Y_cf_d    <- res$Y_hat   # T_post x N_tr_d
    Y_synth_d <- rbind(Y_pre_co %*% W_d, Y_cf_d)  # T x N_tr_d
    gap_d     <- Y_treat_d - Y_synth_d
    att_d     <- mean(gap_d[post_rows, , drop = FALSE], na.rm = TRUE)

    list(
      arm       = as.integer(a),
      n_treated = length(idx_tr_d),
      T_pre     = T_pre,
      T_post    = T_post,
      estimate  = att_d,
      weight    = as.numeric(length(idx_tr_d)) * T_post,
      idx_tr    = idx_tr_d,
      idx_co    = idx_co,
      weights   = W_d,
      Y_cf      = Y_cf_d,
      Y_synth   = Y_synth_d,
      Y_treat   = Y_treat_d,
      gap       = gap_d
    )
  })
  names(arm_fits) <- as.character(treat_arms)

  # Weighted-average ATT: weight proportional to N_tr_d * T_post (Clarke et al. 2023)
  ws   <- vapply(arm_fits, `[[`, numeric(1L), "weight")
  taus <- vapply(arm_fits, `[[`, numeric(1L), "estimate")
  att  <- sum(ws * taus) / sum(ws)

  list(
    method        = "si",
    multi_arm     = TRUE,
    arm_levels    = treat_arms,
    k             = k,
    estimate      = att,
    arm_estimates = setNames(taus, as.character(treat_arms)),
    arm_fits      = arm_fits,
    Y_pre_co      = Y_pre_co,
    Y_post_co     = Y_post_co,
    idx_co        = idx_co,
    times         = times,
    T_pre         = T_pre,
    staggered     = FALSE,
    Y_treat       = NULL,
    Y_synth       = NULL,
    gap           = NULL,
    unit_weights  = NULL
  )
}

# -- Internal: cohort-by-cohort SI for staggered adoption --------------------
# Clarke et al. (2023); Arkhangelsky et al. (2021) Appendix S.8.
# SVD uses only Y_pre_co_g (T_pre_g x N_co_g): future-treated units in the
# "clean" control group are all pre-treatment during periods 1..T_pre_g, so
# there is no contamination from treatment in this window.
.fit_si_staggered <- function(pan, Y, k = NULL, control_group = "clean") {
  TT      <- nrow(Y)
  T_adopt <- pan$T_adopt
  idx_tr  <- pan$idx_treat
  idx_co  <- pan$idx_control

  cohorts <- sort(unique(T_adopt[idx_tr]))

  cohort_list <- lapply(cohorts, function(g) {
    idx_tr_g <- idx_tr[T_adopt[idx_tr] == g]
    T_pre_g  <- g - 1L
    T_post_g <- TT - T_pre_g

    if (control_group == "never_treated") {
      idx_co_g <- idx_co
    } else {
      future_tr <- idx_tr[!is.na(T_adopt[idx_tr]) & T_adopt[idx_tr] > g]
      idx_co_g  <- c(idx_co, future_tr)
    }

    N_co_g <- length(idx_co_g)
    if (N_co_g < 2L || T_pre_g < 1L) {
      warning(sprintf(
        "SI staggered: cohort g=%d skipped (T_pre=%d, N_co=%d).",
        g, T_pre_g, N_co_g), call. = FALSE)
      return(NULL)
    }

    pre_rows  <- seq_len(T_pre_g)
    post_rows <- (T_pre_g + 1L):TT

    Y_pre_co_g  <- Y[pre_rows,  idx_co_g, drop = FALSE]
    Y_post_co_g <- Y[post_rows, idx_co_g, drop = FALSE]
    Y_pre_tr_g  <- Y[pre_rows,  idx_tr_g, drop = FALSE]
    Y_treat_g   <- Y[, idx_tr_g, drop = FALSE]

    k_g <- if (!is.null(k)) {
      max(1L, min(as.integer(k), min(T_pre_g, N_co_g) - 1L))
    } else {
      max(1L, floor(sqrt(min(T_pre_g, N_co_g))))
    }

    res <- tryCatch(
      si_pcr_cpp(Y_pre_co_g, Y_post_co_g, Y_pre_tr_g, k_g),
      error = function(e) {
        warning(sprintf("SI staggered: cohort g=%d failed: %s",
                        g, conditionMessage(e)), call. = FALSE)
        NULL
      }
    )
    if (is.null(res)) return(NULL)

    W_g       <- res$W
    Y_cf_g    <- res$Y_hat
    Y_synth_g <- rbind(Y_pre_co_g %*% W_g, Y_cf_g)
    gap_g     <- Y_treat_g - Y_synth_g
    att_g     <- mean(gap_g[post_rows, , drop = FALSE])

    list(
      cohort    = g,
      n_treated = length(idx_tr_g),
      T_pre     = T_pre_g,
      T_post    = T_post_g,
      estimate  = att_g,
      weight    = as.numeric(length(idx_tr_g)) * T_post_g,
      idx_tr    = idx_tr_g,
      idx_co    = idx_co_g,
      k         = k_g,
      weights   = W_g,
      Y_cf      = Y_cf_g,
      Y_synth   = Y_synth_g,
      Y_treat   = Y_treat_g,
      gap       = gap_g
    )
  })

  valid <- !vapply(cohort_list, is.null, logical(1L))
  if (!any(valid)) stop("All cohort-level SI fits failed.", call. = FALSE)
  r_list <- cohort_list[valid]

  w   <- vapply(r_list, `[[`, numeric(1L), "weight")
  tau <- vapply(r_list, `[[`, numeric(1L), "estimate")
  att <- sum(w * tau) / sum(w)

  cohort_df <- data.frame(
    cohort    = vapply(r_list, `[[`, integer(1L), "cohort"),
    n_treated = vapply(r_list, `[[`, integer(1L), "n_treated"),
    T_pre     = vapply(r_list, `[[`, integer(1L), "T_pre"),
    T_post    = vapply(r_list, `[[`, integer(1L), "T_post"),
    estimate  = tau,
    weight    = w / sum(w),
    stringsAsFactors = FALSE
  )

  list(estimate = att, cohort_estimates = cohort_df, cohort_fits = r_list, k = k)
}

# -- Internal: staggered + multi-arm SI (K>1 arms, staggered adoption) -------
# Combines Phase 16b (staggered) and Phase 21 (multi-arm).
# Within each cohort g, the SVD basis (from Y_pre_co_g) is shared across all
# arms because unit factors v_il are arm-invariant (Agarwal et al. 2025 S.2).
# cohort_fits is a flat list of (cohort, arm) cells -- each compatible with
# .refit_si_cohort() since they carry T_pre, idx_co, idx_tr, k.
.fit_si_staggered_multi <- function(tensor, k = NULL, control_group = "clean") {
  Y          <- tensor$Y
  TT         <- nrow(Y)
  T_adopt    <- tensor$T_adopt
  idx_tr_all <- tensor$idx_treat
  idx_co_all <- tensor$idx_control
  treat_arms <- tensor$arm_levels[tensor$arm_levels != 0L]

  cohorts <- sort(unique(T_adopt[idx_tr_all]))
  cf_list <- list()

  for (g in cohorts) {
    idx_tr_g <- idx_tr_all[!is.na(T_adopt[idx_tr_all]) & T_adopt[idx_tr_all] == g]
    T_pre_g  <- g - 1L
    T_post_g <- TT - T_pre_g

    if (control_group == "never_treated") {
      idx_co_g <- idx_co_all
    } else {
      future_tr <- idx_tr_all[!is.na(T_adopt[idx_tr_all]) & T_adopt[idx_tr_all] > g]
      idx_co_g  <- c(idx_co_all, future_tr)
    }

    N_co_g <- length(idx_co_g)
    if (N_co_g < 2L || T_pre_g < 1L) {
      warning(sprintf(
        "SI staggered-multi: cohort g=%d skipped (T_pre=%d, N_co=%d).",
        g, T_pre_g, N_co_g), call. = FALSE)
      next
    }

    pre_rows  <- seq_len(T_pre_g)
    post_rows <- (T_pre_g + 1L):TT

    Y_pre_co_g  <- Y[pre_rows,  idx_co_g, drop = FALSE]
    Y_post_co_g <- Y[post_rows, idx_co_g, drop = FALSE]

    k_g <- if (!is.null(k)) {
      max(1L, min(as.integer(k), min(T_pre_g, N_co_g) - 1L))
    } else {
      max(1L, floor(sqrt(min(T_pre_g, N_co_g))))
    }

    for (a in as.character(treat_arms)) {
      idx_tr_gd <- intersect(idx_tr_g, tensor$idx_by_arm[[a]])
      if (length(idx_tr_gd) == 0L) next

      Y_pre_tr_gd <- Y[pre_rows,  idx_tr_gd, drop = FALSE]
      Y_treat_gd  <- Y[, idx_tr_gd, drop = FALSE]

      res <- tryCatch(
        si_pcr_cpp(Y_pre_co_g, Y_post_co_g, Y_pre_tr_gd, k_g),
        error = function(e) {
          warning(sprintf("SI staggered-multi: cohort g=%d arm %s failed: %s",
                          g, a, conditionMessage(e)), call. = FALSE)
          NULL
        }
      )
      if (is.null(res)) next

      W_gd       <- res$W
      Y_cf_gd    <- res$Y_hat
      Y_synth_gd <- rbind(Y_pre_co_g %*% W_gd, Y_cf_gd)
      gap_gd     <- Y_treat_gd - Y_synth_gd
      att_gd     <- mean(gap_gd[post_rows, , drop = FALSE], na.rm = TRUE)

      cf_list <- c(cf_list, list(list(
        cohort    = g,
        arm       = as.integer(a),
        n_treated = length(idx_tr_gd),
        T_pre     = T_pre_g,
        T_post    = T_post_g,
        estimate  = att_gd,
        weight    = as.numeric(length(idx_tr_gd)) * T_post_g,
        idx_tr    = idx_tr_gd,
        idx_co    = idx_co_g,
        k         = k_g,
        weights   = W_gd,
        Y_cf      = Y_cf_gd,
        Y_synth   = Y_synth_gd,
        Y_treat   = Y_treat_gd,
        gap       = gap_gd
      )))
    }
  }

  if (length(cf_list) == 0L)
    stop("All (cohort, arm) SI fits failed.", call. = FALSE)

  ws      <- vapply(cf_list, `[[`, numeric(1L), "weight")
  taus    <- vapply(cf_list, `[[`, numeric(1L), "estimate")
  W_total <- sum(ws)
  att     <- sum(ws * taus) / W_total

  arm_estimates <- setNames(
    vapply(as.character(treat_arms), function(a) {
      mask <- vapply(cf_list, function(cf) cf$arm == as.integer(a), logical(1L))
      if (!any(mask) || sum(ws[mask]) == 0) return(NA_real_)
      sum(ws[mask] * taus[mask]) / sum(ws[mask])
    }, numeric(1L)),
    as.character(treat_arms)
  )

  cohort_arm_df <- data.frame(
    cohort    = vapply(cf_list, `[[`, integer(1L),  "cohort"),
    arm       = vapply(cf_list, `[[`, integer(1L),  "arm"),
    n_treated = vapply(cf_list, `[[`, integer(1L),  "n_treated"),
    T_pre     = vapply(cf_list, `[[`, integer(1L),  "T_pre"),
    T_post    = vapply(cf_list, `[[`, integer(1L),  "T_post"),
    estimate  = taus,
    weight    = ws / W_total,
    stringsAsFactors = FALSE
  )

  list(
    method               = "si",
    staggered            = TRUE,
    multi_arm            = TRUE,
    arm_levels           = treat_arms,
    k                    = k,
    estimate             = att,
    arm_estimates        = arm_estimates,
    cohort_arm_estimates = cohort_arm_df,
    cohort_fits          = cf_list,
    Y_all                = Y,
    idx_co               = idx_co_all,
    times                = tensor$times,
    T_pre                = tensor$T_pre,
    Y_treat              = NULL,
    Y_synth              = NULL,
    gap                  = NULL,
    unit_weights         = NULL
  )
}

#' Fit Synthetic Interventions via PCR (Agarwal et al. 2025)
#'
#' Estimates counterfactual outcomes using principal component regression
#' on the pre-treatment control panel. Assumes a low-rank tensor factor model
#' for potential outcomes:
#'   Y_ti^(d) = sum_l u_tl * v_il * lambda_dl + eps_ti^(d)
#'
#' For the standard binary-treatment case (d in {0, 1}), this simplifies to:
#' find weights over control units in the pre-period using k-component PCR,
#' then apply those weights to control post-period outcomes as the counterfactual.
#'
#' Unlike SCM and SDID, SI-PCR weights are unconstrained (can be negative or
#' sum to values other than 1). The `plot(type = "weights")` view shows only
#' positive weights (> 1e-4).
#'
#' @param y    Outcome vector (long format)
#' @param d    Treatment arm indicator (integer, 0 = control, 1 = treated)
#' @param id   Unit identifier (long format)
#' @param time Time identifier (long format)
#' @param k    Number of SVD components for PCR. Default:
#'             `floor(sqrt(min(T_pre, N_co)))`, at least 1.
#' @return A list of class `coresynth` with:
#'   * `method`: `"si"`
#'   * `estimate`: ATT (average treatment effect on the treated)
#'   * `k`: number of SVD components used
#'   * `weights`: Donor weight matrix (N_co x N_tr)
#'   * `unit_weights`: Average donor weights across treated units (named N_co vector)
#'   * `Y_cf`: Counterfactual post-treatment outcomes (T_post x N_tr)
#'   * `Y_treat`: Observed treated outcomes (T x N_tr)
#'   * `Y_synth`: Full synthetic series (T x N_tr; pre = weighted control, post = Y_cf)
#'   * `gap`: Treatment effect series (T x N_tr)
#'   * `times`: Time index vector
#'   * `T_pre`: Number of pre-treatment periods
#' @noRd
fit_si_cpp <- function(y, d, id, time, k = NULL,
                       control_group = c("clean", "never_treated"), ...) {
  control_group <- match.arg(control_group)

  # -- Multi-arm detection: max(d) > 1 means K>=2 treatment arms -------------
  d_int <- as.integer(d)
  if (max(d_int, na.rm = TRUE) > 1L) {
    tensor <- panel_to_tensor(y, d_int, id, time)
    k_use  <- if (is.null(k)) NULL else as.integer(k)
    if (!tensor$is_sharp) {
      return(.fit_si_staggered_multi(tensor, k_use, control_group))
    }
    return(.fit_si_multi(tensor$Y, tensor$idx_by_arm, tensor$arm_levels,
                         tensor$T_pre, k_use, tensor$times))
  }

  pan    <- panel_to_matrices(y, d_int, id, time)

  # -- Staggered adoption path -------------------------------------------------
  if (!pan$is_sharp) {
    res_st <- .fit_si_staggered(pan, pan$Y, k, control_group)
    res_st$method       <- "si"
    res_st$staggered    <- TRUE
    res_st$times        <- pan$times
    res_st$T_pre        <- pan$T_pre
    res_st$Y_treat      <- pan$Y[, pan$idx_treat, drop = FALSE]
    res_st$Y_synth      <- NULL
    res_st$gap          <- NULL
    res_st$unit_weights <- NULL
    res_st$Y_all        <- pan$Y             # T x N -- needed by si_inference()
    res_st$idx_co       <- pan$idx_control
    res_st$idx_tr       <- pan$idx_treat
    return(res_st)
  }

  Y      <- pan$Y
  T_pre  <- pan$T_pre
  idx_tr <- pan$idx_treat
  idx_co <- pan$idx_control

  if (length(idx_co) < 2)
    stop("SI requires at least two control units.")
  if (T_pre < 1)
    stop("SI requires at least one pre-treatment period.")

  TT     <- nrow(Y)
  T_post <- TT - T_pre
  N_co   <- length(idx_co)

  if (T_post < 1)
    stop("No post-treatment periods found.")

  pre_rows  <- seq_len(T_pre)
  post_rows <- (T_pre + 1L):TT

  Y_pre_co  <- Y[pre_rows,  idx_co, drop = FALSE]  # T_pre  x N_co
  Y_post_co <- Y[post_rows, idx_co, drop = FALSE]  # T_post x N_co
  Y_pre_tr  <- Y[pre_rows,  idx_tr, drop = FALSE]  # T_pre  x N_tr

  if (is.null(k)) {
    k <- max(1L, floor(sqrt(min(T_pre, N_co))))
  }
  k <- as.integer(k)

  if (k > min(T_pre, N_co))
    stop(sprintf("k (%d) must be <= min(T_pre=%d, N_co=%d).", k, T_pre, N_co))

  res  <- si_pcr_cpp(Y_pre_co, Y_post_co, Y_pre_tr, k)
  W    <- res$W      # N_co x N_tr
  Y_cf <- res$Y_hat  # T_post x N_tr

  # Full synthetic series for plotting: pre = Y_pre_co %*% W, post = Y_cf
  Y_synth_full <- rbind(Y_pre_co %*% W, Y_cf)  # T x N_tr

  Y_treat <- Y[, idx_tr, drop = FALSE]          # T x N_tr
  gap     <- Y_treat - Y_synth_full
  att     <- mean(gap[post_rows, , drop = FALSE], na.rm = TRUE)

  unit_w        <- rowMeans(W)
  names(unit_w) <- colnames(Y_pre_co)

  list(
    method       = "si",
    k            = k,
    weights      = W,
    unit_weights = unit_w,
    Y_cf         = Y_cf,
    Y_treat      = Y_treat,
    Y_synth      = Y_synth_full,
    gap          = gap,
    times        = pan$times,
    T_pre        = T_pre,
    estimate     = att,
    Y_pre_co     = Y_pre_co,    # T_pre  x N_co -- needed by si_inference()
    Y_post_co    = Y_post_co    # T_post x N_co -- needed by si_inference()
  )
}

# -- Internal helper: re-estimate one SI cohort with given control indices ----
.refit_si_cohort <- function(Y_all, cf) {
  TT        <- nrow(Y_all)
  T_pre_g   <- cf$T_pre
  pre_rows  <- seq_len(T_pre_g)
  post_rows <- (T_pre_g + 1L):TT

  Y_pre_co_g  <- Y_all[pre_rows,  cf$idx_co, drop = FALSE]
  Y_post_co_g <- Y_all[post_rows, cf$idx_co, drop = FALSE]
  Y_pre_tr_g  <- Y_all[pre_rows,  cf$idx_tr, drop = FALSE]

  res <- tryCatch(
    si_pcr_cpp(Y_pre_co_g, Y_post_co_g, Y_pre_tr_g, cf$k),
    error = function(e) NULL
  )
  if (is.null(res)) return(NA_real_)

  Y_treat_post <- Y_all[post_rows, cf$idx_tr, drop = FALSE]
  mean(Y_treat_post - res$Y_hat)
}

# -- Internal: inference for staggered + multi-arm SI (Phase 23) -------------
# Bootstrap: per-cohort resampling -- same resampled controls applied to all arms
#   within a cohort (shared SVD basis requires shared donor bootstrap indices).
# Jackknife: per-cohort LOO refitting all arms simultaneously + delta-method.
# jackknife_global: drop one unique control from ALL cohorts and ALL arms.
.si_inference_staggered_multi <- function(fit, method, n_boot, level,
                                          alternative, seed) {
  if (!is.null(seed)) set.seed(seed)
  alpha      <- 1 - level
  tau_hat    <- fit$estimate
  Y_all      <- fit$Y_all
  cf_list    <- fit$cohort_fits
  ws         <- vapply(cf_list, `[[`, numeric(1L), "weight")
  W_total    <- sum(ws)
  cohort_ids <- vapply(cf_list, `[[`, integer(1L), "cohort")
  uc         <- sort(unique(cohort_ids))

  .p_from_z <- function(z) switch(alternative,
    two.sided = 2 * pnorm(-abs(z)),
    greater   = pnorm(-z),
    less      = pnorm(z))

  .n_co_mean <- round(mean(vapply(uc, function(g) {
    length(cf_list[cohort_ids == g][[1L]]$idx_co)
  }, numeric(1L))))

  if (method == "bootstrap") {
    .boot_one <- function() {
      att_total <- 0
      for (g in uc) {
        cells    <- cf_list[cohort_ids == g]
        idx_co_g <- cells[[1L]]$idx_co
        boot_idx <- idx_co_g[sample(length(idx_co_g), replace = TRUE)]
        for (cf in cells) {
          cf_b <- cf; cf_b$idx_co <- boot_idx
          att_gd <- .refit_si_cohort(Y_all, cf_b)
          att_total <- att_total + cf$weight * att_gd
        }
      }
      att_total / W_total
    }
    boot_ests <- replicate(n_boot, .boot_one())
    n_fail <- sum(is.na(boot_ests))
    if (n_fail > 0.1 * n_boot)
      warning(sprintf("%.0f%% of staggered-multi bootstrap replications failed.",
                      100 * n_fail / n_boot))
    valid    <- boot_ests[!is.na(boot_ests)]
    se       <- sd(valid)
    ci_lower <- unname(quantile(valid, alpha / 2))
    ci_upper <- unname(quantile(valid, 1 - alpha / 2))
    z        <- tau_hat / max(se, .Machine$double.eps)
    return(structure(list(
      estimate    = tau_hat, se = se, p_value = .p_from_z(z),
      ci_lower    = ci_lower, ci_upper = ci_upper,
      method      = method, staggered = TRUE, n_controls = .n_co_mean,
      alternative = alternative, boot_ests = boot_ests
    ), class = "coresynth_inference"))
  }

  if (method == "jackknife_global") {
    all_co    <- sort(unique(unlist(lapply(cf_list, `[[`, "idx_co"))))
    orig_ests <- vapply(cf_list, `[[`, numeric(1L), "estimate")
    jack_ests <- vapply(all_co, function(i) {
      att_total <- 0
      for (ki in seq_along(cf_list)) {
        cf <- cf_list[[ki]]
        if (i %in% cf$idx_co) {
          keep <- cf$idx_co[cf$idx_co != i]
          if (length(keep) < 2L) return(NA_real_)
          cf_loo <- cf; cf_loo$idx_co <- keep
          att_gd <- .refit_si_cohort(Y_all, cf_loo)
          if (is.na(att_gd)) return(NA_real_)
        } else {
          att_gd <- orig_ests[ki]
        }
        att_total <- att_total + cf$weight * att_gd
      }
      att_total / W_total
    }, numeric(1L))
    valid <- jack_ests[!is.na(jack_ests)]
    N_v   <- length(valid)
    se    <- sqrt((N_v - 1L) / N_v * sum((valid - mean(valid))^2))
    z     <- tau_hat / max(se, .Machine$double.eps)
    ci_lower <- tau_hat - qnorm(1 - alpha / 2) * se
    ci_upper <- tau_hat + qnorm(1 - alpha / 2) * se
    return(structure(list(
      estimate    = tau_hat, se = se, p_value = .p_from_z(z),
      ci_lower    = ci_lower, ci_upper = ci_upper,
      method      = method, staggered = TRUE, n_controls = length(all_co),
      alternative = alternative, boot_ests = NULL
    ), class = "coresynth_inference"))
  }

  # jackknife: per-cohort LOO refitting all arms simultaneously + delta-method
  var_components <- vapply(uc, function(g) {
    cells    <- cf_list[cohort_ids == g]
    idx_co_g <- cells[[1L]]$idx_co
    N_co_g   <- length(idx_co_g)
    if (N_co_g < 2L) return(0)
    jack_g <- vapply(seq_len(N_co_g), function(i) {
      keep <- idx_co_g[-i]
      cohort_agg <- 0
      for (cf in cells) {
        cf_loo <- cf; cf_loo$idx_co <- keep
        att_gd_loo <- .refit_si_cohort(Y_all, cf_loo)
        if (is.na(att_gd_loo)) return(NA_real_)
        cohort_agg <- cohort_agg + cf$weight * att_gd_loo
      }
      cohort_agg
    }, numeric(1L))
    valid_g <- jack_g[!is.na(jack_g)]
    if (length(valid_g) < 2L) return(0)
    (N_co_g - 1L) / N_co_g * sum((valid_g - mean(valid_g))^2)
  }, numeric(1L))

  w_g_totals <- vapply(uc, function(g) {
    sum(vapply(cf_list[cohort_ids == g], `[[`, numeric(1L), "weight"))
  }, numeric(1L))
  var_st   <- sum((w_g_totals / W_total)^2 * var_components)
  se       <- sqrt(var_st)
  z        <- tau_hat / max(se, .Machine$double.eps)
  ci_lower <- tau_hat - qnorm(1 - alpha / 2) * se
  ci_upper <- tau_hat + qnorm(1 - alpha / 2) * se
  structure(list(
    estimate    = tau_hat, se = se, p_value = .p_from_z(z),
    ci_lower    = ci_lower, ci_upper = ci_upper,
    method      = method, staggered = TRUE, n_controls = .n_co_mean,
    alternative = alternative, boot_ests = NULL
  ), class = "coresynth_inference")
}

# -- Internal: inference for multi-arm SI -------------------------------------
# Bootstrap: resample control columns (shared across all arms, since arms share
#   the same donor pool). Jackknife: LOO over control columns.
.si_inference_multi <- function(fit, method, n_boot, level, alternative, seed) {
  if (!is.null(seed)) set.seed(seed)
  alpha     <- 1 - level
  tau_hat   <- fit$estimate
  Y_pre_co  <- fit$Y_pre_co
  Y_post_co <- fit$Y_post_co
  k         <- fit$k
  arm_fits  <- fit$arm_fits
  T_pre     <- fit$T_pre
  N_co      <- ncol(Y_pre_co)

  .refit_multi <- function(co_idx) {
    Ypc  <- Y_pre_co[,  co_idx, drop = FALSE]
    Ypoc <- Y_post_co[, co_idx, drop = FALSE]
    post_rows_local <- (T_pre + 1L):nrow(arm_fits[[1L]]$Y_treat)
    ws <- taus <- numeric(length(arm_fits))
    for (i in seq_along(arm_fits)) {
      af  <- arm_fits[[i]]
      Ytr_pre <- af$Y_treat[seq_len(T_pre), , drop = FALSE]
      res <- tryCatch(si_pcr_cpp(Ypc, Ypoc, Ytr_pre, k),
                      error = function(e) NULL)
      ws[i] <- af$weight
      if (is.null(res)) {
        taus[i] <- af$estimate
      } else {
        taus[i] <- mean(
          af$Y_treat[post_rows_local, , drop = FALSE] - res$Y_hat,
          na.rm = TRUE
        )
      }
    }
    sum(ws * taus) / sum(ws)
  }

  if (method == "bootstrap") {
    boot_ests <- replicate(n_boot, .refit_multi(sample(N_co, replace = TRUE)))
    n_fail <- sum(is.na(boot_ests))
    if (n_fail > 0.1 * n_boot)
      warning(sprintf("%.0f%% of multi-arm bootstrap replications failed.",
                      100 * n_fail / n_boot))
    valid    <- boot_ests[!is.na(boot_ests)]
    se       <- sd(valid)
    ci_lower <- unname(quantile(valid, alpha / 2))
    ci_upper <- unname(quantile(valid, 1 - alpha / 2))
    z        <- tau_hat / max(se, .Machine$double.eps)
    p_value  <- switch(alternative,
      two.sided = 2 * pnorm(-abs(z)),
      greater   = pnorm(-z),
      less      = pnorm(z)
    )
    return(structure(list(
      estimate    = tau_hat, se = se, p_value = p_value,
      ci_lower    = ci_lower, ci_upper = ci_upper,
      method      = method, staggered = FALSE, n_controls = N_co,
      alternative = alternative, boot_ests = boot_ests
    ), class = "coresynth_inference"))
  }

  # jackknife
  jack_ests <- vapply(seq_len(N_co),
                      function(i) .refit_multi(seq_len(N_co)[-i]),
                      numeric(1L))
  jack_var <- (N_co - 1) / N_co * sum((jack_ests - mean(jack_ests, na.rm = TRUE))^2)
  se       <- sqrt(jack_var)
  z        <- tau_hat / max(se, .Machine$double.eps)
  ci_lower <- tau_hat - qnorm(1 - alpha / 2) * se
  ci_upper <- tau_hat + qnorm(1 - alpha / 2) * se
  p_value  <- switch(alternative,
    two.sided = 2 * pnorm(-abs(z)),
    greater   = pnorm(-z),
    less      = pnorm(z)
  )
  structure(list(
    estimate    = tau_hat, se = se, p_value = p_value,
    ci_lower    = ci_lower, ci_upper = ci_upper,
    method      = method, staggered = FALSE, n_controls = N_co,
    alternative = alternative, boot_ests = NULL
  ), class = "coresynth_inference")
}

#' Non-parametric Inference for SI (Agarwal et al. 2025)
#'
#' Estimates SE and confidence intervals for the ATT via non-parametric cluster
#' bootstrap or jackknife over control units. Works for both sharp and staggered
#' SI fits. For staggered fits, bootstrap resamples each cohort's control pool
#' independently, and jackknife uses a per-cohort LOO with delta-method variance
#' aggregation.
#'
#' @param fit   A `coresynth` object from [scm_fit()] with `method = "si"`.
#' @param method `"bootstrap"` (default) or `"jackknife"`.
#' @param n_boot Number of bootstrap replications (default 499L; ignored for jackknife).
#' @param level  Confidence level (default 0.95).
#' @param alternative `"two.sided"` (default), `"greater"`, or `"less"`.
#' @param seed  RNG seed for reproducibility (default NULL).
#' @return A list of class `coresynth_inference`.
#' @export
si_inference <- function(
  fit,
  method      = c("bootstrap", "jackknife", "jackknife_global"),
  n_boot      = 499L,
  level       = 0.95,
  alternative = c("two.sided", "greater", "less"),
  seed        = NULL
) {
  method      <- match.arg(method)
  alternative <- match.arg(alternative)

  if (!inherits(fit, "coresynth") || !identical(fit$method, "si"))
    stop("si_inference() requires a coresynth fit with method = 'si'.",
         call. = FALSE)

  tau_hat   <- fit$estimate
  alpha     <- 1 - level
  staggered <- isTRUE(fit$staggered)

  # -- Multi-arm path ----------------------------------------------------------
  if (isTRUE(fit$multi_arm)) {
    if (isTRUE(fit$staggered)) {
      return(.si_inference_staggered_multi(fit, method, n_boot, level, alternative, seed))
    }
    if (method == "jackknife_global")
      stop("jackknife_global is not supported for sharp multi-arm fits; use jackknife instead.",
           call. = FALSE)
    return(.si_inference_multi(fit, method, n_boot, level, alternative, seed))
  }

  if (!is.null(seed)) set.seed(seed)

  # -- Sharp path --------------------------------------------------------------
  if (!staggered) {
    if (method == "jackknife_global")
      stop("si_inference() method='jackknife_global' requires a staggered fit.",
           call. = FALSE)
    if (is.null(fit$Y_pre_co))
      stop("fit does not contain Y_pre_co. Re-estimate with the current version.",
           call. = FALSE)

    Y_pre_co  <- fit$Y_pre_co
    Y_post_co <- fit$Y_post_co
    Y_pre_tr  <- fit$Y_treat[seq_len(fit$T_pre), , drop = FALSE]
    k         <- fit$k
    N_co      <- ncol(Y_pre_co)
    post_rows <- (fit$T_pre + 1L):nrow(fit$Y_treat)

    .refit_sharp <- function(co_idx) {
      res <- tryCatch(
        si_pcr_cpp(Y_pre_co[, co_idx, drop = FALSE],
                   Y_post_co[, co_idx, drop = FALSE],
                   Y_pre_tr, k),
        error = function(e) NULL
      )
      if (is.null(res)) return(NA_real_)
      mean(fit$Y_treat[post_rows, , drop = FALSE] - res$Y_hat)
    }

    if (method == "bootstrap") {
      boot_ests <- replicate(n_boot, {
        .refit_sharp(sample(N_co, replace = TRUE))
      })
      n_fail <- sum(is.na(boot_ests))
      if (n_fail > 0.1 * n_boot)
        warning(sprintf("%.0f%% of bootstrap replications failed.", 100 * n_fail / n_boot))
      valid <- boot_ests[!is.na(boot_ests)]
      se       <- sd(valid)
      ci_lower <- unname(quantile(valid, alpha / 2))
      ci_upper <- unname(quantile(valid, 1 - alpha / 2))
      z        <- tau_hat / max(se, .Machine$double.eps)
      p_value  <- switch(alternative,
        two.sided = 2 * pnorm(-abs(z)),
        greater   = pnorm(-z),
        less      = pnorm(z)
      )
      return(structure(list(
        estimate    = tau_hat, se = se, p_value = p_value,
        ci_lower    = ci_lower, ci_upper = ci_upper,
        method      = method, staggered = FALSE, n_controls = N_co,
        alternative = alternative, boot_ests = boot_ests
      ), class = "coresynth_inference"))
    }

    # jackknife
    jack_ests <- vapply(seq_len(N_co),
                        function(i) .refit_sharp(seq_len(N_co)[-i]),
                        numeric(1))
    jack_var <- (N_co - 1) / N_co * sum((jack_ests - mean(jack_ests))^2)
    se       <- sqrt(jack_var)
    z        <- tau_hat / max(se, .Machine$double.eps)
    ci_lower <- tau_hat - qnorm(1 - alpha / 2) * se
    ci_upper <- tau_hat + qnorm(1 - alpha / 2) * se
    p_value  <- switch(alternative,
      two.sided = 2 * pnorm(-abs(z)),
      greater   = pnorm(-z),
      less      = pnorm(z)
    )
    return(structure(list(
      estimate    = tau_hat, se = se, p_value = p_value,
      ci_lower    = ci_lower, ci_upper = ci_upper,
      method      = method, staggered = FALSE, n_controls = N_co,
      alternative = alternative, boot_ests = NULL
    ), class = "coresynth_inference"))
  }

  # -- Staggered path ----------------------------------------------------------
  if (is.null(fit$Y_all))
    stop("fit does not contain Y_all. Re-estimate with the current version.",
         call. = FALSE)

  Y_all       <- fit$Y_all
  cohort_fits <- fit$cohort_fits
  w_g         <- vapply(cohort_fits, `[[`, numeric(1L), "weight")  # unnorm
  W           <- sum(w_g)

  .boot_one <- function() {
    att_total <- 0
    for (cf in cohort_fits) {
      N_co_g <- length(cf$idx_co)
      idx_b  <- cf$idx_co[sample(N_co_g, replace = TRUE)]
      cf_b   <- cf
      cf_b$idx_co <- idx_b
      att_g_b <- .refit_si_cohort(Y_all, cf_b)
      att_total <- att_total + cf$weight * att_g_b
    }
    att_total / W
  }

  if (method == "bootstrap") {
    boot_ests <- replicate(n_boot, .boot_one())
    n_fail <- sum(is.na(boot_ests))
    if (n_fail > 0.1 * n_boot)
      warning(sprintf("%.0f%% of staggered bootstrap replications failed.",
                      100 * n_fail / n_boot))
    valid    <- boot_ests[!is.na(boot_ests)]
    se       <- sd(valid)
    ci_lower <- unname(quantile(valid, alpha / 2))
    ci_upper <- unname(quantile(valid, 1 - alpha / 2))
    z        <- tau_hat / max(se, .Machine$double.eps)
    p_value  <- switch(alternative,
      two.sided = 2 * pnorm(-abs(z)),
      greater   = pnorm(-z),
      less      = pnorm(z)
    )
    n_co_mean <- round(mean(vapply(cohort_fits, function(cf) length(cf$idx_co), numeric(1L))))
    return(structure(list(
      estimate    = tau_hat, se = se, p_value = p_value,
      ci_lower    = ci_lower, ci_upper = ci_upper,
      method      = method, staggered = TRUE, n_controls = n_co_mean,
      alternative = alternative, boot_ests = boot_ests
    ), class = "coresynth_inference"))
  }

  if (method == "jackknife_global") {
    # Global jackknife: drop one unique control across ALL cohorts simultaneously
    # (Phase 18b). Captures cross-cohort correlation ignored by per-cohort LOO.
    all_co    <- sort(unique(unlist(lapply(cohort_fits, `[[`, "idx_co"))))
    orig_ests <- vapply(cohort_fits, `[[`, numeric(1L), "estimate")
    jack_ests <- vapply(all_co, function(i) {
      att_total <- 0
      for (k in seq_along(cohort_fits)) {
        cf <- cohort_fits[[k]]
        if (i %in% cf$idx_co) {
          keep <- cf$idx_co[cf$idx_co != i]
          if (length(keep) < 2L) return(NA_real_)
          cf_loo        <- cf
          cf_loo$idx_co <- keep
          att_g <- .refit_si_cohort(Y_all, cf_loo)
          if (is.na(att_g)) return(NA_real_)
        } else {
          att_g <- orig_ests[k]
        }
        att_total <- att_total + cf$weight * att_g
      }
      att_total / W
    }, numeric(1L))
    valid <- jack_ests[!is.na(jack_ests)]
    N_v   <- length(valid)
    se    <- sqrt((N_v - 1L) / N_v * sum((valid - mean(valid))^2))
    z        <- tau_hat / max(se, .Machine$double.eps)
    ci_lower <- tau_hat - qnorm(1 - alpha / 2) * se
    ci_upper <- tau_hat + qnorm(1 - alpha / 2) * se
    p_value  <- switch(alternative,
      two.sided = 2 * pnorm(-abs(z)),
      greater   = pnorm(-z),
      less      = pnorm(z)
    )
    return(structure(list(
      estimate    = tau_hat, se = se, p_value = p_value,
      ci_lower    = ci_lower, ci_upper = ci_upper,
      method      = method, staggered = TRUE, n_controls = length(all_co),
      alternative = alternative, boot_ests = NULL
    ), class = "coresynth_inference"))
  }

  # staggered jackknife: per-cohort LOO + delta-method
  var_components <- vapply(cohort_fits, function(cf) {
    N_co_g <- length(cf$idx_co)
    if (N_co_g < 2L) return(0)
    jack_g <- vapply(seq_len(N_co_g), function(i) {
      cf_loo <- cf
      cf_loo$idx_co <- cf$idx_co[-i]
      .refit_si_cohort(Y_all, cf_loo)
    }, numeric(1))
    valid_g <- jack_g[!is.na(jack_g)]
    if (length(valid_g) < 2L) return(0)
    (N_co_g - 1) / N_co_g * sum((valid_g - mean(valid_g))^2)
  }, numeric(1))

  var_st  <- sum((w_g / W)^2 * var_components)
  se      <- sqrt(var_st)
  z       <- tau_hat / max(se, .Machine$double.eps)
  ci_lower <- tau_hat - qnorm(1 - alpha / 2) * se
  ci_upper <- tau_hat + qnorm(1 - alpha / 2) * se
  p_value  <- switch(alternative,
    two.sided = 2 * pnorm(-abs(z)),
    greater   = pnorm(-z),
    less      = pnorm(z)
  )
  n_co_mean <- round(mean(vapply(cohort_fits, function(cf) length(cf$idx_co), numeric(1L))))
  structure(list(
    estimate    = tau_hat, se = se, p_value = p_value,
    ci_lower    = ci_lower, ci_upper = ci_upper,
    method      = method, staggered = TRUE, n_controls = n_co_mean,
    alternative = alternative, boot_ests = NULL
  ), class = "coresynth_inference")
}

#' @export
print.coresynth_inference <- function(x, digits = 4, ...) {
  cat(sprintf("Inference (method=%s, alternative=%s%s)\n",
              x$method, x$alternative,
              if (x$staggered) ", staggered" else ""))
  cat("  Estimate  :", round(x$estimate, digits), "\n")
  cat("  SE        :", round(x$se,       digits), "\n")
  cat("  p-value   :", round(x$p_value,  digits), "\n")
  cat("  CI        : [", round(x$ci_lower, digits), ", ",
      round(x$ci_upper, digits), "]\n", sep = "")
  cat("  Controls  :", x$n_controls, "\n")
  invisible(x)
}

# ── Conformal Inference (Chernozhukov, Wuthrich & Zhu 2021, JASA) ─────────────
# Permutation inference based on permuting blocks of estimated residuals.
# The counterfactual proxy is re-estimated under the null (post-treatment
# treated outcomes imputed as Y1_t - tau0) using ALL T periods, which is
# essential for the finite-sample validity of the procedure (CWZ 2021 §2.2).

# Internal: extract the donor matrix (T x N_co) and treated series (length T)
# from a sharp coresynth fit. Returns NULL if the fit is unsupported.
.conformal_extract <- function(fit) {
  method <- fit$method
  Yco <- if (!is.null(fit$Y_co_pre) && !is.null(fit$Y_co_post)) {
    rbind(fit$Y_co_pre, fit$Y_co_post)        # SCM / SDID
  } else if (!is.null(fit$Y_pre_co) && !is.null(fit$Y_post_co)) {
    rbind(fit$Y_pre_co, fit$Y_post_co)        # SI
  } else if (!is.null(fit$Y_co_all)) {
    fit$Y_co_all                              # GSC / MC
  } else {
    NULL
  }
  if (is.null(Yco)) return(NULL)

  y1 <- if (is.matrix(fit$Y_treat)) rowMeans(fit$Y_treat) else as.numeric(fit$Y_treat)
  list(Yco = Yco, y1 = y1)
}

# Internal: re-estimate the treated counterfactual P_hat (length T) under the
# null, using all T periods of the (null-imputed) treated series y1 and the
# observed donor matrix Yco. Dispatches on the estimator family.
.conformal_refit <- function(method, Yco, y1, fit) {
  TT   <- nrow(Yco)
  N_co <- ncol(Yco)

  if (method == "scm") {
    # Canonical SC over all T: simplex weights minimising ||y1 - Yco w||^2.
    V <- rep(1 / TT, TT)
    w <- drop(scm_inner_weights_cpp(Yco, y1, V))
    return(drop(Yco %*% w))
  }

  if (method == "si") {
    k <- fit$k %||% max(1L, floor(sqrt(min(TT, N_co))))
    k <- as.integer(min(k, min(TT, N_co)))
    res <- si_pcr_cpp(Yco, Yco, matrix(y1, ncol = 1L), k)
    return(drop(res$Y_hat))
  }

  if (method == "sdid") {
    # SDID synthetic series (omega-weighted donors + concentrated intercept),
    # unit weights estimated over all T under the null.
    zeta2 <- fit$zeta2 %||% (sqrt(TT) * var(diff(y1)))
    mu_co <- colMeans(Yco)
    mu_y1 <- mean(y1)
    Yco_c <- sweep(Yco, 2L, mu_co, "-")
    y1_c  <- y1 - mu_y1
    omega  <- drop(sdid_unit_weights_cpp(Yco_c, y1_c, zeta2))
    omega0 <- mu_y1 - sum(omega * mu_co)
    return(omega0 + drop(Yco %*% omega))
  }

  if (method == "gsc") {
    r <- fit$r %||% 2L
    r <- as.integer(min(r, min(TT, N_co)))
    empty_co <- array(0.0, dim = c(TT, N_co, 0L))
    empty_tr <- array(0.0, dim = c(TT, 1L, 0L))
    res <- gsc_ife_cpp(Yco, matrix(y1, ncol = 1L), r, empty_co, empty_tr)
    return(drop(res$Y_tr_hat))
  }

  if (method == "mc") {
    M <- cbind(Yco, y1)                       # T x (N_co + 1), treated last
    O <- matrix(1.0, nrow = TT, ncol = N_co + 1L)  # fully observed under null
    lambda <- fit$lambda %||% (0.01 * svd(M, nu = 0, nv = 0)$d[1])
    L <- soft_impute_cpp(M, O, lambda)
    return(drop(L[, N_co + 1L]))
  }

  stop(sprintf("conformal_inference(): method '%s' is not supported.", method),
       call. = FALSE)
}

# Internal: residual vector (length T) under the null H0: tau = tau0.
.conformal_residuals <- function(method, Yco, y1_obs, T_pre, tau0, fit) {
  TT   <- length(y1_obs)
  post <- (T_pre + 1L):TT
  y1_null <- y1_obs
  y1_null[post] <- y1_null[post] - tau0          # impute counterfactual outcome
  P_hat <- .conformal_refit(method, Yco, y1_null, fit)
  y1_null - P_hat
}

# Internal: moving-block (cyclic) permutation p-value for a residual vector.
.conformal_pval <- function(u, T_pre, q, alternative) {
  TT   <- length(u)
  post <- (T_pre + 1L):TT
  Sstat <- function(v) {
    up <- v[post]
    if (alternative == "two.sided") {
      mean(abs(up)^q)^(1 / q)
    } else {
      mean(up)                                   # signed: average post residual
    }
  }
  S0 <- Sstat(u)
  base <- seq_len(TT) - 1L
  S_perm <- vapply(0:(TT - 1L), function(j) {
    Sstat(u[((base + j) %% TT) + 1L])
  }, numeric(1L))
  switch(alternative,
    two.sided = mean(S_perm >= S0),
    greater   = mean(S_perm >= S0),
    less      = mean(S_perm <= S0)
  )
}

#' Conformal Inference for Synthetic Control Estimators
#'
#' Implements the permutation-based conformal inference procedure of
#' Chernozhukov, Wuthrich & Zhu (2021, JASA). The test inverts a sharp null
#' \eqn{H_0: \tau = \tau_0} by imputing the treated post-treatment
#' counterfactual as \eqn{Y_{1t} - \tau_0}, re-estimating the counterfactual
#' proxy on **all** \eqn{T} periods (imposing the null), and computing a
#' moving-block permutation p-value from the estimated residuals. A confidence
#' interval is obtained by test inversion over a grid of candidate \eqn{\tau_0}.
#'
#' Supported for **sharp** (single-cohort) fits with `method` in
#' `c("scm", "sdid", "gsc", "mc", "si")`. Staggered, multi-arm, and `tasc`
#' fits are not supported (use `sdid_inference()`, `gsc_inference()`, or
#' `si_inference()` instead).
#'
#' @param fit A `coresynth` object from [scm_fit()].
#' @param tau0 Null value of the ATT for the reported p-value (default 0).
#' @param q Exponent of the \eqn{S_q} test statistic
#'   (`S_q = (T_post^{-1} \sum |u_t|^q)^{1/q}`). Default 1, robust to
#'   heavy-tailed data (CWZ 2021). Used only for `alternative = "two.sided"`;
#'   one-sided tests use the signed mean post-treatment residual.
#' @param alternative `"two.sided"` (default), `"greater"`, or `"less"`.
#' @param ci Logical; construct a confidence interval by test inversion
#'   (default `TRUE`).
#' @param level Confidence level for the interval (default 0.95).
#' @param grid Optional numeric vector of candidate \eqn{\tau_0} values for test
#'   inversion. When `NULL` (default), a grid of `n_grid` points is centred on
#'   the point estimate with half-width `grid_mult` times the pre-treatment
#'   residual standard deviation.
#' @param n_grid Number of grid points when `grid = NULL` (default 200).
#' @param grid_mult Half-width multiplier when `grid = NULL` (default 4).
#' @param ... Unused.
#'
#' @return A list of class `c("conformal_inference", "coresynth_inference")`
#'   with `estimate`, `se` (`NA`; conformal has no SE), `p_value` (at `tau0`),
#'   `ci_lower`, `ci_upper`, `method` (`"conformal"`), `n_controls`,
#'   `alternative`, `staggered` (`FALSE`), plus `tau0`, `q`, `grid`, and
#'   `p_grid` (p-values along the grid). Compatible with [tidy()] / [glance()].
#'
#' @references Chernozhukov, V., Wuthrich, K., & Zhu, Y. (2021). An Exact and
#'   Robust Conformal Inference Method for Counterfactual and Synthetic
#'   Controls. *Journal of the American Statistical Association*, 116(536),
#'   1849-1864.
#'
#' @export
conformal_inference <- function(
  fit,
  tau0        = 0,
  q           = 1,
  alternative = c("two.sided", "greater", "less"),
  ci          = TRUE,
  level       = 0.95,
  grid        = NULL,
  n_grid      = 200L,
  grid_mult   = 4,
  ...
) {
  alternative <- match.arg(alternative)
  if (!inherits(fit, "coresynth"))
    stop("conformal_inference() requires a coresynth object.", call. = FALSE)
  if (isTRUE(fit$staggered) || isTRUE(fit$multi_arm))
    stop("conformal_inference() supports sharp (single-cohort) fits only.\n",
         "  For staggered/multi-arm fits use sdid_inference(), gsc_inference(), ",
         "or si_inference().", call. = FALSE)

  method <- fit$method
  if (!method %in% c("scm", "sdid", "gsc", "mc", "si"))
    stop(sprintf("conformal_inference() does not support method = '%s'.\n", method),
         "  Supported: 'scm', 'sdid', 'gsc', 'mc', 'si'.", call. = FALSE)

  ex <- .conformal_extract(fit)
  if (is.null(ex))
    stop("fit does not contain the donor matrix needed for conformal inference. ",
         "Re-estimate with the current version of coresynth.", call. = FALSE)

  Yco    <- ex$Yco
  y1_obs <- ex$y1
  T_pre  <- fit$T_pre
  TT     <- length(y1_obs)
  if (TT - T_pre < 1L)
    stop("conformal_inference() requires at least one post-treatment period.",
         call. = FALSE)

  p_at <- function(t0) {
    u <- .conformal_residuals(method, Yco, y1_obs, T_pre, t0, fit)
    .conformal_pval(u, T_pre, q, alternative)
  }

  p_value <- p_at(tau0)
  alpha   <- 1 - level

  ci_lower <- NA_real_
  ci_upper <- NA_real_
  grid_used <- NULL
  p_grid    <- NULL

  if (isTRUE(ci)) {
    if (is.null(grid)) {
      gap_pre <- (y1_obs - .conformal_refit(method, Yco, y1_obs, fit))[seq_len(T_pre)]
      s <- stats::sd(gap_pre)
      if (!is.finite(s) || s < 1e-8) s <- max(abs(fit$estimate), 1)
      half <- grid_mult * s
      grid_used <- seq(fit$estimate - half, fit$estimate + half, length.out = n_grid)
    } else {
      grid_used <- sort(as.numeric(grid))
    }
    p_grid <- vapply(grid_used, p_at, numeric(1L))
    in_ci  <- p_grid >= alpha
    if (any(in_ci)) {
      ci_lower <- min(grid_used[in_ci])
      ci_upper <- max(grid_used[in_ci])
    }
  }

  structure(list(
    estimate    = fit$estimate,
    se          = NA_real_,
    p_value     = p_value,
    ci_lower    = ci_lower,
    ci_upper    = ci_upper,
    method      = "conformal",
    n_controls  = ncol(Yco),
    alternative = alternative,
    staggered   = FALSE,
    boot_ests   = NULL,
    tau0        = tau0,
    q           = q,
    grid        = grid_used,
    p_grid      = p_grid,
    base_method = method
  ), class = c("conformal_inference", "coresynth_inference"))
}

#' @export
print.conformal_inference <- function(x, digits = 4, ...) {
  cat(sprintf("Conformal Inference (CWZ 2021, base = %s, alternative = %s)\n",
              x$base_method, x$alternative))
  cat("  Estimate (ATT):", round(x$estimate, digits), "\n")
  cat(sprintf("  p-value (H0: tau = %s): %s\n",
              format(x$tau0), round(x$p_value, digits)))
  if (!is.na(x$ci_lower))
    cat("  CI            : [", round(x$ci_lower, digits), ", ",
        round(x$ci_upper, digits), "]\n", sep = "")
  else if (!is.null(x$grid))
    cat("  CI            : empty at this level (widen 'grid' or raise level)\n")
  cat("  Controls      :", x$n_controls, "\n")
  invisible(x)
}

# Generated by using Rcpp::compileAttributes() -> do not edit by hand
# Generator token: 10BE3573-1514-4C36-9D1C-5A225CD40393

#' Fast Interactive Fixed Effects (IFE) for Generalized Synthetic Control
#'
#' Implements Xu (2017) IFE model with optional covariate adjustment.
#' When X_co has p > 0 slices, runs an EM loop alternating between:
#'   E-step: truncated SVD of Y_tilde = Y_co - X_co * beta
#'   M-step: panel OLS to update beta given current factors
#' When X_co has 0 slices (default), falls back to the plain 3-step estimator.
#'
#' @param Y_co     Control units outcome matrix (T x N_co)
#' @param Y_tr_pre Treated units pre-treatment outcomes (T_pre x N_tr)
#' @param r        Number of latent factors (must be <= min(T, N_co))
#' @param X_co     Time-varying covariate cube (T x N_co x p). Pass an empty
#'                 cube (0 slices) for the covariate-free estimator.
#' @param X_tr_pre Time-varying covariate cube for treated units in the
#'                 pre-treatment window (T_pre x N_tr x p). Required for
#'                 correct Step 2 loading estimation per Xu (2017): lambda_hat
#'                 is estimated from Y_tr_pre - X_tr_pre * beta (covariate-
#'                 demeaned). Pass an empty cube (0 slices) to skip demeaning
#'                 (backward-compatible, but biased when beta != 0).
#' @param max_iter Maximum EM iterations (default 50)
#' @param tol      Convergence tolerance on relative beta change (default 1e-6)
#' @return A list with components:
#'   * `F`: estimated time factors (T x r).
#'   * `L_co`: control-unit factor loadings (N_co x r).
#'   * `L_tr`: treated-unit factor loadings (N_tr x r).
#'   * `Y_tr_hat`: estimated treated-unit counterfactual outcomes (T x N_tr).
#'   * `singular_values`: singular values from the final truncated SVD.
#'   * `beta`: estimated covariate coefficients (p x 1), empty when no
#'     covariates are supplied.
#' @export
gsc_ife_cpp <- function(Y_co, Y_tr_pre, r, X_co, X_tr_pre, max_iter = 50L, tol = 1e-6) {
    .Call(`_coresynth_gsc_ife_cpp`, Y_co, Y_tr_pre, r, X_co, X_tr_pre, max_iter, tol)
}

#' Fast Placebo Test for SDID
#'
#' For each control unit, treats it as the "pseudo-treated" unit and
#' estimates the leave-one-out SDID effect. The distribution of these
#' placebo effects provides a permutation-based null distribution for inference.
#'
#' @param Y_pre     Control units pre-treatment outcomes (T_pre x N_co)
#' @param Y_post    Control units post-treatment outcomes (T_post x N_co)
#' @param time_weights Lambda weights for pre-treatment periods (T_pre x 1)
#' @param zeta2     Ridge penalty (same as used in the main estimate)
#' @return A numeric vector of length `N_co`. Each element is the
#'   leave-one-out placebo SDID effect obtained by treating that control unit
#'   as the pseudo-treated unit; the vector serves as a permutation-based null
#'   distribution for inference.
#' @export
sdid_placebo_cpp <- function(Y_pre, Y_post, time_weights, zeta2) {
    .Call(`_coresynth_sdid_placebo_cpp`, Y_pre, Y_post, time_weights, zeta2)
}

#' Fast Leave-One-Out Placebo Test for SCM (Abadie et al. 2010)
#'
#' For each control unit, treats it as pseudo-treated and fits SCM weights
#' from the remaining N_co-1 donors. Returns MSPE components for constructing
#' MSPE-ratio permutation p-values in R.
#'
#' @param Y_pre   Control pre-treatment outcomes (T_pre x N_co)
#' @param Y_post  Control post-treatment outcomes (T_post x N_co)
#' @param max_iter Outer coordinate-descent iterations (default 100)
#' @param tol      Convergence tolerance for V updates (default 1e-4)
#' @return A list with:
#'   * `mspe_pre`:  N_co-vector of pre-treatment MSPE per placebo unit
#'   * `mspe_post`: N_co-vector of post-treatment MSPE per placebo unit
#'   * `effects`:   N_co-vector of mean post-period gap per placebo unit
#' @export
scm_placebo_cpp <- function(Y_pre, Y_post, max_iter = 100L, tol = 1e-4) {
    .Call(`_coresynth_scm_placebo_cpp`, Y_pre, Y_post, max_iter, tol)
}

#' Fast Matrix Completion using Soft-Impute Algorithm
#'
#' Solves: min_L (1/2) ||O o (Y - L)||_F^2 + lambda * ||L||_*
#' via iterative SVD soft-thresholding (Mazumder, Hastie, Tibshirani 2010).
#' Note: lambda is NOT normalized by |O|. Default lambda = 0.01 * sigma_max(Y).
#'
#' @param Y       Observed outcome matrix (N x T). Unobserved entries should be 0.
#' @param O       Binary mask matrix (N x T): 1 = observed, 0 = missing (treated post).
#' @param lambda  Nuclear norm penalty (soft-threshold on singular values).
#' @param max_iter Maximum iterations.
#' @param tol     Convergence tolerance (relative Frobenius norm change).
#' @return A numeric matrix of the same dimension as `Y` (N x T): the
#'   completed low-rank matrix `L` that minimises the soft-thresholded
#'   nuclear-norm objective.
#' @export
soft_impute_cpp <- function(Y, O, lambda, max_iter = 1000L, tol = 1e-5) {
    .Call(`_coresynth_soft_impute_cpp`, Y, O, lambda, max_iter, tol)
}

proj_simplex <- function(y) {
    .Call(`_coresynth_proj_simplex`, y)
}

solve_simplex_qp <- function(Q, c, max_iter = 10000L, tol = 1e-6) {
    .Call(`_coresynth_solve_simplex_qp`, Q, c, max_iter, tol)
}

#' SCM Inner Weights (QP Given V)
#'
#' Solves the inner-loop QP for SCM: given a fixed diagonal metric matrix V,
#' finds donor weights W on the simplex minimising the V-weighted covariate loss.
#'
#' @param X0     Covariate matrix for control units (k x N_co)
#' @param X1     Covariate vector for the treated unit (k x 1)
#' @param V_diag Diagonal of the metric matrix V (k x 1, non-negative, need not sum to 1)
#' @return Donor weight vector W (N_co x 1) on the unit simplex
#' @export
scm_inner_weights_cpp <- function(X0, X1, V_diag) {
    .Call(`_coresynth_scm_inner_weights_cpp`, X0, X1, V_diag)
}

#' SCM Outer Weights (Joint Optimization of W and V)
#'
#' Jointly optimises donor weights W (on the simplex) and the diagonal
#' metric matrix V via coordinate descent on the pre-treatment prediction
#' MSPE, following Abadie, Diamond & Hainmueller (2010).
#'
#' When `t_train > 0`, uses out-of-sample V selection per Abadie (2021)
#' §3.2: V is selected by minimising MSPE on a validation window
#' (rows t_train..T_pre-1 of Z), while W is fitted on the training window
#' (rows 0..t_train-1 of X when X and Z have the same row count, i.e. the
#' outcomes-only case). After selecting V*, W is refit on the full data.
#'
#' @param X0      Covariate matrix for control units (k x N_co, typically pre-treatment outcomes)
#' @param X1      Covariate vector for the treated unit (k x 1)
#' @param Z0      Outcome matrix for control units in the pre-treatment window (T_pre x N_co)
#' @param Z1      Outcome vector for the treated unit in the pre-treatment window (T_pre x 1)
#' @param max_iter Maximum coordinate-descent iterations (default 100)
#' @param tol     Convergence tolerance on MSPE improvement (default 1e-4)
#' @param t_train Training window length for out-of-sample V selection.
#'   -1 (default): in-sample V selection (original behaviour).
#'   Positive: use rows 0..(t_train-1) of Z for fitting W, rows t_train..(T_pre-1)
#'   as the validation window for V selection, then refit W on full data.
#' @return A list with:
#'   * `W`: Donor weight vector (N_co x 1) on the unit simplex
#'   * `V`: Optimal metric diagonal (k x 1, normalised to sum to 1)
#'   * `loss`: Final pre-treatment prediction loss (full pre-treatment window)
#' @export
scm_weights_cpp <- function(X0, X1, Z0, Z1, max_iter = 100L, tol = 1e-4, t_train = -1L) {
    .Call(`_coresynth_scm_weights_cpp`, X0, X1, Z0, Z1, max_iter, tol, t_train)
}

#' Calculate SDID Unit Weights (omega)
#'
#' Solves the regularized QP:
#' min over omega in Delta: sum_t (sum_i omega_i Y_it - Y_tr_t)^2 + zeta^2 * T_pre * ||omega||^2
#'
#' This corresponds to equation (5) in Arkhangelsky et al. (2021).
#'
#' @param Y_pre Pre-treatment outcome matrix for control units (T_pre x N_co)
#' @param Y_tr_pre Pre-treatment outcome vector for treated unit (T_pre x 1), averaged if multiple
#' @param zeta2 Ridge penalty parameter (zeta^2). The code internally multiplies by T_pre per the paper.
#' @return A numeric vector of length `N_co` holding the SDID unit weights
#'   `omega` (non-negative and summing to one).
#' @export
sdid_unit_weights_cpp <- function(Y_pre, Y_tr_pre, zeta2) {
    .Call(`_coresynth_sdid_unit_weights_cpp`, Y_pre, Y_tr_pre, zeta2)
}

#' Calculate SDID Time Weights (lambda)
#'
#' Solves the time-weight QP (with implicit intercept lambda_0 concentrated out):
#'
#' min over lambda in Delta_pre: ||Y_post_target - Y_pre_co^T lambda||^2 + zeta_t^2 * N_co * ||lambda||^2
#'
#' The caller is responsible for pre-demeaning Y_pre_co (row-wise) and
#' Y_post_target (subtract the cross-unit mean) to concentrate out lambda_0,
#' as described in Arkhangelsky et al. (2021) Algorithm 1, Eq. (2.3).
#'
#' @param Y_pre_co  Pre-treatment outcomes for control units, row-demeaned (T_pre x N_co)
#' @param Y_post_target Post-treatment mean per control unit, demeaned (N_co x 1)
#' @param zeta_t    Ridge penalty for time weights (paper: 1e-6 * sigma_hat)
#' @return A numeric vector of length `T_pre` holding the SDID time weights
#'   `lambda` (non-negative and summing to one).
#' @export
sdid_time_weights_cpp <- function(Y_pre_co, Y_post_target, zeta_t) {
    .Call(`_coresynth_sdid_time_weights_cpp`, Y_pre_co, Y_post_target, zeta_t)
}

#' Calculate SDID Estimate (tau_sdid)
#'
#' Given unit weights omega and time weights lambda, computes the SDID
#' estimator as a weighted two-way difference:
#'
#' tau_sdid = (Y_tr_post_mean - Y_tr_pre_wt) - (Y_co_post_wt - Y_co_pre_wt)
#'
#' @param Y_pre_co   Control pre-treatment outcomes (T_pre x N_co)
#' @param Y_post_co  Control post-treatment outcomes (T_post x N_co)
#' @param Y_pre_tr   Treated pre-treatment outcomes (T_pre x 1)
#' @param Y_post_tr  Treated post-treatment outcomes (T_post x 1)
#' @param omega      Unit weights (N_co x 1)
#' @param lambda     Time weights (T_pre x 1)
#' @return A single numeric value: the SDID treatment-effect estimate
#'   `tau_sdid`.
#' @export
sdid_estimate_cpp <- function(Y_pre_co, Y_post_co, Y_pre_tr, Y_post_tr, omega, lambda) {
    .Call(`_coresynth_sdid_estimate_cpp`, Y_pre_co, Y_post_co, Y_pre_tr, Y_post_tr, omega, lambda)
}

#' Tensor Unfolding (Matricization) for Synthetic Interventions
#'
#' @param T_cube A 3D array (cube) of dimensions (n1, n2, n3)
#' @param mode The mode to unfold along (1, 2, or 3)
#' @return A numeric matrix: the mode-`mode` unfolding (matricization) of
#'   `T_cube`, with dimensions `n1 x (n2 * n3)`, `n2 x (n1 * n3)`, or
#'   `n3 x (n1 * n2)` for `mode` 1, 2, or 3 respectively.
#' @export
tensor_unfold_cpp <- function(T_cube, mode) {
    .Call(`_coresynth_tensor_unfold_cpp`, T_cube, mode)
}

#' SI-PCR: Synthetic Interventions via Principal Component Regression
#'
#' Implements the SI-PCR estimator of Agarwal et al. (2025).
#' Uses the top-k SVD of pre-treatment control outcomes to find donor
#' weights that predict each treated unit's pre-treatment trajectory,
#' then applies those weights to post-treatment control outcomes.
#'
#' @param Y_pre_co  Pre-treatment control outcomes (T_pre x N_co)
#' @param Y_post_co Post-treatment control outcomes (T_post x N_co)
#' @param Y_pre_tr  Pre-treatment treated outcomes (T_pre x N_tr)
#' @param k         Number of SVD components to retain
#' @return A list with:
#'   * `W`: Donor weight matrix (N_co x N_tr)
#'   * `Y_hat`: Counterfactual post-treatment outcomes (T_post x N_tr)
#' @export
si_pcr_cpp <- function(Y_pre_co, Y_post_co, Y_pre_tr, k) {
    .Call(`_coresynth_si_pcr_cpp`, Y_pre_co, Y_post_co, Y_pre_tr, k)
}

#' Kalman Filter and RTS Smoother (TASC)
#'
#' Implements the Kalman filter (forward pass) and Rauch-Tung-Striebel smoother
#' (backward pass) for the state-space model in Rho et al. (2026):
#'
#'   State:       z(t+1) = A z(t) + C + eta(t),  eta(t) ~ N(0, Q)
#'   Observation: y_t    = W z_t + eps_t,        eps_t  ~ N(0, R)
#'
#' Observation rows with NA (treated post-intervention) are automatically
#' dropped at each time step so only control-unit rows update the filter.
#'
#' The P update uses the numerically stable Joseph form:
#'   P(t|t) = (I - K W_obs) P(t|t-1) (I - K W_obs)^T + K R_obs K^T
#'
#' @param Y  Observed data matrix (N x T). Use NA for unobserved entries.
#' @param W  Observation / loading matrix (N x r)
#' @param A  State transition matrix (r x r). Pass diag(r) for random-walk dynamics.
#' @param C  State drift vector (r x 1)
#' @param Q  State noise covariance (r x r)
#' @param R  Observation noise covariance (N x N, diagonal in practice)
#' @param z0 Initial state mean (r x 1)
#' @param P0 Initial state covariance (r x r)
#' @return A list with z_smooth, P_smooth, P_cross, z_pred, z_upd.
#'   P_cross is an r x r x (T-1) cube. Slice t (C++ 0-indexed, t=0,...,T-2)
#'   stores P(t+1, t | T) (0-indexed), i.e. P(t+2, t+1 | T) in 1-indexed
#'   Shumway-Stoffer notation. Formula: P(t+1|T) * J_t^T (eq. 6.68-6.69).
#' @export
kalman_smoother_cpp <- function(Y, W, A, C, Q, R, z0, P0) {
    .Call(`_coresynth_kalman_smoother_cpp`, Y, W, A, C, Q, R, z0, P0)
}

#' Fit Time-Aware Synthetic Control (Rho et al. 2026)
#'
#' Uses a state-space model with Kalman Filter + RTS Smoother to estimate
#' counterfactual outcomes accounting for temporal autocorrelation.
#'
#' Model:
#'   z_{t+1} = A z_t + C + eta_t,  eta_t ~ N(0, Q)
#'   y_t     = W z_t + eps_t,       eps_t ~ N(0, R)
#'
#' Parameters (W, A, C, Q, R) are estimated via the Shumway-Stoffer exact EM
#' algorithm, initialised from PCA of control units.
#'
#' @param y      Outcome vector (long format)
#' @param d      Treatment indicator (long format)
#' @param id     Unit identifier (long format)
#' @param time   Time identifier (long format)
#' @param r      Number of latent state dimensions (default 2)
#' @param em_iter Number of EM iterations (default 20)
#' @param fix_A  Logical. If TRUE, keep A = I_r throughout EM (random-walk dynamics).
#'   Default FALSE learns A from data via OLS M-step.
#' @return A list of class `coresynth`.
#' @noRd
fit_tasc_cpp <- function(y, d, id, time, r = 2, em_iter = 20, fix_A = FALSE, ...) {
  pan <- panel_to_matrices(y, d, id, time)
  Y   <- pan$Y; T_pre <- pan$T_pre
  idx_tr <- pan$idx_treat; idx_co <- pan$idx_control

  TT <- nrow(Y); N <- ncol(Y)

  # Build observation matrix Y_obs: NA for each treated unit post its own adoption
  Y_obs <- Y
  for (j in idx_tr) {
    t0 <- pan$T_adopt[j]
    if (!is.na(t0) && t0 <= TT) Y_obs[t0:TT, j] <- NA_real_
  }

  # Initialise parameters from SVD of control-unit data (T_pre x N_co)
  # u: T_pre x r (time factors), v: N_co x r (unit loadings)
  Y_co_pre <- Y[seq_len(T_pre), idx_co, drop = FALSE]
  svd_res  <- svd(Y_co_pre, nu = r, nv = r)

  # W_full: N x r observation/loading matrix
  # Control rows: right singular vectors (unit loadings, N_co x r)
  W_full <- matrix(0.0, nrow = N, ncol = r)
  W_full[idx_co, ] <- svd_res$v[, seq_len(r), drop = FALSE]

  # Treated rows: least-squares loadings from pre-treatment fit
  # F_pre = U * diag(D): T_pre x r time-factor matrix
  if(T_pre >= r) {
    F_pre <- svd_res$u %*% diag(svd_res$d[seq_len(r)], r, r)   # T_pre x r
    L_tr  <- solve(t(F_pre) %*% F_pre + diag(1e-8, r),
                   t(F_pre) %*% Y[seq_len(T_pre), idx_tr, drop = FALSE])
    W_full[idx_tr, ] <- t(L_tr)
  }

  C  <- rep(0.0, r)
  A  <- diag(r)
  Q  <- diag(0.1, r)
  R  <- diag(var(as.vector(Y_co_pre)) * 0.5, N)
  z0 <- rep(0.0, r)
  P0 <- diag(1.0, r)

  # EM loop (E-step = Kalman smoother, M-step = Shumway-Stoffer OLS updates)
  for(em in seq_len(em_iter)) {
    ks <- kalman_smoother_cpp(t(Y_obs), W_full, A, C, Q, R, z0, P0)
    Z    <- ks$z_smooth   # r x T
    P_sm <- ks$P_smooth   # r x r x T (3D array)

    # M-step 1: W update (OLS, per-unit to handle missing observations)
    for (j in seq_len(N)) {
      t_obs <- which(!is.na(Y_obs[, j]))
      if (length(t_obs) == 0L) next
      Z_j <- Z[, t_obs, drop = FALSE]
      y_j <- Y_obs[t_obs, j]
      W_full[j, ] <- drop(t(y_j) %*% t(Z_j) %*%
                       solve(Z_j %*% t(Z_j) + diag(1e-8, r)))
    }

    # M-step 2: A update (Shumway-Stoffer closed-form OLS)
    if (!fix_A) {
      # P_cross: r x r x (T-1) cube
      # C++ slice t (0-indexed) = P_{t+2, t+1 | T} in 1-indexed notation
      P_cross <- ks$P_cross
      num_A <- matrix(0.0, r, r)
      den_A <- matrix(0.0, r, r)
      for (tt in seq_len(TT - 1)) {
        num_A <- num_A + P_cross[,, tt] + outer(Z[, tt + 1], Z[, tt])
        den_A <- den_A + P_sm[,, tt]    + outer(Z[, tt],     Z[, tt])
      }
      A <- num_A %*% solve(den_A + diag(1e-6, r))
    }

    # M-step 3: C update (after A — C = mean(z_{t+1} - A z_t))
    diffs_AC <- Z[, -1] - A %*% Z[, -TT]   # r x (T-1)
    C <- rowMeans(diffs_AC)

    # M-step 4: Q update (process noise covariance from corrected residuals)
    resid_proc <- diffs_AC - C
    Q <- resid_proc %*% t(resid_proc) / (TT - 1) + diag(1e-8, r)

    # M-step 5: R update (diagonal only, per-unit to handle missing observations)
    r_diag <- numeric(N)
    for (j in seq_len(N)) {
      t_obs <- which(!is.na(Y_obs[, j]))
      if (length(t_obs) == 0L) { r_diag[j] <- 1e-6; next }
      resid_j <- Y_obs[t_obs, j] -
                 drop(W_full[j, , drop = FALSE] %*% Z[, t_obs])
      r_diag[j] <- max(mean(resid_j^2), 1e-6)
    }
    R <- diag(r_diag)
  }

  # Final smooth
  ks_final <- kalman_smoother_cpp(t(Y_obs), W_full, A, C, Q, R, z0, P0)
  Z_smooth <- ks_final$z_smooth   # r x T

  Y_hat_all <- t(W_full %*% Z_smooth)   # T x N
  Y_treat   <- Y[, idx_tr, drop = FALSE]
  Y_hat_tr  <- Y_hat_all[, idx_tr, drop = FALSE]

  gap <- Y_treat - Y_hat_tr
  # ATT: average gap over each unit's own post-treatment period
  gap_post_vals <- vapply(seq_along(idx_tr), function(k) {
    j  <- idx_tr[k]
    t0 <- pan$T_adopt[j]
    if (is.na(t0) || t0 > TT) return(NA_real_)
    mean(gap[t0:TT, k, drop = FALSE], na.rm = TRUE)
  }, numeric(1L))
  att <- mean(gap_post_vals, na.rm = TRUE)

  list(
    method       = "tasc",
    r            = r,
    A            = A,
    W            = W_full,
    C            = C,
    Q            = Q,
    R            = R,
    Z_smooth     = Z_smooth,
    Y_hat        = Y_hat_all,
    Y_treat      = Y_treat,
    gap          = gap,
    times        = pan$times,
    T_pre        = T_pre,
    idx_co       = idx_co,
    idx_tr       = idx_tr,
    unit_weights = NULL,
    estimate     = att
  )
}

#' Plot a coresynth model
#'
#' @param x     A `coresynth` object.
#' @param type  One of `"trend"` (observed vs synthetic), `"gap"` (ATT over time),
#'              or `"weights"` (donor unit weight bar chart).
#' @param ...   Ignored.
#' @return A `ggplot2` plot object.
#' @import ggplot2
#' @export
plot.coresynth <- function(x, type = c("trend", "gap", "weights"), ...) {
  type <- match.arg(type)

  if(type %in% c("trend", "gap")) {
    if(is.null(x$times) || is.null(x$Y_treat))
      stop("fit object does not contain time series data for plotting.")

    times <- as.numeric(x$times)
    # Handle matrix Y_treat (multiple treated units) by averaging
    Y_treat <- if(is.matrix(x$Y_treat)) rowMeans(x$Y_treat, na.rm = TRUE) else x$Y_treat
    Y_synth <- if(!is.null(x$Y_synth)) {
      if(is.matrix(x$Y_synth)) rowMeans(x$Y_synth, na.rm = TRUE) else x$Y_synth
    } else {
      # For GSC/MC/TASC, compute from Y_hat
      Y_hat <- if(!is.null(x$Y_hat)) x$Y_hat else x$Y_tr_hat
      if(is.matrix(Y_hat)) rowMeans(Y_hat, na.rm = TRUE) else Y_hat
    }
    treat_time <- if(!is.null(x$T_pre)) times[x$T_pre + 1] else NA

    if(type == "trend") {
      df <- data.frame(
        time     = c(times, times),
        value    = c(Y_treat, Y_synth),
        series   = rep(c("Treated", "Synthetic Control"), each = length(times))
      )
      p <- ggplot(df, aes(x = time, y = value, color = series, linetype = series)) +
        geom_line(linewidth = 0.9) +
        scale_color_manual(values = c("Treated" = "#2166ac", "Synthetic Control" = "#d73027")) +
        scale_linetype_manual(values = c("Treated" = "solid", "Synthetic Control" = "dashed")) +
        {if(!is.na(treat_time)) geom_vline(xintercept = treat_time, linetype = "dotted", color = "gray40")} +
        theme_minimal(base_size = 13) +
        labs(title    = paste0("Synthetic Control Trend  [", toupper(x$method), "]"),
             x = "Time", y = "Outcome", color = NULL, linetype = NULL)
      return(p)
    }

    if(type == "gap") {
      gap <- Y_treat - Y_synth
      df  <- data.frame(time = times, gap = gap)
      p <- ggplot(df, aes(x = time, y = gap)) +
        geom_line(color = "#1a9641", linewidth = 0.9) +
        geom_hline(yintercept = 0, color = "gray50", linetype = "dashed") +
        {if(!is.na(treat_time)) geom_vline(xintercept = treat_time, linetype = "dotted", color = "gray40")} +
        theme_minimal(base_size = 13) +
        labs(title = paste0("Treatment Effect Gap  [", toupper(x$method), "]"),
             x = "Time", y = "Y_treated - Y_synthetic")
      return(p)
    }
  }

  if(type == "weights") {
    w <- x$unit_weights
    if(is.null(w) || all(is.na(w)))
      stop("No unit weights available for this method (GSC/MC/TASC use factor loadings).")

    df <- data.frame(
      unit   = names(w) %||% paste0("Unit_", seq_along(w)),
      weight = as.numeric(w)
    )
    df <- df[df$weight > 1e-4, ]
    if(nrow(df) == 0) stop("All unit weights are negligibly small.")

    p <- ggplot(df, aes(x = reorder(unit, weight), y = weight)) +
      geom_col(fill = "#4575b4", alpha = 0.85) +
      coord_flip() +
      theme_minimal(base_size = 13) +
      labs(title = "Donor Unit Weights", x = NULL, y = "Weight")
    return(p)
  }
}

#' Fit a Synthetic Control Method Model
#'
#' Unified formula interface for Synthetic Control and related causal
#' inference methods.  The formula syntax is:
#'
#'   `outcome ~ treatment | unit_id + time_id`
#'
#' @param formula A `Formula` object, e.g. `y ~ D | unit + time`.
#' @param data    A `data.frame` in **long** format (one row per unit-time).
#' @param method  One of `"scm"`, `"sdid"`, `"gsc"`, `"mc"`, `"tasc"`, `"si"`.
#' @param predictors A `list()` of [pred()] specifications that define the
#'   predictor matrix for SCM (see Abadie et al. 2010, S.2.3). Each [pred()]
#'   entry aggregates one or more variables over a time window. Pass `NULL`
#'   (default) to use all pre-treatment outcome periods as predictors.
#'   Applies to `method = "scm"` only.
#' @param covariates An optional named `list` of additional time-varying
#'   covariates to partial out before estimation. Each element is a character
#'   string naming a column in `data`. Supported for `method = "sdid"`,
#'   `"scm"`, and `"gsc"`.
#' @param v_selection V matrix selection method for `method = "scm"`.
#'   `"insample"` (default) follows Abadie et al. (2010): V is chosen by
#'   minimising in-sample pre-treatment MSPE. `"oos"` follows Abadie (2021)
#'   S.3.2: the pre-treatment window is split in half; V is selected to minimise
#'   MSPE on the validation half, then W is refit on the full window.
#' @param donor_mspe_threshold Donor pool filtering threshold (Abadie 2021 S.4).
#'   For `method = "scm"` only. Each donor's individual pre-treatment MSPE
#'   (using that donor alone as the counterfactual) is divided by the minimum
#'   such MSPE across all donors. Donors whose ratio exceeds this threshold are
#'   excluded from estimation. `Inf` (default) disables filtering.
#' @param lambda_pen Penalised SCM parameter (Abadie & L'Hour 2021, JASA).
#'   For `method = "scm"` only. `NULL` (default) runs standard unpenalised SCM.
#'   `"auto"` selects the penalty via out-of-sample pre-treatment MSPE on the
#'   same validation window as `v_selection = "oos"`. A non-negative number
#'   uses that value directly.
#' @param v_optim Outer V-optimisation method for `method = "scm"`.
#'   `"coord_descent"` (default) uses the existing C++ coordinate descent with
#'   11-point grid search -- fastest when `k = T_pre` is large (outcomes-only).
#'   `"bfgs"` uses R's L-BFGS-B, which requires only O(k^2) inner QP calls and
#'   is faster when `k` is small (e.g. external predictors with k <= 15).
#'   `"auto"` selects `"bfgs"` when `k <= 15`, otherwise `"coord_descent"`.
#' @param ...     Additional arguments forwarded to the specific method
#'                (e.g. `r`, `lambda`, `zeta2`).
#'
#' @return An object of classes `c("coresynth_<method>", "coresynth")`.
#'   All methods return at minimum:
#'   * `method`: estimator name
#'   * `estimate`: average treatment effect (ATT)
#'   * `times`: time index vector
#'   * `T_pre`: number of pre-treatment periods
#'   * `Y_treat`: treated unit outcome series
#'   * `gap`: treatment effect series (Y_treat - counterfactual)
#'
#' @export
#'
#' @examples
#' # Synthetic balanced panel: 10 units over 20 periods, unit 1 treated
#' # after period 15.
#' set.seed(1)
#' panel <- expand.grid(unit = 1:10, year = 1:20)
#' panel$treated <- as.integer(panel$unit == 1 & panel$year > 15)
#' panel$gdp <- panel$unit + 0.5 * panel$year +
#'   rnorm(nrow(panel)) + 3 * panel$treated
#'
#' fit <- scm_fit(gdp ~ treated | unit + year, data = panel, method = "sdid")
#' summary(fit)
#'
#' \donttest{
#' # Visualise the estimated gap (requires ggplot2)
#' plot(fit, type = "gap")
#' }
scm_fit <- function(
  formula,
  data,
  method = c("scm", "sdid", "gsc", "mc", "tasc", "si"),
  predictors = NULL,
  covariates = NULL,
  v_selection = c("insample", "oos"),
  donor_mspe_threshold = Inf,
  lambda_pen = NULL,
  v_optim = c("coord_descent", "auto", "bfgs"),
  ...
) {
  v_selection <- match.arg(v_selection)
  v_optim     <- match.arg(v_optim)
  method <- match.arg(method)

  # Parse Formula
  f_parts <- Formula::Formula(formula)

  if (length(f_parts)[2] < 2) {
    stop(
      "Formula must specify unit and time after '|', e.g. y ~ D | unit + time"
    )
  }

  y_var <- all.vars(formula(f_parts, lhs = 1, rhs = 0))
  d_var <- all.vars(formula(f_parts, lhs = 0, rhs = 1))
  idx_vars <- all.vars(formula(f_parts, lhs = 0, rhs = 2))

  if (length(y_var) != 1) {
    stop("Exactly one outcome variable required.")
  }
  if (length(d_var) != 1) {
    stop("Exactly one treatment variable required.")
  }
  if (length(idx_vars) != 2) {
    stop("Must specify exactly two index variables: unit_id and time_id.")
  }

  id_var <- idx_vars[1]
  time_var <- idx_vars[2]

  # Validate columns exist
  for (v in c(y_var, d_var, id_var, time_var)) {
    if (!v %in% names(data)) {
      stop(paste0("Variable '", v, "' not found in data."))
    }
  }

  y <- as.numeric(data[[y_var]])
  d <- as.integer(data[[d_var]])
  id <- data[[id_var]]
  time <- data[[time_var]]

  # Dispatch
  res <- switch(
    method,
    "scm" = fit_scm_cpp(
      y,
      d,
      id,
      time,
      data = data,
      id_var = id_var,
      time_var = time_var,
      predictors = predictors,
      covariates = covariates,
      v_selection = v_selection,
      donor_mspe_threshold = donor_mspe_threshold,
      lambda_pen = lambda_pen,
      v_optim = v_optim,
      ...
    ),
    "sdid" = fit_sdid_cpp(
      y, d, id, time,
      covariates = covariates,
      data       = data,
      id_var     = id_var,
      time_var   = time_var,
      ...
    ),
    "gsc" = fit_gsc_cpp(
      y,
      d,
      id,
      time,
      data = data,
      id_var = id_var,
      time_var = time_var,
      covariates = covariates,
      ...
    ),
    "mc" = fit_mc_cpp(y, d, id, time, ...),
    "tasc" = fit_tasc_cpp(y, d, id, time, ...),
    "si" = fit_si_cpp(y, d, id, time, ...),
    stop(paste0("Unknown method: '", method, "'"))
  )

  class(res) <- c(paste0("coresynth_", method), "coresynth")
  res
}

#' @export
print.coresynth <- function(x, ...) {
  cat("=== coresynth fit ===\n")
  cat("Method :", toupper(x$method), "\n")
  if (isTRUE(x$multi_arm)) {
    stag_label <- if (isTRUE(x$staggered)) " (staggered)" else ""
    cat(sprintf("Multi-arm SI%s (K = %d arms)\n", stag_label, length(x$arm_levels)))
    cat("Per-arm ATT:",
        paste(names(x$arm_estimates), round(x$arm_estimates, 4),
              sep = "=", collapse = "  "), "\n")
  }
  cat("Estimate (ATT):", round(x$estimate, 4), "\n")
  cat("Pre-treatment periods:", x$T_pre, "\n")
  invisible(x)
}

#' @export
summary.coresynth <- function(object, ...) {
  cat("=== coresynth summary ===\n")
  cat("Method :", toupper(object$method), "\n")
  if (isTRUE(object$multi_arm)) {
    stag_label <- if (isTRUE(object$staggered)) " (staggered)" else ""
    cat(sprintf("Multi-arm SI%s (K = %d treatment arms)\n",
                stag_label, length(object$arm_levels)))
    cat("Per-arm ATT:\n")
    print(round(object$arm_estimates, 6))
  }
  cat(
    "Periods : T_pre =",
    object$T_pre,
    "| T_post =",
    length(object$times) - object$T_pre,
    "\n"
  )
  cat("ATT estimate:", round(object$estimate, 6), "\n")
  if (!is.null(object$unit_weights)) {
    cat("Unit weights (non-zero donors):\n")
    w <- object$unit_weights
    print(round(w[w > 1e-4], 4))
  }
  if (!is.null(object$predictor_table)) {
    cat("\nPredictor balance:\n")
    pt <- object$predictor_table
    pt$treated <- round(pt$treated, 4)
    pt$synthetic <- round(pt$synthetic, 4)
    print(pt, row.names = FALSE)
  }
  invisible(object)
}

# Helper: build k x J predictor matrix for all J units using pred() specs
# Returns list(X_mat = k x J, pred_names = character(k))
build_design_X <- function(data, unit_var, time_var, units, predictors) {
  J    <- length(units)
  rows <- list()
  nms  <- character(0L)

  for (ps in predictors) {
    if (!inherits(ps, "pred_spec")) {
      stop("Each element of 'predictors' must be a pred() object.", call. = FALSE)
    }
    for (var in ps$vars) {
      if (!var %in% names(data)) {
        stop(paste0("Variable '", var, "' not found in data."), call. = FALSE)
      }
      row_vals <- vapply(units, function(u) {
        vals <- data[[var]][data[[unit_var]] == u & data[[time_var]] %in% ps$times]
        switch(ps$op,
          mean   = mean(vals, na.rm = TRUE),
          median = median(vals, na.rm = TRUE),
          sum    = sum(vals, na.rm = TRUE),
          mean(vals, na.rm = TRUE)
        )
      }, numeric(1L))
      rows[[length(rows) + 1L]] <- row_vals
      t_lbl <- if (length(ps$times) == 1L) as.character(ps$times[[1L]])
               else paste0(ps$times[[1L]], ":", ps$times[[length(ps$times)]])
      nms <- c(nms, paste0(var, "[", t_lbl, "]"))
    }
  }

  X_mat <- do.call(rbind, rows)
  colnames(X_mat) <- as.character(units)
  rownames(X_mat) <- nms
  list(X_mat = X_mat, pred_names = nms)
}

#' Experimental Synthetic Control Design
#'
#' Selects which units to assign to the treatment arm (and which to the control
#' arm) in a planned experiment, following Abadie and Zhao (2026).  Both sets
#' of units are chosen by minimising the distance between their weighted-average
#' predictor vectors and the population-average predictor vector \eqn{\bar{X}},
#' so the resulting estimates are less susceptible to post-randomisation bias
#' than pure random assignment.
#'
#' Three design formulations are available:
#'
#' * **`"base"`** (eq. 7): both the synthetic treated and the synthetic control
#'   independently target the population average \eqn{\bar{X}}.
#' * **`"weakly_targeted"`** (eq. 9): the synthetic treated targets
#'   \eqn{\bar{X}}; the synthetic control targets the synthetic treated predictor
#'   vector (controlled by `beta`).
#' * **`"unit_level"`** (eq. 10): each treated unit gets its own synthetic
#'   control; the aggregate control weight is a convex combination (controlled
#'   by `xi`).
#'
#' Inference uses "blank periods" — pre-experimental periods whose outcomes were
#' *not* used to estimate the weights.  Set `T_fit` strictly smaller than the
#' number of pre-experimental periods to enable the permutation test and split-
#' conformal confidence intervals from Section 3 of Abadie and Zhao (2026).
#'
#' @param data        Long-format data frame (one row per unit–time).
#' @param outcome     Name of the outcome column.
#' @param unit        Name of the unit identifier column.
#' @param time        Name of the time identifier column.
#' @param T0          Last pre-experimental period (a value present in the time
#'   column).  Periods after `T0` are the experimental periods.
#' @param T_fit       Number of fitting periods, counted from the **start** of
#'   the pre-experimental phase.  Defaults to `NULL`, which uses all
#'   pre-experimental periods for fitting (no blank periods; inference
#'   disabled).  When `T_fit` is smaller than the total number of
#'   pre-experimental periods, the remaining periods become blank periods used
#'   for inference.
#' @param m_min       Minimum number of units assigned to treatment (default 1).
#' @param m_max       Maximum number of units assigned to treatment (default 1).
#' @param f           Named numeric vector of population weights \eqn{f_j}.
#'   Defaults to uniform weights \eqn{1/J}.  Will be normalised to sum to 1.
#' @param predictors  A `list()` of [pred()] specifications that define the
#'   predictor matrix \eqn{X_j}.  Defaults to `NULL`, which uses all
#'   fitting-period outcome values as predictors.
#' @param design      Design formulation: `"base"` (default), `"weakly_targeted"`,
#'   or `"unit_level"`.
#' @param beta        Trade-off parameter \eqn{\beta > 0} for the Weakly
#'   targeted design (default 1).
#' @param xi          Trade-off parameter \eqn{\xi > 0} for the Unit-level
#'   design (default 1).
#' @param alpha       Significance level for confidence intervals (default 0.05).
#' @param normalize   If `TRUE` (default), each row of the predictor matrix is
#'   divided by its cross-unit standard deviation before optimisation, so
#'   predictors measured on different scales contribute equally.
#' @param max_subsets Maximum number of treatment-set candidates to evaluate
#'   before switching to random sampling (default 100 000).
#'
#' @return An object of class `"scm_design"` with components:
#'   * `treated_units`: unit identifiers selected for treatment
#'   * `control_units`: unit identifiers in the control pool
#'   * `w`: J-length weight vector for the synthetic treated unit (sums to 1)
#'   * `v`: J-length weight vector for the synthetic control unit (sums to 1)
#'   * `tau_hat`: estimated treatment effects for each experimental period
#'   * `p_value`: permutation p-value (NA when blank periods are unavailable)
#'   * `ci_lower`, `ci_upper`: per-period split-conformal confidence interval
#'   * `Y_synth_tr`, `Y_synth_co`: synthetic treated/control series (all periods)
#'   * `estimate`: ATT (mean of `tau_hat`)
#'
#' @references Abadie, A. and Zhao, J. (2026). "Synthetic Controls for
#'   Experimental Design." MIT Working Paper.
#'
#' @export
scm_design <- function(
  data,
  outcome,
  unit,
  time,
  T0,
  T_fit       = NULL,
  m_min       = 1L,
  m_max       = 1L,
  f           = NULL,
  predictors  = NULL,
  design      = c("base", "weakly_targeted", "unit_level"),
  beta        = 1,
  xi          = 1,
  alpha       = 0.05,
  normalize   = TRUE,
  max_subsets = 100000L
) {
  design <- match.arg(design)
  m_min  <- as.integer(m_min)
  m_max  <- as.integer(m_max)

  for (col in c(outcome, unit, time)) {
    if (!col %in% names(data)) {
      stop(paste0("Column '", col, "' not found in data."), call. = FALSE)
    }
  }

  units <- sort(unique(data[[unit]]))
  times <- sort(unique(data[[time]]))
  J     <- length(units)
  TT    <- length(times)

  if (!T0 %in% times) {
    stop("'T0' must be a value present in the time column.", call. = FALSE)
  }
  T0_idx <- which(times == T0)

  if (m_min < 1L || m_max < m_min || m_max >= J) {
    stop("Need 1 <= m_min <= m_max < J (must leave at least one control unit).",
         call. = FALSE)
  }

  # Fitting / blank / experimental row indices
  TE <- if (is.null(T_fit)) T0_idx else as.integer(T_fit)
  if (TE < 1L || TE > T0_idx) {
    stop("'T_fit' must be between 1 and the number of pre-experimental periods.",
         call. = FALSE)
  }
  E_idx    <- seq_len(TE)
  B_idx    <- setdiff(seq_len(T0_idx), E_idx)
  post_idx <- seq.int(T0_idx + 1L, TT)
  if (length(post_idx) == 0L) {
    stop("No experimental periods found after T0.", call. = FALSE)
  }

  # Outcome matrix (TT x J) — vectorised index fill
  Y <- matrix(NA_real_, nrow = TT, ncol = J,
              dimnames = list(as.character(times), as.character(units)))
  ri_all <- match(data[[time]], times)
  ci_all <- match(data[[unit]], units)
  Y[cbind(ri_all, ci_all)] <- data[[outcome]]

  # Population weights f
  f_vec <- if (is.null(f)) {
    setNames(rep(1 / J, J), as.character(units))
  } else {
    fv <- if (!is.null(names(f))) {
      f[as.character(units)]
    } else {
      if (length(f) != J) stop("Unnamed 'f' must have length J.", call. = FALSE)
      setNames(f, as.character(units))
    }
    if (any(is.na(fv))) stop("Some units in 'data' have no entry in 'f'.", call. = FALSE)
    fv / sum(fv)
  }

  # Predictor matrix X_mat (k x J)
  if (!is.null(predictors)) {
    if (!is.list(predictors)) {
      stop("'predictors' must be a list of pred() objects.", call. = FALSE)
    }
    xi_info    <- build_design_X(data, unit, time, units, predictors)
    X_mat      <- xi_info$X_mat
    pred_names <- xi_info$pred_names
  } else {
    X_mat      <- Y[E_idx, , drop = FALSE]   # TE x J
    pred_names <- paste0("Y[", times[E_idx], "]")
  }

  # Row-wise normalisation (divide by cross-unit SD)
  if (normalize) {
    row_sd <- apply(X_mat, 1L, sd, na.rm = TRUE)
    row_sd[row_sd < 1e-10] <- 1
    X_mat <- X_mat / row_sd
  }

  # Population predictor average (k-vector) and uniform V weights
  X_bar  <- drop(X_mat %*% f_vec)
  k      <- nrow(X_mat)
  V_unif <- rep(1, k)

  # ---- Enumerate treatment sets and optimise ----
  best_obj         <- Inf
  best_treated_idx <- NULL
  best_w_full      <- NULL
  best_v_full      <- NULL

  for (m in m_min:m_max) {
    n_total <- choose(J, m)
    if (n_total <= max_subsets) {
      subsets <- combn(J, m, simplify = FALSE)
    } else {
      message(sprintf(
        "C(%d,%d) = %g subsets > max_subsets = %d; using random sampling.",
        J, m, n_total, max_subsets
      ))
      subsets <- replicate(max_subsets, sort(sample.int(J, m)), simplify = FALSE)
    }

    for (S in subsets) {
      S_co <- setdiff(seq_len(J), S)
      if (length(S_co) == 0L) next

      X_S  <- X_mat[, S,    drop = FALSE]   # k x m
      X_co <- X_mat[, S_co, drop = FALSE]   # k x (J-m)

      # Treated weights w: min ||X_S w - X_bar||^2
      w_sub <- tryCatch(
        scm_inner_weights_cpp(X_S, X_bar, V_unif),
        error = function(e) NULL
      )
      if (is.null(w_sub)) next
      X_Sw  <- drop(X_S %*% w_sub)
      obj_w <- sum((X_bar - X_Sw)^2)

      # Control weights v and total objective
      v_sub <- NULL
      obj   <- NA_real_

      if (design == "base") {
        v_sub <- tryCatch(
          scm_inner_weights_cpp(X_co, X_bar, V_unif),
          error = function(e) NULL
        )
        if (!is.null(v_sub)) {
          obj <- obj_w + sum((X_bar - drop(X_co %*% v_sub))^2)
        }

      } else if (design == "weakly_targeted") {
        v_sub <- tryCatch(
          scm_inner_weights_cpp(X_co, X_Sw, V_unif),
          error = function(e) NULL
        )
        if (!is.null(v_sub)) {
          obj <- obj_w + beta * sum((X_Sw - drop(X_co %*% v_sub))^2)
        }

      } else {  # unit_level
        v_mat    <- matrix(0, length(S_co), length(S))
        obj_unit <- 0
        ok       <- TRUE
        for (jj in seq_along(S)) {
          X_j <- X_mat[, S[jj]]
          v_j <- tryCatch(
            scm_inner_weights_cpp(X_co, X_j, V_unif),
            error = function(e) NULL
          )
          if (is.null(v_j)) { ok <- FALSE; break }
          v_mat[, jj] <- v_j
          obj_unit <- obj_unit + w_sub[jj] * sum((X_j - drop(X_co %*% v_j))^2)
        }
        if (ok) {
          v_sub <- drop(v_mat %*% w_sub)   # aggregate per eq. (11)
          obj   <- obj_w + xi * obj_unit
        }
      }

      if (is.null(v_sub) || is.na(obj)) next

      if (obj < best_obj) {
        best_obj         <- obj
        best_treated_idx <- S
        w_full           <- numeric(J); w_full[S]    <- w_sub
        v_full           <- numeric(J); v_full[S_co] <- v_sub
        best_w_full      <- setNames(w_full, as.character(units))
        best_v_full      <- setNames(v_full, as.character(units))
      }
    }
  }

  if (is.null(best_treated_idx)) {
    stop("Optimisation failed: no feasible treatment set found.", call. = FALSE)
  }

  # Synthetic series for all periods
  Y_synth_tr <- drop(Y %*% best_w_full)
  Y_synth_co <- drop(Y %*% best_v_full)

  tau_hat <- (Y_synth_tr - Y_synth_co)[post_idx]
  u_blank  <- (Y_synth_tr - Y_synth_co)[B_idx]

  T_post_n  <- length(post_idx)
  T_blank_n <- length(B_idx)

  # ---- Inference (Section 3 of Abadie & Zhao 2026) ----
  S_obs <- mean(abs(tau_hat))

  if (T_blank_n == 0L) {
    # Warn only when T_fit was explicitly provided but leaves no blank periods;
    # when T_fit = NULL (default) the user is not attempting inference.
    if (!is.null(T_fit)) {
      warning(
        "No blank periods: p_value and CI are NA. ",
        "Set T_fit < number of pre-experimental periods to enable inference.",
        call. = FALSE
      )
    }
    p_value  <- NA_real_
    ci_lower <- rep(NA_real_, T_post_n)
    ci_upper <- rep(NA_real_, T_post_n)
  } else {
    # Permutation test: eq. (16)-(17)
    all_vals  <- c(u_blank, tau_hat)
    n_periods <- T_blank_n + T_post_n
    n_combs   <- choose(n_periods, T_post_n)

    if (n_combs <= 10000L) {
      combs   <- combn(n_periods, T_post_n, simplify = FALSE)
      S_perms <- vapply(
        combs, function(idx) mean(abs(all_vals[idx])), numeric(1L)
      )
    } else {
      set.seed(42L)
      S_perms <- replicate(
        10000L,
        mean(abs(all_vals[sample.int(n_periods, T_post_n)]))
      )
    }
    p_value <- mean(S_perms >= S_obs)

    # Split-conformal CI: eq. (18)-(19)
    q_alpha  <- as.numeric(quantile(abs(u_blank), 1 - alpha))
    ci_lower <- tau_hat - q_alpha
    ci_upper <- tau_hat + q_alpha
  }

  structure(
    list(
      method         = "scm_design",
      design         = design,
      treated_units  = units[best_treated_idx],
      control_units  = units[setdiff(seq_len(J), best_treated_idx)],
      w              = best_w_full,
      v              = best_v_full,
      objective      = best_obj,
      tau_hat        = tau_hat,
      u_blank        = u_blank,
      p_value        = p_value,
      ci_lower       = ci_lower,
      ci_upper       = ci_upper,
      Y_synth_tr     = Y_synth_tr,
      Y_synth_co     = Y_synth_co,
      times_post     = times[post_idx],
      times_blank    = times[B_idx],
      times_fit      = times[E_idx],
      times          = times,
      T0             = T0,
      T_fit          = TE,
      J              = J,
      m_min          = m_min,
      m_max          = m_max,
      f              = f_vec,
      units          = units,
      alpha          = alpha,
      pred_names     = pred_names,
      estimate       = mean(tau_hat),
      T_pre          = T0_idx,
      Y              = Y
    ),
    class = "scm_design"
  )
}

#' @export
print.scm_design <- function(x, ...) {
  cat("=== scm_design (Abadie & Zhao 2026) ===\n")
  cat("Design variant :", x$design, "\n")
  cat("Treated units  :", paste(x$treated_units, collapse = ", "), "\n")
  cat("ATT estimate   :", round(x$estimate, 4L), "\n")
  if (!is.na(x$p_value)) {
    cat(sprintf("p-value        : %.4f (alpha = %.2f)\n", x$p_value, x$alpha))
  } else {
    cat("p-value        : NA (no blank periods)\n")
  }
  invisible(x)
}

#' @export
summary.scm_design <- function(object, ...) {
  cat("=== scm_design summary (Abadie & Zhao 2026) ===\n")
  cat("Design variant      :", object$design, "\n")
  cat("Number of units : J =", object$J, "\n")
  cat("Fitting periods : TE =", object$T_fit, "\n")
  cat("Blank periods   : TB =", length(object$times_blank), "\n")
  cat("Exp. periods    : T_post =", length(object$times_post), "\n\n")
  cat("Treated units  :", paste(object$treated_units, collapse = ", "), "\n")
  w_nz <- object$w[object$w > 1e-4]
  if (length(w_nz) > 0L) {
    cat("Treated weights (non-zero):\n")
    print(round(w_nz, 4L))
  }
  cat("\nControl weights (non-zero):\n")
  v_nz <- object$v[object$v > 1e-4]
  if (length(v_nz) > 0L) print(round(v_nz, 4L))
  cat("\nATT estimate :", round(object$estimate, 6L), "\n")
  if (!is.na(object$p_value)) {
    cat("p-value      :", round(object$p_value, 4L), "\n\n")
    ci_df <- data.frame(
      time     = object$times_post,
      tau_hat  = round(object$tau_hat, 4L),
      ci_lower = round(object$ci_lower, 4L),
      ci_upper = round(object$ci_upper, 4L)
    )
    cat(sprintf(
      "Confidence intervals (1 - alpha = %.0f%%):\n",
      100 * (1 - object$alpha)
    ))
    print(ci_df, row.names = FALSE)
  }
  invisible(object)
}

#' Plot an scm_design object
#'
#' @param x    An `scm_design` object.
#' @param type `"outcome"` (default): synthetic treated vs synthetic control
#'   outcome series over all periods. `"gap"`: estimated treatment effect in the
#'   experimental periods, with split-conformal confidence intervals.
#' @param ...  Currently ignored.
#' @return A `ggplot` object: for `type = "outcome"`, the synthetic treated and
#'   synthetic control outcome series; for `type = "gap"`, the estimated
#'   treatment effect over the experimental periods with split-conformal
#'   confidence intervals. The object is returned for printing or further
#'   customisation.
#' @export
plot.scm_design <- function(x, type = c("outcome", "gap"), ...) {
  type <- match.arg(type)

  if (type == "outcome") {
    df <- data.frame(
      time    = x$times,
      Treated = x$Y_synth_tr,
      Control = x$Y_synth_co
    )
    df_long <- data.frame(
      time  = rep(x$times, 2L),
      value = c(x$Y_synth_tr, x$Y_synth_co),
      group = rep(
        c("Synthetic Treated", "Synthetic Control"),
        each = length(x$times)
      )
    )
    ggplot2::ggplot(df_long, ggplot2::aes(x = time, y = value,
                                          linetype = group)) +
      ggplot2::geom_line() +
      ggplot2::geom_vline(xintercept = x$T0, linetype = "dotted",
                          colour = "grey50") +
      ggplot2::scale_linetype_manual(
        values = c("Synthetic Treated" = "solid",
                   "Synthetic Control" = "dashed")
      ) +
      ggplot2::labs(
        title    = "Synthetic Control Design: Outcome Series",
        x        = "Time", y = "Outcome", linetype = NULL
      ) +
      ggplot2::theme_bw() +
      ggplot2::theme(legend.position = "bottom")
  } else {
    df <- data.frame(
      time     = x$times_post,
      tau_hat  = x$tau_hat,
      ci_lower = x$ci_lower,
      ci_upper = x$ci_upper
    )
    p <- ggplot2::ggplot(df, ggplot2::aes(x = time, y = tau_hat)) +
      ggplot2::geom_line() +
      ggplot2::geom_point() +
      ggplot2::geom_hline(yintercept = 0, linetype = "dashed", colour = "grey50")
    if (!any(is.na(df$ci_lower))) {
      p <- p + ggplot2::geom_ribbon(
        ggplot2::aes(ymin = ci_lower, ymax = ci_upper),
        alpha = 0.2, fill = "steelblue"
      )
    }
    p + ggplot2::labs(
      title = "Treatment Effect Estimate",
      x = "Time", y = "Estimated Effect"
    ) + ggplot2::theme_bw()
  }
}

#' Fit Synthetic Difference-in-Differences (Arkhangelsky et al. 2021)
#'
#' Implements Algorithm 1 from Arkhangelsky et al. (2021) exactly:
#'
#' 1. **Unit weights** omega (with implicit intercept omega_0 concentrated out):
#'    zeta = (N_tr * T_post)^{1/4} * sigma_hat, where sigma_hat^2 = Var of
#'    first differences of control outcomes in pre-period (Eq. 2.2).
#'    Column-demeaning Y_co_pre/Y_tr_pre eliminates omega_0 from the QP.
#'
#' 2. **Time weights** lambda (with implicit intercept lambda_0 concentrated out):
#'    Row-demeaning Y_co_pre and demeaning Ybar_co_post eliminates lambda_0.
#'    Tiny zeta_t = 1e-6 * sigma_hat for uniqueness (Eq. 2.3, footnote 3).
#'
#' 3. **SDID estimate** (closed-form DiD; omega_0/lambda_0 cancel algebraically):
#'    tau = (Ybar_tr_post - lambda'Y_tr_pre) - (omega'Ybar_co_post - omega'Y_co_pre'lambda)
#'
#' For staggered adoption, applies the cohort-based approach from Arkhangelsky
#' et al. (2021) Appendix Section 8: SDID is run separately for each adoption
#' cohort and the resulting estimates are averaged with weights proportional to
#' N_treated * T_post for that cohort.
#'
#' @param y     Outcome vector (long format)
#' @param d     Treatment indicator (long format)
#' @param id    Unit identifier (long format)
#' @param time  Time identifier (long format)
#' @param zeta2 Override zeta^2 (penalty BEFORE T_pre multiplication). Default: auto.
#' @param zeta_t Base tiny ridge scale for time weight uniqueness (default 1e-6).
#' @param covariates Character vector of time-varying covariate column names in
#'   `data`. When provided, covariates are partialled out of Y via OLS on the
#'   control-unit pre-period block before computing SDID weights and the ATT
#'   (Arkhangelsky et al. 2021 §4; Clarke et al. 2023). Requires `data`,
#'   `id_var`, and `time_var` to be supplied (done automatically when called
#'   via [scm_fit()]). For staggered adoption, a global partial-out using all
#'   Wit=0 observations is applied before cohort-level SDID (Clarke et al. 2023
#'   §2.2 "projected" method, Kranz 2022).
#' @param data        Full long-format data frame (passed by [scm_fit()]).
#' @param id_var      Name of the unit identifier column (passed by [scm_fit()]).
#' @param time_var    Name of the time identifier column (passed by [scm_fit()]).
#' @param control_group For staggered adoption: which units serve as controls
#'   for each cohort. `"clean"` (default) uses never-treated units plus units
#'   whose adoption date is strictly later than cohort g (Clarke et al. 2023).
#'   `"never_treated"` uses only units that are never treated.
#' @return A list of class `coresynth`.
#' @noRd

# ── Internal: covariate partial-out (sharp) ──────────────────────────────────
# OLS of Y on X using control units x pre-period block.
# Returns Y_tilde = Y - X * beta_hat (all units, all periods).
.sdid_partial_out <- function(Y, X_arr, idx_co, T_pre) {
  p        <- dim(X_arr)[3]
  pre_rows <- seq_len(T_pre)
  N_co     <- length(idx_co)

  X_mat <- matrix(NA_real_, T_pre * N_co, p)
  y_vec <- as.vector(Y[pre_rows, idx_co, drop = FALSE])
  for (j in seq_len(p))
    X_mat[, j] <- as.vector(X_arr[pre_rows, idx_co, j])

  beta_hat <- drop(solve(crossprod(X_mat) + diag(1e-8, p),
                         crossprod(X_mat, y_vec)))

  Y_tilde <- Y
  for (j in seq_len(p))
    Y_tilde <- Y_tilde - X_arr[, , j] * beta_hat[j]

  list(Y_tilde = Y_tilde, beta_hat = beta_hat)
}

# ── Internal: covariate partial-out (staggered) ───────────────────────────────
# OLS of Y on X using ALL Wit=0 observations:
#   - control units x all T periods
#   - treated unit j x periods 1..(T_adopt[j]-1)
# This is Clarke et al. (2023) §2.2 "projected" approach (Kranz 2022).
# Returns Y_tilde = Y - X * beta_hat (all units, all periods).
.sdid_partial_out_staggered <- function(Y, X_arr, pan) {
  p       <- dim(X_arr)[3]
  TT      <- nrow(Y)
  idx_co  <- pan$idx_control
  idx_tr  <- pan$idx_treat
  T_adopt <- pan$T_adopt

  # Collect (row, col) pairs for Wit = 0
  co_pairs <- do.call(rbind, lapply(idx_co, function(j)
    data.frame(row = seq_len(TT), col = j, stringsAsFactors = FALSE)))

  tr_pairs <- do.call(rbind, lapply(idx_tr, function(j) {
    t0 <- T_adopt[j]
    if (is.na(t0) || t0 <= 1L) return(NULL)
    data.frame(row = seq_len(t0 - 1L), col = j, stringsAsFactors = FALSE)
  }))

  cells <- rbind(co_pairs, tr_pairs)
  K     <- nrow(cells)

  idx_mat <- cbind(cells$row, cells$col)
  y_vec   <- Y[idx_mat]

  X_mat <- matrix(NA_real_, K, p)
  for (k in seq_len(p))
    X_mat[, k] <- X_arr[cbind(cells$row, cells$col, rep(k, K))]

  # Drop rows with any NA (missing covariate values)
  ok <- complete.cases(X_mat)
  if (sum(ok) < p + 1L)
    stop("Too few complete cases for covariate partial-out in staggered SDID.",
         call. = FALSE)
  X_mat <- X_mat[ok, , drop = FALSE]
  y_vec <- y_vec[ok]

  beta_hat <- drop(solve(crossprod(X_mat) + diag(1e-8, p),
                         crossprod(X_mat, y_vec)))

  Y_tilde <- Y
  for (k in seq_len(p))
    Y_tilde <- Y_tilde - X_arr[, , k] * beta_hat[k]

  list(Y_tilde = Y_tilde, beta_hat = beta_hat)
}

# ── Internal: SDID from pre-built matrices ────────────────────────────────────
# Core estimation logic, callable from both sharp and staggered paths.
# Colnames of Y_co_pre are used to name the returned unit_weights vector.
.fit_sdid_matrices <- function(Y_co_pre, Y_co_post,
                               Y_tr_pre_mean, Y_tr_post_mean,
                               N_tr = 1L, zeta2 = NULL, zeta_t = 1e-6) {
  T_pre  <- nrow(Y_co_pre)
  T_post <- nrow(Y_co_post)

  if (T_pre < 2L)
    stop("at least 2 pre-treatment periods required to estimate sigma_hat.",
         call. = FALSE)

  # sigma_hat^2 (Eq. 2.2)
  delta      <- diff(Y_co_pre)
  sigma2_hat <- mean((delta - mean(delta))^2)
  sigma_hat  <- sqrt(max(sigma2_hat, .Machine$double.eps))

  if (is.null(zeta2))
    zeta2 <- sqrt(N_tr * T_post) * sigma2_hat

  # Unit weights: column-demeaning concentrates out omega_0
  mu_co      <- colMeans(Y_co_pre)
  mu_tr_pre  <- mean(Y_tr_pre_mean)
  Y_co_pre_c <- sweep(Y_co_pre, 2L, mu_co, "-")
  Y_tr_c     <- Y_tr_pre_mean - mu_tr_pre

  omega  <- sdid_unit_weights_cpp(Y_co_pre_c, Y_tr_c, zeta2)
  omega0 <- mu_tr_pre - sum(omega * mu_co)

  # Time weights: row-demeaning concentrates out lambda_0
  Ybar_co_post <- colMeans(Y_co_post)
  mu_t         <- rowMeans(Y_co_pre)
  mu_post      <- mean(Ybar_co_post)
  Y_co_pre_rc  <- sweep(Y_co_pre, 1L, mu_t, "-")
  b_tilde      <- Ybar_co_post - mu_post
  zeta_t_eff   <- zeta_t * sigma_hat

  lambda  <- sdid_time_weights_cpp(Y_co_pre_rc, b_tilde, zeta_t_eff)
  lambda0 <- mu_post - sum(lambda * mu_t)

  # SDID estimate
  tau_hat <- sdid_estimate_cpp(
    Y_co_pre, Y_co_post,
    Y_tr_pre_mean, Y_tr_post_mean,
    omega, lambda
  )

  names(omega) <- colnames(Y_co_pre)

  list(
    estimate     = tau_hat,
    unit_weights = omega,
    omega0       = omega0,
    time_weights = drop(lambda),
    lambda0      = lambda0,
    zeta2        = zeta2,
    sigma2_hat   = sigma2_hat
  )
}

# ── Internal: re-estimate one SDID cohort with given control indices ──────────
# Used by sdid_inference() staggered bootstrap/jackknife (Phase 18a).
# zeta2 = NULL: recompute from resampled data (bootstrap).
# zeta2 = cf$zeta2: fixed from original fit (jackknife).
.refit_sdid_cohort <- function(Y_mat, cf, zeta2 = NULL) {
  TT        <- nrow(Y_mat)
  pre_rows  <- seq_len(cf$T_pre)
  post_rows <- (cf$T_pre + 1L):TT
  res <- tryCatch(
    .fit_sdid_matrices(
      Y_co_pre       = Y_mat[pre_rows,  cf$idx_co, drop = FALSE],
      Y_co_post      = Y_mat[post_rows, cf$idx_co, drop = FALSE],
      Y_tr_pre_mean  = rowMeans(Y_mat[pre_rows,  cf$idx_tr, drop = FALSE]),
      Y_tr_post_mean = rowMeans(Y_mat[post_rows, cf$idx_tr, drop = FALSE]),
      N_tr  = length(cf$idx_tr),
      zeta2 = zeta2
    ),
    error = function(e) NULL
  )
  if (is.null(res)) NA_real_ else res$estimate
}

# ── Internal: cohort-by-cohort SDID for staggered adoption ───────────────────
# Arkhangelsky et al. (2021) Appendix §8; Clarke et al. (2023).
# Y: optional pre-residualised outcome matrix (Phase 14a). If NULL, uses pan$Y.
.fit_sdid_staggered <- function(pan, Y = NULL, zeta2 = NULL, zeta_t = 1e-6,
                                control_group = "clean") {
  Y       <- if (!is.null(Y)) Y else pan$Y
  T_adopt <- pan$T_adopt
  idx_tr  <- pan$idx_treat
  idx_co  <- pan$idx_control
  TT      <- nrow(Y)

  cohorts <- sort(unique(T_adopt[idx_tr]))

  cohort_list <- lapply(cohorts, function(g) {
    idx_tr_g <- idx_tr[T_adopt[idx_tr] == g]
    T_pre_g  <- g - 1L
    T_post_g <- TT - T_pre_g

    if (control_group == "never_treated") {
      idx_co_g <- idx_co
    } else {
      future_tr <- idx_tr[!is.na(T_adopt[idx_tr]) & T_adopt[idx_tr] > g]
      idx_co_g  <- c(idx_co, future_tr)
    }

    if (length(idx_co_g) < 2L || T_pre_g < 2L) {
      warning(sprintf(
        "SDID staggered: cohort g=%d skipped (T_pre=%d, N_co=%d).",
        g, T_pre_g, length(idx_co_g)
      ), call. = FALSE)
      return(NULL)
    }

    pre_rows  <- seq_len(T_pre_g)
    post_rows <- (T_pre_g + 1L):TT

    fit <- tryCatch(
      .fit_sdid_matrices(
        Y_co_pre       = Y[pre_rows,  idx_co_g, drop = FALSE],
        Y_co_post      = Y[post_rows, idx_co_g, drop = FALSE],
        Y_tr_pre_mean  = rowMeans(Y[pre_rows,  idx_tr_g, drop = FALSE]),
        Y_tr_post_mean = rowMeans(Y[post_rows, idx_tr_g, drop = FALSE]),
        N_tr   = length(idx_tr_g),
        zeta2  = zeta2,
        zeta_t = zeta_t
      ),
      error = function(e) {
        warning(sprintf("SDID staggered: cohort g=%d failed: %s",
                        g, conditionMessage(e)), call. = FALSE)
        NULL
      }
    )
    if (is.null(fit)) return(NULL)

    fit$cohort    <- g
    fit$n_treated <- length(idx_tr_g)
    fit$T_pre     <- T_pre_g
    fit$T_post    <- T_post_g
    fit$weight    <- as.numeric(length(idx_tr_g)) * T_post_g
    fit$idx_co    <- idx_co_g   # column indices in Y_mat (for inference resampling)
    fit$idx_tr    <- idx_tr_g   # column indices in Y_mat
    fit
  })

  valid <- !vapply(cohort_list, is.null, logical(1L))
  if (!any(valid)) stop("All cohort-level SDID fits failed.", call. = FALSE)
  r <- cohort_list[valid]

  w   <- vapply(r, `[[`, numeric(1L), "weight")
  tau <- vapply(r, `[[`, numeric(1L), "estimate")
  att <- sum(w * tau) / sum(w)

  cohort_df <- data.frame(
    cohort    = vapply(r, `[[`, integer(1L), "cohort"),
    n_treated = vapply(r, `[[`, integer(1L), "n_treated"),
    T_pre     = vapply(r, `[[`, integer(1L), "T_pre"),
    T_post    = vapply(r, `[[`, integer(1L), "T_post"),
    estimate  = tau,
    weight    = w / sum(w),
    stringsAsFactors = FALSE
  )

  list(estimate = att, cohort_estimates = cohort_df, cohort_fits = r)
}

# ── Public wrapper ─────────────────────────────────────────────────────────────
fit_sdid_cpp <- function(y, d, id, time,
                         zeta2 = NULL, zeta_t = 1e-6,
                         covariates = NULL,
                         data = NULL, id_var = NULL, time_var = NULL,
                         control_group = c("clean", "never_treated"),
                         ...) {
  control_group <- match.arg(control_group)
  pan    <- panel_to_matrices(y, d, id, time)
  Y      <- pan$Y
  T_pre  <- pan$T_pre
  idx_tr <- pan$idx_treat
  idx_co <- pan$idx_control

  use_cov <- !is.null(covariates) && length(covariates) > 0L && !is.null(data)

  # ── Staggered path (Arkhangelsky 2021 Appendix §8) ───────────────────────
  if (!pan$is_sharp) {
    Y_work   <- pan$Y
    beta_hat <- numeric(0)
    if (use_cov) {
      X_arr    <- build_covariate_array(data, id_var, time_var,
                                       covariates, pan$units, pan$times)
      po       <- .sdid_partial_out_staggered(pan$Y, X_arr, pan)
      Y_work   <- po$Y_tilde
      beta_hat <- po$beta_hat
    }
    stag <- .fit_sdid_staggered(pan,
                                Y             = Y_work,
                                zeta2         = zeta2,
                                zeta_t        = zeta_t,
                                control_group = control_group)
    return(list(
      method           = "sdid",
      staggered        = TRUE,
      control_group    = control_group,
      estimate         = stag$estimate,
      cohort_estimates = stag$cohort_estimates,
      cohort_fits      = stag$cohort_fits,
      Y_treat          = pan$Y[, pan$idx_treat, drop = FALSE],
      Y_mat            = Y_work,
      Y_synth          = NULL,
      gap              = NULL,
      unit_weights     = NULL,
      time_weights     = NULL,
      times            = pan$times,
      T_pre            = pan$T_pre,
      beta_hat         = beta_hat,
      covariates       = covariates
    ))
  }

  # ── Sharp path ───────────────────────────────────────────────────────────
  if (length(idx_co) < 2)
    stop("SDID requires at least two control units.", call. = FALSE)

  TT     <- nrow(Y)
  T_post <- TT - T_pre
  N_tr   <- length(idx_tr)

  if (T_post < 1)
    stop("No post-treatment periods found.", call. = FALSE)

  # Partial-out covariates (Arkhangelsky 2021 §4; Clarke et al. 2023)
  if (use_cov) {
    X_arr    <- build_covariate_array(data, id_var, time_var,
                                     covariates, pan$units, pan$times)
    po       <- .sdid_partial_out(Y, X_arr, idx_co, T_pre)
    Y_work   <- po$Y_tilde
    beta_hat <- po$beta_hat
  } else {
    Y_work   <- Y
    beta_hat <- numeric(0)
  }

  pre_rows  <- seq_len(T_pre)
  post_rows <- (T_pre + 1L):TT

  mfit <- .fit_sdid_matrices(
    Y_co_pre       = Y_work[pre_rows,  idx_co, drop = FALSE],
    Y_co_post      = Y_work[post_rows, idx_co, drop = FALSE],
    Y_tr_pre_mean  = rowMeans(Y_work[pre_rows,  idx_tr, drop = FALSE]),
    Y_tr_post_mean = rowMeans(Y_work[post_rows, idx_tr, drop = FALSE]),
    N_tr   = N_tr,
    zeta2  = zeta2,
    zeta_t = zeta_t
  )

  omega   <- mfit$unit_weights
  Y_synth <- drop(Y_work[, idx_co, drop = FALSE] %*% omega)
  Y_treat <- rowMeans(Y_work[, idx_tr, drop = FALSE])

  list(
    method         = "sdid",
    staggered      = FALSE,
    unit_weights   = omega,
    omega0         = mfit$omega0,
    time_weights   = mfit$time_weights,
    lambda0        = mfit$lambda0,
    estimate       = mfit$estimate,
    zeta2          = mfit$zeta2,
    sigma2_hat     = mfit$sigma2_hat,
    Y_synth        = Y_synth,
    Y_treat        = Y_treat,
    gap            = Y_treat - Y_synth,
    times          = pan$times,
    T_pre          = T_pre,
    covariates     = covariates,
    beta_hat       = beta_hat,
    Y_co_pre       = Y_work[pre_rows,  idx_co, drop = FALSE],
    Y_co_post      = Y_work[post_rows, idx_co, drop = FALSE],
    Y_tr_pre_mean  = rowMeans(Y_work[pre_rows,  idx_tr, drop = FALSE]),
    Y_tr_post_mean = rowMeans(Y_work[post_rows, idx_tr, drop = FALSE]),
    N_tr           = N_tr
  )
}

# ── SDID Inference (Clarke et al. 2023, Algorithms 2-4) ────────────────────────

#' Inference for Synthetic Difference-in-Differences
#'
#' Computes standard errors and p-values for a SDID estimate using one of
#' three methods: permutation placebo test (Algorithm 4), cluster bootstrap
#' (Algorithm 2), or leave-one-out jackknife (Algorithm 3), following
#' Clarke et al. (2023).
#'
#' @param fit A `coresynth` object with `method = "sdid"` (sharp adoption only).
#' @param method Inference method: `"placebo"` (permutation), `"bootstrap"`, or
#'   `"jackknife"`.
#' @param n_boot Number of bootstrap replications (only for `method = "bootstrap"`).
#' @param level Confidence level for the interval (only for `method = "bootstrap"`
#'   or `"jackknife"`).
#' @param alternative Direction of the alternative hypothesis: `"two.sided"`,
#'   `"greater"`, or `"less"`.
#' @param seed Integer seed for reproducibility (only for `method = "bootstrap"`).
#'
#' @return A list with:
#' * `estimate`: The SDID point estimate.
#' * `se`: Standard error (bootstrap / jackknife only).
#' * `p_value`: Permutation or normal-approximation p-value.
#' * `ci_lower`, `ci_upper`: Confidence interval bounds (bootstrap / jackknife).
#' * `method`: The inference method used.
#' * `n_controls`: Number of control units.
#' * `alternative`: The alternative hypothesis direction.
#' * `placebo_effects`: Named vector of LOO placebo effects (placebo only).
#' * `boot_ests`: Bootstrap estimate distribution (bootstrap only).
#'
#' @export
sdid_inference <- function(
  fit,
  method      = c("placebo", "bootstrap", "jackknife", "jackknife_global"),
  n_boot      = 200L,
  level       = 0.95,
  alternative = c("two.sided", "greater", "less"),
  seed        = NULL
) {
  method      <- match.arg(method)
  alternative <- match.arg(alternative)

  # ── Input validation ────────────────────────────────────────────────────────
  if (!inherits(fit, "coresynth") || !identical(fit$method, "sdid"))
    stop("sdid_inference() requires a coresynth fit with method = 'sdid'.",
         call. = FALSE)
  tau_hat   <- fit$estimate
  alpha     <- 1 - level
  staggered <- isTRUE(fit$staggered)

  # ── Staggered path (Phase 18a) ───────────────────────────────────────────────
  if (staggered) {
    if (is.null(fit$Y_mat))
      stop("fit$Y_mat not found. Re-estimate with the current version of coresynth.",
           call. = FALSE)

    cohort_fits <- fit$cohort_fits
    W    <- sum(vapply(cohort_fits, `[[`, numeric(1L), "weight"))
    w_g  <- vapply(cohort_fits, `[[`, numeric(1L), "weight")
    Y_mat <- fit$Y_mat

    if (!is.null(seed)) set.seed(seed)

    if (method == "placebo") {
      # never-treated controls: intersection of all cohort control pools.
      # future-treated units appear in early cohorts only; the intersection
      # yields the set that is untreated throughout all cohorts.
      never_co <- Reduce(intersect, lapply(cohort_fits, `[[`, "idx_co"))
      if (length(never_co) < 2L)
        stop("sdid_inference() staggered placebo: fewer than 2 never-treated ",
             "controls available. Use method='bootstrap' or 'jackknife'.",
             call. = FALSE)

      TT <- nrow(Y_mat)

      # Call sdid_placebo_cpp once per cohort (K calls total).
      # lambda (time_weights) and zeta2 are fixed from the original cohort fit.
      plac_by_cohort <- lapply(cohort_fits, function(cf) {
        pre_rows  <- seq_len(cf$T_pre)
        post_rows <- (cf$T_pre + 1L):TT
        sdid_placebo_cpp(
          Y_mat[pre_rows,  cf$idx_co, drop = FALSE],
          Y_mat[post_rows, cf$idx_co, drop = FALSE],
          cf$time_weights,
          cf$zeta2
        )
      })

      # For each never-treated unit j, aggregate per-cohort placebos with
      # the same cohort weights used for the staggered ATT.
      placebo_effects <- vapply(never_co, function(j) {
        att_total <- 0
        for (k in seq_along(cohort_fits)) {
          cf    <- cohort_fits[[k]]
          pos_j <- which(cf$idx_co == j)   # length 1: j always in idx_co_k
          att_total <- att_total + cf$weight * plac_by_cohort[[k]][pos_j]
        }
        att_total / W
      }, numeric(1L))

      nm <- colnames(Y_mat)
      names(placebo_effects) <-
        if (!is.null(nm)) nm[never_co] else as.character(never_co)

      p_value <- switch(alternative,
        two.sided = (1 + sum(abs(placebo_effects) >= abs(tau_hat))) / (length(never_co) + 1),
        greater   = (1 + sum(placebo_effects >= tau_hat))           / (length(never_co) + 1),
        less      = (1 + sum(placebo_effects <= tau_hat))           / (length(never_co) + 1)
      )

      return(structure(list(
        estimate        = tau_hat,
        se              = NULL,
        p_value         = p_value,
        ci_lower        = NULL,
        ci_upper        = NULL,
        method          = method,
        n_controls      = length(never_co),
        alternative     = alternative,
        placebo_effects = placebo_effects,
        boot_ests       = NULL,
        staggered       = TRUE
      ), class = c("sdid_inference", "coresynth_inference")))
    }

    if (method == "bootstrap") {
      boot_ests <- replicate(n_boot, {
        att_total <- 0
        for (cf in cohort_fits) {
          N_co_g <- length(cf$idx_co)
          idx_b  <- cf$idx_co[sample(N_co_g, replace = TRUE)]
          cf_b   <- cf; cf_b$idx_co <- idx_b
          att_g  <- tryCatch(.refit_sdid_cohort(Y_mat, cf_b),
                             error = function(e) NA_real_)
          att_total <- att_total + cf$weight * att_g
        }
        att_total / W
      })
      valid    <- boot_ests[!is.na(boot_ests)]
      se       <- sd(valid)
      ci_lower <- unname(quantile(valid, alpha / 2))
      ci_upper <- unname(quantile(valid, 1 - alpha / 2))
      z        <- tau_hat / max(se, .Machine$double.eps)
      p_value  <- switch(alternative,
        two.sided = 2 * pnorm(-abs(z)),
        greater   = pnorm(-z),
        less      = pnorm(z)
      )
      return(structure(list(
        estimate        = tau_hat,
        se              = se,
        p_value         = p_value,
        ci_lower        = ci_lower,
        ci_upper        = ci_upper,
        method          = method,
        n_controls      = mean(vapply(cohort_fits, function(cf) length(cf$idx_co), numeric(1L))),
        alternative     = alternative,
        placebo_effects = NULL,
        boot_ests       = boot_ests,
        staggered       = TRUE
      ), class = c("sdid_inference", "coresynth_inference")))
    }

    if (method == "jackknife_global") {
      # Global jackknife: drop one unique control across ALL cohorts simultaneously
      # (Phase 18b). Captures cross-cohort correlation ignored by per-cohort LOO.
      all_co    <- sort(unique(unlist(lapply(cohort_fits, `[[`, "idx_co"))))
      orig_ests <- vapply(cohort_fits, `[[`, numeric(1L), "estimate")
      jack_ests <- vapply(all_co, function(i) {
        att_total <- 0
        for (k in seq_along(cohort_fits)) {
          cf   <- cohort_fits[[k]]
          if (i %in% cf$idx_co) {
            keep <- cf$idx_co[cf$idx_co != i]
            if (length(keep) < 2L) return(NA_real_)
            cf_loo        <- cf
            cf_loo$idx_co <- keep
            att_g <- .refit_sdid_cohort(Y_mat, cf_loo, zeta2 = cf$zeta2)
            if (is.na(att_g)) return(NA_real_)
          } else {
            att_g <- orig_ests[k]
          }
          att_total <- att_total + cf$weight * att_g
        }
        att_total / W
      }, numeric(1L))
      valid <- jack_ests[!is.na(jack_ests)]
      N_v   <- length(valid)
      se    <- sqrt((N_v - 1L) / N_v * sum((valid - mean(valid))^2))
      z        <- tau_hat / max(se, .Machine$double.eps)
      ci_lower <- tau_hat - qnorm(1 - alpha / 2) * se
      ci_upper <- tau_hat + qnorm(1 - alpha / 2) * se
      p_value  <- switch(alternative,
        two.sided = 2 * pnorm(-abs(z)),
        greater   = pnorm(-z),
        less      = pnorm(z)
      )
      return(structure(list(
        estimate        = tau_hat,
        se              = se,
        p_value         = p_value,
        ci_lower        = ci_lower,
        ci_upper        = ci_upper,
        method          = method,
        n_controls      = length(all_co),
        alternative     = alternative,
        placebo_effects = NULL,
        boot_ests       = NULL,
        staggered       = TRUE
      ), class = c("sdid_inference", "coresynth_inference")))
    }

    # jackknife (per-cohort LOO + delta-method; Clarke et al. 2023 Alg 3)
    var_components <- vapply(cohort_fits, function(cf) {
      N_co_g <- length(cf$idx_co)
      if (N_co_g < 2L) return(0)
      jack_g <- vapply(seq_len(N_co_g), function(i) {
        cf_loo        <- cf
        cf_loo$idx_co <- cf$idx_co[-i]
        .refit_sdid_cohort(Y_mat, cf_loo, zeta2 = cf$zeta2)
      }, numeric(1L))
      valid_g <- jack_g[!is.na(jack_g)]
      if (length(valid_g) < 2L) return(0)
      (N_co_g - 1L) / N_co_g * sum((valid_g - mean(valid_g))^2)
    }, numeric(1L))
    var_st   <- sum((w_g / W)^2 * var_components)
    se       <- sqrt(var_st)
    z        <- tau_hat / max(se, .Machine$double.eps)
    ci_lower <- tau_hat - qnorm(1 - alpha / 2) * se
    ci_upper <- tau_hat + qnorm(1 - alpha / 2) * se
    p_value  <- switch(alternative,
      two.sided = 2 * pnorm(-abs(z)),
      greater   = pnorm(-z),
      less      = pnorm(z)
    )
    return(structure(list(
      estimate        = tau_hat,
      se              = se,
      p_value         = p_value,
      ci_lower        = ci_lower,
      ci_upper        = ci_upper,
      method          = method,
      n_controls      = mean(vapply(cohort_fits, function(cf) length(cf$idx_co), numeric(1L))),
      alternative     = alternative,
      placebo_effects = NULL,
      boot_ests       = NULL,
      staggered       = TRUE
    ), class = c("sdid_inference", "coresynth_inference")))
  }

  # ── Sharp path ───────────────────────────────────────────────────────────────
  if (method == "jackknife_global")
    stop("sdid_inference() method='jackknife_global' requires a staggered fit.",
         call. = FALSE)
  if (is.null(fit$Y_co_pre))
    stop("fit does not contain Y_co_pre. Re-estimate with the current version of ",
         "coresynth to enable inference.", call. = FALSE)

  N_co    <- ncol(fit$Y_co_pre)

  # ── Method dispatch ─────────────────────────────────────────────────────────
  if (method == "placebo") {
    effects <- drop(sdid_placebo_cpp(fit$Y_co_pre, fit$Y_co_post,
                                     fit$time_weights, fit$zeta2))
    names(effects) <- colnames(fit$Y_co_pre)

    p_value <- switch(alternative,
      two.sided = (1 + sum(abs(effects) >= abs(tau_hat))) / (N_co + 1),
      greater   = (1 + sum(effects >= tau_hat))           / (N_co + 1),
      less      = (1 + sum(effects <= tau_hat))           / (N_co + 1)
    )

    return(structure(list(
      estimate        = tau_hat,
      se              = NULL,
      p_value         = p_value,
      ci_lower        = NULL,
      ci_upper        = NULL,
      method          = method,
      n_controls      = N_co,
      alternative     = alternative,
      placebo_effects = effects,
      boot_ests       = NULL
    ), class = c("sdid_inference", "coresynth_inference")))
  }

  # bootstrap / jackknife share re-estimation via .fit_sdid_matrices()
  .refit <- function(Y_co_pre, Y_co_post) {
    .fit_sdid_matrices(
      Y_co_pre       = Y_co_pre,
      Y_co_post      = Y_co_post,
      Y_tr_pre_mean  = fit$Y_tr_pre_mean,
      Y_tr_post_mean = fit$Y_tr_post_mean,
      N_tr           = fit$N_tr,
      zeta2          = fit$zeta2
    )$estimate
  }

  if (method == "bootstrap") {
    if (!is.null(seed)) set.seed(seed)
    boot_ests <- replicate(n_boot, {
      idx <- sample(N_co, replace = TRUE)
      .refit(fit$Y_co_pre[, idx, drop = FALSE],
             fit$Y_co_post[, idx, drop = FALSE])
    })
    se       <- sd(boot_ests)
    ci_lower <- unname(quantile(boot_ests, alpha / 2))
    ci_upper <- unname(quantile(boot_ests, 1 - alpha / 2))
    z        <- tau_hat / max(se, .Machine$double.eps)
    p_value  <- switch(alternative,
      two.sided = 2 * pnorm(-abs(z)),
      greater   = pnorm(-z),
      less      = pnorm(z)
    )
    return(structure(list(
      estimate   = tau_hat,
      se         = se,
      p_value    = p_value,
      ci_lower   = ci_lower,
      ci_upper   = ci_upper,
      method     = method,
      n_controls = N_co,
      alternative = alternative,
      placebo_effects = NULL,
      boot_ests  = boot_ests
    ), class = c("sdid_inference", "coresynth_inference")))
  }

  # jackknife
  jack_ests <- vapply(seq_len(N_co), function(i) {
    .refit(fit$Y_co_pre[, -i, drop = FALSE],
           fit$Y_co_post[, -i, drop = FALSE])
  }, numeric(1))
  jack_var <- (N_co - 1) / N_co * sum((jack_ests - mean(jack_ests))^2)
  se       <- sqrt(jack_var)
  z        <- tau_hat / max(se, .Machine$double.eps)
  ci_lower <- tau_hat - qnorm(1 - alpha / 2) * se
  ci_upper <- tau_hat + qnorm(1 - alpha / 2) * se
  p_value  <- switch(alternative,
    two.sided = 2 * pnorm(-abs(z)),
    greater   = pnorm(-z),
    less      = pnorm(z)
  )
  structure(list(
    estimate    = tau_hat,
    se          = se,
    p_value     = p_value,
    ci_lower    = ci_lower,
    ci_upper    = ci_upper,
    method      = method,
    n_controls  = N_co,
    alternative = alternative,
    placebo_effects = NULL,
    boot_ests   = NULL
  ), class = c("sdid_inference", "coresynth_inference"))
}

#' @export
print.sdid_inference <- function(x, digits = 4, ...) {
  cat("SDID Inference (", x$method, ", alternative = ", x$alternative, ")\n",
      sep = "")
  cat("  Estimate  :", round(x$estimate, digits), "\n")
  if (!is.null(x$se))
    cat("  SE        :", round(x$se, digits), "\n")
  cat("  p-value   :", round(x$p_value, digits), "\n")
  if (!is.null(x$ci_lower))
    cat("  CI        : [", round(x$ci_lower, digits), ", ",
        round(x$ci_upper, digits), "]\n", sep = "")
  cat("  Controls  :", x$n_controls, "\n")
  invisible(x)
}

---
title: "Get started with coresynth"
output: rmarkdown::html_vignette
vignette: >
  %\VignetteIndexEntry{Get started with coresynth}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

```{r, include = FALSE}
knitr::opts_chunk$set(
  collapse = TRUE,
  comment  = "#>",
  fig.width = 7,
  fig.height = 4,
  dpi = 96
)
```

**coresynth** provides six causal-inference estimators for panel data behind a
single formula interface, with the computational core (QP solving, SVD, Kalman
filtering) written in C++ via RcppArmadillo. This vignette walks through the
basics: fitting a model, comparing methods, and pulling results out with
`broom` and `plot()`.

```{r setup}
library(coresynth)
```

## The unified formula

Every estimator is reached through `scm_fit()` with the same formula syntax:

```
outcome ~ treatment | unit_id + time_id
```

The data must be a **long-format** balanced panel (one row per unit–time), and
`treatment` is a 0/1 indicator that switches on for treated units in
post-treatment periods. `method =` selects the estimator.

## A toy panel

We simulate a balanced panel of 10 units over 20 periods. Unit `u1` is treated
from period 11 onward with a true ATT of 2.0.

```{r dgp}
set.seed(42)
N <- 10; TT <- 20; T_pre <- 10
f   <- cumsum(rnorm(TT, 0, 0.5))     # common factor
lam <- rnorm(N, 1, 0.3)              # unit loadings
dat <- expand.grid(time = seq_len(TT), id = paste0("u", seq_len(N)))
dat$y <- as.vector(outer(f, lam)) + rnorm(nrow(dat), 0, 0.3)
dat$d <- as.integer(dat$id == "u1" & dat$time > T_pre)
dat$y[dat$d == 1] <- dat$y[dat$d == 1] + 2.0   # inject the treatment effect

head(dat)
```

## Fitting one method

```{r fit-one}
fit <- scm_fit(y ~ d | id + time, data = dat, method = "scm")
fit
```

The estimated ATT lives in `fit$estimate`:

```{r estimate}
fit$estimate
```

## Comparing all six methods

Because the interface is shared, swapping estimators is a one-word change. Here
we run all six on the same data (true ATT = 2.0).

```{r compare}
methods <- c("scm", "sdid", "gsc", "mc", "tasc", "si")
fits    <- lapply(methods, function(m) scm_fit(y ~ d | id + time, data = dat, method = m))
names(fits) <- methods

data.frame(
  method   = methods,
  estimate = round(sapply(fits, `[[`, "estimate"), 3)
)
```

| Method | Reference |
|----|----|
| `scm`  | Abadie, Diamond & Hainmueller (2010) |
| `sdid` | Arkhangelsky et al. (2021) |
| `gsc`  | Xu (2017) |
| `mc`   | Athey et al. (2021) |
| `tasc` | Rho et al. (2026) |
| `si`   | Agarwal et al. (2025) |

## Visualizing a fit

`plot.coresynth()` offers three views via `type =`.

```{r plot-trend}
plot(fits$sdid, type = "trend")   # observed vs. synthetic
```

```{r plot-gap}
plot(fits$scm, type = "gap")      # treatment effect over time
```

```{r plot-weights}
plot(fits$scm, type = "weights")  # donor weights
```

## tidy / glance / augment

coresynth integrates with `broom`, so results drop straight into tidy
workflows and paper tables.

```{r broom}
library(broom)

tidy(fits$scm)     # donor weights as a data frame
glance(fits$scm)   # one-row model summary
```

`export_json()` writes a fit to disk as JSON for reproducibility or downstream
(e.g. AI) workflows:

```{r export, eval = FALSE}
export_json(fits$scm, file = "scm_result.json")
```

## Where to next

- **[Estimators](https://yo5uke.com/coresynth/articles/estimators.html)** —
  covariates, predictors, and method-specific options for all six estimators
  plus the experimental-design variant.
- **[Inference](https://yo5uke.com/coresynth/articles/inference.html)** —
  placebo, bootstrap, jackknife, parametric, and conformal inference.
- **[Staggered adoption](https://yo5uke.com/coresynth/articles/staggered.html)** —
  cohort-based estimation when units adopt treatment at different times.

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/RcppExports.R
\name{soft_impute_cpp}
\alias{soft_impute_cpp}
\title{Fast Matrix Completion using Soft-Impute Algorithm}
\usage{
soft_impute_cpp(Y, O, lambda, max_iter = 1000L, tol = 1e-05)
}
\arguments{
\item{Y}{Observed outcome matrix (N x T). Unobserved entries should be 0.}

\item{O}{Binary mask matrix (N x T): 1 = observed, 0 = missing (treated post).}

\item{lambda}{Nuclear norm penalty (soft-threshold on singular values).}

\item{max_iter}{Maximum iterations.}

\item{tol}{Convergence tolerance (relative Frobenius norm change).}
}
\value{
A numeric matrix of the same dimension as \code{Y} (N x T): the
completed low-rank matrix \code{L} that minimises the soft-thresholded
nuclear-norm objective.
}
\description{
Solves: min_L (1/2) ||O o (Y - L)||\emph{F^2 + lambda * ||L||}*
via iterative SVD soft-thresholding (Mazumder, Hastie, Tibshirani 2010).
Note: lambda is NOT normalized by |O|. Default lambda = 0.01 * sigma_max(Y).
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/sdid.R
\name{sdid_inference}
\alias{sdid_inference}
\title{Inference for Synthetic Difference-in-Differences}
\usage{
sdid_inference(
  fit,
  method = c("placebo", "bootstrap", "jackknife", "jackknife_global"),
  n_boot = 200L,
  level = 0.95,
  alternative = c("two.sided", "greater", "less"),
  seed = NULL
)
}
\arguments{
\item{fit}{A \code{coresynth} object with \code{method = "sdid"} (sharp adoption only).}

\item{method}{Inference method: \code{"placebo"} (permutation), \code{"bootstrap"}, or
\code{"jackknife"}.}

\item{n_boot}{Number of bootstrap replications (only for \code{method = "bootstrap"}).}

\item{level}{Confidence level for the interval (only for \code{method = "bootstrap"}
or \code{"jackknife"}).}

\item{alternative}{Direction of the alternative hypothesis: \code{"two.sided"},
\code{"greater"}, or \code{"less"}.}

\item{seed}{Integer seed for reproducibility (only for \code{method = "bootstrap"}).}
}
\value{
A list with:
\itemize{
\item \code{estimate}: The SDID point estimate.
\item \code{se}: Standard error (bootstrap / jackknife only).
\item \code{p_value}: Permutation or normal-approximation p-value.
\item \code{ci_lower}, \code{ci_upper}: Confidence interval bounds (bootstrap / jackknife).
\item \code{method}: The inference method used.
\item \code{n_controls}: Number of control units.
\item \code{alternative}: The alternative hypothesis direction.
\item \code{placebo_effects}: Named vector of LOO placebo effects (placebo only).
\item \code{boot_ests}: Bootstrap estimate distribution (bootstrap only).
}
}
\description{
Computes standard errors and p-values for a SDID estimate using one of
three methods: permutation placebo test (Algorithm 4), cluster bootstrap
(Algorithm 2), or leave-one-out jackknife (Algorithm 3), following
Clarke et al. (2023).
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/broom.R
\name{tidy.coresynth_inference}
\alias{tidy.coresynth_inference}
\title{Tidy an inference result}
\usage{
\method{tidy}{coresynth_inference}(x, conf.int = TRUE, ...)
}
\arguments{
\item{x}{A \code{coresynth_inference} (or \code{sdid_inference}) object returned by
\code{\link[=sdid_inference]{sdid_inference()}}, \code{\link[=gsc_inference]{gsc_inference()}}, or \code{\link[=si_inference]{si_inference()}}.}

\item{conf.int}{Logical. Include \code{conf.low}/\code{conf.high} columns when CI is
available (default \code{TRUE}). Permutation placebo SE/CI are \code{NA}.}

\item{...}{Unused.}
}
\value{
A one-row \code{data.frame} with columns \code{term}, \code{estimate}, \code{std.error},
\code{statistic}, \code{p.value}, \code{conf.low}, \code{conf.high}, \code{method}, \code{alternative},
\code{n_controls}, \code{staggered}.
}
\description{
Coerces a \code{coresynth_inference} or \code{sdid_inference} object to a one-row
tidy \code{data.frame} with broom-style column names so it can be combined
with regression output for paper tables.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/scm_fit.R
\name{scm_fit}
\alias{scm_fit}
\title{Fit a Synthetic Control Method Model}
\usage{
scm_fit(
  formula,
  data,
  method = c("scm", "sdid", "gsc", "mc", "tasc", "si"),
  predictors = NULL,
  covariates = NULL,
  v_selection = c("insample", "oos"),
  donor_mspe_threshold = Inf,
  lambda_pen = NULL,
  v_optim = c("coord_descent", "auto", "bfgs"),
  ...
)
}
\arguments{
\item{formula}{A \code{Formula} object, e.g. \code{y ~ D | unit + time}.}

\item{data}{A \code{data.frame} in \strong{long} format (one row per unit-time).}

\item{method}{One of \code{"scm"}, \code{"sdid"}, \code{"gsc"}, \code{"mc"}, \code{"tasc"}, \code{"si"}.}

\item{predictors}{A \code{list()} of \code{\link[=pred]{pred()}} specifications that define the
predictor matrix for SCM (see Abadie et al. 2010, S.2.3). Each \code{\link[=pred]{pred()}}
entry aggregates one or more variables over a time window. Pass \code{NULL}
(default) to use all pre-treatment outcome periods as predictors.
Applies to \code{method = "scm"} only.}

\item{covariates}{An optional named \code{list} of additional time-varying
covariates to partial out before estimation. Each element is a character
string naming a column in \code{data}. Supported for \code{method = "sdid"},
\code{"scm"}, and \code{"gsc"}.}

\item{v_selection}{V matrix selection method for \code{method = "scm"}.
\code{"insample"} (default) follows Abadie et al. (2010): V is chosen by
minimising in-sample pre-treatment MSPE. \code{"oos"} follows Abadie (2021)
S.3.2: the pre-treatment window is split in half; V is selected to minimise
MSPE on the validation half, then W is refit on the full window.}

\item{donor_mspe_threshold}{Donor pool filtering threshold (Abadie 2021 S.4).
For \code{method = "scm"} only. Each donor's individual pre-treatment MSPE
(using that donor alone as the counterfactual) is divided by the minimum
such MSPE across all donors. Donors whose ratio exceeds this threshold are
excluded from estimation. \code{Inf} (default) disables filtering.}

\item{lambda_pen}{Penalised SCM parameter (Abadie & L'Hour 2021, JASA).
For \code{method = "scm"} only. \code{NULL} (default) runs standard unpenalised SCM.
\code{"auto"} selects the penalty via out-of-sample pre-treatment MSPE on the
same validation window as \code{v_selection = "oos"}. A non-negative number
uses that value directly.}

\item{v_optim}{Outer V-optimisation method for \code{method = "scm"}.
\code{"coord_descent"} (default) uses the existing C++ coordinate descent with
11-point grid search -- fastest when \code{k = T_pre} is large (outcomes-only).
\code{"bfgs"} uses R's L-BFGS-B, which requires only O(k^2) inner QP calls and
is faster when \code{k} is small (e.g. external predictors with k <= 15).
\code{"auto"} selects \code{"bfgs"} when \code{k <= 15}, otherwise \code{"coord_descent"}.}

\item{...}{Additional arguments forwarded to the specific method
(e.g. \code{r}, \code{lambda}, \code{zeta2}).}
}
\value{
An object of classes \code{c("coresynth_<method>", "coresynth")}.
All methods return at minimum:
\itemize{
\item \code{method}: estimator name
\item \code{estimate}: average treatment effect (ATT)
\item \code{times}: time index vector
\item \code{T_pre}: number of pre-treatment periods
\item \code{Y_treat}: treated unit outcome series
\item \code{gap}: treatment effect series (Y_treat - counterfactual)
}
}
\description{
Unified formula interface for Synthetic Control and related causal
inference methods.  The formula syntax is:
}
\details{
\code{outcome ~ treatment | unit_id + time_id}
}
\examples{
# Synthetic balanced panel: 10 units over 20 periods, unit 1 treated
# after period 15.
set.seed(1)
panel <- expand.grid(unit = 1:10, year = 1:20)
panel$treated <- as.integer(panel$unit == 1 & panel$year > 15)
panel$gdp <- panel$unit + 0.5 * panel$year +
  rnorm(nrow(panel)) + 3 * panel$treated

fit <- scm_fit(gdp ~ treated | unit + year, data = panel, method = "sdid")
summary(fit)

\donttest{
# Visualise the estimated gap (requires ggplot2)
plot(fit, type = "gap")
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/scm_design.R
\name{plot.scm_design}
\alias{plot.scm_design}
\title{Plot an scm_design object}
\usage{
\method{plot}{scm_design}(x, type = c("outcome", "gap"), ...)
}
\arguments{
\item{x}{An \code{scm_design} object.}

\item{type}{\code{"outcome"} (default): synthetic treated vs synthetic control
outcome series over all periods. \code{"gap"}: estimated treatment effect in the
experimental periods, with split-conformal confidence intervals.}

\item{...}{Currently ignored.}
}
\value{
A \code{ggplot} object: for \code{type = "outcome"}, the synthetic treated and
synthetic control outcome series; for \code{type = "gap"}, the estimated
treatment effect over the experimental periods with split-conformal
confidence intervals. The object is returned for printing or further
customisation.
}
\description{
Plot an scm_design object
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/RcppExports.R
\name{tensor_unfold_cpp}
\alias{tensor_unfold_cpp}
\title{Tensor Unfolding (Matricization) for Synthetic Interventions}
\usage{
tensor_unfold_cpp(T_cube, mode)
}
\arguments{
\item{T_cube}{A 3D array (cube) of dimensions (n1, n2, n3)}

\item{mode}{The mode to unfold along (1, 2, or 3)}
}
\value{
A numeric matrix: the mode-\code{mode} unfolding (matricization) of
\code{T_cube}, with dimensions \verb{n1 x (n2 * n3)}, \verb{n2 x (n1 * n3)}, or
\verb{n3 x (n1 * n2)} for \code{mode} 1, 2, or 3 respectively.
}
\description{
Tensor Unfolding (Matricization) for Synthetic Interventions
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/RcppExports.R
\name{scm_inner_weights_cpp}
\alias{scm_inner_weights_cpp}
\title{SCM Inner Weights (QP Given V)}
\usage{
scm_inner_weights_cpp(X0, X1, V_diag)
}
\arguments{
\item{X0}{Covariate matrix for control units (k x N_co)}

\item{X1}{Covariate vector for the treated unit (k x 1)}

\item{V_diag}{Diagonal of the metric matrix V (k x 1, non-negative, need not sum to 1)}
}
\value{
Donor weight vector W (N_co x 1) on the unit simplex
}
\description{
Solves the inner-loop QP for SCM: given a fixed diagonal metric matrix V,
finds donor weights W on the simplex minimising the V-weighted covariate loss.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/gsc.R
\name{gsc_boot}
\alias{gsc_boot}
\title{Parametric Bootstrap Inference for GSC (Xu 2017 §3)}
\usage{
gsc_boot(fit, B = 499L, alpha = 0.05, seed = NULL)
}
\arguments{
\item{fit}{A \code{coresynth} object from \code{\link[=scm_fit]{scm_fit()}} with \code{method = "gsc"}.}

\item{B}{Bootstrap replications (default 499L).}

\item{alpha}{Significance level for the confidence interval (default 0.05).}

\item{seed}{RNG seed for reproducibility (default NULL).}
}
\value{
A list with:
\itemize{
\item \code{p_value}:   Two-sided p-value: mean(|ATT*| >= |ATT_obs|)
\item \code{ci_lower}:  Lower bound of (1-alpha)*100\% bootstrap CI
\item \code{ci_upper}:  Upper bound of (1-alpha)*100\% bootstrap CI
\item \code{se}:        Bootstrap standard error
\item \code{boot_dist}: Numeric vector of length B (bootstrap ATT* values)
\item \code{att_obs}:   Observed ATT from the original fit
}
}
\description{
Generates the null distribution of the ATT under H0 (no treatment effect)
by parametric resampling from the estimated IFE factor model. Under H0,
both the control panel and treated unit are generated from the fitted
factor model with homoskedastic noise. When the fit includes covariate
adjustment (beta), the covariate contribution is included in the simulated
DGP and re-estimated in each bootstrap replicate.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/RcppExports.R
\name{scm_placebo_cpp}
\alias{scm_placebo_cpp}
\title{Fast Leave-One-Out Placebo Test for SCM (Abadie et al. 2010)}
\usage{
scm_placebo_cpp(Y_pre, Y_post, max_iter = 100L, tol = 1e-04)
}
\arguments{
\item{Y_pre}{Control pre-treatment outcomes (T_pre x N_co)}

\item{Y_post}{Control post-treatment outcomes (T_post x N_co)}

\item{max_iter}{Outer coordinate-descent iterations (default 100)}

\item{tol}{Convergence tolerance for V updates (default 1e-4)}
}
\value{
A list with:
\itemize{
\item \code{mspe_pre}:  N_co-vector of pre-treatment MSPE per placebo unit
\item \code{mspe_post}: N_co-vector of post-treatment MSPE per placebo unit
\item \code{effects}:   N_co-vector of mean post-period gap per placebo unit
}
}
\description{
For each control unit, treats it as pseudo-treated and fits SCM weights
from the remaining N_co-1 donors. Returns MSPE components for constructing
MSPE-ratio permutation p-values in R.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/RcppExports.R
\name{si_pcr_cpp}
\alias{si_pcr_cpp}
\title{SI-PCR: Synthetic Interventions via Principal Component Regression}
\usage{
si_pcr_cpp(Y_pre_co, Y_post_co, Y_pre_tr, k)
}
\arguments{
\item{Y_pre_co}{Pre-treatment control outcomes (T_pre x N_co)}

\item{Y_post_co}{Post-treatment control outcomes (T_post x N_co)}

\item{Y_pre_tr}{Pre-treatment treated outcomes (T_pre x N_tr)}

\item{k}{Number of SVD components to retain}
}
\value{
A list with:
\itemize{
\item \code{W}: Donor weight matrix (N_co x N_tr)
\item \code{Y_hat}: Counterfactual post-treatment outcomes (T_post x N_tr)
}
}
\description{
Implements the SI-PCR estimator of Agarwal et al. (2025).
Uses the top-k SVD of pre-treatment control outcomes to find donor
weights that predict each treated unit's pre-treatment trajectory,
then applies those weights to post-treatment control outcomes.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/coresynth-package.R
\docType{package}
\name{coresynth-package}
\alias{coresynth}
\alias{coresynth-package}
\title{coresynth: Fast and Unified Synthetic Control Methods}
\description{
\if{html}{\figure{logo.png}{options: style='float: right' alt='logo' width='120'}}

A unified 'Formula' interface to the Synthetic Control Method (SCM) and related panel-data causal inference estimators: Synthetic Difference-in-Differences (SDID), Generalized Synthetic Control (GSC), Matrix Completion (MC), Time-Aware Synthetic Control (TASC), and Synthetic Interventions (SI), together with an experimental-design variant. Computational bottlenecks (quadratic programming, singular value decomposition, and Kalman filtering) are implemented in 'C++' via 'RcppArmadillo'. Methods are described in Abadie, Diamond and Hainmueller (2010) \doi{10.1198/jasa.2009.ap08746}, Arkhangelsky, Athey, Hirshberg, Imbens and Wager (2021) \doi{10.1257/aer.20190159}, Xu (2017) \doi{10.1017/pan.2016.2}, Athey, Bayati, Doudchenko, Imbens and Khosravi (2021) \doi{10.1080/01621459.2021.1891924}, and Agarwal, Shah and Shen (2025) \doi{10.1287/opre.2025.1590}.
}
\seealso{
Useful links:
\itemize{
  \item \url{https://github.com/yo5uke/coresynth}
  \item \url{https://yo5uke.com/coresynth/}
  \item Report bugs at \url{https://github.com/yo5uke/coresynth/issues}
}

}
\author{
\strong{Maintainer}: Yosuke Abe \email{yosuke.abe0507@gmail.com}

Authors:
\itemize{
  \item Yosuke Abe \email{yosuke.abe0507@gmail.com}
}

}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/export.R
\name{export_json}
\alias{export_json}
\title{Export coresynth Results to JSON}
\usage{
export_json(x, file = "coresynth_results.json", inference = NULL, digits = 6L)
}
\arguments{
\item{x}{A \code{coresynth} object from \code{\link[=scm_fit]{scm_fit()}}.}

\item{file}{Output file path. Default \code{"coresynth_results.json"}. Pass
\code{NULL} to skip writing and return the R list invisibly.}

\item{inference}{Optional list from \code{\link[=mspe_ratio_pval]{mspe_ratio_pval()}} or \code{\link[=gsc_boot]{gsc_boot()}}.
When provided, populates the \code{inference} section and updates \code{estimate}
with \code{p_value}, \code{se}, \code{ci_lower}, \code{ci_upper}.}

\item{digits}{Number of significant digits applied to numeric values
(default 6L).}
}
\value{
Invisibly, the R list that was (or would be) serialized.
}
\description{
Generates a comprehensive, standardized JSON record covering all six
estimators. Suitable for reproducibility workflows (Xu & Yang 2026) and
downstream tooling. Pass the result of \code{\link[=mspe_ratio_pval]{mspe_ratio_pval()}} or \code{\link[=gsc_boot]{gsc_boot()}}
via the \code{inference} argument to include inference results.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/RcppExports.R
\name{sdid_unit_weights_cpp}
\alias{sdid_unit_weights_cpp}
\title{Calculate SDID Unit Weights (omega)}
\usage{
sdid_unit_weights_cpp(Y_pre, Y_tr_pre, zeta2)
}
\arguments{
\item{Y_pre}{Pre-treatment outcome matrix for control units (T_pre x N_co)}

\item{Y_tr_pre}{Pre-treatment outcome vector for treated unit (T_pre x 1), averaged if multiple}

\item{zeta2}{Ridge penalty parameter (zeta^2). The code internally multiplies by T_pre per the paper.}
}
\value{
A numeric vector of length \code{N_co} holding the SDID unit weights
\code{omega} (non-negative and summing to one).
}
\description{
Solves the regularized QP:
min over omega in Delta: sum_t (sum_i omega_i Y_it - Y_tr_t)^2 + zeta^2 * T_pre * ||omega||^2
}
\details{
This corresponds to equation (5) in Arkhangelsky et al. (2021).
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/scm.R
\name{mspe_ratio_pval}
\alias{mspe_ratio_pval}
\title{Permutation Inference via MSPE Ratio for SCM}
\usage{
mspe_ratio_pval(
  fit,
  mspe_threshold = 0,
  max_iter = 100L,
  tol = 1e-04,
  use_covariates = FALSE,
  alternative = c("two.sided", "greater", "less")
)
}
\arguments{
\item{fit}{A \code{coresynth} object from \code{\link[=scm_fit]{scm_fit()}} with \code{method = "scm"}.}

\item{mspe_threshold}{Minimum pre-treatment MSPE for including a control
unit in the two-sided test. Ignored for one-sided tests.
Default: 0 (no filtering).}

\item{max_iter}{Passed to \code{\link[=scm_placebo_cpp]{scm_placebo_cpp()}}. Default 100L.}

\item{tol}{Passed to \code{\link[=scm_placebo_cpp]{scm_placebo_cpp()}}. Default 1e-4.}

\item{use_covariates}{If \code{TRUE} and the fit used predictor covariates,
applies the same covariate spec to each placebo unit (R-level loop).
Default \code{FALSE} (faster C++ outcomes-only placebos).}

\item{alternative}{Direction of the alternative hypothesis:
\code{"two.sided"} (default) uses the MSPE ratio statistic;
\code{"greater"} tests whether the treatment increased the outcome;
\code{"less"} tests whether the treatment decreased the outcome.
One-sided tests use the signed ATT as the test statistic.}
}
\value{
A list with:
\itemize{
\item \code{p_value}: Permutation p-value between 0 and 1
\item \code{mspe_ratio_treated}: MSPE_post / MSPE_pre for the treated unit (two.sided only)
\item \code{mspe_ratios_all}: Named numeric vector (treated first, then controls); two.sided only
\item \code{placebo_effects}: Named N_co-vector of placebo ATT estimates
\item \code{treated_effect}: ATT estimate for the treated unit
\item \code{n_placebo_used}: Number of control units used
}
}
\description{
Computes the Abadie et al. (2010) / Abadie (2021) permutation p-value.
For each control unit, a leave-one-out synthetic control is fitted.
}
\details{
When \code{alternative = "two.sided"} (default), the test statistic is the
post/pre MSPE ratio, following Abadie et al. (2010). When
\code{alternative = "greater"} or \code{"less"}, the test statistic is the signed
average post-treatment gap (ATT), giving a one-sided permutation test as
recommended by Abadie (2021) S.3.5 for improved power when the direction
of the treatment effect is known.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/broom.R
\name{glance.coresynth_inference}
\alias{glance.coresynth_inference}
\title{Glance at an inference result}
\usage{
\method{glance}{coresynth_inference}(x, ...)
}
\arguments{
\item{x}{An inference object.}

\item{...}{Unused.}
}
\value{
A one-row \code{data.frame} with columns \code{method}, \code{n_controls},
\code{staggered}, \code{estimate}, \code{std.error}, \code{p.value}, \code{conf.low},
\code{conf.high}, \code{alternative}, \code{n_boot_valid}.
}
\description{
One-row summary of a \code{coresynth_inference} (or \code{sdid_inference}) object.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/scm_design.R
\name{scm_design}
\alias{scm_design}
\title{Experimental Synthetic Control Design}
\usage{
scm_design(
  data,
  outcome,
  unit,
  time,
  T0,
  T_fit = NULL,
  m_min = 1L,
  m_max = 1L,
  f = NULL,
  predictors = NULL,
  design = c("base", "weakly_targeted", "unit_level"),
  beta = 1,
  xi = 1,
  alpha = 0.05,
  normalize = TRUE,
  max_subsets = 100000L
)
}
\arguments{
\item{data}{Long-format data frame (one row per unit–time).}

\item{outcome}{Name of the outcome column.}

\item{unit}{Name of the unit identifier column.}

\item{time}{Name of the time identifier column.}

\item{T0}{Last pre-experimental period (a value present in the time
column).  Periods after \code{T0} are the experimental periods.}

\item{T_fit}{Number of fitting periods, counted from the \strong{start} of
the pre-experimental phase.  Defaults to \code{NULL}, which uses all
pre-experimental periods for fitting (no blank periods; inference
disabled).  When \code{T_fit} is smaller than the total number of
pre-experimental periods, the remaining periods become blank periods used
for inference.}

\item{m_min}{Minimum number of units assigned to treatment (default 1).}

\item{m_max}{Maximum number of units assigned to treatment (default 1).}

\item{f}{Named numeric vector of population weights \eqn{f_j}.
Defaults to uniform weights \eqn{1/J}.  Will be normalised to sum to 1.}

\item{predictors}{A \code{list()} of \code{\link[=pred]{pred()}} specifications that define the
predictor matrix \eqn{X_j}.  Defaults to \code{NULL}, which uses all
fitting-period outcome values as predictors.}

\item{design}{Design formulation: \code{"base"} (default), \code{"weakly_targeted"},
or \code{"unit_level"}.}

\item{beta}{Trade-off parameter \eqn{\beta > 0} for the Weakly
targeted design (default 1).}

\item{xi}{Trade-off parameter \eqn{\xi > 0} for the Unit-level
design (default 1).}

\item{alpha}{Significance level for confidence intervals (default 0.05).}

\item{normalize}{If \code{TRUE} (default), each row of the predictor matrix is
divided by its cross-unit standard deviation before optimisation, so
predictors measured on different scales contribute equally.}

\item{max_subsets}{Maximum number of treatment-set candidates to evaluate
before switching to random sampling (default 100 000).}
}
\value{
An object of class \code{"scm_design"} with components:
\itemize{
\item \code{treated_units}: unit identifiers selected for treatment
\item \code{control_units}: unit identifiers in the control pool
\item \code{w}: J-length weight vector for the synthetic treated unit (sums to 1)
\item \code{v}: J-length weight vector for the synthetic control unit (sums to 1)
\item \code{tau_hat}: estimated treatment effects for each experimental period
\item \code{p_value}: permutation p-value (NA when blank periods are unavailable)
\item \code{ci_lower}, \code{ci_upper}: per-period split-conformal confidence interval
\item \code{Y_synth_tr}, \code{Y_synth_co}: synthetic treated/control series (all periods)
\item \code{estimate}: ATT (mean of \code{tau_hat})
}
}
\description{
Selects which units to assign to the treatment arm (and which to the control
arm) in a planned experiment, following Abadie and Zhao (2026).  Both sets
of units are chosen by minimising the distance between their weighted-average
predictor vectors and the population-average predictor vector \eqn{\bar{X}},
so the resulting estimates are less susceptible to post-randomisation bias
than pure random assignment.
}
\details{
Three design formulations are available:
\itemize{
\item \strong{\code{"base"}} (eq. 7): both the synthetic treated and the synthetic control
independently target the population average \eqn{\bar{X}}.
\item \strong{\code{"weakly_targeted"}} (eq. 9): the synthetic treated targets
\eqn{\bar{X}}; the synthetic control targets the synthetic treated predictor
vector (controlled by \code{beta}).
\item \strong{\code{"unit_level"}} (eq. 10): each treated unit gets its own synthetic
control; the aggregate control weight is a convex combination (controlled
by \code{xi}).
}

Inference uses "blank periods" — pre-experimental periods whose outcomes were
\emph{not} used to estimate the weights.  Set \code{T_fit} strictly smaller than the
number of pre-experimental periods to enable the permutation test and split-
conformal confidence intervals from Section 3 of Abadie and Zhao (2026).
}
\references{
Abadie, A. and Zhao, J. (2026). "Synthetic Controls for
Experimental Design." MIT Working Paper.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/plot.R
\name{plot.coresynth}
\alias{plot.coresynth}
\title{Plot a coresynth model}
\usage{
\method{plot}{coresynth}(x, type = c("trend", "gap", "weights"), ...)
}
\arguments{
\item{x}{A \code{coresynth} object.}

\item{type}{One of \code{"trend"} (observed vs synthetic), \code{"gap"} (ATT over time),
or \code{"weights"} (donor unit weight bar chart).}

\item{...}{Ignored.}
}
\value{
A \code{ggplot2} plot object.
}
\description{
Plot a coresynth model
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/RcppExports.R
\name{sdid_estimate_cpp}
\alias{sdid_estimate_cpp}
\title{Calculate SDID Estimate (tau_sdid)}
\usage{
sdid_estimate_cpp(Y_pre_co, Y_post_co, Y_pre_tr, Y_post_tr, omega, lambda)
}
\arguments{
\item{Y_pre_co}{Control pre-treatment outcomes (T_pre x N_co)}

\item{Y_post_co}{Control post-treatment outcomes (T_post x N_co)}

\item{Y_pre_tr}{Treated pre-treatment outcomes (T_pre x 1)}

\item{Y_post_tr}{Treated post-treatment outcomes (T_post x 1)}

\item{omega}{Unit weights (N_co x 1)}

\item{lambda}{Time weights (T_pre x 1)}
}
\value{
A single numeric value: the SDID treatment-effect estimate
\code{tau_sdid}.
}
\description{
Given unit weights omega and time weights lambda, computes the SDID
estimator as a weighted two-way difference:
}
\details{
tau_sdid = (Y_tr_post_mean - Y_tr_pre_wt) - (Y_co_post_wt - Y_co_pre_wt)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utils.R
\name{pred}
\alias{pred}
\title{Predictor Specification for SCM}
\usage{
pred(vars, times, op = "mean")
}
\arguments{
\item{vars}{Character vector of variable names. All variables share the
same \code{times} window and \code{op} operator. Use separate \code{pred()} calls for
variables with different time windows.}

\item{times}{Numeric/integer vector of time values to aggregate over.}

\item{op}{Aggregation operator applied to each variable over \code{times}.
One of \code{"mean"} (default), \code{"median"}, or \code{"sum"}.}
}
\value{
A \code{pred_spec} object (a named list with class \code{"pred_spec"}).
}
\description{
Creates a single predictor specification for use in \code{\link[=scm_fit]{scm_fit()}} with
\code{method = "scm"}. Pass a \code{list()} of \code{pred()} calls as the \code{predictors}
argument to define the full covariate matrix.
}
\examples{
# Three variables averaged over the same window
pred(c("lnincome", "retprice", "age15to24"), 1980:1988)

# Single variable at a specific year
pred("cigsale", 1975)

# Single variable averaged over a range
pred("beer", 1984:1988)

# Abadie, Diamond & Hainmueller (2010) California Prop 99 style: combine
# several covariates aggregated over different windows plus three outcome
# lags at specific years. The resulting list is passed to
# scm_fit(..., predictors = predictors).
predictors <- list(
  pred(c("lnincome", "retprice", "age15to24"), 1980:1988),
  pred("beer",    1984:1988),
  pred("cigsale", 1988),
  pred("cigsale", 1980),
  pred("cigsale", 1975)
)
predictors
}
\seealso{
\code{\link[=scm_fit]{scm_fit()}} for the \code{predictors} argument that consumes a \code{list()}
of \code{pred_spec} objects.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/si.R
\name{si_inference}
\alias{si_inference}
\title{Non-parametric Inference for SI (Agarwal et al. 2025)}
\usage{
si_inference(
  fit,
  method = c("bootstrap", "jackknife", "jackknife_global"),
  n_boot = 499L,
  level = 0.95,
  alternative = c("two.sided", "greater", "less"),
  seed = NULL
)
}
\arguments{
\item{fit}{A \code{coresynth} object from \code{\link[=scm_fit]{scm_fit()}} with \code{method = "si"}.}

\item{method}{\code{"bootstrap"} (default) or \code{"jackknife"}.}

\item{n_boot}{Number of bootstrap replications (default 499L; ignored for jackknife).}

\item{level}{Confidence level (default 0.95).}

\item{alternative}{\code{"two.sided"} (default), \code{"greater"}, or \code{"less"}.}

\item{seed}{RNG seed for reproducibility (default NULL).}
}
\value{
A list of class \code{coresynth_inference}.
}
\description{
Estimates SE and confidence intervals for the ATT via non-parametric cluster
bootstrap or jackknife over control units. Works for both sharp and staggered
SI fits. For staggered fits, bootstrap resamples each cohort's control pool
independently, and jackknife uses a per-cohort LOO with delta-method variance
aggregation.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/RcppExports.R
\name{gsc_ife_cpp}
\alias{gsc_ife_cpp}
\title{Fast Interactive Fixed Effects (IFE) for Generalized Synthetic Control}
\usage{
gsc_ife_cpp(Y_co, Y_tr_pre, r, X_co, X_tr_pre, max_iter = 50L, tol = 1e-06)
}
\arguments{
\item{Y_co}{Control units outcome matrix (T x N_co)}

\item{Y_tr_pre}{Treated units pre-treatment outcomes (T_pre x N_tr)}

\item{r}{Number of latent factors (must be <= min(T, N_co))}

\item{X_co}{Time-varying covariate cube (T x N_co x p). Pass an empty
cube (0 slices) for the covariate-free estimator.}

\item{X_tr_pre}{Time-varying covariate cube for treated units in the
pre-treatment window (T_pre x N_tr x p). Required for
correct Step 2 loading estimation per Xu (2017): lambda_hat
is estimated from Y_tr_pre - X_tr_pre * beta (covariate-
demeaned). Pass an empty cube (0 slices) to skip demeaning
(backward-compatible, but biased when beta != 0).}

\item{max_iter}{Maximum EM iterations (default 50)}

\item{tol}{Convergence tolerance on relative beta change (default 1e-6)}
}
\value{
A list with components:
\itemize{
\item \code{F}: estimated time factors (T x r).
\item \code{L_co}: control-unit factor loadings (N_co x r).
\item \code{L_tr}: treated-unit factor loadings (N_tr x r).
\item \code{Y_tr_hat}: estimated treated-unit counterfactual outcomes (T x N_tr).
\item \code{singular_values}: singular values from the final truncated SVD.
\item \code{beta}: estimated covariate coefficients (p x 1), empty when no
covariates are supplied.
}
}
\description{
Implements Xu (2017) IFE model with optional covariate adjustment.
When X_co has p > 0 slices, runs an EM loop alternating between:
E-step: truncated SVD of Y_tilde = Y_co - X_co * beta
M-step: panel OLS to update beta given current factors
When X_co has 0 slices (default), falls back to the plain 3-step estimator.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/RcppExports.R
\name{sdid_placebo_cpp}
\alias{sdid_placebo_cpp}
\title{Fast Placebo Test for SDID}
\usage{
sdid_placebo_cpp(Y_pre, Y_post, time_weights, zeta2)
}
\arguments{
\item{Y_pre}{Control units pre-treatment outcomes (T_pre x N_co)}

\item{Y_post}{Control units post-treatment outcomes (T_post x N_co)}

\item{time_weights}{Lambda weights for pre-treatment periods (T_pre x 1)}

\item{zeta2}{Ridge penalty (same as used in the main estimate)}
}
\value{
A numeric vector of length \code{N_co}. Each element is the
leave-one-out placebo SDID effect obtained by treating that control unit
as the pseudo-treated unit; the vector serves as a permutation-based null
distribution for inference.
}
\description{
For each control unit, treats it as the "pseudo-treated" unit and
estimates the leave-one-out SDID effect. The distribution of these
placebo effects provides a permutation-based null distribution for inference.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/RcppExports.R
\name{sdid_time_weights_cpp}
\alias{sdid_time_weights_cpp}
\title{Calculate SDID Time Weights (lambda)}
\usage{
sdid_time_weights_cpp(Y_pre_co, Y_post_target, zeta_t)
}
\arguments{
\item{Y_pre_co}{Pre-treatment outcomes for control units, row-demeaned (T_pre x N_co)}

\item{Y_post_target}{Post-treatment mean per control unit, demeaned (N_co x 1)}

\item{zeta_t}{Ridge penalty for time weights (paper: 1e-6 * sigma_hat)}
}
\value{
A numeric vector of length \code{T_pre} holding the SDID time weights
\code{lambda} (non-negative and summing to one).
}
\description{
Solves the time-weight QP (with implicit intercept lambda_0 concentrated out):
}
\details{
min over lambda in Delta_pre: ||Y_post_target - Y_pre_co^T lambda||^2 + zeta_t^2 * N_co * ||lambda||^2

The caller is responsible for pre-demeaning Y_pre_co (row-wise) and
Y_post_target (subtract the cross-unit mean) to concentrate out lambda_0,
as described in Arkhangelsky et al. (2021) Algorithm 1, Eq. (2.3).
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/scm.R
\name{augment_scm}
\alias{augment_scm}
\title{Augmented Synthetic Control Method (Ridge ASCM)}
\usage{
augment_scm(fit, lambda_ridge = NULL)
}
\arguments{
\item{fit}{A \code{coresynth} object from \code{\link[=scm_fit]{scm_fit()}} with \code{method = "scm"}.}

\item{lambda_ridge}{Ridge penalty (non-negative). \code{NULL} (default) selects
the penalty by leave-one-out cross-validation on the control units.}
}
\value{
A list with:
\itemize{
\item \code{att_aug}: Augmented ATT estimate
\item \code{delta}: Bias correction term (m_tr_post - sum_j W_j m_j_post)
\item \code{att_scm}: Original SCM ATT for comparison
\item \code{lambda_ridge}: Ridge penalty used
\item \code{beta_hat}: Ridge regression coefficients (length T_pre)
}
}
\description{
Applies a ridge-regression-based bias correction to a fitted SCM object,
following Ben-Michael, Feller & Rothstein (2021, JASA). The corrected
estimator is:
}
\details{
tau_aug = tau_SCM + (m_tr_post - sum_j W_j * m_j_post)

where m_i_post = Y_pre_i' beta_hat is the ridge outcome model prediction
for unit i's mean post-treatment outcome, and beta_hat is estimated by
ridge regression across control units.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/conformal.R
\name{conformal_inference}
\alias{conformal_inference}
\title{Conformal Inference for Synthetic Control Estimators}
\usage{
conformal_inference(
  fit,
  tau0 = 0,
  q = 1,
  alternative = c("two.sided", "greater", "less"),
  ci = TRUE,
  level = 0.95,
  grid = NULL,
  n_grid = 200L,
  grid_mult = 4,
  ...
)
}
\arguments{
\item{fit}{A \code{coresynth} object from \code{\link[=scm_fit]{scm_fit()}}.}

\item{tau0}{Null value of the ATT for the reported p-value (default 0).}

\item{q}{Exponent of the \eqn{S_q} test statistic
(\verb{S_q = (T_post^\{-1\} \\sum |u_t|^q)^\{1/q\}}). Default 1, robust to
heavy-tailed data (CWZ 2021). Used only for \code{alternative = "two.sided"};
one-sided tests use the signed mean post-treatment residual.}

\item{alternative}{\code{"two.sided"} (default), \code{"greater"}, or \code{"less"}.}

\item{ci}{Logical; construct a confidence interval by test inversion
(default \code{TRUE}).}

\item{level}{Confidence level for the interval (default 0.95).}

\item{grid}{Optional numeric vector of candidate \eqn{\tau_0} values for test
inversion. When \code{NULL} (default), a grid of \code{n_grid} points is centred on
the point estimate with half-width \code{grid_mult} times the pre-treatment
residual standard deviation.}

\item{n_grid}{Number of grid points when \code{grid = NULL} (default 200).}

\item{grid_mult}{Half-width multiplier when \code{grid = NULL} (default 4).}

\item{...}{Unused.}
}
\value{
A list of class \code{c("conformal_inference", "coresynth_inference")}
with \code{estimate}, \code{se} (\code{NA}; conformal has no SE), \code{p_value} (at \code{tau0}),
\code{ci_lower}, \code{ci_upper}, \code{method} (\code{"conformal"}), \code{n_controls},
\code{alternative}, \code{staggered} (\code{FALSE}), plus \code{tau0}, \code{q}, \code{grid}, and
\code{p_grid} (p-values along the grid). Compatible with \code{\link[generics:tidy]{tidy()}} / \code{\link[generics:glance]{glance()}}.
}
\description{
Implements the permutation-based conformal inference procedure of
Chernozhukov, Wuthrich & Zhu (2021, JASA). The test inverts a sharp null
\eqn{H_0: \tau = \tau_0} by imputing the treated post-treatment
counterfactual as \eqn{Y_{1t} - \tau_0}, re-estimating the counterfactual
proxy on \strong{all} \eqn{T} periods (imposing the null), and computing a
moving-block permutation p-value from the estimated residuals. A confidence
interval is obtained by test inversion over a grid of candidate \eqn{\tau_0}.
}
\details{
Supported for \strong{sharp} (single-cohort) fits with \code{method} in
\code{c("scm", "sdid", "gsc", "mc", "si")}. Staggered, multi-arm, and \code{tasc}
fits are not supported (use \code{sdid_inference()}, \code{gsc_inference()}, or
\code{si_inference()} instead).
}
\references{
Chernozhukov, V., Wuthrich, K., & Zhu, Y. (2021). An Exact and
Robust Conformal Inference Method for Counterfactual and Synthetic
Controls. \emph{Journal of the American Statistical Association}, 116(536),
1849-1864.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/RcppExports.R
\name{scm_weights_cpp}
\alias{scm_weights_cpp}
\title{SCM Outer Weights (Joint Optimization of W and V)}
\usage{
scm_weights_cpp(X0, X1, Z0, Z1, max_iter = 100L, tol = 1e-04, t_train = -1L)
}
\arguments{
\item{X0}{Covariate matrix for control units (k x N_co, typically pre-treatment outcomes)}

\item{X1}{Covariate vector for the treated unit (k x 1)}

\item{Z0}{Outcome matrix for control units in the pre-treatment window (T_pre x N_co)}

\item{Z1}{Outcome vector for the treated unit in the pre-treatment window (T_pre x 1)}

\item{max_iter}{Maximum coordinate-descent iterations (default 100)}

\item{tol}{Convergence tolerance on MSPE improvement (default 1e-4)}

\item{t_train}{Training window length for out-of-sample V selection.
-1 (default): in-sample V selection (original behaviour).
Positive: use rows 0..(t_train-1) of Z for fitting W, rows t_train..(T_pre-1)
as the validation window for V selection, then refit W on full data.}
}
\value{
A list with:
\itemize{
\item \code{W}: Donor weight vector (N_co x 1) on the unit simplex
\item \code{V}: Optimal metric diagonal (k x 1, normalised to sum to 1)
\item \code{loss}: Final pre-treatment prediction loss (full pre-treatment window)
}
}
\description{
Jointly optimises donor weights W (on the simplex) and the diagonal
metric matrix V via coordinate descent on the pre-treatment prediction
MSPE, following Abadie, Diamond & Hainmueller (2010).
}
\details{
When \code{t_train > 0}, uses out-of-sample V selection per Abadie (2021)
§3.2: V is selected by minimising MSPE on a validation window
(rows t_train..T_pre-1 of Z), while W is fitted on the training window
(rows 0..t_train-1 of X when X and Z have the same row count, i.e. the
outcomes-only case). After selecting V*, W is refit on the full data.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/RcppExports.R
\name{kalman_smoother_cpp}
\alias{kalman_smoother_cpp}
\title{Kalman Filter and RTS Smoother (TASC)}
\usage{
kalman_smoother_cpp(Y, W, A, C, Q, R, z0, P0)
}
\arguments{
\item{Y}{Observed data matrix (N x T). Use NA for unobserved entries.}

\item{W}{Observation / loading matrix (N x r)}

\item{A}{State transition matrix (r x r). Pass diag(r) for random-walk dynamics.}

\item{C}{State drift vector (r x 1)}

\item{Q}{State noise covariance (r x r)}

\item{R}{Observation noise covariance (N x N, diagonal in practice)}

\item{z0}{Initial state mean (r x 1)}

\item{P0}{Initial state covariance (r x r)}
}
\value{
A list with z_smooth, P_smooth, P_cross, z_pred, z_upd.
P_cross is an r x r x (T-1) cube. Slice t (C++ 0-indexed, t=0,...,T-2)
stores P(t+1, t | T) (0-indexed), i.e. P(t+2, t+1 | T) in 1-indexed
Shumway-Stoffer notation. Formula: P(t+1|T) * J_t^T (eq. 6.68-6.69).
}
\description{
Implements the Kalman filter (forward pass) and Rauch-Tung-Striebel smoother
(backward pass) for the state-space model in Rho et al. (2026):
}
\details{
State:       z(t+1) = A z(t) + C + eta(t),  eta(t) ~ N(0, Q)
Observation: y_t    = W z_t + eps_t,        eps_t  ~ N(0, R)

Observation rows with NA (treated post-intervention) are automatically
dropped at each time step so only control-unit rows update the filter.

The P update uses the numerically stable Joseph form:
P(t|t) = (I - K W_obs) P(t|t-1) (I - K W_obs)^T + K R_obs K^T
}
